@@ -226,6 +226,7 @@ namespace Magick
   using MagickCore::RLECompression;
   using MagickCore::ZipCompression;
 
+  // Dispose methods
   using MagickCore::DisposeType;
   using MagickCore::UndefinedDispose;
   using MagickCore::NoneDispose;
@@ -498,6 +499,26 @@ namespace Magick
   using MagickCore::OverlineDecoration;
   using MagickCore::LineThroughDecoration;
 
+  // Virtual pixel methods
+  using MagickCore::VirtualPixelMethod;
+  using MagickCore::UndefinedVirtualPixelMethod;
+  using MagickCore::BackgroundVirtualPixelMethod;
+  using MagickCore::DitherVirtualPixelMethod;
+  using MagickCore::EdgeVirtualPixelMethod;
+  using MagickCore::MirrorVirtualPixelMethod;
+  using MagickCore::RandomVirtualPixelMethod;
+  using MagickCore::TileVirtualPixelMethod;
+  using MagickCore::TransparentVirtualPixelMethod;
+  using MagickCore::MaskVirtualPixelMethod;
+  using MagickCore::BlackVirtualPixelMethod;
+  using MagickCore::GrayVirtualPixelMethod;
+  using MagickCore::WhiteVirtualPixelMethod;
+  using MagickCore::HorizontalTileVirtualPixelMethod;
+  using MagickCore::VerticalTileVirtualPixelMethod;
+  using MagickCore::HorizontalTileEdgeVirtualPixelMethod;
+  using MagickCore::VerticalTileEdgeVirtualPixelMethod;
+  using MagickCore::CheckerTileVirtualPixelMethod;
+
 #if defined(MAGICKCORE_IMPLEMENTATION)
   //
   // ImageMagick symbols used in implementation code
@@ -543,6 +564,7 @@ namespace Magick
   using MagickCore::CoderError;
   using MagickCore::CoderFatalError;
   using MagickCore::CoderWarning;
+  using MagickCore::ColorDecisionListImage;
   using MagickCore::ColorizeImage;
   using MagickCore::ColorPacket;
   using MagickCore::CompositeImage;
@@ -723,8 +745,11 @@ namespace Magick
   using MagickCore::GetStringInfoLength;
   using MagickCore::GetTypeMetrics;
   using MagickCore::GetVirtualIndexQueue;
+  using MagickCore::GetImageVirtualPixelMethod;
   using MagickCore::GlobExpression;
+  using MagickCore::GravityAdjustGeometry;
   using MagickCore::GreaterValue;
+  using MagickCore::HaldClutImage;
   using MagickCore::HeightValue;
   using MagickCore::ImageError;
   using MagickCore::ImageFatalError;
@@ -792,10 +817,12 @@ namespace Magick
   using MagickCore::QuantumInfo;
   using MagickCore::QueryColorDatabase;
   using MagickCore::QueryMagickColor;
+  using MagickCore::QueueAuthenticPixels;
+  using MagickCore::QueueCacheViewAuthenticPixels;
   using MagickCore::RaiseImage;
   using MagickCore::RandomThresholdImageChannel;
   using MagickCore::ReadImage;
-  using MagickCore::RectangleInfo;
+  using MagickCore::RecolorImage;
   using MagickCore::RectangleInfo;
   using MagickCore::ReduceNoiseImage;
   using MagickCore::RegisterMagickInfo;
@@ -817,8 +844,8 @@ namespace Magick
   using MagickCore::ScaleImage;
   using MagickCore::SegmentImage;
   using MagickCore::SeparateImageChannel;
-  using MagickCore::QueueCacheViewAuthenticPixels;
   using MagickCore::SetClientName;
+  using MagickCore::SetGeometry;
   using MagickCore::SetImageBackgroundColor;
   using MagickCore::SetImageChannelDepth;
   using MagickCore::SetImageClipMask;
@@ -827,7 +854,6 @@ namespace Magick
   using MagickCore::SetImageInfo;
   using MagickCore::SetImageOpacity;
   using MagickCore::SetImageOption;
-  using MagickCore::QueueAuthenticPixels;
   using MagickCore::SetImageProfile;
   using MagickCore::SetImageProperty;
   using MagickCore::SetImageRegistry;
@@ -836,6 +862,7 @@ namespace Magick
   using MagickCore::SetMagickInfo;
   using MagickCore::SetMagickResourceLimit;
   using MagickCore::SetStringInfoDatum;
+  using MagickCore::SetImageVirtualPixelMethod;
   using MagickCore::ShadeImage;
   using MagickCore::SharpenImage;
   using MagickCore::SharpenImageChannel;
@@ -845,6 +872,7 @@ namespace Magick
   using MagickCore::SignatureImage;
   using MagickCore::SolarizeImage;
   using MagickCore::SparseColorImage;
+  using MagickCore::SpliceImage;
   using MagickCore::SpreadImage;
   using MagickCore::SteganoImage;
   using MagickCore::StereoImage;
@@ -871,7 +899,7 @@ namespace Magick
   using MagickCore::UnregisterMagickInfo;
   using MagickCore::UnsharpMaskImage;
   using MagickCore::UnsharpMaskImageChannel;
-  using MagickCore::ViewInfo;
+  using MagickCore::CacheView;
   using MagickCore::WaveImage;
   using MagickCore::WidthValue;
   using MagickCore::WriteImage;
