@@ -607,20 +607,20 @@ double Magick::Options::strokeWidth ( void ) const
 
 void Magick::Options::subImage ( unsigned int subImage_ )
 {
-  _imageInfo->subimage = subImage_;
+  _imageInfo->scene = subImage_;
 }
 unsigned int Magick::Options::subImage ( void ) const
 {
-  return _imageInfo->subimage;
+  return _imageInfo->scene;
 }
 
 void Magick::Options::subRange ( unsigned int subRange_ )
 {
-  _imageInfo->subrange = subRange_;
+  _imageInfo->number_scenes = subRange_;
 }
 unsigned int Magick::Options::subRange ( void ) const
 {
-  return _imageInfo->subrange;
+  return _imageInfo->number_scenes;
 }
 
 // Annotation text encoding (e.g. "UTF-16")
@@ -799,6 +799,15 @@ bool Magick::Options::verbose ( void ) const
   return static_cast<bool>(_imageInfo->verbose);
 }
 
+void Magick::Options::virtualPixelMethod ( VirtualPixelMethod virtual_pixel_method_ )
+{
+  _imageInfo->virtual_pixel_method = virtual_pixel_method_;
+}
+Magick::VirtualPixelMethod Magick::Options::virtualPixelMethod ( void ) const
+{
+  return static_cast<Magick::VirtualPixelMethod>(_imageInfo->virtual_pixel_method);
+}
+
 void Magick::Options::view ( const std::string &view_ )
 {
   if ( view_.length() == 0 )
