@@ -44,6 +44,7 @@
 #include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/draw.h"
@@ -392,8 +393,7 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -864,6 +864,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image=DestroyImageList(image);
           return((Image *) NULL);
         }
+      DuplicateBlob(flipped_image,image);
       image=DestroyImage(image);
       image=flipped_image;
     }
@@ -1008,7 +1009,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
     Initialize DIB raster file header.
   */
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   if (image->storage_class == DirectClass)
     {
       /*
