@@ -113,8 +113,7 @@ void increase (void *pixel, int byteType){
  %
  %  A description of each parameter follows:
  %
- %    o magick: This string is generally the first few bytes of an image file
- %      or blob.
+ %    o magick: compare image format pattern against these bytes.
  %
  %    o length: Specifies the length of the magick string.
  %
@@ -123,7 +122,7 @@ static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"data",4) == 0)
+  if (LocaleNCompare((const char *) magick,"data",4) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -162,11 +161,11 @@ static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
  */
 
 void SetHeaderFromIPL(Image *image, IPLInfo *ipl){
-	image->columns = ipl->width;
-	image->rows = ipl->height;
-	image->depth = ipl->depth;
-	image->x_resolution = 1;
-	image->y_resolution = 1;
+  image->columns = ipl->width;
+  image->rows = ipl->height;
+  image->depth = ipl->depth;
+  image->x_resolution = 1;
+  image->y_resolution = 1;
 }
 
 
@@ -326,8 +325,8 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
      pixels=GetQuantumPixels(quantum_info); 
      if(image->columns != ipl_info.width){
 /*
- 		printf("Columns not set correctly!  Wanted: %lu, got: %lu\n",
- 			ipl_info.width, image->columns);
+     printf("Columns not set correctly!  Wanted: %lu, got: %lu\n",
+       ipl_info.width, image->columns);
 */
      }
 
@@ -341,11 +340,11 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
                 break;
-      	(void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
           GrayQuantum,pixels,exception);
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
-  	}
+    }
   }
   else{
       for(y = 0; y < (long) image->rows; y++){
@@ -353,8 +352,8 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
                 break;
-      	(void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
-          RedQuantum,pixels,exception);	
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+          RedQuantum,pixels,exception);  
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
       }
@@ -363,7 +362,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
                 break;
-      	(void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
           GreenQuantum,pixels,exception);
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
@@ -373,7 +372,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
                 break;
-      	(void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+        (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
           BlueQuantum,pixels,exception);
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
@@ -573,7 +572,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   ipl_info.height = image->rows;
   
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   
   if(image->colorspace == RGBColorspace) { ipl_info.colors = 3; }
   else{ ipl_info.colors = 1; }
@@ -591,14 +590,14 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
   the code in here, but it is all but useless if you want to use IPLab. */
 
   if(image_info->endian == MSBEndian)
-    (void) WriteBlob(image, 4, (unsigned char *) "mmmm");
+    (void) WriteBlob(image, 4, (const unsigned char *) "mmmm");
   else{
     image->endian = LSBEndian;
-    (void) WriteBlob(image, 4, (unsigned char *) "iiii");
+    (void) WriteBlob(image, 4, (const unsigned char *) "iiii");
   }
   (void) WriteBlobLong(image, 4);
-  (void) WriteBlob(image, 4, (unsigned char *) "100f");
-  (void) WriteBlob(image, 4, (unsigned char *) "data");
+  (void) WriteBlob(image, 4, (const unsigned char *) "100f");
+  (void) WriteBlob(image, 4, (const unsigned char *) "data");
   (void) WriteBlobLong(image, ipl_info.size);
   (void) WriteBlobLong(image, ipl_info.width); 
   (void) WriteBlobLong(image, ipl_info.height);
@@ -623,7 +622,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
     p=GetAuthenticPixels(image,0,y,image->columns,1,exception);
     if (p == (PixelPacket *) NULL)
       break;
-      (void) ExportQuantumPixels(image,(const ViewInfo *) NULL, quantum_info,
+      (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
         GrayQuantum, pixels,&image->exception);
       (void) WriteBlob(image, image->columns*image->depth/8, pixels);
   }
@@ -635,7 +634,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
     p=GetAuthenticPixels(image,0,y,image->columns,1,exception);
     if (p == (PixelPacket *) NULL)
       break;
-      (void) ExportQuantumPixels(image,(const ViewInfo *) NULL, quantum_info,
+      (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
         RedQuantum, pixels,&image->exception);
       (void) WriteBlob(image, image->columns*image->depth/8, pixels);
   }
@@ -645,7 +644,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
       p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (PixelPacket *) NULL)
         break;
-        (void) ExportQuantumPixels(image,(const ViewInfo *) NULL, quantum_info,
+        (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
           GreenQuantum, pixels,&image->exception);
         (void) WriteBlob(image, image->columns*image->depth/8, pixels);
     }
@@ -654,7 +653,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
       p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (PixelPacket *) NULL)
         break;
-      (void) ExportQuantumPixels(image,(const ViewInfo *) NULL, quantum_info,
+      (void) ExportQuantumPixels(image,(const CacheView *) NULL, quantum_info,
         BlueQuantum, pixels,&image->exception);
       (void) WriteBlob(image, image->columns*image->depth/8, pixels);
       if (image->previous == (Image *) NULL)
@@ -675,7 +674,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
         break;
     }while (image_info->adjoin != MagickFalse);
 
-  (void) WriteBlob(image, 4, (unsigned char *) "fini");
+  (void) WriteBlob(image, 4, (const unsigned char *) "fini");
   (void) WriteBlobLong(image, 0);
 
 CloseBlob(image);
