@@ -10,14 +10,14 @@
 %                             JJ    P      22222                              %
 %                                                                             %
 %                                                                             %
-%                     Read/Write JPEG-2000 Image Format.                      %
+%                     Read/Write JPEG-2000 Image Format                       %
 %                                                                             %
 %                                John Cristy                                  %
 %                                Nathan Brown                                 %
 %                                 June 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,8 +40,10 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
@@ -53,12 +55,21 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/profile.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
-#if defined(HasJP2)
+#include "magick/module.h"
+#if defined(MAGICKCORE_JP2_DELEGATE)
+#ifndef JAS_IMAGE_CM_GRAY
+#define JAS_IMAGE_CM_GRAY JAS_IMAGE_CS_GRAY
+#endif
+#ifndef JAS_IMAGE_CM_RGB
+#define JAS_IMAGE_CM_RGB JAS_IMAGE_CS_RGB
+#endif
 #if !defined(uchar)
 #define uchar  unsigned char
 #endif
@@ -75,13 +86,27 @@
 #define ulonglong  unsigned long long
 #endif
 
+#ifdef __VMS
+#define JAS_VERSION 1.700.0
+#define PACKAGE jasper
+#define VERSION 1.700.0
+#endif
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
 #include "jasper/jasper.h"
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+
 #endif
 
 /*
   Forward declarations.
 */
-#if defined(HasJP2)
+#if defined(MAGICKCORE_JP2_DELEGATE)
 static MagickBooleanType
   WriteJP2Image(const ImageInfo *,Image *);
 #endif
@@ -106,12 +131,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsJP2(const unsigned char *magick,const size_t length)
 {
@@ -142,12 +165,10 @@ static MagickBooleanType IsJP2(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsJPC(const unsigned char *magick,const size_t length)
 {
@@ -183,12 +204,12 @@ static MagickBooleanType IsJPC(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
 */
-#if defined(HasJP2)
+#if defined(MAGICKCORE_JP2_DELEGATE)
 
 typedef struct _StreamManager
 {
@@ -204,65 +225,104 @@ static int BlobRead(jas_stream_obj_t *object,char *buffer,const int length)
   ssize_t
     count;
 
-  count=ReadBlob(((StreamManager *) object)->image,(size_t) length,
-    (void *) buffer);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  count=ReadBlob(source->image,(size_t) length,(unsigned char *) buffer);
   return((int) count);
 }
 
 static int BlobWrite(jas_stream_obj_t *object,char *buffer,const int length)
 {
-  size_t
+  ssize_t
     count;
 
-  count=WriteBlob(((StreamManager *) object)->image,(size_t) length,
-    (void *) buffer);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  count=WriteBlob(source->image,(size_t) length,(unsigned char *) buffer);
   return((int) count);
 }
 
 static long BlobSeek(jas_stream_obj_t *object,long offset,int origin)
 {
-  return(SeekBlob(((StreamManager *) object)->image,offset,origin));
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  return((long) SeekBlob(source->image,offset,origin));
 }
 
 static int BlobClose(jas_stream_obj_t *object)
 {
-  CloseBlob(((StreamManager *) object)->image);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  (void) CloseBlob(source->image);
+  source=(StreamManager *) RelinquishMagickMemory(source);
   return(0);
 }
 
-static jas_stream_ops_t
-  StreamOperators =
-  {
-    BlobRead,
-    BlobWrite,
-    BlobSeek,
-    BlobClose
-  };
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
 
 static jas_stream_t *JP2StreamManager(Image *image)
 {
+  static jas_stream_ops_t
+    StreamOperators =
+    {
+      BlobRead,
+      BlobWrite,
+      BlobSeek,
+      BlobClose
+    };
+
   jas_stream_t
     *stream;
 
   StreamManager
     *source;
 
-  stream=(jas_stream_t *) malloc(sizeof(*stream));
+  stream=(jas_stream_t *) jas_malloc(sizeof(*stream));
   if (stream == (jas_stream_t *) NULL)
     return((jas_stream_t *) NULL);
   (void) ResetMagickMemory(stream,0,sizeof(*stream));
   stream->rwlimit_=(-1);
-  stream->obj_=(jas_stream_obj_t *) malloc(sizeof(StreamManager));
+  stream->obj_=(jas_stream_obj_t *) jas_malloc(sizeof(StreamManager));
   if (stream->obj_ == (jas_stream_obj_t *) NULL)
     return((jas_stream_t *) NULL);
   (void) ResetMagickMemory(stream->obj_,0,sizeof(StreamManager));
   stream->ops_=(&StreamOperators);
   stream->openmode_=JAS_STREAM_READ | JAS_STREAM_WRITE | JAS_STREAM_BINARY;
-  stream->bufbase_=stream->tinybuf_;
-  stream->bufsize_=1;
+  stream->bufbase_=(unsigned char *) jas_malloc(JAS_STREAM_BUFSIZE+
+    JAS_STREAM_MAXPUTBACK);
+  if (stream->bufbase_ == (void *) NULL)
+    {
+      stream->bufbase_=stream->tinybuf_;
+      stream->bufsize_=1;
+    }
+  else
+    {
+      stream->bufmode_=JAS_STREAM_FREEBUF | JAS_STREAM_BUFMODEMASK;
+      stream->bufsize_=JAS_STREAM_BUFSIZE;
+    }
   stream->bufstart_=(&stream->bufbase_[JAS_STREAM_MAXPUTBACK]);
   stream->ptr_=stream->bufstart_;
-  stream->bufmode_|=JAS_STREAM_UNBUF & JAS_STREAM_BUFMODEMASK;
+  stream->cnt_=0;
   source=(StreamManager *) stream->obj_;
   source->image=image;
   return(stream);
@@ -273,9 +333,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  long
-    y;
-
   jas_cmprof_t
     *cm_profile;
 
@@ -291,9 +348,18 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   jas_stream_t
     *jp2_stream;
 
+  long
+    components[4],
+    y;
+
   MagickBooleanType
     status;
 
+  QuantumAny
+    pixel,
+    *map[4],
+    range;
+
   register long
     i,
     x;
@@ -301,18 +367,12 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   register PixelPacket
     *q;
 
-  int
-    components[4];
-
-  unsigned int
+  unsigned long
     maximum_component_depth,
-    scale[4],
+    number_components,
     x_step[4],
     y_step[4];
 
-  unsigned long
-    number_components;
-
   /*
     Open image file.
   */
@@ -323,7 +383,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -406,65 +466,79 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   image->columns=jas_image_width(jp2_image);
   image->rows=jas_image_height(jp2_image);
+  image->compression=JPEG2000Compression;
   for (i=0; i < (long) number_components; i++)
   {
-    if ((((jas_image_cmptwidth(jp2_image,components[i])*
-          jas_image_cmpthstep(jp2_image,components[i])) != image->columns)) ||
-        (((jas_image_cmptheight(jp2_image,components[i])*
-          jas_image_cmptvstep(jp2_image,components[i])) != image->rows)) ||
+    unsigned long
+      height,
+      width;
+
+    width=(unsigned long) (jas_image_cmptwidth(jp2_image,components[i])*
+      jas_image_cmpthstep(jp2_image,components[i]));
+    height=(unsigned long) (jas_image_cmptheight(jp2_image,components[i])*
+      jas_image_cmptvstep(jp2_image,components[i]));
+    x_step[i]=(unsigned int) jas_image_cmpthstep(jp2_image,components[i]);
+    y_step[i]=(unsigned int) jas_image_cmptvstep(jp2_image,components[i]);
+    if ((width != image->columns) || (height != image->rows) ||
         (jas_image_cmpttlx(jp2_image,components[i]) != 0) ||
         (jas_image_cmpttly(jp2_image,components[i]) != 0) ||
+        (x_step[i] != 1) || (y_step[i] != 1) ||
         (jas_image_cmptsgnd(jp2_image,components[i]) != MagickFalse))
       {
         (void) jas_stream_close(jp2_stream);
         jas_image_destroy(jp2_image);
         ThrowReaderException(CoderError,"IrregularChannelGeometryNotSupported");
       }
-    x_step[i]=jas_image_cmpthstep(jp2_image,components[i]);
-    y_step[i]=jas_image_cmptvstep(jp2_image,components[i]);
   }
   /*
     Convert JPEG 2000 pixels.
   */
+  image->matte=number_components > 3 ? MagickTrue : MagickFalse;
+  maximum_component_depth=0;
+  for (i=0; i < (long) number_components; i++)
+  {
+    maximum_component_depth=(unsigned int) MagickMax((size_t)
+      jas_image_cmptprec(jp2_image,components[i]),(size_t)
+      maximum_component_depth);
+    pixels[i]=jas_matrix_create(1,(int) (image->columns/x_step[i]));
+    if (pixels[i] == (jas_matrix_t *) NULL)
+      {
+        for (--i; i >= 0; i--)
+          jas_matrix_destroy(pixels[i]);
+        jas_image_destroy(jp2_image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      }
+  }
+  image->depth=maximum_component_depth;
   if (image_info->ping != MagickFalse)
     {
       (void) jas_stream_close(jp2_stream);
       jas_image_destroy(jp2_image);
       return(GetFirstImageInList(image));
     }
-  image->matte=number_components > 3;
-  maximum_component_depth=0;
   for (i=0; i < (long) number_components; i++)
   {
-    maximum_component_depth=Max(jas_image_cmptprec(jp2_image,components[i]),
-      maximum_component_depth);
-    pixels[i]=jas_matrix_create(1,((unsigned int) image->columns)/x_step[i]);
-    if (pixels[i] == (jas_matrix_t *) NULL)
+    long
+      j;
+
+    map[i]=(QuantumAny *) AcquireQuantumMemory(MaxMap+1,sizeof(**map));
+    if (map[i] == (QuantumAny *) NULL)
       {
+        for (--i; i >= 0; i--)
+          map[i]=(QuantumAny *) RelinquishMagickMemory(map[i]);
+        for (i=0; i < (long) number_components; i++)
+          jas_matrix_destroy(pixels[i]);
         jas_image_destroy(jp2_image);
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
+    range=GetQuantumRange((unsigned long) jas_image_cmptprec(jp2_image,
+      components[i]));
+    for (j=0; j <= (long) MaxMap; j++)
+      map[i][j]=ScaleQuantumToMap(ScaleAnyToQuantum((QuantumAny) j,range));
   }
-  if (maximum_component_depth <= 8)
-    image->depth=Min(QuantumDepth,8);
-  else
-    image->depth=Min(QuantumDepth,16);
-  for (i=0; i < (long) number_components; i++)
-  {
-    scale[i]=1;
-    if (jas_image_cmptprec(jp2_image,components[i]) < 16)
-      scale[i]=(1 << (16-jas_image_cmptprec(jp2_image,components[i])))+1;
-  }
-  if (number_components == 1)
-    {
-      image->storage_class=PseudoClass;
-      image->colors=(image->depth == 8 ? 256 : MaxColormapSize);
-      if (AllocateImageColormap(image,image->colors) == MagickFalse)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    }
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
     for (i=0; i < (long) number_components; i++)
@@ -475,34 +549,17 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       case 1:
       {
-        register IndexPacket
-          *indexes;
-
-        IndexPacket
-          index;
-
         /*
           Grayscale.
         */
-        indexes=GetIndexes(image);
-        if (image->depth == 8)
-          for (x=0; x < (long) image->columns; x++)
-          {
-            index=ScaleQuantumToChar(ScaleShortToQuantum((unsigned short)
-              (scale[0]*jas_matrix_getv(pixels[0],x))));
-            index=ConstrainColormapIndex(image,index);
-            *indexes++=index;
-            *q++=image->colormap[index];
-          }
-        else
-          for (x=0; x < (long) image->columns; x++)
-          {
-            index=(unsigned short) (scale[0]*
-              jas_matrix_getv(pixels[0],x/x_step[0]));
-            index=ConstrainColormapIndex(image,index);
-            *indexes++=index;
-            *q++=image->colormap[index];
-          }
+        for (x=0; x < (long) image->columns; x++)
+        {
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=(Quantum) map[0][pixel];
+          q->green=q->red;
+          q->blue=q->red;
+          q++;
+        }
         break;
       }
       case 3:
@@ -512,12 +569,12 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
-            jas_matrix_getv(pixels[0],x/x_step[0])));
-          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
-            jas_matrix_getv(pixels[1],x/x_step[1])));
-          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
-            jas_matrix_getv(pixels[2],x/x_step[2])));
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=(Quantum) map[0][pixel];
+          pixel=(QuantumAny) jas_matrix_getv(pixels[1],x/x_step[1]);
+          q->green=(Quantum) map[1][pixel];
+          pixel=(QuantumAny) jas_matrix_getv(pixels[2],x/x_step[2]);
+          q->blue=(Quantum) map[2][pixel];
           q++;
         }
         break;
@@ -529,31 +586,27 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
-            jas_matrix_getv(pixels[0],x/x_step[0])));
-          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
-            jas_matrix_getv(pixels[1],x/x_step[1])));
-          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
-            jas_matrix_getv(pixels[2],x/x_step[2])));
-          q->opacity=QuantumRange-ScaleShortToQuantum((unsigned short) (scale[3]*
-            jas_matrix_getv(pixels[3],x/x_step[3])));
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=(Quantum) map[0][pixel];
+          pixel=(QuantumAny) jas_matrix_getv(pixels[1],x/x_step[1]);
+          q->green=(Quantum) map[1][pixel];
+          pixel=(QuantumAny) jas_matrix_getv(pixels[2],x/x_step[2]);
+          q->blue=(Quantum) map[2][pixel];
+          pixel=(QuantumAny) jas_matrix_getv(pixels[3],x/x_step[3]);
+          q->opacity=(Quantum) (QuantumRange-map[3][pixel]);
           q++;
         }
         break;
       }
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+    status=SetImageProgress(image,LoadImageTag,y,image->rows);
+    if (status == MagickFalse)
       break;
-    if (image->previous == (Image *) NULL)
-     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(LoadImagesTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
   }
+  for (i=0; i < (long) number_components; i++)
+    map[i]=(QuantumAny *) RelinquishMagickMemory(map[i]);
   cm_profile=jas_image_cmprof(jp2_image);
   icc_profile=(jas_iccprof_t *) NULL;
   if (cm_profile != (jas_cmprof_t *) NULL)
@@ -568,13 +621,13 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
           (jas_iccprof_save(icc_profile,icc_stream) == 0) &&
           (jas_stream_flush(icc_stream) == 0))
         {
-          jas_stream_memobj_t
-            *blob;
-
           StringInfo
             *icc_profile,
             *profile;
 
+          jas_stream_memobj_t
+            *blob;
+
           /*
             Extract the icc profile, handle errors without much noise.
           */
@@ -584,11 +637,11 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
               "Profile: ICC, %lu bytes",(unsigned long) blob->len_);
           profile=AcquireStringInfo(blob->len_);
           SetStringInfoDatum(profile,blob->buf_);
-          icc_profile=GetImageProfile(image,"icc");
+          icc_profile=(StringInfo *) GetImageProfile(image,"icc");
           if (icc_profile == (StringInfo *) NULL)
-            SetImageProfile(image,"icc",profile);
+            (void) SetImageProfile(image,"icc",profile);
           else
-            ConcatenateStringInfo(icc_profile,profile);
+            (void) ConcatenateStringInfo(icc_profile,profile);
           profile=DestroyStringInfo(profile);
           (void) jas_stream_close(icc_stream);
         }
@@ -620,64 +673,65 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterJP2Image method is:
 %
-%      RegisterJP2Image(void)
+%      unsigned long RegisterJP2Image(void)
 %
 */
-ModuleExport void RegisterJP2Image(void)
+ModuleExport unsigned long RegisterJP2Image(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("JP2");
-  entry->description=AcquireString("JPEG-2000 File Format Syntax");
-  entry->module=AcquireString("JP2");
-  entry->magick=(MagickHandler *) IsJP2;
+  entry->description=ConstantString("JPEG-2000 File Format Syntax");
+  entry->module=ConstantString("JP2");
+  entry->magick=(IsImageFormatHandler *) IsJP2;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=MagickFalse;
-#if defined(HasJP2)
-  entry->decoder=(DecoderHandler *) ReadJP2Image;
-  entry->encoder=(EncoderHandler *) WriteJP2Image;
+  entry->thread_support=NoThreadSupport;
+#if defined(MAGICKCORE_JP2_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadJP2Image;
+  entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JPC");
-  entry->description=AcquireString("JPEG-2000 Code Stream Syntax");
-  entry->module=AcquireString("JPC");
-  entry->magick=(MagickHandler *) IsJPC;
+  entry->description=ConstantString("JPEG-2000 Code Stream Syntax");
+  entry->module=ConstantString("JPC");
+  entry->magick=(IsImageFormatHandler *) IsJPC;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=MagickFalse;
-#if defined(HasJP2)
-  entry->decoder=(DecoderHandler *) ReadJP2Image;
-  entry->encoder=(EncoderHandler *) WriteJP2Image;
+  entry->thread_support=NoThreadSupport;
+#if defined(MAGICKCORE_JP2_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadJP2Image;
+  entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JPX");
-  entry->description=AcquireString("JPEG-2000 File Format Syntax");
-  entry->module=AcquireString("JPX");
-  entry->magick=(MagickHandler *) IsJPC;
+  entry->description=ConstantString("JPEG-2000 File Format Syntax");
+  entry->module=ConstantString("JPX");
+  entry->magick=(IsImageFormatHandler *) IsJPC;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=MagickFalse;
-#if defined(HasJP2)
-  entry->decoder=(DecoderHandler *) ReadJP2Image;
-  entry->encoder=(EncoderHandler *) WriteJP2Image;
+  entry->thread_support=NoThreadSupport;
+#if defined(MAGICKCORE_JP2_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadJP2Image;
+  entry->encoder=(EncodeImageHandler *) WriteJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PGX");
-  entry->description=AcquireString("JPEG-2000 VM Format");
-  entry->module=AcquireString("PGX");
-  entry->magick=(MagickHandler *) IsJPC;
+  entry->description=ConstantString("JPEG-2000 VM Format");
+  entry->module=ConstantString("PGX");
+  entry->magick=(IsImageFormatHandler *) IsJPC;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=MagickFalse;
-#if defined(HasJP2)
-  entry->decoder=(DecoderHandler *) ReadJP2Image;
+  entry->thread_support=NoThreadSupport;
+#if defined(MAGICKCORE_JP2_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadJP2Image;
 #endif
   (void) RegisterMagickInfo(entry);
-#if defined(HasJP2)
+#if defined(MAGICKCORE_JP2_DELEGATE)
   jas_init();
 #endif
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -704,12 +758,12 @@ ModuleExport void UnregisterJP2Image(void)
   (void) UnregisterMagickInfo("JP2");
   (void) UnregisterMagickInfo("JPC");
   (void) UnregisterMagickInfo("PGX");
-#if defined(HasJP2)
+#if defined(MAGICKCORE_JP2_DELEGATE)
   jas_cleanup();
 #endif
 }
 
-#if defined(HasJP2)
+#if defined(MAGICKCORE_JP2_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -731,11 +785,10 @@ ModuleExport void UnregisterJP2Image(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
 {
@@ -747,10 +800,8 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   const char
     *option;
 
-  int
-    format;
-
   long
+    format,
     y;
 
   jas_image_cmptparm_t
@@ -768,6 +819,9 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
+  QuantumAny
+    range;
+
   register const PixelPacket
     *p;
 
@@ -775,6 +829,9 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     i,
     x;
 
+  unsigned short
+    *map;
+
   unsigned long
     number_components;
 
@@ -793,14 +850,19 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   /*
     Intialize JPEG 2000 API.
   */
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) TransformImageColorspace(image,RGBColorspace);
   jp2_stream=JP2StreamManager(image);
   if (jp2_stream == (jas_stream_t *) NULL)
     ThrowWriterException(DelegateError,"UnableToManageJP2Stream");
-  number_components=image->matte ? 4 : 3;
+  number_components=image->matte ? 4UL : 3UL;
   if ((image_info->type != TrueColorType) &&
       IsGrayImage(image,&image->exception))
     number_components=1;
+  if ((image->columns != (unsigned int) image->columns) ||
+      (image->rows != (unsigned int) image->rows))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+  (void) ResetMagickMemory(&component_info,0,sizeof(component_info));
   for (i=0; i < (long) number_components; i++)
   {
     component_info[i].tlx=0;
@@ -809,10 +871,11 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     component_info[i].vstep=1;
     component_info[i].width=(unsigned int) image->columns;
     component_info[i].height=(unsigned int) image->rows;
-    component_info[i].prec=(unsigned int) image->depth <= 8 ? 8 : 16;
+    component_info[i].prec=(int) MagickMax(MagickMin(image->depth,16),2);
     component_info[i].sgnd=MagickFalse;
+    
   }
-  jp2_image=jas_image_create(number_components,component_info,
+  jp2_image=jas_image_create((int) number_components,component_info,
     JAS_CLRSPC_UNKNOWN);
   if (jp2_image == (jas_image_t *) NULL)
     ThrowWriterException(DelegateError,"UnableToCreateImage");
@@ -830,8 +893,6 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       /*
         sRGB.
       */
-      if (number_components == 4 )
-        jas_image_setcmpttype(jp2_image,3,JAS_IMAGE_CT_OPACITY);
       jas_image_setclrspc(jp2_image,JAS_CLRSPC_SRGB);
       jas_image_setcmpttype(jp2_image,0,
         JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_R));
@@ -839,13 +900,15 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
         JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_G));
       jas_image_setcmpttype(jp2_image,2,
         JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_B));
+      if (number_components == 4)
+        jas_image_setcmpttype(jp2_image,3,JAS_IMAGE_CT_OPACITY);
     }
   /*
     Convert to JPEG 2000 pixels.
   */
   for (i=0; i < (long) number_components; i++)
   {
-    pixels[i]=jas_matrix_create(1,(unsigned int) image->columns);
+    pixels[i]=jas_matrix_create(1,(int) image->columns);
     if (pixels[i] == (jas_matrix_t *) NULL)
       {
         for (x=0; x < i; x++)
@@ -854,58 +917,47 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       }
   }
+  range=GetQuantumRange((unsigned long) component_info[0].prec);
+  map=(unsigned short *) AcquireQuantumMemory(MaxMap+1,sizeof(*map));
+  for (i=0; i <= (long) MaxMap; i++)
+    map[i]=(unsigned short) ScaleQuantumToMap((Quantum)
+      ScaleQuantumToAny((Quantum) i,range));
+  if (map == (unsigned short *) NULL)
+    {
+      for (i=0; i < (long) number_components; i++)
+        jas_matrix_destroy(pixels[i]);
+      jas_image_destroy(jp2_image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    }
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    if (image->depth <= 8)
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if (number_components == 1)
-          jas_matrix_setv(pixels[0],x,
-            ScaleQuantumToChar(PixelIntensityToQuantum(p)));
-        else
-          {
-            jas_matrix_setv(pixels[0],x,ScaleQuantumToChar(p->red));
-            jas_matrix_setv(pixels[1],x,ScaleQuantumToChar(p->green));
-            jas_matrix_setv(pixels[2],x,ScaleQuantumToChar(p->blue));
-            if (number_components > 3)
-              jas_matrix_setv(pixels[3],x,
-                ScaleQuantumToChar(QuantumRange-p->opacity));
-          }
-        p++;
-      }
-    else
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if (number_components == 1)
-          jas_matrix_setv(pixels[0],x,
-            ScaleQuantumToShort(PixelIntensityToQuantum(p)));
-        else
-          {
-            jas_matrix_setv(pixels[0],x,ScaleQuantumToShort(p->red));
-            jas_matrix_setv(pixels[1],x,ScaleQuantumToShort(p->green));
-            jas_matrix_setv(pixels[2],x,ScaleQuantumToShort(p->blue));
-            if (number_components > 3)
-              jas_matrix_setv(pixels[3],x,
-                ScaleQuantumToShort(QuantumRange-p->opacity));
-          }
-        p++;
-      }
+    for (x=0; x < (long) image->columns; x++)
+    {
+      if (number_components == 1)
+        jas_matrix_setv(pixels[0],x,map[ScaleQuantumToMap(
+          PixelIntensityToQuantum(p))]);
+      else
+        {
+          jas_matrix_setv(pixels[0],x,map[ScaleQuantumToMap(p->red)]);
+          jas_matrix_setv(pixels[1],x,map[ScaleQuantumToMap(p->green)]);
+          jas_matrix_setv(pixels[2],x,map[ScaleQuantumToMap(p->blue)]);
+          if (number_components > 3)
+            jas_matrix_setv(pixels[3],x,map[ScaleQuantumToMap((Quantum)
+              (QuantumRange-p->opacity))]);
+        }
+      p++;
+    }
     for (i=0; i < (long) number_components; i++)
       (void) jas_image_writecmpt(jp2_image,(short) i,0,(unsigned int) y,
         (unsigned int) image->columns,1,pixels[i]);
-    if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
+  map=(unsigned short *) RelinquishMagickMemory(map);
   (void) CopyMagickString(magick,image_info->magick,MaxTextExtent);
   LocaleLower(magick);
   format=jas_image_strtofmt(magick);
@@ -919,19 +971,23 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       {
         if (LocaleNCompare(key,"jp2:",4) == 0)
           {
-            ConcatenateString(&options,key+4);
-            ConcatenateString(&options,"=");
-            ConcatenateString(&options,option);
-            ConcatenateString(&options," ");
+            (void) ConcatenateString(&options,key+4);
+            if (*option != '\0')
+              {
+                (void) ConcatenateString(&options,"=");
+                (void) ConcatenateString(&options,option);
+              }
+            (void) ConcatenateString(&options," ");
           }
        }
     key=GetNextImageOption(image_info);
   }
   option=GetImageOption(image_info,"jp2:rate");
   if ((option == (const char *) NULL) &&
-      (image->compression != LosslessJPEGCompression) &&
+      (image_info->compression != LosslessJPEGCompression) &&
       (image->quality != UndefinedCompressionQuality) &&
-      (image->quality < 99.5) && ((image->rows*image->columns) > 2500))
+      ((double) image->quality <= 99.5) &&
+      ((image->rows*image->columns) > 2500))
     {
       char
         option[MaxTextExtent];
@@ -943,7 +999,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
         rate,
         target_size;
 
-      alpha=115-image->quality;
+      alpha=115.0-image->quality;
       rate=100.0/(alpha*alpha);
       header_size=550.0;
       header_size+=(number_components-1)*142;
@@ -951,10 +1007,11 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
         (GetImageQuantumDepth(image,MagickTrue)/8);
       target_size=(number_pixels*rate)+header_size;
       rate=target_size/number_pixels;
-      FormatMagickString(option,MaxTextExtent,"rate=%g",rate);
-      ConcatenateString(&options,option);
+      (void) FormatMagickString(option,MaxTextExtent,"rate=%g",rate);
+      (void) ConcatenateString(&options,option);
     }
-  status=jas_image_encode(jp2_image,jp2_stream,format,options);
+  status=jas_image_encode(jp2_image,jp2_stream,format,options) != 0 ?
+    MagickTrue : MagickFalse;
   (void) jas_stream_close(jp2_stream);
   for (i=0; i < (long) number_components; i++)
     jas_matrix_destroy(pixels[i]);
