@@ -48,6 +48,7 @@
 #include "magick/blob-private.h"
 #include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
@@ -161,8 +162,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -272,6 +272,26 @@ static void InitializeSource(j_decompress_ptr cinfo)
   source->start_of_blob=TRUE;
 }
 
+static MagickBooleanType IsITUFaxImage(const Image *image)
+{
+  const StringInfo
+    *profile;
+
+  const unsigned char
+    *datum;
+
+  profile=GetImageProfile(image,"8bim");
+  if (profile == (const StringInfo *) NULL)
+    return(MagickFalse);
+  if (GetStringInfoLength(profile) < 5)
+    return(MagickFalse);
+  datum=GetStringInfoDatum(profile);
+  if ((datum[0] == 0x47) && (datum[1] == 0x33) && (datum[2] == 0x46) &&
+      (datum[3] == 0x41) && (datum[4] == 0x58))
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
 static void JPEGErrorHandler(j_common_ptr jpeg_info)
 {
   ErrorManager
@@ -564,21 +584,21 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
         (void) CopyMagickString(name,"exif",MaxTextExtent);
       if ((length > 5) && (LocaleNCompare((char *) p,"http:",5) == 0))
         {
-          ssize_t
-            i;
+          long
+            j;
 
           /*
             Extract namespace from XMP profile.
           */
           p=GetStringInfoDatum(profile);
-          for (i=0; i < (long) GetStringInfoLength(profile); i++)
+          for (j=0; j < (long) GetStringInfoLength(profile); j++)
           {
             if (*p == '\0')
               break;
             p++;
           }
-          if (i < (long) GetStringInfoLength(profile))
-            DestroyStringInfo(SplitStringInfo(profile,i+1));
+          if (j < (long) GetStringInfoLength(profile))
+            (void) DestroyStringInfo(SplitStringInfo(profile,(size_t) (j+1)));
           (void) CopyMagickString(name,"xmp",MaxTextExtent);
         }
     }
@@ -636,24 +656,232 @@ static void JPEGSourceManager(j_decompress_ptr cinfo,Image *image)
   source->image=image;
 }
 
+static void JPEGSetImageQuality(struct jpeg_decompress_struct *jpeg_info,
+  Image *image)
+{
+  image->quality=UndefinedCompressionQuality;
+#if defined(D_PROGRESSIVE_SUPPORTED)
+  if (image->compression == LosslessJPEGCompression)
+    {
+      image->quality=100;
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Quality: 100 (lossless)");
+    }
+  else
+#endif
+  {
+    long
+      j,
+      qvalue,
+      sum;
+
+    register long
+      i;
+
+    /*
+      Determine the JPEG compression quality from the quantization tables.
+    */
+    sum=0;
+    for (i=0; i < NUM_QUANT_TBLS; i++)
+    {
+      if (jpeg_info->quant_tbl_ptrs[i] != NULL)
+        for (j=0; j < DCTSIZE2; j++)
+          sum+=jpeg_info->quant_tbl_ptrs[i]->quantval[j];
+     }
+     if ((jpeg_info->quant_tbl_ptrs[0] != NULL) &&
+         (jpeg_info->quant_tbl_ptrs[1] != NULL))
+       {
+         long
+           hash[101] =
+           {
+             1020, 1015,  932,  848,  780,  735,  702,  679,  660,  645,
+              632,  623,  613,  607,  600,  594,  589,  585,  581,  571,
+              555,  542,  529,  514,  494,  474,  457,  439,  424,  410,
+              397,  386,  373,  364,  351,  341,  334,  324,  317,  309,
+              299,  294,  287,  279,  274,  267,  262,  257,  251,  247,
+              243,  237,  232,  227,  222,  217,  213,  207,  202,  198,
+              192,  188,  183,  177,  173,  168,  163,  157,  153,  148,
+              143,  139,  132,  128,  125,  119,  115,  108,  104,   99,
+               94,   90,   84,   79,   74,   70,   64,   59,   55,   49,
+               45,   40,   34,   30,   25,   20,   15,   11,    6,    4,
+                0
+           },
+           sums[101] =
+           {
+             32640, 32635, 32266, 31495, 30665, 29804, 29146, 28599, 28104,
+             27670, 27225, 26725, 26210, 25716, 25240, 24789, 24373, 23946,
+             23572, 22846, 21801, 20842, 19949, 19121, 18386, 17651, 16998,
+             16349, 15800, 15247, 14783, 14321, 13859, 13535, 13081, 12702,
+             12423, 12056, 11779, 11513, 11135, 10955, 10676, 10392, 10208,
+              9928,  9747,  9564,  9369,  9193,  9017,  8822,  8639,  8458,
+              8270,  8084,  7896,  7710,  7527,  7347,  7156,  6977,  6788,
+              6607,  6422,  6236,  6054,  5867,  5684,  5495,  5305,  5128,
+              4945,  4751,  4638,  4442,  4248,  4065,  3888,  3698,  3509,
+              3326,  3139,  2957,  2775,  2586,  2405,  2216,  2037,  1846,
+              1666,  1483,  1297,  1109,   927,   735,   554,   375,   201,
+               128,     0
+           };
+
+         qvalue=(long) (jpeg_info->quant_tbl_ptrs[0]->quantval[2]+
+           jpeg_info->quant_tbl_ptrs[0]->quantval[53]+
+           jpeg_info->quant_tbl_ptrs[1]->quantval[0]+
+           jpeg_info->quant_tbl_ptrs[1]->quantval[DCTSIZE2-1]);
+         for (i=0; i < 100; i++)
+         {
+           if ((qvalue < hash[i]) && (sum < sums[i]))
+             continue;
+           if ((qvalue <= hash[i]) && (sum <= sums[i]))
+             image->quality=(unsigned long) i+1;
+           if (image->debug != MagickFalse)
+             {
+               if (image->quality != UndefinedCompressionQuality)
+                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                   "Quality: %ld",image->quality);
+               else
+                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                   "Quality: %ld (approximate)",i+1);
+             }
+           break;
+         }
+       }
+     else
+       if (jpeg_info->quant_tbl_ptrs[0] != NULL)
+         {
+           long
+             hash[101] =
+             {
+               510,  505,  422,  380,  355,  338,  326,  318,  311,  305,
+               300,  297,  293,  291,  288,  286,  284,  283,  281,  280,
+               279,  278,  277,  273,  262,  251,  243,  233,  225,  218,
+               211,  205,  198,  193,  186,  181,  177,  172,  168,  164,
+               158,  156,  152,  148,  145,  142,  139,  136,  133,  131,
+               129,  126,  123,  120,  118,  115,  113,  110,  107,  105,
+               102,  100,   97,   94,   92,   89,   87,   83,   81,   79,
+                76,   74,   70,   68,   66,   63,   61,   57,   55,   52,
+                50,   48,   44,   42,   39,   37,   34,   31,   29,   26,
+                24,   21,   18,   16,   13,   11,    8,    6,    3,    2,
+                 0
+             },
+             sums[101] =
+             {
+               16320, 16315, 15946, 15277, 14655, 14073, 13623, 13230, 12859,
+               12560, 12240, 11861, 11456, 11081, 10714, 10360, 10027,  9679,
+                9368,  9056,  8680,  8331,  7995,  7668,  7376,  7084,  6823,
+                6562,  6345,  6125,  5939,  5756,  5571,  5421,  5240,  5086,
+                4976,  4829,  4719,  4616,  4463,  4393,  4280,  4166,  4092,
+                3980,  3909,  3835,  3755,  3688,  3621,  3541,  3467,  3396,
+                3323,  3247,  3170,  3096,  3021,  2952,  2874,  2804,  2727,
+                2657,  2583,  2509,  2437,  2362,  2290,  2211,  2136,  2068,
+                1996,  1915,  1858,  1773,  1692,  1620,  1552,  1477,  1398,
+                1326,  1251,  1179,  1109,  1031,   961,   884,   814,   736,
+                 667,   592,   518,   441,   369,   292,   221,   151,    86,
+                  64,     0
+             };
+
+           qvalue=(long) (jpeg_info->quant_tbl_ptrs[0]->quantval[2]+
+             jpeg_info->quant_tbl_ptrs[0]->quantval[53]);
+           for (i=0; i < 100; i++)
+           {
+             if ((qvalue < hash[i]) && (sum < sums[i]))
+               continue;
+             image->quality=(unsigned long) i+1;
+             if (image->debug != MagickFalse)
+               {
+                 if ((qvalue > hash[i]) || (sum > sums[i]))
+                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                     "Quality: %ld (approximate)",i+1);
+                 else
+                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                     "Quality: %ld",i+1);
+               }
+             break;
+           }
+         }
+  }
+}
+
+static void JPEGSetImageSamplingFactor(struct jpeg_decompress_struct *jpeg_info,  Image *image)
+{
+  char
+    sampling_factor[MaxTextExtent];
+
+  switch (jpeg_info->out_color_space)
+  {
+    case JCS_CMYK:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: CMYK");
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,
+        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor,
+        jpeg_info->comp_info[1].h_samp_factor,
+        jpeg_info->comp_info[1].v_samp_factor,
+        jpeg_info->comp_info[2].h_samp_factor,
+        jpeg_info->comp_info[2].v_samp_factor,
+        jpeg_info->comp_info[3].h_samp_factor,
+        jpeg_info->comp_info[3].v_samp_factor);
+        break;
+    }
+    case JCS_GRAYSCALE:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Colorspace: GRAYSCALE");
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,"%dx%d",
+        jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor);
+      break;
+    }
+    case JCS_RGB:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: RGB");
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,
+        "%dx%d,%dx%d,%dx%d",jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor,
+        jpeg_info->comp_info[1].h_samp_factor,
+        jpeg_info->comp_info[1].v_samp_factor,
+        jpeg_info->comp_info[2].h_samp_factor,
+        jpeg_info->comp_info[2].v_samp_factor);
+      break;
+    }
+    default:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: %d",
+        jpeg_info->out_color_space);
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,
+        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor,
+        jpeg_info->comp_info[1].h_samp_factor,
+        jpeg_info->comp_info[1].v_samp_factor,
+        jpeg_info->comp_info[2].h_samp_factor,
+        jpeg_info->comp_info[2].v_samp_factor,
+        jpeg_info->comp_info[3].h_samp_factor,
+        jpeg_info->comp_info[3].v_samp_factor);
+      break;
+    }
+  }
+  (void) SetImageProperty(image,"jpeg:sampling-factor",sampling_factor);
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+    "Sampling Factors: %s",sampling_factor);
+}
+
 static Image *ReadJPEGImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   char
-    sampling_factor[MaxTextExtent],
     value[MaxTextExtent];
 
+  const char
+    *option;
+
   ErrorManager
     error_manager;
 
   IndexPacket
-    index;
+    jindex;
 
   Image
     *image;
 
   long
-    x,
     y;
 
   JSAMPLE
@@ -669,9 +897,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   MagickSizeType
     number_pixels;
 
-  register IndexPacket
-    *indexes;
-
   register long
     i;
 
@@ -684,9 +909,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   register JSAMPLE
     *p;
 
-  register PixelPacket
-    *q;
-
   unsigned long
     precision,
     units;
@@ -722,13 +944,12 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   if (setjmp(error_manager.error_recovery) != 0)
     {
       jpeg_destroy_decompress(&jpeg_info);
-      InheritException(exception,&image->exception);
       (void) CloseBlob(image);
       number_pixels=(MagickSizeType) image->columns*image->rows;
       if (number_pixels != 0)
         return(GetFirstImageInList(image));
-      image=DestroyImage(image);
-      return(image);
+      InheritException(exception,&image->exception);
+      return(DestroyImage(image));
     }
   jpeg_info.client_data=(void *) &error_manager;
   jpeg_create_decompress(&jpeg_info);
@@ -740,15 +961,18 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     if ((i != 2) && (i != 13) && (i != 14))
       jpeg_set_marker_processor(&jpeg_info,(int) (JPEG_APP0+i),ReadProfile);
   i=jpeg_read_header(&jpeg_info,MagickTrue);
-  if ((image->colorspace == YCbCrColorspace) ||
-      (image->colorspace == Rec601YCbCrColorspace) ||
-      (image->colorspace == Rec709YCbCrColorspace))
+  if ((image_info->colorspace == YCbCrColorspace) ||
+      (image_info->colorspace == Rec601YCbCrColorspace) ||
+      (image_info->colorspace == Rec709YCbCrColorspace))
+    jpeg_info.out_color_space=JCS_YCbCr;
+  if (IsITUFaxImage(image) != MagickFalse)
     {
+      image->colorspace=LabColorspace;
       jpeg_info.out_color_space=JCS_YCbCr;
-      image->colorspace=YCbCrColorspace;
     }
-  if (jpeg_info.out_color_space == JCS_CMYK)
-    image->colorspace=CMYKColorspace;
+  else
+    if (jpeg_info.out_color_space == JCS_CMYK)
+      image->colorspace=CMYKColorspace;
   /*
     Set image resolution.
   */
@@ -765,32 +989,41 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   if (units == 2)
     image->units=PixelsPerCentimeterResolution;
   number_pixels=(MagickSizeType) image->columns*image->rows;
-  if (image_info->size != (char *) NULL)
+  option=GetImageOption(image_info,"jpeg:size");
+  if (option != (const char *) NULL)
     {
       double
         scale_factor;
 
+      GeometryInfo
+        geometry_info;
+
+      int
+        flags;
+
       /*
-        Let the JPEG library subsample for us.
+        Scale the image.
       */
+      flags=ParseGeometry(option,&geometry_info);
+      if ((flags & SigmaValue) == 0)
+        geometry_info.sigma=geometry_info.rho;
       jpeg_calc_output_dimensions(&jpeg_info);
       image->magick_columns=jpeg_info.output_width;
       image->magick_rows=jpeg_info.output_height;
-      scale_factor=(double) jpeg_info.output_width/image->columns;
-      if (scale_factor > ((double) jpeg_info.output_height/image->rows))
-        scale_factor=(double) jpeg_info.output_height/image->rows;
+      scale_factor=1.0;
+      if (geometry_info.rho != 0.0)
+        scale_factor=jpeg_info.output_width/geometry_info.rho;
+      if ((geometry_info.sigma != 0.0) &&
+          (scale_factor > (jpeg_info.output_height/geometry_info.sigma)))
+        scale_factor=jpeg_info.output_height/geometry_info.sigma;
+      jpeg_info.scale_num=1U;
       jpeg_info.scale_denom=(unsigned int) scale_factor;
       jpeg_calc_output_dimensions(&jpeg_info);
       if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Scale factor: %ld",
           (long) scale_factor);
     }
-  if (image_info->subrange != 0)
-    {
-      jpeg_info.scale_denom=(unsigned int) image_info->subrange;
-      jpeg_calc_output_dimensions(&jpeg_info);
-    }
-  precision=jpeg_info.data_precision;
+  precision=(unsigned long) jpeg_info.data_precision;
 #if (JPEG_LIB_VERSION >= 61) && defined(D_PROGRESSIVE_SUPPORTED)
 #if defined(D_LOSSLESS_SUPPORTED)
   image->interlace=jpeg_info.process == JPROC_PROGRESSIVE ?
@@ -812,7 +1045,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   image->compression=JPEGCompression;
   image->interlace=JPEGInterlace;
 #endif
-  if ((image_info->colors != 0) && (image_info->colors <= 256))
+  if ((image_info->colors > 8) && (image_info->colors <= 256))
     {
       /*
         Let the JPEG library quantize for us.
@@ -820,10 +1053,28 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       jpeg_info.quantize_colors=MagickTrue;
       jpeg_info.desired_number_of_colors=(int) image_info->colors;
     }
+  option=GetImageOption(image_info,"jpeg:block-smoothing");
+  if (option != (const char *) NULL)
+    {
+      jpeg_info.do_block_smoothing=MagickFalse;
+      if (IsMagickTrue(option) != MagickFalse)
+        jpeg_info.do_block_smoothing=MagickTrue;
+    }
+  option=GetImageOption(image_info,"jpeg:fancy-upsampling");
+  if (option != (const char *) NULL)
+    {
+      jpeg_info.do_fancy_upsampling=MagickFalse;
+      if (IsMagickTrue(option) != MagickFalse)
+        jpeg_info.do_fancy_upsampling=MagickTrue;
+    }
   (void) jpeg_start_decompress(&jpeg_info);
   image->columns=jpeg_info.output_width;
   image->rows=jpeg_info.output_height;
   image->depth=(unsigned long) jpeg_info.data_precision;
+  if (jpeg_info.out_color_space == JCS_YCbCr)
+    image->colorspace=YCbCrColorspace;
+  if (jpeg_info.out_color_space == JCS_CMYK)
+    image->colorspace=CMYKColorspace;
   if ((image_info->colors != 0) && (image_info->colors <= 256))
     if (AcquireImageColormap(image,image_info->colors) == MagickFalse)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -850,206 +1101,11 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %dx%d",
         (int) jpeg_info.output_width,(int) jpeg_info.output_height);
     }
-  image->quality=UndefinedCompressionQuality;
-#if defined(D_PROGRESSIVE_SUPPORTED)
-  if (image->compression == LosslessJPEGCompression)
-    {
-      image->quality=100;
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-        "Quality: 100 (lossless)");
-    }
-  else
-#endif
-  {
-    long
-      value,
-      sum;
-
-    register long
-      j;
-
-    /*
-      Determine the JPEG compression quality from the quantization tables.
-    */
-    sum=0;
-    for (i=0; i < NUM_QUANT_TBLS; i++)
-    {
-      if (jpeg_info.quant_tbl_ptrs[i] != NULL)
-        for (j=0; j < DCTSIZE2; j++)
-          sum+=jpeg_info.quant_tbl_ptrs[i]->quantval[j];
-     }
-     if ((jpeg_info.quant_tbl_ptrs[0] != NULL) &&
-         (jpeg_info.quant_tbl_ptrs[1] != NULL))
-       {
-         long
-           hash[101] =
-           {
-             1020, 1015,  932,  848,  780,  735,  702,  679,  660,  645,
-              632,  623,  613,  607,  600,  594,  589,  585,  581,  571,
-              555,  542,  529,  514,  494,  474,  457,  439,  424,  410,
-              397,  386,  373,  364,  351,  341,  334,  324,  317,  309,
-              299,  294,  287,  279,  274,  267,  262,  257,  251,  247,
-              243,  237,  232,  227,  222,  217,  213,  207,  202,  198,
-              192,  188,  183,  177,  173,  168,  163,  157,  153,  148,
-              143,  139,  132,  128,  125,  119,  115,  108,  104,   99,
-               94,   90,   84,   79,   74,   70,   64,   59,   55,   49,
-               45,   40,   34,   30,   25,   20,   15,   11,    6,    4,
-                0
-           },
-           sums[101] =
-           {
-             32640, 32635, 32266, 31495, 30665, 29804, 29146, 28599, 28104,
-             27670, 27225, 26725, 26210, 25716, 25240, 24789, 24373, 23946,
-             23572, 22846, 21801, 20842, 19949, 19121, 18386, 17651, 16998,
-             16349, 15800, 15247, 14783, 14321, 13859, 13535, 13081, 12702,
-             12423, 12056, 11779, 11513, 11135, 10955, 10676, 10392, 10208,
-              9928,  9747,  9564,  9369,  9193,  9017,  8822,  8639,  8458,
-              8270,  8084,  7896,  7710,  7527,  7347,  7156,  6977,  6788,
-              6607,  6422,  6236,  6054,  5867,  5684,  5495,  5305,  5128,
-              4945,  4751,  4638,  4442,  4248,  4065,  3888,  3698,  3509,
-              3326,  3139,  2957,  2775,  2586,  2405,  2216,  2037,  1846,
-              1666,  1483,  1297,  1109,   927,   735,   554,   375,   201,
-               128,     0
-           };
-
-         value=(long) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
-           jpeg_info.quant_tbl_ptrs[0]->quantval[53]+
-           jpeg_info.quant_tbl_ptrs[1]->quantval[0]+
-           jpeg_info.quant_tbl_ptrs[1]->quantval[DCTSIZE2-1]);
-         for (i=0; i < 100; i++)
-         {
-           if ((value < hash[i]) && (sum < sums[i]))
-             continue;
-           if ((value <= hash[i]) && (sum <= sums[i]))
-             image->quality=(unsigned long) i+1;
-           if (image->debug != MagickFalse)
-             {
-               if (image->quality != UndefinedCompressionQuality)
-                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                   "Quality: %ld",image->quality);
-               else
-                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                   "Quality: %ld (approximate)",i+1);
-             }
-           break;
-         }
-       }
-     else
-       if (jpeg_info.quant_tbl_ptrs[0] != NULL)
-         {
-           long
-             hash[101] =
-             {
-               510,  505,  422,  380,  355,  338,  326,  318,  311,  305,
-               300,  297,  293,  291,  288,  286,  284,  283,  281,  280,
-               279,  278,  277,  273,  262,  251,  243,  233,  225,  218,
-               211,  205,  198,  193,  186,  181,  177,  172,  168,  164,
-               158,  156,  152,  148,  145,  142,  139,  136,  133,  131,
-               129,  126,  123,  120,  118,  115,  113,  110,  107,  105,
-               102,  100,   97,   94,   92,   89,   87,   83,   81,   79,
-                76,   74,   70,   68,   66,   63,   61,   57,   55,   52,
-                50,   48,   44,   42,   39,   37,   34,   31,   29,   26,
-                24,   21,   18,   16,   13,   11,    8,    6,    3,    2,
-                 0
-             },
-             sums[101] =
-             {
-               16320, 16315, 15946, 15277, 14655, 14073, 13623, 13230, 12859,
-               12560, 12240, 11861, 11456, 11081, 10714, 10360, 10027,  9679,
-                9368,  9056,  8680,  8331,  7995,  7668,  7376,  7084,  6823,
-                6562,  6345,  6125,  5939,  5756,  5571,  5421,  5240,  5086,
-                4976,  4829,  4719,  4616,  4463,  4393,  4280,  4166,  4092,
-                3980,  3909,  3835,  3755,  3688,  3621,  3541,  3467,  3396,
-                3323,  3247,  3170,  3096,  3021,  2952,  2874,  2804,  2727,
-                2657,  2583,  2509,  2437,  2362,  2290,  2211,  2136,  2068,
-                1996,  1915,  1858,  1773,  1692,  1620,  1552,  1477,  1398,
-                1326,  1251,  1179,  1109,  1031,   961,   884,   814,   736,
-                 667,   592,   518,   441,   369,   292,   221,   151,    86,
-                  64,     0
-             };
-
-           value=(long) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
-             jpeg_info.quant_tbl_ptrs[0]->quantval[53]);
-           for (i=0; i < 100; i++)
-           {
-             if ((value < hash[i]) && (sum < sums[i]))
-               continue;
-             image->quality=(unsigned long) i+1;
-             if (image->debug != MagickFalse)
-               {
-                 if ((value > hash[i]) || (sum > sums[i]))
-                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                     "Quality: %ld (approximate)",i+1);
-                 else
-                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                     "Quality: %ld",i+1);
-               }
-             break;
-           }
-         }
-  }
+  JPEGSetImageQuality(&jpeg_info,image);
+  JPEGSetImageSamplingFactor(&jpeg_info,image);
   (void) FormatMagickString(value,MaxTextExtent,"%ld",(long)
     jpeg_info.out_color_space);
   (void) SetImageProperty(image,"jpeg:colorspace",value);
-  /*
-    Set image sampling factors.
-  */
-  switch (jpeg_info.out_color_space)
-  {
-    case JCS_CMYK:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: CMYK");
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,
-        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor,
-        jpeg_info.comp_info[1].h_samp_factor,
-        jpeg_info.comp_info[1].v_samp_factor,
-        jpeg_info.comp_info[2].h_samp_factor,
-        jpeg_info.comp_info[2].v_samp_factor,
-        jpeg_info.comp_info[3].h_samp_factor,
-        jpeg_info.comp_info[3].v_samp_factor);
-        break;
-    }
-    case JCS_GRAYSCALE:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-        "Colorspace: GRAYSCALE");
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,"%dx%d",
-        jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor);
-      break;
-    }
-    case JCS_RGB:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: RGB");
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,
-        "%dx%d,%dx%d,%dx%d",jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor,
-        jpeg_info.comp_info[1].h_samp_factor,
-        jpeg_info.comp_info[1].v_samp_factor,
-        jpeg_info.comp_info[2].h_samp_factor,
-        jpeg_info.comp_info[2].v_samp_factor);
-      break;
-    }
-    default:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: %d",
-        jpeg_info.out_color_space);
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,
-        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor,
-        jpeg_info.comp_info[1].h_samp_factor,
-        jpeg_info.comp_info[1].v_samp_factor,
-        jpeg_info.comp_info[2].h_samp_factor,
-        jpeg_info.comp_info[2].v_samp_factor,
-        jpeg_info.comp_info[3].h_samp_factor,
-        jpeg_info.comp_info[3].v_samp_factor);
-      break;
-    }
-  }
-  (void) SetImageProperty(image,"jpeg:sampling-factor",sampling_factor);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-    "Sampling Factors: %s",sampling_factor);
   if (image_info->ping != MagickFalse)
     {
       jpeg_destroy_decompress(&jpeg_info);
@@ -1065,15 +1121,14 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   */
   if (setjmp(error_manager.error_recovery) != 0)
     {
-      jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
+      if (jpeg_pixels != (unsigned char *) NULL)
+        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
       jpeg_destroy_decompress(&jpeg_info);
-      InheritException(exception,&image->exception);
       (void) CloseBlob(image);
       number_pixels=(MagickSizeType) image->columns*image->rows;
       if (number_pixels != 0)
         return(GetFirstImageInList(image));
-      image=DestroyImage(image);
-      return(image);
+      return(DestroyImage(image));
     }
   if (jpeg_info.quantize_colors != MagickFalse)
     {
@@ -1098,9 +1153,18 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   scanline[0]=(JSAMPROW) jpeg_pixels;
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *__restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *__restrict q;
+
     if (jpeg_read_scanlines(&jpeg_info,scanline,1) != 1)
       {
-        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        (void) ThrowMagickException(exception,GetMagickModule(),
           CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
         continue;
       }
@@ -1121,9 +1185,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
               pixel=(unsigned long) GETJSAMPLE(*p);
             else
               pixel=(unsigned long) ((GETJSAMPLE(*p) ^ 0x80) << 4);
-            index=ConstrainColormapIndex(image,pixel);
-            indexes[x]=index;
-            *q++=image->colormap[(int) index];
+            jindex=ConstrainColormapIndex(image,pixel);
+            indexes[x]=jindex;
+            *q++=image->colormap[(int) jindex];
             p++;
           }
         else
@@ -1158,9 +1222,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       if (jpeg_info.output_components == 1)
         for (x=0; x < (long) image->columns; x++)
         {
-          index=ConstrainColormapIndex(image,(unsigned long) GETJSAMPLE(*p));
-          indexes[x]=(IndexPacket) index;
-          *q++=image->colormap[(int) index];
+          jindex=ConstrainColormapIndex(image,(unsigned long) GETJSAMPLE(*p));
+          indexes[x]=(IndexPacket) jindex;
+          *q++=image->colormap[(int) jindex];
           p++;
         }
       else
@@ -1199,7 +1263,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   jpeg_destroy_decompress(&jpeg_info);
   jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
   (void) CloseBlob(image);
-  InheritException(exception,&image->exception);
   return(GetFirstImageInList(image));
 }
 #endif
@@ -1229,9 +1292,19 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 */
 ModuleExport unsigned long RegisterJPEGImage(void)
 {
+  char
+    version[MaxTextExtent];
+
   MagickInfo
     *entry;
 
+  static const char
+    description[] = "Joint Photographic Experts Group JFIF format";
+
+  *version='\0';
+#if defined(JPEG_LIB_VERSION)
+  (void) FormatMagickString(version,MaxTextExtent,"%d",JPEG_LIB_VERSION);
+#endif
   entry=SetMagickInfo("JPEG");
   entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_JPEG_DELEGATE)
@@ -1240,17 +1313,9 @@ ModuleExport unsigned long RegisterJPEGImage(void)
 #endif
   entry->magick=(IsImageFormatHandler *) IsJPEG;
   entry->adjoin=MagickFalse;
-  entry->description=ConstantString(
-    "Joint Photographic Experts Group JFIF format");
-#if defined(JPEG_LIB_VERSION)
-  {
-    char
-      version[MaxTextExtent];
-
-    (void) FormatMagickString(version,MaxTextExtent,"%d",JPEG_LIB_VERSION);
+  entry->description=ConstantString(description);
+  if (*version != '\0')
     entry->version=ConstantString(version);
-  }
-#endif
   entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JPG");
@@ -1260,8 +1325,9 @@ ModuleExport unsigned long RegisterJPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteJPEGImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->description=ConstantString(
-    "Joint Photographic Experts Group JFIF format");
+  entry->description=ConstantString(description);
+  if (*version != '\0')
+    entry->version=ConstantString(version);
   entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PJPEG");
@@ -1271,8 +1337,9 @@ ModuleExport unsigned long RegisterJPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteJPEGImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->description=ConstantString(
-    "Progessive Joint Photographic Experts Group JFIF");
+  entry->description=ConstantString(description);
+  if (*version != '\0')
+    entry->version=ConstantString(version);
   entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
@@ -1393,6 +1460,9 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
   const StringInfo
     *profile;
 
+  MagickBooleanType
+    iptc;
+
   register long
     i;
 
@@ -1408,6 +1478,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
   /*
     Save image profile as a APP marker.
   */
+  iptc=MagickFalse;
   custom_profile=AcquireStringInfo(65535L);
   ResetImageProfileIterator(image);
   for (name=GetNextImageProfile(image); name != (const char *) NULL; )
@@ -1440,7 +1511,8 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
             custom_profile),(unsigned int) (length+tag_length));
         }
       }
-    if ((LocaleCompare(name,"IPTC") == 0) || (LocaleCompare(name,"8BIM") == 0))
+    if (((LocaleCompare(name,"IPTC") == 0) ||
+        (LocaleCompare(name,"8BIM") == 0)) && (iptc == MagickFalse))
       {
         register unsigned char
           *p;
@@ -1448,6 +1520,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
         unsigned long
           roundup;
 
+        iptc=MagickTrue;
         p=GetStringInfoDatum(custom_profile);
         if (LocaleNCompare((char *) GetStringInfoDatum(profile),"8BIM",4) == 0)
           {
@@ -1482,9 +1555,9 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
         /*
           Add namespace to XMP profile.
         */
-        xmp_profile=StringToStringInfo("http://ns.adobe.com/xap/1.0/ ");
+        xmp_profile=StringToStringInfo("http://ns.adobe.com/xap/1.0/");
         ConcatenateStringInfo(xmp_profile,profile);
-        GetStringInfoDatum(xmp_profile)[29]='\0';
+        GetStringInfoDatum(xmp_profile)[28]='\0';
         for (i=0; i < (long) GetStringInfoLength(xmp_profile); i+=65533L)
         {
           length=MagickMin(GetStringInfoLength(xmp_profile)-i,65533L);
@@ -1586,18 +1659,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
   register JSAMPLE
     *q;
 
   register long
-    i,
-    x;
+    i;
 
   struct jpeg_compress_struct
     jpeg_info;
@@ -1631,14 +1697,15 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   if (setjmp(error_manager.error_recovery) != 0)
     {
       jpeg_destroy_compress(&jpeg_info);
-      if (jpeg_pixels != (unsigned char *) NULL)
-        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
       (void) CloseBlob(image);
       return(MagickFalse);
     }
   jpeg_info.client_data=(void *) &error_manager;
   jpeg_create_compress(&jpeg_info);
   JPEGDestinationManager(&jpeg_info,image);
+  if ((image->columns != (unsigned int) image->columns) ||
+      (image->rows != (unsigned int) image->rows))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   jpeg_info.image_width=(unsigned int) image->columns;
   jpeg_info.image_height=(unsigned int) image->rows;
   jpeg_info.input_components=3;
@@ -1950,7 +2017,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
         case RGBColorspace:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            " Image colorspace is RGB");
+            "Image colorspace is RGB");
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Sampling factors: %dx%d,%dx%d,%dx%d",
             jpeg_info.comp_info[0].h_samp_factor,
@@ -2012,12 +2079,26 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     jpeg_info.input_components*sizeof(*jpeg_pixels));
   if (jpeg_pixels == (JSAMPLE *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  if (setjmp(error_manager.error_recovery) != 0)
+    {
+      jpeg_destroy_compress(&jpeg_info);
+      if (jpeg_pixels != (unsigned char *) NULL)
+        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
+      (void) CloseBlob(image);
+      return(MagickFalse);
+    }
   scanline[0]=(JSAMPROW) jpeg_pixels;
   if (jpeg_info.data_precision > 8)
     {
       if (jpeg_info.in_color_space == JCS_GRAYSCALE)
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *p;
+
+          register long
+            x;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
@@ -2037,6 +2118,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             (jpeg_info.in_color_space == JCS_YCbCr))
           for (y=0; y < (long) image->rows; y++)
           {
+            register const PixelPacket
+              *p;
+
+            register long
+              x;
+
             p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
@@ -2056,6 +2143,15 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
         else
           for (y=0; y < (long) image->rows; y++)
           {
+            register const IndexPacket
+              *indexes;
+
+            register const PixelPacket
+              *p;
+
+            register long
+              x;
+
             p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
@@ -2082,6 +2178,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     if (jpeg_info.in_color_space == JCS_GRAYSCALE)
       for (y=0; y < (long) image->rows; y++)
       {
+        register const PixelPacket
+          *p;
+
+        register long
+          x;
+
         p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
@@ -2100,6 +2202,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           (jpeg_info.in_color_space == JCS_YCbCr))
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *p;
+
+          register long
+            x;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
@@ -2118,6 +2226,15 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       else
         for (y=0; y < (long) image->rows; y++)
         {
+          register const IndexPacket
+            *indexes;
+
+          register const PixelPacket
+            *p;
+
+          register long
+            x;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
