@@ -42,23 +42,31 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/colormap.h"
+#include "magick/colormap-private.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
+#include "magick/histogram.h"
 #include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
+#include "magick/property.h"
 #include "magick/quantize.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
 
 /*
@@ -66,11 +74,11 @@
 */
 #define PALM_IS_COMPRESSED_FLAG  0x8000
 #define PALM_HAS_COLORMAP_FLAG  0x4000
+#define PALM_HAS_FOUR_BYTE_FIELD  0x0200
 #define PALM_HAS_TRANSPARENCY_FLAG  0x2000
 #define PALM_IS_INDIRECT  0x1000
 #define PALM_IS_FOR_SCREEN  0x0800
 #define PALM_IS_DIRECT_COLOR  0x0400
-
 #define PALM_COMPRESSION_SCANLINE  0x00
 #define PALM_COMPRESSION_RLE  0x01
 #define PALM_COMPRESSION_NONE  0xFF
@@ -177,7 +185,6 @@ static MagickBooleanType
 %
 %    o pixel: a pointer to the PixelPacket to be matched.
 %
-%
 */
 static int FindColor(PixelPacket *pixel)
 {
@@ -218,8 +225,22 @@ static int FindColor(PixelPacket *pixel)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
+static inline long MagickMax(const long x,const long y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline ssize_t MagickMin(const ssize_t x,const ssize_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static Image *ReadPALMImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -290,11 +311,11 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      (void) DestroyImageList(image);
       return((Image *) NULL);
     }
   totalOffset=0;
@@ -302,6 +323,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   {
     image->columns=ReadBlobMSBShort(image);
     image->rows=ReadBlobMSBShort(image);
+    if (EOFBlob(image) != MagickFalse)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     /*
       Copied from coders/pnm.c. TODO other checks ...
     */
@@ -309,60 +332,67 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
       ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
     bytes_per_row=ReadBlobMSBShort(image);
     flags=ReadBlobMSBShort(image);
-    bits_per_pixel=ReadBlobByte(image);
+    bits_per_pixel=(unsigned long) ReadBlobByte(image);
     if (bits_per_pixel > 16)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-    version=ReadBlobByte(image);
-    nextDepthOffset=ReadBlobMSBShort(image);
-    transparentIndex=ReadBlobByte(image);
-    compressionType=ReadBlobByte(image);
+    version=(unsigned long) ReadBlobByte(image);
+    nextDepthOffset=(unsigned long) ReadBlobMSBShort(image);
+    transparentIndex=(unsigned long) ReadBlobByte(image);
+    compressionType=(unsigned long) ReadBlobByte(image);
     pad=ReadBlobMSBShort(image);
     /*
       Initialize image colormap.
     */
     if ((bits_per_pixel < 16) &&
-        !AllocateImageColormap(image,1L << bits_per_pixel))
+        (AcquireImageColormap(image,1L << bits_per_pixel) == MagickFalse))
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     GetMagickPixelPacket(image,&transpix);
     if (bits_per_pixel == 16)  /* Direct Color */
       {
-        redbits=ReadBlobByte(image);  /* # of bits of red */
-        greenbits=ReadBlobByte(image);  /* # of bits of green */
-        bluebits=ReadBlobByte(image);  /* # of bits of blue */
+        redbits=(unsigned long) ReadBlobByte(image);  /* # of bits of red */
+        greenbits=(unsigned long) ReadBlobByte(image);  /* # of bits of green */
+        bluebits=(unsigned long) ReadBlobByte(image);  /* # of bits of blue */
         ReadBlobByte(image);  /* reserved by Palm */
         ReadBlobByte(image);  /* reserved by Palm */
-        transpix.red=QuantumRange*ReadBlobByte(image)/31;
-        transpix.green=QuantumRange*ReadBlobByte(image)/63;
-        transpix.blue=QuantumRange*ReadBlobByte(image)/31;
+        transpix.red=(MagickRealType) (QuantumRange*ReadBlobByte(image)/31);
+        transpix.green=(MagickRealType) (QuantumRange*ReadBlobByte(image)/63);
+        transpix.blue=(MagickRealType) (QuantumRange*ReadBlobByte(image)/31);
       }
     if (bits_per_pixel == 8)
       {
+        IndexPacket
+          index;
+
         if (flags & PALM_HAS_COLORMAP_FLAG)
           {
-            count=ReadBlobMSBShort(image);
+            count=(ssize_t) ReadBlobMSBShort(image);
             for (i=0; i < (long) count; i++)
             {
               ReadBlobByte(image);
-              image->colormap[255-i].red=
-                ScaleCharToQuantum(ReadBlobByte(image));
-              image->colormap[255-i].green=
-                ScaleCharToQuantum(ReadBlobByte(image));
-              image->colormap[255-i].blue=
-                ScaleCharToQuantum(ReadBlobByte(image));
+              index=ConstrainColormapIndex(image,255-i);
+              image->colormap[(int) index].red=
+                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+              image->colormap[(int) index].green=
+                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+              image->colormap[(int) index].blue=
+                ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
           }
         }
       else
         {
-          for (i = 0; i < (long) (1L << bits_per_pixel); i++)
+          for (i=0; i < (long) (1L << bits_per_pixel); i++)
           {
-            image->colormap[255-i].red=ScaleCharToQuantum(PalmPalette[i][0]);
-            image->colormap[255-i].green=ScaleCharToQuantum(PalmPalette[i][1]);
-            image->colormap[255-i].blue=ScaleCharToQuantum(PalmPalette[i][2]);
+            index=ConstrainColormapIndex(image,255-i);
+            image->colormap[(int) index].red=
+              ScaleCharToQuantum(PalmPalette[i][0]);
+            image->colormap[(int) index].green=
+              ScaleCharToQuantum(PalmPalette[i][1]);
+            image->colormap[(int) index].blue=
+              ScaleCharToQuantum(PalmPalette[i][2]);
           }
         }
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)
-      /* TODO unused: cast to void? */
       size=ReadBlobMSBShort(image);
     image->storage_class=DirectClass;
     if (bits_per_pixel < 16)
@@ -370,16 +400,17 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         image->storage_class=PseudoClass;
         image->depth=8;
       }
-    one_row=(unsigned char *) AcquireMagickMemory(bytes_per_row);
+    one_row=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
+      sizeof(*one_row));
     if (one_row == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     lastrow=(unsigned char *) NULL;
     if (compressionType == PALM_COMPRESSION_SCANLINE) {
-      lastrow=(unsigned char *) AcquireMagickMemory(bytes_per_row);
-      if (lastrow == (unsigned char *) NULL)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      lastrow=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
+        sizeof(*lastrow));
+    if (lastrow == (unsigned char *) NULL)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
-    /* TODO move to point just before first use */
     mask=(1l << bits_per_pixel)-1;
     for (y = 0; y < (long) image->rows; y++)
     {
@@ -397,9 +428,10 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
               image->compression=RLECompression;
               for (i=0; i < (long) bytes_per_row; )
               {
-                count=ReadBlobByte(image);
-                byte=ReadBlobByte(image);
-                (void) ResetMagickMemory(one_row+i,(int) byte,count);
+                count=(ssize_t) ReadBlobByte(image);
+                count=MagickMin(count,(ssize_t) bytes_per_row-i);
+                byte=(unsigned long) ReadBlobByte(image);
+                (void) ResetMagickMemory(one_row+i,(int) byte,(size_t) count);
                 i+=count;
               }
           }
@@ -410,33 +442,35 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
               image->compression=FaxCompression;
               for (i=0; i < (long) bytes_per_row; i+=8)
               {
-                count=ReadBlobByte(image);
-                byte=Min(bytes_per_row-i,8);
+                count=(ssize_t) ReadBlobByte(image);
+                byte=1UL*MagickMin((ssize_t) bytes_per_row-i,8);
                 for (bit=0; bit < byte; bit++)
                 {
                   if ((y == 0) || (count & (1 << (7 - bit))))
-                    one_row[i+bit]=ReadBlobByte(image);
+                    one_row[i+bit]=(unsigned char) ReadBlobByte(image);
                   else
                     one_row[i+bit]=lastrow[i+bit];
                 }
               }
-              CopyMagickMemory(lastrow, one_row, bytes_per_row);
+              (void) CopyMagickMemory(lastrow, one_row, bytes_per_row);
             }
         }
       ptr=one_row;
-      q=SetImagePixels(image,0,y,image->columns,1);
+      q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
+      indexes=GetAuthenticIndexQueue(image);
       if (bits_per_pixel == 16)
         {
+          if (image->columns > (2*bytes_per_row))
+            ThrowReaderException(CorruptImageError,"CorruptImage");
           for (x=0; x < (long) image->columns; x++)
           {
             color16=(*ptr++ << 8);
             color16|=(*ptr++);
-            q->red=(QuantumRange*((color16 >> 11) & 0x1f))/0x1f;
-            q->green=(QuantumRange*((color16 >> 5) & 0x3f))/0x3f;
-            q->blue=(QuantumRange*((color16 >> 0) & 0x1f))/0x1f;
+            q->red=(Quantum) ((QuantumRange*((color16 >> 11) & 0x1f))/0x1f);
+            q->green=(Quantum) ((QuantumRange*((color16 >> 5) & 0x3f))/0x3f);
+            q->blue=(Quantum) ((QuantumRange*((color16 >> 0) & 0x1f))/0x1f);
             q->opacity=OpaqueOpacity;
             q++;
           }
@@ -446,9 +480,11 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
           bit=8-bits_per_pixel;
           for (x=0; x < (long) image->columns; x++)
           {
+            if ((size_t) (ptr-one_row) >= bytes_per_row)
+              ThrowReaderException(CorruptImageError,"CorruptImage");
             index=(IndexPacket) (mask-(((*ptr) & (mask << bit)) >> bit));
             indexes[x]=index;
-            *q++=image->colormap[index];
+            *q++=image->colormap[(long) index];
             if (bit)
               bit-=bits_per_pixel;
             else
@@ -457,16 +493,23 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
                 bit=8-bits_per_pixel;
               }
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
         }
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
     if (flags & PALM_HAS_TRANSPARENCY_FLAG)
       {
         if (bits_per_pixel != 16)
-          SetMagickPixelPacket(image->colormap+(mask-transparentIndex),
+          SetMagickPixelPacket(image,image->colormap+(mask-transparentIndex),
             (const IndexPacket *) NULL,&transpix);
-        PaintTransparentImage(image,&transpix,TransparentOpacity);
+        (void) TransparentPaintImage(image,&transpix,(Quantum)
+          TransparentOpacity,MagickFalse);
       }
     one_row=(unsigned char *) RelinquishMagickMemory(one_row);
     if (compressionType == PALM_COMPRESSION_SCANLINE)
@@ -496,24 +539,20 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         /*
           Allocate next image structure. Copied from coders/pnm.c
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            (void) DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        /* TODO Analyze this ...
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }*/
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (nextDepthOffset != 0);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -528,29 +567,30 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterPALMImage() adds attributes for the PALM image format to the list of
-%  supported formats.  The attributes include the image format tag, a method to
+%  RegisterPALMImage() adds properties for the PALM image format to the list of
+%  supported formats.  The properties include the image format tag, a method to
 %  read and/or write the format, whether the format supports the saving of more
 %  than one frame to the same file or blob, whether the format supports native
 %  in-memory I/O, and a brief description of the format.
 %
 %  The format of the RegisterPALMImage method is:
 %
-%      RegisterPALMImage(void)
+%      unsigned long RegisterPALMImage(void)
 %
 */
-ModuleExport void RegisterPALMImage(void)
+ModuleExport unsigned long RegisterPALMImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("PALM");
-  entry->decoder=(DecoderHandler *) ReadPALMImage;
-  entry->encoder=(EncoderHandler *) WritePALMImage;
+  entry->decoder=(DecodeImageHandler *) ReadPALMImage;
+  entry->encoder=(EncodeImageHandler *) WritePALMImage;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Palm pixmap");
-  entry->module=AcquireString("PALM");
+  entry->description=ConstantString("Palm pixmap");
+  entry->module=ConstantString("PALM");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -574,7 +614,7 @@ ModuleExport void RegisterPALMImage(void)
 */
 ModuleExport void UnregisterPALMImage(void)
 {
-  UnregisterMagickInfo("PALM");
+  (void) UnregisterMagickInfo("PALM");
 }
 
 /*
@@ -592,7 +632,8 @@ ModuleExport void UnregisterPALMImage(void)
 %
 %  The format of the WritePALMImage method is:
 %
-%      MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePALMImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -600,19 +641,13 @@ ModuleExport void UnregisterPALMImage(void)
 %
 %    o image:  A pointer to a Image structure.
 %
-%
 */
-static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePALMImage(const ImageInfo *image_info,
+  Image *image)
 {
   int
     y;
 
-  const ImageAttribute
-    *attribute;
-
-  Image
-    *map;
-
   ExceptionInfo
     exception;
 
@@ -620,7 +655,9 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
     status;
 
   MagickOffsetType
-    currentOffset;
+    currentOffset,
+    offset,
+    scene;
 
   MagickSizeType
     cc;
@@ -629,7 +666,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
     transpix;
 
   QuantizeInfo
-    quantize_info;
+    *quantize_info;
 
   register IndexPacket
     *indexes;
@@ -647,10 +684,10 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
     *lastrow,
     *one_row,
     *ptr,
-    version = 0;
+    version;
 
   unsigned int
-    transparentIndex = 0;
+    transparentIndex;
 
   unsigned long
     count,
@@ -659,9 +696,8 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
     nextDepthOffset;
 
   unsigned short
-    offset,
     color16,
-    flags = 0;
+    flags;
 
 
   /*
@@ -677,51 +713,60 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
   if (status == MagickFalse)
     return(status);
   GetExceptionInfo(&exception);
+  quantize_info=AcquireQuantizeInfo(image_info);
+  flags=0;
   currentOffset=0;
+  transparentIndex=0;
   transpix.red=0;
   transpix.green=0;
   transpix.blue=0;
   transpix.opacity=0;
+  version=0;
+  scene=0;
   do
   {
-    attribute=GetImageAttribute(image, "Comment");
-    if (attribute != (ImageAttribute *)NULL)
-      if (LocaleCompare("COLORMAP",attribute->value) == 0)
-        flags|=PALM_HAS_COLORMAP_FLAG;
-    count = GetNumberColors(image, NULL, &exception);
-    for (bits_per_pixel=1;  (1UL << bits_per_pixel) < count;  bits_per_pixel*=2);
+    if (image->colorspace != RGBColorspace)
+      (void) TransformImageColorspace(image,RGBColorspace);
+    count=GetNumberColors(image,NULL,&exception);
+    for (bits_per_pixel=1;  (1UL << bits_per_pixel) < count;  bits_per_pixel*=2) ;
     if (image_info->depth > 100)
       bits_per_pixel=image_info->depth-100;
     if (bits_per_pixel < 16)
-      SetImageColorspace(image,image->colorspace);
+      (void) TransformImageColorspace(image,image->colorspace);
     if (bits_per_pixel < 8)
       {
-        SetImageColorspace(image,GRAYColorspace);
+        (void) TransformImageColorspace(image,GRAYColorspace);
+        (void) SetImageType(image,PaletteType);
         (void) SortColormapByIntensity(image);
       }
-    if (bits_per_pixel > 8)
+    if ((image->storage_class == PseudoClass) && (image->colors > 256))
+      (void) SetImageStorageClass(image,DirectClass);
+    if (image->storage_class == PseudoClass)
+      flags|=PALM_HAS_COLORMAP_FLAG;
+    else
       flags|=PALM_IS_DIRECT_COLOR;
-    (void) WriteBlobMSBShort(image,image->columns);  /* width */
-    (void) WriteBlobMSBShort(image,image->rows );  /* height */
-    bytes_per_row=((image->columns+(16/bits_per_pixel-1))/(16/bits_per_pixel))*2;
-    (void) WriteBlobMSBShort(image, bytes_per_row);
-    if ((image->compression == RLECompression) ||
-        (image->compression == FaxCompression))
+    (void) WriteBlobMSBShort(image,(unsigned short) image->columns); /* width */
+    (void) WriteBlobMSBShort(image,(unsigned short) image->rows);  /* height */
+    bytes_per_row=((image->columns+(16/bits_per_pixel-1))/(16/
+      bits_per_pixel))*2;
+    (void) WriteBlobMSBShort(image,(unsigned short) bytes_per_row);
+    if ((image_info->compression == RLECompression) ||
+        (image_info->compression == FaxCompression))
       flags|=PALM_IS_COMPRESSED_FLAG;
     (void) WriteBlobMSBShort(image, flags);
-    (void) WriteBlobByte(image, bits_per_pixel);
+    (void) WriteBlobByte(image,(unsigned char) bits_per_pixel);
     if (bits_per_pixel > 1)
       version=1;
-    if ((image->compression == RLECompression) ||
-        (image->compression == FaxCompression))
+    if ((image_info->compression == RLECompression) ||
+        (image_info->compression == FaxCompression))
       version=2;
     (void) WriteBlobByte(image,version);
     (void) WriteBlobMSBShort(image,0);  /* nextDepthOffset */
-    (void) WriteBlobByte(image,transparentIndex);
-    if (image->compression == RLECompression)
+    (void) WriteBlobByte(image,(unsigned char) transparentIndex);
+    if (image_info->compression == RLECompression)
       (void) WriteBlobByte(image,PALM_COMPRESSION_RLE);
     else
-      if (image->compression == FaxCompression)
+      if (image_info->compression == FaxCompression)
         (void) WriteBlobByte(image,PALM_COMPRESSION_SCANLINE);
       else
         (void) WriteBlobByte(image,PALM_COMPRESSION_NONE);
@@ -729,72 +774,81 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
     offset=16;
     if (bits_per_pixel == 16)
       {
-        WriteBlobByte(image,5);  /* # of bits of red */
-        WriteBlobByte(image,6);  /* # of bits of green */
-        WriteBlobByte(image,5);  /* # of bits of blue */
-        WriteBlobByte(image,0);  /* reserved by Palm */
-        WriteBlobMSBLong(image,0);  /* no transparent color, YET */
+        (void) WriteBlobByte(image,5);  /* # of bits of red */
+        (void) WriteBlobByte(image,6);  /* # of bits of green */
+        (void) WriteBlobByte(image,5);  /* # of bits of blue */
+        (void) WriteBlobByte(image,0);  /* reserved by Palm */
+        (void) WriteBlobMSBLong(image,0);  /* no transparent color, YET */
         offset+=8;
       }
     if (bits_per_pixel == 8)
       {
         if (flags & PALM_HAS_COLORMAP_FLAG)  /* Write out colormap */
           {
-            GetQuantizeInfo(&quantize_info);
-            quantize_info.dither=IsPaletteImage(image,&image->exception);
-            quantize_info.number_colors=image->colors;
-            (void) QuantizeImage(&quantize_info,image);
-            WriteBlobMSBShort(image,image->colors);
+            quantize_info->dither=IsPaletteImage(image,&image->exception);
+            quantize_info->number_colors=image->colors;
+            (void) QuantizeImage(quantize_info,image);
+            (void) WriteBlobMSBShort(image,(unsigned short) image->colors);
             for (count = 0; count < image->colors; count++)
             {
-              WriteBlobByte(image,count);
-              WriteBlobByte(image,ScaleQuantumToChar(image->colormap[count].red));
-              WriteBlobByte(image,
+              (void) WriteBlobByte(image,(unsigned char) count);
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                image->colormap[count].red));
+              (void) WriteBlobByte(image,
                 ScaleQuantumToChar(image->colormap[count].green));
-              WriteBlobByte(image,
+              (void) WriteBlobByte(image,
                 ScaleQuantumToChar(image->colormap[count].blue));
             }
             offset+=2+count*4;
           }
       else  /* Map colors to Palm standard colormap */
         {
-          map=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,&exception);
-          SetImageColorspace(map,map->colorspace);
-          MapImage(image,map,image_info->dither);
+          Image
+            *affinity_image;
+
+          affinity_image=ConstituteImage(256,1,"RGB",CharPixel,&PalmPalette,
+            &exception);
+          (void) TransformImageColorspace(affinity_image,affinity_image->colorspace);
+          (void) RemapImage(quantize_info,image,affinity_image);
           for (y=0; y < (long) image->rows; y++)
           {
-            p=GetImagePixels(image,0,y,image->columns,1);
-            indexes=GetIndexes(image);
+            p=GetAuthenticPixels(image,0,y,image->columns,1,&exception);
+            indexes=GetAuthenticIndexQueue(image);
             for (x=0; x < (long) image->columns; x++)
-              indexes[x]=FindColor(&image->colormap[indexes[x]]);
+              indexes[x]=(IndexPacket) FindColor(&image->colormap[(long)
+                indexes[x]]);
           }
-          DestroyImage(map);
+          affinity_image=DestroyImage(affinity_image);
         }
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)
-      WriteBlobMSBShort(image,0);  /* fill in size later */
+      (void) WriteBlobMSBShort(image,0);  /* fill in size later */
     lastrow=(unsigned char *) NULL;
-    if (image->compression == FaxCompression)
-      lastrow=(unsigned char *) AcquireMagickMemory(bytes_per_row);
+    if (image_info->compression == FaxCompression)
+      lastrow=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
+        sizeof(*lastrow));
       /* TODO check whether memory really was acquired? */
-    one_row=(unsigned char *) AcquireMagickMemory(bytes_per_row);
+    one_row=(unsigned char *) AcquireQuantumMemory(bytes_per_row,
+      sizeof(*one_row));
     if (one_row == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=0; y < (int) image->rows; y++)
     {
       ptr=one_row;
       (void) ResetMagickMemory(ptr,0,bytes_per_row);
-      p=GetImagePixels(image,0,y,image->columns,1);
+      p=GetAuthenticPixels(image,0,y,image->columns,1,&exception);
       if (p == (PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
+      indexes=GetAuthenticIndexQueue(image);
       if (bits_per_pixel == 16)
         {
           for (x=0; x < (int) image->columns; x++)
           {
-            color16=(((31*p->red)/QuantumRange) << 11) | (((63*p->green)/QuantumRange) << 5) |
-              ((31*p->blue)/QuantumRange);
-            if (p->opacity == TransparentOpacity)
+            color16=(unsigned short) ((((31*(unsigned long) p->red)/
+              (unsigned long) QuantumRange) << 11) |
+              (((63*(unsigned long) p->green)/(unsigned long) QuantumRange) << 5) |
+              ((31*(unsigned long) p->blue)/(unsigned long) QuantumRange));
+            if (p->opacity == (Quantum) TransparentOpacity)
               {
                 transpix.red=p->red;
                 transpix.green=p->green;
@@ -802,8 +856,8 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
                 transpix.opacity=p->opacity;
                 flags|=PALM_HAS_TRANSPARENCY_FLAG;
               }
-            *ptr++=(color16 >> 8) & 0xff;
-            *ptr++=color16 & 0xff;
+            *ptr++=(unsigned char) ((color16 >> 8) & 0xff);
+            *ptr++=(unsigned char) (color16 & 0xff);
             p++;
           }
         }
@@ -817,7 +871,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
               color=(unsigned char) indexes[x];
             else
               color=(unsigned char) (indexes[x]*((1 << bits_per_pixel)-1)/
-                Max(image->colors-1,1));
+                MagickMax(1L*image->colors-1L,1L));
             byte|=color << bit;
             if (bit != 0)
               bit-=bits_per_pixel;
@@ -831,7 +885,7 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
           if ((image->columns % (8/bits_per_pixel)) != 0)
             *ptr++=byte;
         }
-      if (image->compression == RLECompression)
+      if (image_info->compression == RLECompression)
         {
           x=0;
           while (x < (long) bytes_per_row)
@@ -841,12 +895,12 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
             while ((one_row[++x] == byte) && (count < 255) &&
                    (x < (long) bytes_per_row))
               count++;
-            (void) WriteBlobByte(image,count);
-            (void) WriteBlobByte(image,byte);
+            (void) WriteBlobByte(image,(unsigned char) count);
+            (void) WriteBlobByte(image,(unsigned char) byte);
           }
         }
       else
-        if (image->compression == FaxCompression)
+        if (image_info->compression == FaxCompression)
           {
             char
               tmpbuf[8],
@@ -855,12 +909,12 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
             for (x = 0;  x < (long) bytes_per_row;  x += 8)
             {
               tptr = tmpbuf;
-              for (bit = 0, byte = 0; bit < Min(8, bytes_per_row - x); bit++)
+              for (bit=0, byte=0; bit < (unsigned char) MagickMin(8,(ssize_t) bytes_per_row-x); bit++)
               {
                 if ((y == 0) || (lastrow[x + bit] != one_row[x + bit]))
                   {
                   byte |= (1 << (7 - bit));
-                  *tptr++ = one_row[x + bit];
+                  *tptr++ = (char) one_row[x + bit];
                 }
               }
               (void) WriteBlobByte(image, byte);
@@ -873,23 +927,24 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
       }
     if (flags & PALM_HAS_TRANSPARENCY_FLAG)
       {
-        SeekBlob(image,currentOffset+6,SEEK_SET);
+        offset=SeekBlob(image,currentOffset+6,SEEK_SET);
         (void) WriteBlobMSBShort(image,flags);
-        SeekBlob(image,currentOffset+12,SEEK_SET);
-        (void) WriteBlobByte(image,transparentIndex);  /* trans index */
+        offset=SeekBlob(image,currentOffset+12,SEEK_SET);
+        (void) WriteBlobByte(image,(unsigned char) transparentIndex);  /* trans index */
       }
     if (bits_per_pixel == 16)
       {
-        SeekBlob(image,currentOffset+20,SEEK_SET);
-        WriteBlobByte(image,0);  /* reserved by Palm */
-        WriteBlobByte(image,(31*transpix.red)/QuantumRange);
-        WriteBlobByte(image,(63*transpix.green)/QuantumRange);
-        WriteBlobByte(image,(31*transpix.blue)/QuantumRange);
+        offset=SeekBlob(image,currentOffset+20,SEEK_SET);
+        (void) WriteBlobByte(image,0);  /* reserved by Palm */
+        (void) WriteBlobByte(image,(unsigned char) ((31*transpix.red)/QuantumRange));
+        (void) WriteBlobByte(image,(unsigned char) ((63*transpix.green)/QuantumRange));
+        (void) WriteBlobByte(image,(unsigned char) ((31*transpix.blue)/QuantumRange));
       }
     if (flags & PALM_IS_COMPRESSED_FLAG)  /* fill in size now */
       {
-        SeekBlob(image,currentOffset+offset,SEEK_SET);
-        (void) WriteBlobMSBShort(image,GetBlobSize(image)-currentOffset-offset);
+        offset=SeekBlob(image,currentOffset+offset,SEEK_SET);
+        (void) WriteBlobMSBShort(image,(unsigned short) (GetBlobSize(image)-
+          currentOffset-offset));
       }
     if (one_row != (unsigned char *) NULL) 
       one_row=(unsigned char *) RelinquishMagickMemory(one_row);
@@ -903,14 +958,19 @@ static MagickBooleanType WritePALMImage(const ImageInfo *image_info,Image *image
       (void) WriteBlobByte(image,0);
     } 
     /* write nextDepthOffset and return to end of image */
-    SeekBlob(image,currentOffset+10,SEEK_SET);
+    offset=SeekBlob(image,currentOffset+10,SEEK_SET);
     nextDepthOffset=(unsigned long) ((GetBlobSize(image)-currentOffset)/4);
-    (void) WriteBlobMSBShort(image,nextDepthOffset);
+    (void) WriteBlobMSBShort(image,(unsigned short) nextDepthOffset);
     currentOffset=(MagickOffsetType) GetBlobSize(image);
-    SeekBlob(image,currentOffset,SEEK_SET);
+    offset=SeekBlob(image,currentOffset,SEEK_SET);
     image=SyncNextImageInList(image);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
-  DestroyExceptionInfo(&exception);
+  quantize_info=DestroyQuantizeInfo(quantize_info);
+  (void) CloseBlob(image);
+  (void) DestroyExceptionInfo(&exception);
   return(MagickTrue);
 }
