@@ -60,6 +60,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
+#include "magick/option.h"
 #include "magick/profile.h"
 #include "magick/resource_.h"
 #include "magick/quantum-private.h"
@@ -96,12 +97,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsPCL(const unsigned char *magick,const size_t length)
 {
@@ -354,7 +353,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,density,options,
     read_info->filename,input_filename);
-  status=SystemCommand(read_info->verbose,command) != 0 ? MagickTrue :
+  status=SystemCommand(read_info->verbose,command,exception) != 0 ? MagickTrue :
     MagickFalse;
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
@@ -399,7 +398,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %  RegisterPCLImage() adds attributes for the PCL image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
-%  supports the saving of more than one frame to the same file or blob,
+%  supports the saving of more than one frame to the i file or blob,
 %  whether the format supports native in-memory I/O, and a brief
 %  description of the format.
 %
@@ -417,7 +416,6 @@ ModuleExport unsigned long RegisterPCLImage(void)
   entry->decoder=(DecodeImageHandler *) ReadPCLImage;
   entry->encoder=(EncodeImageHandler *) WritePCLImage;
   entry->magick=(IsImageFormatHandler *) IsPCL;
-  entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->thread_support=EncoderThreadSupport;
@@ -438,8 +436,8 @@ ModuleExport unsigned long RegisterPCLImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  UnregisterPCLImage() removes format registrations made by the
-%  PCL module from the list of supported formats.
+%  UnregisterPCLImage() removes format registrations made by the PCL module
+%  from the list of supported formats.
 %
 %  The format of the UnregisterPCLImage method is:
 %
@@ -476,26 +474,218 @@ ModuleExport void UnregisterPCLImage(void)
 %    o image:  The image.
 %
 */
+
+static size_t PCLDeltaCompressImage(const size_t length,
+  const unsigned char *previous_pixels,const unsigned char *pixels,
+  unsigned char *compress_pixels)
+{
+  int
+    delta,
+    j,
+    replacement;
+
+  register long
+    i,
+    x;
+
+  register unsigned char
+    *q;
+
+  q=compress_pixels;
+  for (x=0; x < (long) length; )
+  {
+    j=0;
+    for (i=0; x < (long) length; x++)
+    {
+      if (*pixels++ != *previous_pixels++)
+        {
+          i=1;
+          break;
+        }
+      j++;
+    }
+    for ( ; x < (long) length; x++)
+    {
+      if (*pixels == *previous_pixels)
+        break;
+      i++;
+      previous_pixels++;
+      pixels++;
+    }
+    if (i == 0)
+      break;
+    replacement=j >= 31 ? 31 : j;
+    j-=replacement;
+    delta=i >= 8 ? 8 : i;
+    *q++=(unsigned char) (((delta-1) << 5) | replacement);
+    if (replacement == 31)
+      {
+        for (replacement=255; j != 0; )
+        {
+          if (replacement > j)
+            replacement=j;
+          *q++=(unsigned char) replacement;
+          j-=replacement;
+        }
+        if (replacement == 255)
+          *q++='\0';
+      }
+    for (pixels-=i; i != 0; )
+    {
+      for (i-=delta; delta != 0; delta--)
+        *q++=(*pixels++);
+      if (i == 0)
+        break;
+      delta=i;
+      if (i >= 8)
+        delta=8;
+      *q++=(unsigned char) ((delta-1) << 5);
+    }
+  }
+  return((size_t) (q-compress_pixels));
+}
+
+static size_t PCLPackbitsCompressImage(const size_t length,
+  const unsigned char *pixels,unsigned char *compress_pixels)
+{
+  int
+    count;
+
+  long
+    j;
+
+  register long
+    x;
+
+  register unsigned char
+    *q;
+
+  unsigned char
+    packbits[128];
+
+  /*
+    Compress pixels with Packbits encoding.
+  */
+  q=compress_pixels;
+  for (x=(long) length; x != 0; )
+  {
+    switch (x)
+    {
+      case 1:
+      {
+        x--;
+        *q++=0;
+        *q++=(*pixels);
+        break;
+      }
+      case 2:
+      {
+        x-=2;
+        *q++=1;
+        *q++=(*pixels);
+        *q++=pixels[1];
+        break;
+      }
+      case 3:
+      {
+        x-=3;
+        if ((*pixels == *(pixels+1)) && (*(pixels+1) == *(pixels+2)))
+          {
+            *q++=(unsigned char) ((256-3)+1);
+            *q++=(*pixels);
+            break;
+          }
+        *q++=2;
+        *q++=(*pixels);
+        *q++=pixels[1];
+        *q++=pixels[2];
+        break;
+      }
+      default:
+      {
+        if ((*pixels == *(pixels+1)) && (*(pixels+1) == *(pixels+2)))
+          {
+            /*
+              Packed run.
+            */
+            count=3;
+            while (((long) count < x) && (*pixels == *(pixels+count)))
+            {
+              count++;
+              if (count >= 127)
+                break;
+            }
+            x-=count;
+            *q++=(unsigned char) ((256-count)+1);
+            *q++=(*pixels);
+            pixels+=count;
+            break;
+          }
+        /*
+          Literal run.
+        */
+        count=0;
+        while ((*(pixels+count) != *(pixels+count+1)) ||
+               (*(pixels+count+1) != *(pixels+count+2)))
+        {
+          packbits[count+1]=pixels[count];
+          count++;
+          if (((long) count >= (x-3)) || (count >= 127))
+            break;
+        }
+        x-=count;
+        *packbits=(unsigned char) (count-1);
+        for (j=0; j <= (long) count; j++)
+          *q++=packbits[j];
+        pixels+=count;
+        break;
+      }
+    }
+  }
+  *q++=128; /* EOD marker */
+  return((size_t) (q-compress_pixels));
+}
+
 static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
 {
   char
     buffer[MaxTextExtent];
 
+  const char
+    *option;
+
   long
     y;
 
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    scene;
+
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
   register long
+    i,
     x;
 
   register unsigned char
     *q;
 
+  size_t
+    length,
+    packets;
+
+  unsigned char
+    bits_per_pixel,
+    *compress_pixels,
+    *pixels,
+    *previous_pixels;
+
   unsigned long
     density;
 
@@ -511,13 +701,6 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
-  /*
-    Initialize the printer.
-  */
-  (void) WriteBlobString(image,"\033E");  /* printer reset */
-  (void) WriteBlobString(image,"\033&l0E");  /* top margin 0 */
   density=75;
   if (image_info->density != (char *) NULL)
     {
@@ -527,106 +710,268 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
       (void) ParseGeometry(image_info->density,&geometry);
       density=(unsigned long) geometry.rho;
     }
-  if (IsMonochromeImage(image,&image->exception) != MagickFalse)
-    {
-      register unsigned char
-        bit,
-        byte;
-
-      /*
-        Write PCL monochrome image.
-      */
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%ldR",density);
-      (void) WriteBlobString(image,buffer);
-      (void) WriteBlobString(image,"\033*r1A");  /* start graphics */
-      (void) WriteBlobString(image,"\033*b0M");  /* no compression */
-      for (y=0; y < (long) image->rows; y++)
+  scene=0;
+  do
+  {
+    if (image->colorspace != RGBColorspace)
+      (void) TransformImageColorspace(image,RGBColorspace);
+    /*
+      Initialize the printer.
+    */
+    (void) WriteBlobString(image,"\033E");  /* printer reset */
+    (void) WriteBlobString(image,"\033*r3F");  /* set presentation mode */
+    (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%lus%luT",
+      image->columns,image->rows);
+    (void) WriteBlobString(image,buffer);
+    (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%ldR",density);
+    (void) WriteBlobString(image,buffer);
+    (void) WriteBlobString(image,"\033&l0E");  /* top margin 0 */
+    if (IsMonochromeImage(image,&image->exception) != MagickFalse)
       {
-        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-        if (p == (const PixelPacket *) NULL)
-          break;
-        bit=0;
-        byte=0;
-        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
-          (image->columns+7)/8);
+        /*
+          Monochrome image.
+        */
+        bits_per_pixel=1;
+        (void) WriteBlobString(image,"\033*v6W"); /* set color mode... */
+        (void) WriteBlobByte(image,0); /* RGB */
+        (void) WriteBlobByte(image,1); /* indexed by pixel */
+        (void) WriteBlobByte(image,bits_per_pixel); /* bits per index */
+        (void) WriteBlobByte(image,8); /* bits per red component */
+        (void) WriteBlobByte(image,8); /* bits per green component */
+        (void) WriteBlobByte(image,8); /* bits per blue component */
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*v0a0b0c0I");
+        (void) WriteBlobString(image,buffer);
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*v1a1b1c1I");
         (void) WriteBlobString(image,buffer);
-        for (x=0; x < (long) image->columns; x++)
-        {
-          byte<<=1;
-          if (PixelIntensity(p) < ((MagickRealType) QuantumRange/2.0))
-            byte|=0x01;
-          bit++;
-          if (bit == 8)
-            {
-              (void) WriteBlobByte(image,byte);
-              bit=0;
-              byte=0;
-            }
-          p++;
-        }
-        if (bit != 0)
-          (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
-        status=SetImageProgress(image,SaveImageTag,y,image->rows);
-        if (status == MagickFalse)
-          break;
       }
-      (void) WriteBlobString(image,"\033*rB");  /* end graphics */
-    }
-  else
-    {
-      static char
-        color_mode[6] = { 0, 3, 0, 8, 8, 8 };
-
-      unsigned char
-        *pixels;
-
-      /*
-        Allocate pixel buffers.
-      */
-      pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
-        3UL*sizeof(*pixels));
-      if (pixels == (unsigned char *) NULL)
-        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-      /*
-        Write PCL color image.
-      */
-      (void) WriteBlobString(image,"\033*r3F");  /* set presentation mode */
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%luR",density);  /* set resolution */
-      (void) WriteBlobString(image,buffer);
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%luT",image->rows);
-      (void) WriteBlobString(image,buffer);
-      (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%luS",
-        image->columns);
-      (void) WriteBlobString(image,buffer);
-      (void) WriteBlobString(image,"\033*v6W");  /* set color mode */
-      (void) WriteBlob(image,6,(unsigned char *) color_mode);
+    else
+      if (image->storage_class == DirectClass)
+        {
+          /*
+            DirectClass image.
+          */
+          bits_per_pixel=24;
+          (void) WriteBlobString(image,"\033*v6W"); /* set color mode */
+          (void) WriteBlobByte(image,0); /* RGB */
+          (void) WriteBlobByte(image,3); /* direct by pixel */
+          (void) WriteBlobByte(image,0); /* bits per index (ignored) */
+          (void) WriteBlobByte(image,8); /* bits per red component */
+          (void) WriteBlobByte(image,8); /* bits per green component */
+          (void) WriteBlobByte(image,8); /* bits per blue component */
+        }
+      else
+        {
+          /*
+            Colormapped image.
+          */
+          bits_per_pixel=8;
+          (void) WriteBlobString(image,"\033*v6W"); /* set color mode... */
+          (void) WriteBlobByte(image,0); /* RGB */
+          (void) WriteBlobByte(image,1); /* indexed by pixel */
+          (void) WriteBlobByte(image,bits_per_pixel); /* bits per index */
+          (void) WriteBlobByte(image,8); /* bits per red component */
+          (void) WriteBlobByte(image,8); /* bits per green component */
+          (void) WriteBlobByte(image,8); /* bits per blue component */
+          for (i=0; i < (long) image->colors; i++)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,
+              "\033*v%da%db%dc%ldI",ScaleQuantumToChar(image->colormap[i].red),
+              ScaleQuantumToChar(image->colormap[i].green),
+              ScaleQuantumToChar(image->colormap[i].blue),i);
+            (void) WriteBlobString(image,buffer);
+          }
+          for ( ; i < (1L << bits_per_pixel); i++)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,"\033*v%luI",i);
+            (void) WriteBlobString(image,buffer);
+          }
+        }
+    option=GetImageOption(image_info,"pcl:fit-to-page");
+    if ((option != (const char *) NULL) &&
+        (IsMagickTrue(option) != MagickFalse))
+      (void) WriteBlobString(image,"\033*r3A");
+    else
       (void) WriteBlobString(image,"\033*r1A");  /* start raster graphics */
-      (void) WriteBlobString(image,"\033*b0Y");  /* set y offset */
-      (void) WriteBlobString(image,"\033*b0M");  /* no compression */
-      for (y=0; y < (long) image->rows; y++)
+    (void) WriteBlobString(image,"\033*b0Y");  /* set y offset */
+    length=(image->columns*bits_per_pixel+7)/8;
+    pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
+    if (pixels == (unsigned char *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    compress_pixels=(unsigned char *) NULL;
+    previous_pixels=(unsigned char *) NULL;
+    switch (image->compression)
+    {
+      case NoCompression:
       {
-        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-        if (p == (const PixelPacket *) NULL)
-          break;
-        q=pixels;
-        for (x=0; x < (long) image->columns; x++)
-        {
-          *q++=ScaleQuantumToChar(p->red);
-          *q++=ScaleQuantumToChar(p->green);
-          *q++=ScaleQuantumToChar(p->blue);
-          p++;
-        }
-        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
-          3*image->columns);  /* send row */
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b0M");
         (void) WriteBlobString(image,buffer);
-        (void) WriteBlob(image,3*image->columns,pixels);
-        status=SetImageProgress(image,SaveImageTag,y,image->rows);
-        if (status == MagickFalse)
-          break;
+        break;
+      }
+      case RLECompression:
+      {
+        compress_pixels=(unsigned char *) AcquireQuantumMemory(length+256,
+          sizeof(*compress_pixels));
+        if (compress_pixels == (unsigned char *) NULL)
+          {
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b2M");
+        (void) WriteBlobString(image,buffer);
+        break;
+      }
+      default:
+      {
+        compress_pixels=(unsigned char *) AcquireQuantumMemory(length+
+          (length >> 3),sizeof(*compress_pixels));
+        if (compress_pixels == (unsigned char *) NULL)
+          {
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+        previous_pixels=(unsigned char *) AcquireQuantumMemory(length,
+          sizeof(*previous_pixels));
+        if (previous_pixels == (unsigned char *) NULL)
+          {
+            compress_pixels=(unsigned char *) RelinquishMagickMemory(
+              compress_pixels);
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b3M");
+        (void) WriteBlobString(image,buffer);
+        break;
       }
-      (void) WriteBlobString(image,"\033*r0C");  /* end graphics */
-      pixels=(unsigned char *) RelinquishMagickMemory(pixels);
     }
+    for (y=0; y < (long) image->rows; y++)
+    {
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+      if (p == (const PixelPacket *) NULL)
+        break;
+      indexes=GetAuthenticIndexQueue(image);
+      q=pixels;
+      switch (bits_per_pixel)
+      {
+        case 1:
+        {
+          register unsigned char
+            bit,
+            byte;
+
+          /*
+            Monochrome image.
+          */
+          bit=0;
+          byte=0;
+          for (x=0; x < (long) image->columns; x++)
+          {
+            byte<<=1;
+            if (PixelIntensity(p) >= ((MagickRealType) QuantumRange/2.0))
+              byte|=0x01;
+            bit++;
+            if (bit == 8)
+              {
+                *q++=byte;
+                bit=0;
+                byte=0;
+              }
+            p++;
+          }
+          if (bit != 0)
+            *q++=byte << (8-bit);
+          break;
+        }
+        case 8:
+        {
+          /*
+            Colormapped image.
+          */
+          for (x=0; x < (long) image->columns; x++)
+            *q++=(unsigned char) indexes[x];
+          break;
+        }
+        case 24:
+        case 32:
+        {
+          /*
+            Truecolor image.
+          */
+          for (x=0; x < (long) image->columns; x++)
+          {
+            *q++=ScaleQuantumToChar(p->red);
+            *q++=ScaleQuantumToChar(p->green);
+            *q++=ScaleQuantumToChar(p->blue);
+            p++;
+          }
+          break;
+        }
+      }
+      switch (image->compression)
+      {
+        case NoCompression:
+        {
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
+            (unsigned long) length);
+          (void) WriteBlobString(image,buffer);
+          (void) WriteBlob(image,length,pixels);
+          break;
+        }
+        case RLECompression:
+        {
+          packets=PCLPackbitsCompressImage(length,pixels,
+            compress_pixels);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
+            (unsigned long) packets);
+          (void) WriteBlobString(image,buffer);
+          (void) WriteBlob(image,packets,compress_pixels);
+          break;
+        }
+        default:
+        {
+          if (y == 0)
+            for (i=0; i < (long) length; i++)
+              previous_pixels[i]=(~pixels[i]);
+          packets=PCLDeltaCompressImage(length,previous_pixels,pixels,
+            compress_pixels);
+          (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
+            (unsigned long) packets);
+          (void) WriteBlobString(image,buffer);
+          (void) WriteBlob(image,packets,compress_pixels);
+          (void) CopyMagickMemory(previous_pixels,pixels,length*
+            sizeof(*pixels));
+          break;
+        }
+      }
+    }
+    (void) WriteBlobString(image,"\033*rB");  /* end graphics */
+    switch (image->compression)
+    {
+      case NoCompression:
+        break;
+      case RLECompression:
+      {
+        compress_pixels=(unsigned char *) RelinquishMagickMemory(
+          compress_pixels);
+        break;
+      }
+      default:
+      {
+        previous_pixels=(unsigned char *) RelinquishMagickMemory(
+          previous_pixels);
+        compress_pixels=(unsigned char *) RelinquishMagickMemory(
+          compress_pixels);
+        break;
+      }
+    }
+    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+    if (GetNextImageInList(image) == (Image *) NULL)
+      break;
+    image=SyncNextImageInList(image);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
+  } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"\033E");
   (void) CloseBlob(image);
   return(MagickTrue);
