@@ -10,20 +10,20 @@
 %                            P      DDDD   BBBB                               %
 %                                                                             %
 %                                                                             %
-%               Read/Write Palm Database ImageViewer Image Format.            %
+%               Read/Write Palm Database ImageViewer Image Format             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -34,6 +34,13 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %
+    20071202 TS * rewrote RLE decoder - old version could cause buffer overflows
+                * failure of RLE decoding now thows error RLEDecoderError
+                * fixed bug in RLE decoding - now all rows are decoded, not just
+      the first one
+    * fixed bug in reader - record offsets now handled correctly
+    * fixed bug in reader - only bits 0..2 indicate compression type
+                * in writer: now using image color count instead of depth
 */
 
 /*
@@ -42,17 +49,27 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
-#include "magick/color_private.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/colormap-private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Typedef declarations.
@@ -137,45 +154,48 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image: The address of a structure of type Image.
+%    o image: the address of a structure of type Image.
 %
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
 %      the decoding process.
 %
+%    o length:  Number of bytes to read into buffer 'pixels'.
 %
 */
-static MagickBooleanType DecodeImage(Image *image,unsigned char *pixels,
+static MagickBooleanType DecodeImage(Image *image, unsigned char *pixels,
   const size_t length)
 {
-  long
-    pixel;
+#define RLE_MODE_NONE -1
+#define RLE_MODE_COPY  0
+#define RLE_MODE_RUN   1
 
-  register long
-    i;
+  int           data = 0, count = 0;
+  unsigned char *p;
+  int           mode = RLE_MODE_NONE;
 
-  register unsigned char
-    *p;
-
-  ssize_t
-    count;
-
-  p=pixels;
-  while (p < (pixels+length))
-  {
-    pixel=ReadBlobByte(image);
-    if (pixel <= 0x80)
-      {
-        count=pixel+1;
-        for (i=0; i < count; i++)
-          *p++=ReadBlobByte(image);
-        continue;
+  for (p = pixels; p < pixels + length; p++) {
+    if (0 == count) {
+      data = ReadBlobByte( image );
+      if (-1 == data) return MagickFalse;
+      if (data > 128) {
+        mode  = RLE_MODE_RUN;
+        count = data - 128 + 1;
+        data  = ReadBlobByte( image );
+        if (-1 == data) return MagickFalse;
+      } else {
+        mode  = RLE_MODE_COPY;
+        count = data + 1;
       }
-    count=pixel+1-0x80;
-    pixel=ReadBlobByte(image);
-    for (i=0; i < count; i++)
-      *p++=(unsigned char) pixel;
+    }
+
+    if (RLE_MODE_COPY == mode) {
+      data = ReadBlobByte( image );
+      if (-1 == data) return MagickFalse;
+    }
+    *p = (unsigned char)data;
+    --count;
   }
-  return(MagickTrue);
+  return MagickTrue;
 }
 
 /*
@@ -198,12 +218,10 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *pixels,
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 {
@@ -235,16 +253,15 @@ static MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  char
-    record_type,
+  unsigned char
+    attributes, /* TS */
     tag[3];
 
   Image
@@ -254,7 +271,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     index;
 
   long
-    offset,
+    img_offset, /* TS */
+    comment_offset = 0,
     y;
 
   MagickBooleanType
@@ -278,13 +296,14 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
-  size_t
+  ssize_t
     count;
 
   unsigned char
     *pixels;
 
   unsigned long
+    num_pad_bytes, /* TS */
     bits_per_pixel,
     packets;
 
@@ -294,117 +313,113 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
-    Determine if this is a PDB image file.
+    Determine if this a PDB image file.
   */
   count=ReadBlob(image,32,(unsigned char *) pdb_info.name);
-  pdb_info.attributes=ReadBlobMSBShort(image);
-  pdb_info.version=ReadBlobMSBShort(image);
+  pdb_info.attributes=(short) ReadBlobMSBShort(image);
+  pdb_info.version=(short) ReadBlobMSBShort(image);
   pdb_info.create_time=ReadBlobMSBLong(image);
   pdb_info.modify_time=ReadBlobMSBLong(image);
   pdb_info.archive_time=ReadBlobMSBLong(image);
   pdb_info.modify_number=ReadBlobMSBLong(image);
   pdb_info.application_info=ReadBlobMSBLong(image);
   pdb_info.sort_info=ReadBlobMSBLong(image);
-  (void) ReadBlob(image,4,(unsigned char *) pdb_info.type);
-  (void) ReadBlob(image,4,(unsigned char *) pdb_info.id);
+  count=ReadBlob(image,4,(unsigned char *) pdb_info.type);
+  count=ReadBlob(image,4,(unsigned char *) pdb_info.id);
   if ((count == 0) || (memcmp(pdb_info.type,"vIMG",4) != 0) ||
       (memcmp(pdb_info.id,"View",4) != 0))
-    ThrowReaderException(CoderError,"ImproperImageHeader");
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   pdb_info.seed=ReadBlobMSBLong(image);
   pdb_info.next_record=ReadBlobMSBLong(image);
-  pdb_info.number_records=ReadBlobMSBShort(image);
+  pdb_info.number_records=(short) ReadBlobMSBShort(image);
   if (pdb_info.next_record != 0)
     ThrowReaderException(CoderError,"MultipleRecordListNotSupported");
   /*
     Read record header.
   */
-  offset=(long) ReadBlobMSBLong(image);
-  (void) ReadBlob(image,3,(unsigned char *) tag);
-  record_type=ReadBlobByte(image);
-  if (((record_type != 0x00) && (record_type != 0x01)) ||
-      (memcmp(tag,"\x40\x6f\x80",3) != 0))
+  img_offset=(long) ReadBlobMSBLong(image); /* TS */
+  attributes=(unsigned char) ReadBlobByte(image);
+  count=ReadBlob(image,3,(unsigned char *) tag);
+  if (count != 3  ||  memcmp(tag,"\x6f\x80\x00",3) != 0)
     ThrowReaderException(CorruptImageError,"CorruptImage");
-  if ((offset-TellBlob(image)) == 6)
-    {
-      (void) ReadBlobByte(image);
-      (void) ReadBlobByte(image);
-    }
+
   if (pdb_info.number_records > 1)
     {
-      offset=(long) ReadBlobMSBLong(image);
-      (void) ReadBlob(image,3,(unsigned char *) tag);
-      record_type=ReadBlobByte(image);
-      if (((record_type != 0x00) && (record_type != 0x01)) ||
-          (memcmp(tag,"\x40\x6f\x80",3) != 0))
+      comment_offset=(long) ReadBlobMSBLong(image);
+      attributes=(unsigned char) ReadBlobByte(image);
+      count=ReadBlob(image,3,(unsigned char *) tag);
+      if (count != 3  ||  memcmp(tag,"\x6f\x80\x01",3) != 0)
         ThrowReaderException(CorruptImageError,"CorruptImage");
-      if ((offset-TellBlob(image)) == 6)
-        {
-          (void) ReadBlobByte(image);
-          (void) ReadBlobByte(image);
-        }
     }
+
+  num_pad_bytes = (unsigned long) (img_offset - TellBlob( image ));
+  while (num_pad_bytes--) ReadBlobByte( image );
   /*
     Read image header.
   */
-  (void) ReadBlob(image,32,(unsigned char *) pdb_image.name);
+  count=ReadBlob(image,32,(unsigned char *) pdb_image.name);
   pdb_image.version=ReadBlobByte(image);
   pdb_image.type=ReadBlobByte(image);
   pdb_image.reserved_1=ReadBlobMSBLong(image);
   pdb_image.note=ReadBlobMSBLong(image);
-  pdb_image.x_last=ReadBlobMSBShort(image);
-  pdb_image.y_last=ReadBlobMSBShort(image);
+  pdb_image.x_last=(short) ReadBlobMSBShort(image);
+  pdb_image.y_last=(short) ReadBlobMSBShort(image);
   pdb_image.reserved_2=ReadBlobMSBLong(image);
-  pdb_image.x_anchor=ReadBlobMSBShort(image);
-  pdb_image.y_anchor=ReadBlobMSBShort(image);
-  pdb_image.width=ReadBlobMSBShort(image);
-  pdb_image.height=ReadBlobMSBShort(image);
+  pdb_image.x_anchor=(short) ReadBlobMSBShort(image);
+  pdb_image.y_anchor=(short) ReadBlobMSBShort(image);
+  pdb_image.width=(short) ReadBlobMSBShort(image);
+  pdb_image.height=(short) ReadBlobMSBShort(image);
   /*
     Initialize image structure.
   */
-  image->columns=pdb_image.width;
-  image->rows=pdb_image.height;
+  image->columns=(unsigned long) pdb_image.width;
+  image->rows=(unsigned long) pdb_image.height;
   image->depth=8;
   image->storage_class=PseudoClass;
-  bits_per_pixel=pdb_image.type == 0 ? 2 : pdb_image.type == 2 ? 4 : 1;
-  if (AllocateImageColormap(image,1 << bits_per_pixel) == MagickFalse)
+  bits_per_pixel=pdb_image.type == 0 ? 2UL : pdb_image.type == 2 ? 4UL : 1UL;
+  if (AcquireImageColormap(image,1 << bits_per_pixel) == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  packets=(bits_per_pixel*image->columns/8)*image->rows;
-  pixels=(unsigned char *) AcquireMagickMemory(packets+256);
+  packets=bits_per_pixel*image->columns/8;
+  pixels=(unsigned char *) AcquireQuantumMemory(packets+256UL,image->rows*
+    sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  switch (pdb_image.version)
+
+  switch (pdb_image.version & 7) /* TS */
   {
     case 0:
     {
       image->compression=NoCompression;
-      (void) ReadBlob(image,packets,pixels);
+      count=(ssize_t) ReadBlob(image, packets * image -> rows, pixels);
       break;
     }
     case 1:
     {
       image->compression=RLECompression;
-      (void) DecodeImage(image,pixels,packets);
+      if (!DecodeImage(image, pixels, packets * image -> rows))
+        ThrowReaderException( CorruptImageError, "RLEDecoderError" ); /* TS */
       break;
     }
     default:
       ThrowReaderException(CorruptImageError,
-        "UnrecognizedImageCompressionType");
+         "UnrecognizedImageCompressionType" );
   }
   p=pixels;
   switch (bits_per_pixel)
@@ -419,25 +434,25 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       for (y=0; y < (long) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < ((long) image->columns-7); x+=8)
         {
           for (bit=0; bit < 8; bit++)
           {
-            index=(*p & (0x80 >> bit) ? 0x00 : 0x01);
+            index=(IndexPacket) (*p & (0x80 >> bit) ? 0x00 : 0x01);
             indexes[x+bit]=index;
-            *q++=image->colormap[index];
+            *q++=image->colormap[(long) index];
           }
           p++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
+          break;
+        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        if (status == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
       }
       break;
     }
@@ -448,31 +463,31 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       for (y=0; y < (long) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < (long) image->columns; x+=4)
         {
-          index=ConstrainColormapIndex(image,3-((*p >> 6) & 0x03));
+          index=ConstrainColormapIndex(image,3UL-((*p >> 6) & 0x03));
           indexes[x]=index;
-          *q++=image->colormap[index];
-          index=ConstrainColormapIndex(image,3-((*p >> 4) & 0x03));
+          *q++=image->colormap[(long) index];
+          index=ConstrainColormapIndex(image,3UL-((*p >> 4) & 0x03));
           indexes[x+1]=index;
-          *q++=image->colormap[index];
-          index=ConstrainColormapIndex(image,3-((*p >> 2) & 0x03));
+          *q++=image->colormap[(long) index];
+          index=ConstrainColormapIndex(image,3UL-((*p >> 2) & 0x03));
           indexes[x+2]=index;
-          *q++=image->colormap[index];
-          index=ConstrainColormapIndex(image,3-((*p) & 0x03));
+          *q++=image->colormap[(long) index];
+          index=ConstrainColormapIndex(image,3UL-((*p) & 0x03));
           indexes[x+3]=index;
-          *q++=image->colormap[index];
+          *q++=image->colormap[(long) index];
           p++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
+          break;
+        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        if (status == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
       }
       break;
     }
@@ -483,36 +498,37 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       for (y=0; y < (long) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < (long) image->columns; x+=2)
         {
-          index=ConstrainColormapIndex(image,15-((*p >> 4) & 0x0f));
+          index=ConstrainColormapIndex(image,15UL-((*p >> 4) & 0x0f));
           indexes[x]=index;
-          *q++=image->colormap[index];
-          index=ConstrainColormapIndex(image,15-((*p) & 0x0f));
+          *q++=image->colormap[(long) index];
+          index=ConstrainColormapIndex(image,15UL-((*p) & 0x0f));
           indexes[x+1]=index;
-          *q++=image->colormap[index];
+          *q++=image->colormap[(long) index];
           p++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
+          break;
+        status=SetImageProgress(image,LoadImageTag,y,image->rows);
+        if (status == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-            break;
       }
       break;
     }
     default:
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   }
+
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  if ((offset-TellBlob(image)) == 0)
+  if (pdb_info.number_records > 1) /* TS */
     {
       char
         *comment;
@@ -526,6 +542,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       unsigned long
         length;
 
+      num_pad_bytes = (unsigned long) (comment_offset - TellBlob( image ));
+      while (num_pad_bytes--) ReadBlobByte( image );
+
       /*
         Read comment.
       */
@@ -539,8 +558,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             *p='\0';
             length<<=1;
             length+=MaxTextExtent;
-            comment=(char *) ResizeMagickMemory(comment,
-              (length+MaxTextExtent)*sizeof(*comment));
+            comment=(char *) ResizeQuantumMemory(comment,length+MaxTextExtent,
+              sizeof(*comment));
             if (comment == (char *) NULL)
               break;
             p=comment+strlen(comment);
@@ -551,10 +570,10 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       *p='\0';
       if (comment == (char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      (void) SetImageAttribute(image,"comment",comment);
-      comment=(char *) RelinquishMagickMemory(comment);
+      (void) SetImageProperty(image,"comment",comment);
+      comment=DestroyString(comment);
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -569,8 +588,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterPDBImage() adds attributes for the PDB image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterPDBImage() adds properties for the PDB image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -578,21 +597,22 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPDBImage method is:
 %
-%      RegisterPDBImage(void)
+%      unsigned long RegisterPDBImage(void)
 %
 */
-ModuleExport void RegisterPDBImage(void)
+ModuleExport unsigned long RegisterPDBImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("PDB");
-  entry->decoder=(DecoderHandler *) ReadPDBImage;
-  entry->encoder=(EncoderHandler *) WritePDBImage;
-  entry->magick=(MagickHandler *) IsPDB;
-  entry->description=AcquireString("Palm Database ImageViewer Format");
-  entry->module=AcquireString("PDB");
+  entry->decoder=(DecodeImageHandler *) ReadPDBImage;
+  entry->encoder=(EncodeImageHandler *) WritePDBImage;
+  entry->magick=(IsImageFormatHandler *) IsPDB;
+  entry->description=ConstantString("Palm Database ImageViewer Format");
+  entry->module=ConstantString("PDB");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -638,7 +658,7 @@ ModuleExport void UnregisterPDBImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -649,12 +669,12 @@ static unsigned char *EncodeRLE(unsigned char *destination,
   unsigned char *source,unsigned long literal,unsigned long repeat)
 {
   if (literal > 0)
-    *destination++=literal-1;
-  CopyMagickMemory(destination,source,literal);
+    *destination++=(unsigned char) (literal-1);
+  (void) CopyMagickMemory(destination,source,literal);
   destination+=literal;
   if (repeat > 0)
     {
-      *destination++=0x80 | (repeat-1);
+      *destination++=(unsigned char) (0x80 | (repeat-1));
       *destination++=source[literal];
     }
   return(destination);
@@ -662,6 +682,9 @@ static unsigned char *EncodeRLE(unsigned char *destination,
 
 static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
 {
+  const char
+    *comment;
+
   int
     bits;
 
@@ -677,6 +700,9 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   PDBInfo
     pdb_info;
 
+  QuantumInfo
+    *quantum_info;
+
   register const PixelPacket
     *p;
 
@@ -700,9 +726,6 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     packets,
     repeat;
 
-  const ImageAttribute
-    *comment;
-
   /*
     Open output image file.
   */
@@ -711,16 +734,23 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
-  bits_per_pixel=image->depth;
-  if (GetImageType(image,&image->exception) == BilevelType)
-    bits_per_pixel=1;
-  if ((bits_per_pixel != 1) && (bits_per_pixel != 2))
-    bits_per_pixel=4;
+  if (image->colorspace != RGBColorspace)
+    (void) TransformImageColorspace(image,RGBColorspace);
+
+  if (image -> colors <= 2  ||  GetImageType( image, &image -> exception ) == BilevelType) { /* TS */
+    bits_per_pixel = 1;
+  } else if (image -> colors <= 4) {
+    bits_per_pixel = 2;
+  } else if (image -> colors <= 8) {
+    bits_per_pixel = 3;
+  } else {
+    bits_per_pixel = 4;
+  }
+
   (void) ResetMagickMemory(pdb_info.name,0,32);
   (void) CopyMagickString(pdb_info.name,image_info->filename,32);
   pdb_info.attributes=0;
@@ -731,15 +761,15 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   pdb_info.modify_number=0;
   pdb_info.application_info=0;
   pdb_info.sort_info=0;
-  CopyMagickMemory(pdb_info.type,"vIMG",4);
-  CopyMagickMemory(pdb_info.id,"View",4);
+  (void) CopyMagickMemory(pdb_info.type,"vIMG",4);
+  (void) CopyMagickMemory(pdb_info.id,"View",4);
   pdb_info.seed=0;
   pdb_info.next_record=0;
-  comment=GetImageAttribute(image,"comment");
-  pdb_info.number_records=(comment == (ImageAttribute *) NULL ? 1 : 2);
+  comment=GetImageProperty(image,"comment");
+  pdb_info.number_records=(comment == (const char *) NULL ? 1 : 2);
   (void) WriteBlob(image,32,(unsigned char *) pdb_info.name);
-  (void) WriteBlobMSBShort(image,pdb_info.attributes);
-  (void) WriteBlobMSBShort(image,pdb_info.version);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_info.attributes);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_info.version);
   (void) WriteBlobMSBLong(image,pdb_info.create_time);
   (void) WriteBlobMSBLong(image,pdb_info.modify_time);
   (void) WriteBlobMSBLong(image,pdb_info.archive_time);
@@ -748,9 +778,9 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBLong(image,pdb_info.sort_info);
   (void) WriteBlob(image,4,(unsigned char *) pdb_info.type);
   (void) WriteBlob(image,4,(unsigned char *) pdb_info.id);
-  (void) WriteBlobMSBLong(image,pdb_info.seed);
-  (void) WriteBlobMSBLong(image,pdb_info.next_record);
-  (void) WriteBlobMSBShort(image,pdb_info.number_records);
+  (void) WriteBlobMSBLong(image,(unsigned long) pdb_info.seed);
+  (void) WriteBlobMSBLong(image,(unsigned long) pdb_info.next_record);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_info.number_records);
   (void) CopyMagickString(pdb_image.name,pdb_info.name,32);
   pdb_image.version=1;  /* RLE Compressed */
   switch (bits_per_pixel)
@@ -771,20 +801,26 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     pdb_image.width=(short) (16*(image->columns/16+1));
   pdb_image.height=(short) image->rows;
   packets=(bits_per_pixel*image->columns/8)*image->rows;
-  runlength=(unsigned char *) AcquireMagickMemory(2*packets);
+  runlength=(unsigned char *) AcquireQuantumMemory(2UL*packets,
+    sizeof(*runlength));
   if (runlength == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  buffer=(unsigned char *) AcquireMagickMemory(256);
+  buffer=(unsigned char *) AcquireQuantumMemory(256UL,sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   packet_size=(size_t) (image->depth > 8 ? 2: 1);
-  scanline=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
+  scanline=(unsigned char *) AcquireQuantumMemory(image->columns,packet_size*
+    sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Convert to GRAY raster scanline.
   */
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   bits=8/(long) bits_per_pixel-1;  /* start at most significant bits */
   literal=0;
   repeat=0;
@@ -792,10 +828,11 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   buffer[0]=0x00;
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) PopImagePixels(image,GrayQuantum,scanline);
+    (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+      GrayQuantum,scanline,&image->exception);
     for (x=0; x < pdb_image.width; x++)
     {
       if (x < (long) image->columns)
@@ -835,7 +872,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
               if (0x7f < literal)
                 {
                   q=EncodeRLE(q,buffer,(literal < 0x80 ? literal : 0x80),0);
-                  CopyMagickMemory(buffer,buffer+literal+repeat,0x80);
+                  (void) CopyMagickMemory(buffer,buffer+literal+repeat,0x80);
                   literal-=0x80;
                 }
             }
@@ -843,13 +880,14 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
         buffer[literal+repeat]=0x00;
       }
     }
-    if (QuantumTick(y,image->rows) != MagickFalse)
-      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
-        break;
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   q=EncodeRLE(q,buffer,literal,repeat);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  quantum_info=DestroyQuantumInfo(quantum_info);
   /*
     Write the Image record header.
   */
@@ -864,7 +902,8 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
       /*
         Write the comment record header.
       */
-      (void) WriteBlobMSBLong(image,TellBlob(image)+8+58+q-runlength);
+      (void) WriteBlobMSBLong(image,(unsigned long) (TellBlob(image)+8+58+q-
+        runlength));
       (void) WriteBlobByte(image,0x40);
       (void) WriteBlobByte(image,0x6f);
       (void) WriteBlobByte(image,0x80);
@@ -874,21 +913,21 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     Write the Image data.
   */
   (void) WriteBlob(image,32,(unsigned char *) pdb_image.name);
-  (void) WriteBlobByte(image,pdb_image.version);
-  (void) WriteBlobByte(image,pdb_image.type);
+  (void) WriteBlobByte(image,(unsigned char) pdb_image.version);
+  (void) WriteBlobByte(image,(unsigned char) pdb_image.type);
   (void) WriteBlobMSBLong(image,pdb_image.reserved_1);
   (void) WriteBlobMSBLong(image,pdb_image.note);
-  (void) WriteBlobMSBShort(image,pdb_image.x_last);
-  (void) WriteBlobMSBShort(image,pdb_image.y_last);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.x_last);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.y_last);
   (void) WriteBlobMSBLong(image,pdb_image.reserved_2);
-  (void) WriteBlobMSBShort(image,pdb_image.x_anchor);
-  (void) WriteBlobMSBShort(image,pdb_image.y_anchor);
-  (void) WriteBlobMSBShort(image,pdb_image.width);
-  (void) WriteBlobMSBShort(image,pdb_image.height);
-  (void) WriteBlob(image,q-runlength,runlength);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.x_anchor);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.y_anchor);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.width);
+  (void) WriteBlobMSBShort(image,(unsigned short) pdb_image.height);
+  (void) WriteBlob(image,(size_t) (q-runlength),runlength);
   runlength=(unsigned char *) RelinquishMagickMemory(runlength);
   if (pdb_info.number_records > 1)
-    WriteBlobString(image,comment->value);
-  CloseBlob(image);
+    (void) WriteBlobString(image,comment);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
