@@ -10,20 +10,20 @@
 %                            P      SSSSS  22222                              %
 %                                                                             %
 %                                                                             %
-%                      Write Postscript Level II Format.                      %
+%                      Write Postscript Level II Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,24 +40,38 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
-#include "magick/color_private.h"
+#include "magick/color-private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/draw.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/monitor-private.h"
+#include "magick/option.h"
 #include "magick/resource_.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
-#if defined(HasTIFF)
+
+/*
+  Define declarations.
+*/
+#if defined(MAGICKCORE_TIFF_DELEGATE)
 #define CCITTParam  "-1"
 #else
 #define CCITTParam  "0"
@@ -80,8 +94,8 @@ static MagickBooleanType
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterPS2Image() adds attributes for the PS2 image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterPS2Image() adds properties for the PS2 image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -89,27 +103,28 @@ static MagickBooleanType
 %
 %  The format of the RegisterPS2Image method is:
 %
-%      RegisterPS2Image(void)
+%      unsigned long RegisterPS2Image(void)
 %
 */
-ModuleExport void RegisterPS2Image(void)
+ModuleExport unsigned long RegisterPS2Image(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("EPS2");
-  entry->encoder=(EncoderHandler *) WritePS2Image;
+  entry->encoder=(EncodeImageHandler *) WritePS2Image;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Level II Encapsulated PostScript");
-  entry->module=AcquireString("PS2");
+  entry->description=ConstantString("Level II Encapsulated PostScript");
+  entry->module=ConstantString("PS2");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS2");
-  entry->encoder=(EncoderHandler *) WritePS2Image;
+  entry->encoder=(EncodeImageHandler *) WritePS2Image;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Level II PostScript");
-  entry->module=AcquireString("PS2");
+  entry->description=ConstantString("Level II PostScript");
+  entry->module=ConstantString("PS2");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -159,12 +174,49 @@ ModuleExport void UnregisterPS2Image(void)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
-%
-%    o image: The image.
+%    o image_info: the image info.
 %
+%    o image: the image.
 %
 */
+
+static MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
+  Image *image,Image *inject_image)
+{
+  Image
+    *group4_image;
+
+  ImageInfo
+    *write_info;
+
+  MagickBooleanType
+    status;
+
+  size_t
+    length;
+
+  unsigned char
+    *group4;
+
+  status=MagickTrue;
+  write_info=CloneImageInfo(image_info);
+  (void) CopyMagickString(write_info->filename,"GROUP4:",MaxTextExtent);
+  (void) CopyMagickString(write_info->magick,"GROUP4",MaxTextExtent);
+  group4_image=CloneImage(inject_image,0,0,MagickTrue,&image->exception);
+  if (group4_image == (Image *) NULL)
+    return(MagickFalse);
+  group4=(unsigned char *) ImageToBlob(write_info,group4_image,&length,
+    &image->exception);
+  group4_image=DestroyImage(group4_image);
+  if (group4 == (unsigned char *) NULL)
+    return(MagickFalse);
+  write_info=DestroyImageInfo(write_info);
+  if (WriteBlob(image,length,group4) != (ssize_t) length)
+    status=MagickFalse;
+  group4=(unsigned char *) RelinquishMagickMemory(group4);
+  return(status);
+}
+
 static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
 {
   static const char
@@ -192,8 +244,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       "      /Decode [0 1 0 1 0 1]",
       "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
       "      compression 0 gt",
-      "      { /DataSource pixel_stream /%s filter }",
-      "      { /DataSource pixel_stream /%s filter } ifelse",
+      "      { /DataSource pixel_stream %s }",
+      "      { /DataSource pixel_stream %s } ifelse",
       "    >> image",
       "  }",
       "  {",
@@ -206,8 +258,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       "      /Decode [1 0 1 0 1 0 1 0]",
       "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
       "      compression 0 gt",
-      "      { /DataSource pixel_stream /%s filter }",
-      "      { /DataSource pixel_stream /%s filter } ifelse",
+      "      { /DataSource pixel_stream %s }",
+      "      { /DataSource pixel_stream %s } ifelse",
       "    >> image",
       "  } ifelse",
       "} bind def",
@@ -238,9 +290,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       "      /Decode [0 1]",
       "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
       "      compression 0 gt",
-      "      { /DataSource pixel_stream /%s filter }",
+      "      { /DataSource pixel_stream %s }",
       "      {",
-      "        /DataSource pixel_stream /%s filter",
+      "        /DataSource pixel_stream %s",
       "        <<",
       "           /K "CCITTParam,
       "           /Columns columns",
@@ -266,8 +318,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       "      /Decode [0 255]",
       "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
       "      compression 0 gt",
-      "      { /DataSource pixel_stream /%s filter }",
-      "      { /DataSource pixel_stream /%s filter } ifelse",
+      "      { /DataSource pixel_stream %s }",
+      "      { /DataSource pixel_stream %s } ifelse",
       "    >> image",
       "  } ifelse",
       "} bind def",
@@ -317,7 +369,6 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       "  currentfile buffer readline pop",
       "  token pop /compression exch def pop",
       "  class 0 gt { PseudoClassImage } { DirectClassImage } ifelse",
-      "  grestore",
       (char *) NULL
     };
 
@@ -327,22 +378,15 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     page_geometry[MaxTextExtent],
     **labels;
 
-  const char
-    **q;
-
   CompressionType
     compression;
 
-  const ImageAttribute
-    *attribute;
+  const char
+    **q,
+    *value;
 
   double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
+    pointsize;
 
   GeometryInfo
     geometry_info;
@@ -357,25 +401,34 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     stop;
 
   MagickBooleanType
+    progress,
     status;
 
+  MagickOffsetType
+    offset;
+
   MagickSizeType
     number_pixels;
 
   MagickStatusType
     flags;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
     page_info;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -406,103 +459,113 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
   compression=image->compression;
+  if (image_info->compression != UndefinedCompression)
+    compression=image_info->compression;
   switch (compression)
   {
-#if !defined(HasJPEG)
+#if !defined(MAGICKCORE_JPEG_DELEGATE)
     case JPEGCompression:
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JPEGLibraryIsNotAvailable",image->filename);
+        MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (JPEG)",
+        image->filename);
       break;
     }
 #endif
     default:
       break;
   }
+  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   page=1;
   scene=0;
   do
   {
     /*
-      Scale image to size of Postscript page.
+      Scale relative to dots-per-inch.
     */
-    text_size=0;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
+    resolution.x=image->x_resolution;
+    resolution.y=image->y_resolution;
+    if ((resolution.x == 0.0) || (resolution.y == 0.0))
+      {
+        flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
+    if (image_info->density != (char *) NULL)
+      {
+        flags=ParseGeometry(image_info->density,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
+    if (image->units == PixelsPerCentimeterResolution)
+      {
+        resolution.x*=2.54;
+        resolution.y*=2.54;
+      }
     SetGeometry(image,&geometry);
-    geometry.y=(long) text_size;
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
       image->columns,image->rows);
     if (image_info->page != (char *) NULL)
       (void) CopyMagickString(page_geometry,image_info->page,MaxTextExtent);
     else
       if ((image->page.width != 0) && (image->page.height != 0))
-          (void) FormatMagickString(page_geometry,MaxTextExtent,
-          "%lux%lu%+ld%+ld",image->page.width,image->page.height,
-           image->page.x,image->page.y);
+        (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
+          image->page.width,image->page.height,image->page.x,image->page.y);
       else
-        if (LocaleCompare(image_info->magick,"PS2") == 0)
-          (void) strcpy(page_geometry,PSPageGeometry);
+        if ((image->gravity != UndefinedGravity) &&
+            (LocaleCompare(image_info->magick,"PS") == 0))
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
+    (void) ConcatenateMagickString(page_geometry,">",MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
-       &geometry.width,&geometry.height);
+      &geometry.width,&geometry.height);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
-    geometry.x=(-page_info.x);
-    geometry.y=(long) (media_info.height+page_info.y-image->rows);
-    /*
-      Scale relative to dots-per-inch.
-    */
-    dx_resolution=72.0;
-    dy_resolution=72.0;
-    x_resolution=72.0;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
-    if (image_info->density != (char *) NULL)
+    (void) ParseGravityGeometry(image,page_geometry,&page_info,
+      &image->exception);
+    if (image->gravity != UndefinedGravity)
       {
-        flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
-        if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+        geometry.x=(-page_info.x);
+        geometry.y=(long) (media_info.height+page_info.y-image->rows);
       }
-    if (image->units == PixelsPerCentimeterResolution)
-      {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
-      }
-    x_scale=(double) ((geometry.width*dx_resolution)/x_resolution);
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) ((geometry.height*dy_resolution)/y_resolution);
-    geometry.height=(unsigned long) (y_scale+0.5);
+    pointsize=12.0;
+    if (image_info->pointsize != 0.0)
+      pointsize=image_info->pointsize;
+    text_size=0;
+    value=GetImageProperty(image,"label");
+    if (value != (const char *) NULL)
+      text_size=(unsigned long) (MultilineCensus(value)*pointsize+12);
     if (page == 1)
       {
         /*
           Output Postscript header.
         */
         if (LocaleCompare(image_info->magick,"PS2") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0\n",MaxTextExtent);
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",
+            MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"%%Creator: (ImageMagick)\n");
         (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Title: (%s)\n",
           image->filename);
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
-        (void) localtime(&timer);
-        (void) CopyMagickString(date,ctime(&timer),MaxTextExtent);
-        date[strlen(date)-1]='\0';
+        (void) FormatMagickTime(timer,MaxTextExtent,date);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%CreationDate: (%s)\n",date);
         (void) WriteBlobString(image,buffer);
@@ -510,8 +573,10 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
         bounds.y1=(double) geometry.y;
         bounds.x2=(double) geometry.x+geometry.width;
         bounds.y2=(double) geometry.y+geometry.height+text_size;
-        if ((image_info->adjoin != MagickFalse) && (image->next != (Image *) NULL))
-          (void) strcpy(buffer,"%%BoundingBox: (atend)\n");
+        if ((image_info->adjoin != MagickFalse) &&
+            (GetNextImageInList(image) != (Image *) NULL))
+          (void) CopyMagickString(buffer,"%%BoundingBox: (atend)\n",
+            MaxTextExtent);
         else
           {
             (void) FormatMagickString(buffer,MaxTextExtent,
@@ -524,8 +589,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               bounds.x2,bounds.y2);
           }
         (void) WriteBlobString(image,buffer);
-        attribute=GetImageAttribute(image,"label");
-        if (attribute != (const ImageAttribute *) NULL)
+        value=GetImageProperty(image,"label");
+        if (value != (const char *) NULL)
           (void) WriteBlobString(image,
             "%%DocumentNeededResources: font Helvetica\n");
         (void) WriteBlobString(image,"%%LanguageLevel: 2\n");
@@ -536,7 +601,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             if (image_info->adjoin == MagickFalse)
-              (void) strcpy(buffer,"%%Pages: 1\n");
+              (void) CopyMagickString(buffer,"%%Pages: 1\n",MaxTextExtent);
             else
               (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Pages: %lu\n",
                 (unsigned long) GetImageListLength(image));
@@ -555,45 +620,45 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
             case NoCompression:
             {
               (void) FormatMagickString(buffer,MaxTextExtent,*q,
-                "ASCII85Decode");
+                "/ASCII85Decode filter");
               break;
             }
             case JPEGCompression:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,*q,"DCTDecode");
+              (void) FormatMagickString(buffer,MaxTextExtent,*q,
+                "/DCTDecode filter");
               break;
             }
             case LZWCompression:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,*q,"LZWDecode");
+              (void) FormatMagickString(buffer,MaxTextExtent,*q,
+                "/LZWDecode filter");
               break;
             }
             case FaxCompression:
             case Group4Compression:
             {
-              (void) FormatMagickString(buffer,MaxTextExtent,*q,
-                "ASCII85Decode");
+              (void) FormatMagickString(buffer,MaxTextExtent,*q," ");
               break;
             }
             default:
             {
               (void) FormatMagickString(buffer,MaxTextExtent,*q,
-                "RunLengthDecode");
+                "/RunLengthDecode filter");
               break;
             }
           }
           (void) WriteBlobString(image,buffer);
           (void) WriteBlobByte(image,'\n');
         }
-        attribute=GetImageAttribute(image,"label");
-        if (attribute != (const ImageAttribute *) NULL)
-          for (j=(long) MultilineCensus(attribute->value)-1; j >= 0; j--)
+        value=GetImageProperty(image,"label");
+        if (value != (const char *) NULL)
+          for (j=(long) MultilineCensus(value)-1; j >= 0; j--)
           {
             (void) WriteBlobString(image,"  /label 512 string def\n");
             (void) WriteBlobString(image,"  currentfile label readline pop\n");
             (void) FormatMagickString(buffer,MaxTextExtent,
-              "  0 y %g add moveto label show pop\n",j*image_info->pointsize+
-              12);
+              "  0 y %g add moveto label show pop\n",j*pointsize+12);
             (void) WriteBlobString(image,buffer);
           }
         for (q=PostscriptEpilog; *q; q++)
@@ -621,8 +686,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       bounds.x2=(double) geometry.x+geometry.width-1;
     if ((double) (geometry.y+(geometry.height+text_size)-1) > bounds.y2)
       bounds.y2=(double) geometry.y+(geometry.height+text_size)-1;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
+    value=GetImageProperty(image,"label");
+    if (value != (const char *) NULL)
       (void) WriteBlobString(image,"%%PageResources: font Times-Roman\n");
     if (LocaleCompare(image_info->magick,"PS2") != 0)
       (void) WriteBlobString(image,"userdict begin\n");
@@ -637,12 +702,12 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       Output image data.
     */
     (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,x_scale,y_scale,image_info->pointsize);
+      geometry.x,geometry.y,scale.x,scale.y,pointsize);
     (void) WriteBlobString(image,buffer);
     labels=(char **) NULL;
-    attribute=GetImageAttribute(image,"label");
-    if (attribute != (const ImageAttribute *) NULL)
-      labels=StringToList(attribute->value);
+    value=GetImageProperty(image,"label");
+    if (value != (const char *) NULL)
+      labels=StringToList(value);
     if (labels != (char **) NULL)
       {
         for (i=0; labels[i] != (char *) NULL; i++)
@@ -650,7 +715,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           (void) FormatMagickString(buffer,MaxTextExtent,"%s \n",
             labels[i]);
           (void) WriteBlobString(image,buffer);
-          labels[i]=(char *) RelinquishMagickMemory(labels[i]);
+          labels[i]=DestroyString(labels[i]);
         }
         labels=(char **) RelinquishMagickMemory(labels);
       }
@@ -681,17 +746,16 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
               {
-                (void) HuffmanEncodeImage(image_info,image);
+                (void) HuffmanEncodeImage(image_info,image,image);
                 break;
               }
-#if defined(HasTIFF)
-            (void) Huffman2DEncodeImage(image_info,image);
-#endif
+            (void) Huffman2DEncodeImage(image_info,image,image);
             break;
           }
           case JPEGCompression:
           {
-            status=JPEGEncodeImage(image_info,image);
+            status=InjectImageBlob(image_info,image,image,"jpeg",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
@@ -706,7 +770,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               Allocate pixel array.
             */
             length=(size_t) number_pixels;
-            pixels=(unsigned char *) AcquireMagickMemory(length);
+            pixels=(unsigned char *) AcquireQuantumMemory(length,
+              sizeof(*pixels));
             if (pixels == (unsigned char *) NULL)
               ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
             /*
@@ -715,7 +780,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
             q=pixels;
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
@@ -724,15 +789,11 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 *q++=ScaleQuantumToChar(PixelIntensityToQuantum(p));
                 p++;
               }
-              if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
-                  {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
-                    if (status == MagickFalse)
-                      break;
-                  }
+              progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (progress == MagickFalse)
+                break;
             }
+            length=(size_t) (q-pixels);
             if (compression == LZWCompression)
               status=LZWEncodeImage(image,length,pixels);
             else
@@ -740,7 +801,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
             if (status == MagickFalse)
               {
-                CloseBlob(image);
+                (void) CloseBlob(image);
                 return(MagickFalse);
               }
             break;
@@ -753,7 +814,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
             Ascii85Initialize(image);
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
@@ -763,14 +824,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                   ScaleQuantumToChar(PixelIntensityToQuantum(p)));
                 p++;
               }
-              if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows) != MagickFalse)
-                  {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
-                    if (status == MagickFalse)
-                      break;
-                  }
+              progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (progress == MagickFalse)
+                break;
             }
             Ascii85Flush(image);
             break;
@@ -792,7 +848,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           {
             case JPEGCompression:
             {
-              status=JPEGEncodeImage(image_info,image);
+              status=InjectImageBlob(image_info,image,image,"jpeg",
+                &image->exception);
               if (status == MagickFalse)
                 ThrowWriterException(CoderError,image->exception.reason);
               break;
@@ -806,9 +863,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               /*
                 Allocate pixel array.
               */
-              length=(size_t) ((image->colorspace == CMYKColorspace ? 4 : 3)*
-                number_pixels);
-              pixels=(unsigned char *) AcquireMagickMemory(length);
+              length=(size_t) number_pixels;
+              pixels=(unsigned char *) AcquireQuantumMemory(length,
+                4*sizeof(*pixels));
               if (pixels == (unsigned char *) NULL)
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
@@ -818,19 +875,19 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               q=pixels;
               for (y=0; y < (long) image->rows; y++)
               {
-                p=AcquireImagePixels(image,0,y,image->columns,1,
+                p=GetVirtualPixels(image,0,y,image->columns,1,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=GetVirtualIndexQueue(image);
                 for (x=0; x < (long) image->columns; x++)
                 {
                   if ((image->matte != MagickFalse) &&
-                      (p->opacity == TransparentOpacity))
+                      (p->opacity == (Quantum) TransparentOpacity))
                     {
-                      *q++=ScaleQuantumToChar(MaxRGB);
-                      *q++=ScaleQuantumToChar(MaxRGB);
-                      *q++=ScaleQuantumToChar(MaxRGB);
+                      *q++=ScaleQuantumToChar((Quantum) QuantumRange);
+                      *q++=ScaleQuantumToChar((Quantum) QuantumRange);
+                      *q++=ScaleQuantumToChar((Quantum) QuantumRange);
                     }
                   else
                     if (image->colorspace != CMYKColorspace)
@@ -848,22 +905,18 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                       }
                   p++;
                 }
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
               }
+              length=(size_t) (q-pixels);
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
               else
                 status=PackbitsEncodeImage(image,length,pixels);
               if (status == MagickFalse)
                 {
-                  CloseBlob(image);
+                  (void) CloseBlob(image);
                   return(MagickFalse);
                 }
               pixels=(unsigned char *) RelinquishMagickMemory(pixels);
@@ -877,19 +930,22 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               Ascii85Initialize(image);
               for (y=0; y < (long) image->rows; y++)
               {
-                p=AcquireImagePixels(image,0,y,image->columns,1,
+                p=GetVirtualPixels(image,0,y,image->columns,1,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=GetVirtualIndexQueue(image);
                 for (x=0; x < (long) image->columns; x++)
                 {
                   if ((image->matte != MagickFalse) &&
-                      (p->opacity == TransparentOpacity))
+                      (p->opacity == (Quantum) TransparentOpacity))
                     {
-                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
-                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
-                      Ascii85Encode(image,ScaleQuantumToChar(MaxRGB));
+                      Ascii85Encode(image,ScaleQuantumToChar((Quantum)
+                        QuantumRange));
+                      Ascii85Encode(image,ScaleQuantumToChar((Quantum)
+                        QuantumRange));
+                      Ascii85Encode(image,ScaleQuantumToChar((Quantum)
+                        QuantumRange));
                     }
                   else
                     if (image->colorspace != CMYKColorspace)
@@ -907,14 +963,9 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                       }
                   p++;
                 }
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
               }
               Ascii85Flush(image);
               break;
@@ -955,7 +1006,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
                 Allocate pixel array.
               */
               length=(size_t) number_pixels;
-              pixels=(unsigned char *) AcquireMagickMemory(length);
+              pixels=(unsigned char *) AcquireQuantumMemory(length,
+                sizeof(*pixels));
               if (pixels == (unsigned char *) NULL)
                 ThrowWriterException(ResourceLimitError,
                   "MemoryAllocationFailed");
@@ -965,22 +1017,18 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               q=pixels;
               for (y=0; y < (long) image->rows; y++)
               {
-                p=AcquireImagePixels(image,0,y,image->columns,1,
+                p=GetVirtualPixels(image,0,y,image->columns,1,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=GetVirtualIndexQueue(image);
                 for (x=0; x < (long) image->columns; x++)
                   *q++=(unsigned char) indexes[x];
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
               }
+              length=(size_t) (q-pixels);
               if (compression == LZWCompression)
                 status=LZWEncodeImage(image,length,pixels);
               else
@@ -988,7 +1036,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               pixels=(unsigned char *) RelinquishMagickMemory(pixels);
               if (status == MagickFalse)
                 {
-                  CloseBlob(image);
+                  (void) CloseBlob(image);
                   return(MagickFalse);
                 }
               break;
@@ -1001,21 +1049,16 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
               Ascii85Initialize(image);
               for (y=0; y < (long) image->rows; y++)
               {
-                p=AcquireImagePixels(image,0,y,image->columns,1,
+                p=GetVirtualPixels(image,0,y,image->columns,1,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=GetVirtualIndexQueue(image);
                 for (x=0; x < (long) image->columns; x++)
                   Ascii85Encode(image,(unsigned char) indexes[x]);
-                if (image->previous == (Image *) NULL)
-                  if (QuantumTick(y,image->rows) != MagickFalse)
-                    {
-                      status=MagickMonitor(SaveImageTag,y,image->rows,
-                        &image->exception);
-                      if (status == MagickFalse)
-                        break;
-                    }
+                progress=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (progress == MagickFalse)
+                  break;
               }
               Ascii85Flush(image);
               break;
@@ -1025,21 +1068,23 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,'\n');
     length=(size_t) (TellBlob(image)-stop);
     stop=TellBlob(image);
-    (void) SeekBlob(image,start,SEEK_SET);
+    offset=SeekBlob(image,start,SEEK_SET);
+    if (offset < 0)
+      ThrowWriterException(CorruptImageError,"ImproperImageHeader");
     (void) FormatMagickString(buffer,MaxTextExtent,
       "%%%%BeginData:%13ld %s Bytes\n",(long) length,
       compression == NoCompression ? "ASCII" : "Binary");
     (void) WriteBlobString(image,buffer);
-    (void) SeekBlob(image,stop,SEEK_SET);
+    offset=SeekBlob(image,stop,SEEK_SET);
     (void) WriteBlobString(image,"%%EndData\n");
     if (LocaleCompare(image_info->magick,"PS2") != 0)
       (void) WriteBlobString(image,"end\n");
     (void) WriteBlobString(image,"%%PageTrailer\n");
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
@@ -1056,6 +1101,6 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
