@@ -10,7 +10,7 @@
 %                            P      SSSSS  33333                              %
 %                                                                             %
 %                                                                             %
-%                     Write Postscript Level III Format.                      %
+%                     Write Postscript Level III Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,9 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/compress.h"
@@ -58,18 +58,17 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
 #include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/token.h"
 #include "magick/utility.h"
-#include "magick/version.h"
-#if defined(HasTIFF)
-#define CCITTParam  "-1"
-#else
-#define CCITTParam  "0"
-#endif
+#include "magick/module.h"
 
 /*
   Define declarations.
@@ -86,6 +85,12 @@
 
 #define PS3_DirectClass "0"
 #define PS3_PseudoClass "1"
+
+#if defined(MAGICKCORE_TIFF_DELEGATE)
+#define CCITTParam  "-1"
+#else
+#define CCITTParam  "0"
+#endif
 
 /*
   Forward declarations.
@@ -104,32 +109,33 @@ static MagickBooleanType
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterPS3Image() adds attributes for the PS3 image format to the list of
-%  supported formats.  The attributes include the image format tag, a method to
+%  RegisterPS3Image() adds properties for the PS3 image format to the list of
+%  supported formats.  The properties include the image format tag, a method to
 %  read and/or write the format, whether the format supports the saving of more
 %  than one frame to the same file or blob, whether the format supports native
 %  in-memory I/O, and a brief description of the format.
 %
 %  The format of the RegisterPS3Image method is:
 %
-%      RegisterPS3Image(void)
+%      unsigned long RegisterPS3Image(void)
 %
 */
-ModuleExport void RegisterPS3Image(void)
+ModuleExport unsigned long RegisterPS3Image(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("EPS3");
-  entry->encoder=(EncoderHandler *) WritePS3Image;
-  entry->description=AcquireString("Level III Encapsulated PostScript");
-  entry->module=AcquireString("PS3");
+  entry->encoder=(EncodeImageHandler *) WritePS3Image;
+  entry->description=ConstantString("Level III Encapsulated PostScript");
+  entry->module=ConstantString("PS3");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS3");
-  entry->encoder=(EncoderHandler *) WritePS3Image;
-  entry->description=AcquireString("Level III PostScript");
-  entry->module=AcquireString("PS3");
+  entry->encoder=(EncodeImageHandler *) WritePS3Image;
+  entry->description=ConstantString("Level III PostScript");
+  entry->module=ConstantString("PS3");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -181,23 +187,60 @@ ModuleExport void UnregisterPS3Image(void)
 %
 %    o image_info: Specifies a pointer to a ImageInfo structure.
 %
-%    o image: The image.
-%
+%    o image: the image.
 %
 */
 
+static MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
+  Image *image,Image *inject_image)
+{
+  Image
+    *group4_image;
+
+  ImageInfo
+    *write_info;
+
+  MagickBooleanType
+    status;
+
+  size_t
+    length;
+
+  unsigned char
+    *group4;
+
+  status=MagickTrue;
+  write_info=CloneImageInfo(image_info);
+  (void) CopyMagickString(write_info->filename,"GROUP4:",MaxTextExtent);
+  (void) CopyMagickString(write_info->magick,"GROUP4",MaxTextExtent);
+  group4_image=CloneImage(inject_image,0,0,MagickTrue,&image->exception);
+  if (group4_image == (Image *) NULL)
+    return(MagickFalse);
+  group4=(unsigned char *) ImageToBlob(write_info,group4_image,&length,
+    &image->exception);
+  group4_image=DestroyImage(group4_image);
+  if (group4 == (unsigned char *) NULL)
+    return(MagickFalse);
+  write_info=DestroyImageInfo(write_info);
+  if (WriteBlob(image,length,group4) != (ssize_t) length)
+    status=MagickFalse;
+  group4=(unsigned char *) RelinquishMagickMemory(group4);
+  return(status);
+}
+
+
 static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   Image *image,unsigned char **pixels,size_t *length)
 {
-  IndexPacket
-    *indexes;
-
   long
     y;
 
   MagickBooleanType
     status;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
@@ -214,16 +257,16 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   status=MagickTrue;
   *length=(image->colorspace == CMYKColorspace ? 4 : 3)*
     (size_t) image->columns*image->rows;
-  *pixels=(unsigned char *) AcquireMagickMemory(*length);
+  *pixels=(unsigned char *) AcquireQuantumMemory(*length,sizeof(**pixels));
   if (*pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   q=(*pixels);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     if (image->colorspace != CMYKColorspace)
       for (x=0; x < (long) image->columns; x++)
       {
@@ -242,14 +285,11 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
         p++;
       }
     if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+      {
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
@@ -287,17 +327,16 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
-  pack=(unsigned long)
-    (IsMonochromeImage(image,&image->exception) != MagickFalse ? 8 : 1);
+  pack=IsMonochromeImage(image,&image->exception) == MagickFalse ? 1UL : 8UL;
   padded_columns=((image->columns+pack-1)/pack)*pack;
   *length=(size_t) padded_columns*image->rows/pack;
-  *pixels=(unsigned char *) AcquireMagickMemory(*length);
+  *pixels=(unsigned char *) AcquireQuantumMemory(*length,sizeof(**pixels));
   if (*pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   q=(*pixels);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
     if (pack == 1)
@@ -313,8 +352,8 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
         {
           bit=(unsigned char) 0x00;
           if (x < (long) image->columns)
-            bit=(unsigned char)
-              (PixelIntensityToQuantum(p) == TransparentOpacity ? 0x01 : 0x00);
+            bit=(unsigned char) (PixelIntensityToQuantum(p) ==
+              (Quantum) TransparentOpacity ? 0x01 : 0x00);
           code=(code << 1)+bit;
           if (((x+1) % pack) == 0)
             {
@@ -324,15 +363,9 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
           p++;
         }
       }
-    if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
@@ -348,12 +381,12 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -366,27 +399,24 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
   *length=(size_t) image->columns*image->rows;
-  *pixels=(unsigned char *) AcquireMagickMemory(*length);
+  *pixels=(unsigned char *) AcquireQuantumMemory(*length,sizeof(**pixels));
   if (*pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   q=(*pixels);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (long) image->columns; x++)
       *q++=(unsigned char) indexes[x];
     if (image->previous == (Image *) NULL)
-     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-         (QuantumTick(y,image->rows) != MagickFalse))
-       {
-         status=image->progress_monitor(SaveImageTag,y,image->rows,
-           image->client_data);
-         if (status == MagickFalse)
-           break;
-       }
+      {
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
+      }
   }
   if (status == MagickFalse)
     *pixels=(unsigned char *) RelinquishMagickMemory(*pixels);
@@ -394,14 +424,11 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
 }
 
 static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
-  Image *image)
+  Image *image,const CompressionType compression)
 {
   char
     buffer[MaxTextExtent];
 
-  CompressionType
-    compression;
-
   Image
     *mask_image;
 
@@ -409,6 +436,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     status;
 
   MagickOffsetType
+    offset,
     start,
     stop;
 
@@ -421,9 +449,6 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   unsigned char
     *pixels;
 
-  unsigned long
-    quality;
-
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
@@ -432,9 +457,6 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->matte != MagickFalse);
   status=MagickTrue;
-  compression=image->compression;
-  quality=(unsigned long)
-    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   /*
     Note BeginData DSC comment for update later.
   */
@@ -450,24 +472,18 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   switch (compression)
   {
     case NoCompression:
+    default:
     {
       (void) FormatMagickString(buffer,MaxTextExtent,
         "currentfile %lu %lu "PS3_NoCompression" ByteStreamDecodeFilter\n",
         image->columns,image->rows);
+      break;
     }
     case FaxCompression:
     case Group4Compression:
-    default:
     {
       (void) FormatMagickString(buffer,MaxTextExtent,
         "currentfile %lu %lu "PS3_FaxCompression" ByteStreamDecodeFilter\n",
-        image->columns, image->rows);
-      break;
-    }
-    case JPEGCompression:
-    {
-      (void) FormatMagickString(buffer,MaxTextExtent,
-        "currentfile %lu %lu "PS3_JPEGCompression" ByteStreamDecodeFilter\n",
         image->columns,image->rows);
       break;
     }
@@ -505,12 +521,14 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
       return(MagickFalse);
     }
   (void) SetImageType(mask_image,BilevelType);
+  (void) SetImageType(mask_image,PaletteType);
   mask_image->matte=MagickFalse;
   pixels=(unsigned char *) NULL;
   length=0;
   switch (compression)
   {
     case NoCompression:
+    default:
     {
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
       if (status == MagickFalse)
@@ -524,17 +542,12 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     }
     case FaxCompression:
     case Group4Compression:
-    default:
     {
-      if (LocaleCompare(CCITTParam,"0") == 0)
-        status=HuffmanEncodeImage(image_info,mask_image);
+      if ((compression == FaxCompression) ||
+          (LocaleCompare(CCITTParam,"0") == 0))
+        status=HuffmanEncodeImage(image_info,image,mask_image);
       else
-        status=Huffman2DEncodeImage(image_info,mask_image);
-      break;
-    }
-    case JPEGCompression:
-    {
-      status=JPEGEncodeImage(image_info,mask_image);
+        status=Huffman2DEncodeImage(image_info,image,mask_image);
       break;
     }
     case LZWCompression:
@@ -560,7 +573,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
       if (status == MagickFalse)
         break;
-      status=ZLIBEncodeImage(image,length,quality,pixels);
+      status=ZLIBEncodeImage(image,length,pixels);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       break;
     }
@@ -569,12 +582,16 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   (void) WriteBlobByte(image,'\n');
   length=(size_t) (TellBlob(image)-stop);
   stop=TellBlob(image);
-  (void) SeekBlob(image,start,SEEK_SET);
+  offset=SeekBlob(image,start,SEEK_SET);
+  if (offset < 0)
+    ThrowWriterException(CorruptImageError,"ImproperImageHeader");
   (void) FormatMagickString(buffer,MaxTextExtent,
     "%%%%BeginData:%13ld %s Bytes\n",(long) length,
     compression == NoCompression ? "ASCII" : "BINARY");
   (void) WriteBlobString(image,buffer);
-  (void) SeekBlob(image,stop,SEEK_SET);
+  offset=SeekBlob(image,stop,SEEK_SET);
+  if (offset < 0)
+    ThrowWriterException(CorruptImageError,"ImproperImageHeader");
   (void) WriteBlobString(image,"%%EndData\n");
   (void) WriteBlobString(image, "/mask_stream exch def\n");
   return(status);
@@ -718,13 +735,11 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "/DisplayImage",
       "{",
       "  /buffer 512 string def",
-      "",
       "  % Translation.",
       "  currentfile buffer readline pop",
       "  token pop /x exch def",
       "  token pop /y exch def pop",
       "  x y translate",
-      "",
       "  % Image size and font size.",
       "  currentfile buffer readline pop",
       "  token pop /x exch def",
@@ -739,7 +754,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  % Clipping path.",
       "  currentfile buffer readline pop",
       "  token pop /clipped exch def pop",
-      "  % EPS.",
+      "  % Showpage.",
       "  currentfile buffer readline pop",
       "  token pop /sp exch def pop",
       "  % Image pixel size.",
@@ -762,15 +777,13 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  currentfile buffer readline pop",
       "  token pop /compression exch def pop",
       "  % Clip and render.",
-      "  /pixel_stream currentfile columns rows compression",
-      "    ByteStreamDecodeFilter def",
+      "  /pixel_stream currentfile columns rows compression ByteStreamDecodeFilter def",
       "  clipped { ClipImage } if",
       "  alpha stencil not and",
       "  { MaskedImageDict mask_stream resetfile }",
       "  { NonMaskedImageDict }",
       "  ifelse",
       "  stencil { 0 setgray imagemask } { image } ifelse",
-      "  grestore",
       "  sp { showpage } if",
       "} bind def",
       (char *) NULL
@@ -782,15 +795,16 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     **labels,
     page_geometry[MaxTextExtent];
 
-  const char
-    *option,
-    **q;
-
   CompressionType
     compression;
 
-  const ImageAttribute
-    *attribute;
+  const char
+    *option,
+    **q,
+    *value;
+
+  double
+    pointsize;
 
   GeometryInfo
     geometry_info;
@@ -802,6 +816,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     status;
 
   MagickOffsetType
+    offset,
     scene,
     start,
     stop;
@@ -852,24 +867,35 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(MagickFalse);
   compression=image->compression;
+  if (image_info->compression != UndefinedCompression)
+    compression=image_info->compression;
   switch (compression)
   {
-#if !defined(HasJPEG)
+    case FaxCompression:
+    case Group4Compression:
+    { 
+      if ((IsMonochromeImage(image,&image->exception) == MagickFalse) ||
+          (image->matte != MagickFalse))
+        compression=RLECompression;
+      break;
+    }
+#if !defined(MAGICKCORE_JPEG_DELEGATE)
     case JPEGCompression:
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"JPEGLibraryIsNotAvailable","`%s'",
+        MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (JPEG)",
         image->filename);
       break;
     }
 #endif
-#if !defined(HasZLIB)
+#if !defined(MAGICKCORE_ZLIB_DELEGATE)
     case ZipCompression:
     {
       compression=RLECompression;
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        MissingDelegateError,"ZipLibraryIsNotAvailable","`%s'",image->filename);
+        MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (ZLIB)",
+        image->filename);
       break;
     }
 #endif
@@ -886,15 +912,16 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     */
     delta.x=DefaultResolution;
     delta.y=DefaultResolution;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    resolution.x=geometry_info.rho;
-    resolution.y=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      resolution.y=resolution.x;
-    if (image->x_resolution != 0.0)
-      resolution.x=image->x_resolution;
-    if (image->y_resolution != 0.0)
-      resolution.y=image->y_resolution;
+    resolution.x=image->x_resolution;
+    resolution.y=image->y_resolution;
+    if ((resolution.x == 0.0) || (resolution.y == 0.0))
+      {
+        flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
@@ -920,7 +947,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PS") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
+    (void) ConcatenateMagickString(page_geometry,">",MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
@@ -928,17 +956,20 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     scale.y=(double) (geometry.height*delta.y)/resolution.y;
     geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info,
+      &image->exception);
     if (image->gravity != UndefinedGravity)
       {
         geometry.x=(-page_info.x);
         geometry.y=(long) (media_info.height+page_info.y-image->rows);
       }
+    pointsize=12.0;
+    if (image_info->pointsize != 0.0)
+      pointsize=image_info->pointsize;
     text_size=0;
-    attribute=GetImageAttribute(image,"Label");
-    if (attribute != (const ImageAttribute *) NULL)
-      text_size=(unsigned long)
-        (MultilineCensus(attribute->value)*image_info->pointsize+12);
+    value=GetImageProperty(image,"label");
+    if (value != (const char *) NULL)
+      text_size=(unsigned long) (MultilineCensus(value)*pointsize+12);
     page++;
     if (page == 1)
       {
@@ -946,9 +977,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
           Postscript header on the first page.
         */
         if (LocaleCompare(image_info->magick,"PS3") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0\n",MaxTextExtent);
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",
+            MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%Creator: ImageMagick %s\n",MagickLibVersionText);
@@ -957,9 +989,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
           image->filename);
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
-        (void) localtime(&timer);
-        (void) CopyMagickString(date,ctime(&timer),MaxTextExtent);
-        date[strlen(date)-1]='\0';
+        (void) FormatMagickTime(timer,MaxTextExtent,date);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%CreationDate: %s\n",date);
         (void) WriteBlobString(image,buffer);
@@ -994,8 +1024,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         /*
           Font resources
         */
-        attribute=GetImageAttribute(image,"Label");
-        if (attribute != (const ImageAttribute *) NULL)
+        value=GetImageProperty(image,"label");
+        if (value != (const char *) NULL)
           (void) WriteBlobString(image,
             "%%DocumentNeededResources: font Helvetica\n");
         (void) WriteBlobString(image,"%%LanguageLevel: 3\n");
@@ -1009,7 +1039,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             if (image_info->adjoin == MagickFalse)
-              (void) strcpy(buffer,"%%Pages: 1\n");
+              (void) CopyMagickString(buffer,"%%Pages: 1\n",MaxTextExtent);
             else
               (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Pages: %lu\n",
                 (unsigned long) GetImageListLength(image));
@@ -1028,20 +1058,20 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         /*
           One label line for each line in label string.
         */
-        attribute=GetImageAttribute(image,"Label");
-        if (attribute != (const ImageAttribute *) NULL)
-        {
-          (void) WriteBlobString(image,"\n  % Labels.\n  /Helvetica findfont "
-            "pointsize scalefont setfont\n");
-          for (i=(long) MultilineCensus(attribute->value)-1; i >= 0; i--)
+        value=GetImageProperty(image,"label");
+        if (value != (const char *) NULL)
           {
-            (void) WriteBlobString(image,
-              "  currentfile buffer readline pop token pop\n");
-            (void) FormatMagickString(buffer,MaxTextExtent,
-              "  0 y %g add moveto show pop\n",i*image_info->pointsize+12);
-            (void) WriteBlobString(image,buffer);
+              (void) WriteBlobString(image,"\n  %% Labels.\n  /Helvetica "
+              " findfont pointsize scalefont setfont\n");
+            for (i=(long) MultilineCensus(value)-1; i >= 0; i--)
+            {
+              (void) WriteBlobString(image,
+                "  currentfile buffer readline pop token pop\n");
+              (void) FormatMagickString(buffer,MaxTextExtent,
+                "  0 y %g add moveto show pop\n",i*pointsize+12);
+              (void) WriteBlobString(image,buffer);
+            }
           }
-        }
         /*
           The static postscript procedures epilog.
         */
@@ -1084,40 +1114,40 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     /*
       Page font resource if there's a label.
     */
-    attribute=GetImageAttribute(image,"Label");
-    if (attribute != (const ImageAttribute *) NULL)
+    value=GetImageProperty(image,"label");
+    if (value != (const char *) NULL)
       (void) WriteBlobString(image,"%%PageResources: font Helvetica\n");
     /*
       PS clipping path from Photoshop clipping path.
     */
     if ((image->clip_mask == (Image *) NULL) ||
         (LocaleNCompare("8BIM:",image->clip_mask->magick_filename,5) != 0))
-      (void)WriteBlobString(image,"/ClipImage {} def\n");
+      (void) WriteBlobString(image,"/ClipImage {} def\n");
     else
       {
-        const ImageAttribute
-          *attribute;
+        const char
+          *value;
 
-        attribute=GetImageAttribute(image,image->clip_mask->magick_filename);
-        if (attribute == (const ImageAttribute *) NULL)
+        value=GetImageProperty(image,image->clip_mask->magick_filename);
+        if (value == (const char *) NULL)
           return(MagickFalse);
-        (void) WriteBlobString(image,attribute->value);
+        (void) WriteBlobString(image,value);
         (void) WriteBlobByte(image,'\n');
       }
     /*
-      Push a dictionary for our own def's if this is an EPS.
+      Push a dictionary for our own def's if this an EPS.
     */
     if (LocaleCompare(image_info->magick,"PS3") != 0)
       (void) WriteBlobString(image,"userdict begin\n");
     /*
       Image mask.
     */
-    if ((LocaleCompare(image_info->magick,"EPS3") != 0) && image->matte)
-      if (WritePS3MaskImage(image_info,image) == MagickFalse)
-        {
-          CloseBlob(image);
-          return(MagickFalse);
-        }
+    if ((image->matte != MagickFalse) &&
+        (WritePS3MaskImage(image_info,image,compression) == MagickFalse))
+      {
+        (void) CloseBlob(image);
+        return(MagickFalse);
+      }
     /*
       Remember position of BeginData comment so we can update it.
     */
@@ -1132,15 +1162,15 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       Translate, scale, and font point size.
     */
     (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,scale.x,scale.y,image_info->pointsize);
+      geometry.x,geometry.y,scale.x,scale.y,pointsize);
     (void) WriteBlobString(image,buffer);
     /*
       Output labels.
     */
     labels=(char **) NULL;
-    attribute=GetImageAttribute(image,"Label");
-    if (attribute != (const ImageAttribute *) NULL)
-      labels=StringToList(attribute->value);
+    value=GetImageProperty(image,"label");
+    if (value != (const char *) NULL)
+      labels=StringToList(value);
     if (labels != (char **) NULL)
       {
         for (i=0; labels[i] != (char *) NULL; i++)
@@ -1159,7 +1189,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
                 Ascii85Encode(image,(unsigned char) labels[i][j]);
               Ascii85Flush(image);
             }
-          labels[i]=(char *) RelinquishMagickMemory(labels[i]);
+          labels[i]=DestroyString(labels[i]);
         }
         labels=(char **) RelinquishMagickMemory(labels);
       }
@@ -1171,13 +1201,11 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,"true\n");
       else
         (void) WriteBlobString(image,"false\n");
-    if ((compression == FaxCompression) || (compression == Group4Compression))
-      (void) SetImageType(image,BilevelType);
     /*
       Showpage for non-EPS.
     */
-    (void) WriteBlobString(image,
-      LocaleCompare(image_info->magick,"PS3") == 0 ?  "true\n" : "false\n");
+    (void) WriteBlobString(image, LocaleCompare(image_info->magick,"PS3") == 0 ?
+      "true\n" : "false\n");
     /*
       Image columns, rows, and color space.
     */
@@ -1188,15 +1216,15 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     /*
       Masked image?
     */
-    (void) WriteBlobString(image,
-      (LocaleCompare(image_info->magick,"EPS3") != 0) && image->matte ?
+    (void) WriteBlobString(image,image->matte != MagickFalse ?
       "true\n" : "false\n");
     /*
       Render with imagemask operator?
     */
     option=GetImageOption(image_info,"ps3:imagemask");
-    (void) WriteBlobString(image,
-      option != (const char *) NULL ? "true\n" : "false\n");
+    (void) WriteBlobString(image,((option != (const char *) NULL) &&
+      (IsMonochromeImage(image,&image->exception) != MagickFalse)) ?
+      "true\n" : "false\n");
     /*
       Output pixel data.
     */
@@ -1263,51 +1291,55 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
           Image data.
         */
         if (compression == JPEGCompression)
-          status=JPEGEncodeImage(image_info,image);
+          status=InjectImageBlob(image_info,image,image,"jpeg",
+            &image->exception);
         else
           if ((compression == FaxCompression) ||
               (compression == Group4Compression))
-           {
-             if (LocaleCompare(CCITTParam,"0") == 0)
-               status=HuffmanEncodeImage(image_info,image);
-             else
-               status=Huffman2DEncodeImage(image_info,image);
-           }
-        else
-          {
-            status=SerializeImageChannel(image_info,image,&pixels,&length);
-            if (status == MagickFalse)
-              {
-                CloseBlob(image);
-                return(MagickFalse);
-              }
-            switch (compression)
             {
-              case NoCompression:
-              default:
+              if (LocaleCompare(CCITTParam,"0") == 0)
+                status=HuffmanEncodeImage(image_info,image,image);
+              else
+                status=Huffman2DEncodeImage(image_info,image,image);
+            }
+          else
+            {
+              status=SerializeImageChannel(image_info,image,&pixels,&length);
+              if (status == MagickFalse)
+                {
+                  (void) CloseBlob(image);
+                  return(MagickFalse);
+                }
+              switch (compression)
               {
-                for (i=0; i < (long) length; i++)
-                  (void) WriteBlobByte(image,pixels[i]);
-                break;
+                case NoCompression:
+                default:
+                {
+                  Ascii85Initialize(image);
+                  for (i=0; i < (long) length; i++)
+                    Ascii85Encode(image,pixels[i]);
+                  Ascii85Flush(image);
+                  status=MagickTrue;
+                  break;
+                }
+                case LZWCompression:
+                {
+                  status=LZWEncodeImage(image,length,pixels);
+                  break;
+                }
+                case RLECompression:
+                {
+                  status=PackbitsEncodeImage(image,length,pixels);
+                  break;
+                }
+                case ZipCompression:
+                {
+                  status=ZLIBEncodeImage(image,length,pixels);
+                  break;
+                }
               }
-              case LZWCompression:
-              {
-                status=LZWEncodeImage(image,length,pixels);
-                break;
-              }
-              case RLECompression:
-              {
-                status=PackbitsEncodeImage(image,length,pixels);
-                break;
-              }
-              case ZipCompression:
-              {
-                status=ZLIBEncodeImage(image,length,image->quality,pixels);
-                break;
-              }
+              pixels=(unsigned char *) RelinquishMagickMemory(pixels);
             }
-            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-          }
       }
     else
       if ((image->storage_class == DirectClass) || (image->colors > 256) ||
@@ -1350,7 +1382,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             Image data.
           */
           if (compression == JPEGCompression)
-            status=JPEGEncodeImage(image_info,image);
+            status=InjectImageBlob(image_info,image,image,"jpeg",
+              &image->exception);
           else
             {
               /*
@@ -1359,7 +1392,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               status=SerializeImage(image_info,image,&pixels,&length);
               if (status == MagickFalse)
                 {
-                  CloseBlob(image);
+                  (void) CloseBlob(image);
                   return(MagickFalse);
                 }
               switch (compression)
@@ -1386,7 +1419,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
                 }
                 case ZipCompression:
                 {
-                  status=ZLIBEncodeImage(image,length,image->quality,pixels);
+                  status=ZLIBEncodeImage(image,length,pixels);
                   break;
                 }
               }
@@ -1437,7 +1470,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             /*
               Color map - uncompressed.
             */
-            if (compression != NoCompression)
+            if ((compression != NoCompression) &&
+                (compression != UndefinedCompression))
               {
                 for (i=0; i < (long) image->colors; i++)
                 {
@@ -1466,7 +1500,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             status=SerializeImageIndexes(image_info,image,&pixels,&length);
             if (status == MagickFalse)
               {
-                CloseBlob(image);
+                (void) CloseBlob(image);
                 return(MagickFalse);
               }
             switch (compression)
@@ -1483,7 +1517,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case JPEGCompression:
               {
-                status=JPEGEncodeImage(image_info,image);
+                status=InjectImageBlob(image_info,image,image,"jpeg",
+                  &image->exception);
                 break;
               }
               case RLECompression:
@@ -1498,7 +1533,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image->quality,pixels);
+                status=ZLIBEncodeImage(image,length,pixels);
                 break;
               }
             }
@@ -1507,7 +1542,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,'\n');
     if (status == MagickFalse)
       {
-        CloseBlob(image);
+        (void) CloseBlob(image);
         return(MagickFalse);
       }
     /*
@@ -1515,15 +1550,17 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     */
     length=(size_t) (TellBlob(image)-stop);
     stop=TellBlob(image);
-    (void) SeekBlob(image,start,SEEK_SET);
+    offset=SeekBlob(image,start,SEEK_SET);
+    if (offset < 0)
+      ThrowWriterException(CorruptImageError,"ImproperImageHeader");
     (void) FormatMagickString(buffer,MaxTextExtent,
       "%%%%BeginData:%13ld %s Bytes\n",(long) length,
       compression == NoCompression ? "ASCII" : "BINARY");
     (void) WriteBlobString(image,buffer);
-    (void) SeekBlob(image,stop,SEEK_SET);
+    offset=SeekBlob(image,stop,SEEK_SET);
     (void) WriteBlobString(image,"%%EndData\n");
     /*
-      End private dictionary if this is an EPS.
+      End private dictionary if this an EPS.
     */
     if (LocaleCompare(image_info->magick,"PS3") != 0)
       (void) WriteBlobString(image,"end\n");
@@ -1531,14 +1568,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
   (void) WriteBlobString(image,"%%Trailer\n");
   if (page > 1)
@@ -1553,6 +1586,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
