@@ -10,20 +10,20 @@
 %                            R  R   A   A  W   W                              %
 %                                                                             %
 %                                                                             %
-%                       Read/Write RAW Image Format.                          %
+%                       Read/Write RAW Image Format                           %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,16 +41,25 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Forward declarations.
@@ -79,43 +88,44 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
-static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
+static Image *ReadRAWImage(const ImageInfo *image_info,
+  ExceptionInfo *exception)
 {
   Image
+    *canvas_image,
     *image;
 
   long
-    j,
     y;
 
   MagickBooleanType
     status;
 
-  register long
-    i,
-    x;
+  MagickOffsetType
+    scene;
 
-  register PixelPacket
-    *q;
+  QuantumInfo
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
+
+  register long
+    i;
 
   ssize_t
     count;
 
   size_t
-    packet_size;
+    length;
 
   unsigned char
-    *scanline;
-
-  unsigned long
-    depth,
-    width;
+    *pixels;
 
   /*
     Open image file.
@@ -123,46 +133,37 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  switch (*image->magick)
-  {
-    case 'C':
-    case 'c':
-    case 'M':
-    case 'm':
-    case 'Y':
-    case 'y':
-    case 'K':
-    case 'k':
-    {
-      image->colorspace=CMYKColorspace;
-      break;
-    }
-    default:
-      break;
-  }
   for (i=0; i < image->offset; i++)
-    (void) ReadBlobByte(image);
+    if (ReadBlobByte(image) == EOF)
+      {
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
   /*
-    Allocate memory for a scanline.
+    Create virtual canvas to support cropping (i.e. image.gray[100x100+10+20]).
   */
-  depth=GetImageQuantumDepth(image,MagickTrue);
-  packet_size=(size_t) (depth/8);
-  scanline=(unsigned char *)
-    AcquireMagickMemory(packet_size*image->extract_info.width);
-  if (scanline == (unsigned char *) NULL)
+  canvas_image=CloneImage(image,image->extract_info.width,1,MagickFalse,
+    exception);
+  (void) SetImageVirtualPixelMethod(canvas_image,BlackVirtualPixelMethod);
+  quantum_type=GrayQuantum;
+  quantum_info=AcquireQuantumInfo(image_info,canvas_image);
+  if (quantum_info == (QuantumInfo *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  pixels=GetQuantumPixels(quantum_info);
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
     {
@@ -170,75 +171,112 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
         Skip to next image.
       */
       image->scene++;
+      length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
       for (y=0; y < (long) image->rows; y++)
-        (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+      {
+        count=ReadBlob(image,length,pixels);
+        if (count != (ssize_t) length)
+          break;
+      }
     }
-  x=(long) (packet_size*image->extract_info.x);
+  scene=0;
+  count=0;
+  length=0;
   do
   {
     /*
-      Convert raster image to pixel packets.
+      Read pixels to virtual canvas image then push to image.
     */
-    if (AllocateImageColormap(image,1 << image->depth) == MagickFalse)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    for (y=0; y < image->extract_info.y; y++)
-      (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-    for (y=0; y < (long) image->rows; y++)
+    if (scene == 0)
+      {
+        length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
+        count=ReadBlob(image,length,pixels);
+      }
+    for (y=0; y < (long) image->extract_info.height; y++)
     {
-      if ((y > 0) || (image->previous == (Image *) NULL))
-        (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-      q=SetImagePixels(image,0,y,image->columns,1);
+      register const PixelPacket
+        *__restrict p;
+
+      register long
+        x;
+
+      register PixelPacket
+        *__restrict q;
+
+      if (count != (ssize_t) length)
+        {
+          ThrowFileException(exception,CorruptImageError,
+            "UnexpectedEndOfFile",image->filename);
+          break;
+        }
+      q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      (void) PushImagePixels(image,GrayQuantum,scanline+x);
-      if (SyncImagePixels(image) == MagickFalse)
+      length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,quantum_info,
+        quantum_type,pixels,exception);
+      if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
         break;
-      if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
+      if (((y-image->extract_info.y) >= 0) && 
+          ((y-image->extract_info.y) < (long) image->rows))
+        {
+          p=GetVirtualPixels(canvas_image,canvas_image->extract_info.x,0,
+            image->columns,1,exception);
+          q=QueueAuthenticPixels(image,0,y-image->extract_info.y,image->columns,
+            1,exception);
+          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
+          for (x=0; x < (long) image->columns; x++)
+          {
+            q->red=p->red;
+            q->green=p->green;
+            q->blue=p->blue;
+            p++;
+            q++;
+          }
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
+            break;
+        }
+      if (image->previous == (Image *) NULL)
+        {
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
+      count=ReadBlob(image,length,pixels);
     }
-    width=image->extract_info.height-image->rows-image->extract_info.y;
-    for (j=0; j < (long) width; j++)
-      (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-    if (EOFBlob(image) != MagickFalse)
-      {
-        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
-          image->filename);
-        break;
-      }
-    if (image->colorspace == CMYKColorspace)
-      image->storage_class=DirectClass;
+    SetQuantumImageType(image,quantum_type);
     /*
       Proceed to next image.
     */
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
-    if (count != 0)
+    if (count == (ssize_t) length)
       {
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        AcquireNextImage(image_info,image);
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
         if (status == MagickFalse)
           break;
       }
-  } while (count != 0);
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-  CloseBlob(image);
+    scene++;
+  } while (count == (ssize_t) length);
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  InheritException(&image->exception,&canvas_image->exception);
+  canvas_image=DestroyImage(canvas_image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -261,86 +299,95 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterRAWImage method is:
 %
-%      RegisterRAWImage(void)
+%      unsigned long RegisterRAWImage(void)
 %
 */
-ModuleExport void RegisterRAWImage(void)
+ModuleExport unsigned long RegisterRAWImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("R");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw red samples");
-  entry->module=AcquireString("RAW");
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Raw red samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("C");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw cyan samples");
-  entry->module=AcquireString("RAW");
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Raw cyan samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("G");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw green samples");
-  entry->module=AcquireString("RAW");
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Raw green samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw magenta samples");
-  entry->module=AcquireString("RAW");
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Raw magenta samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("B");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw blue samples");
-  entry->module=AcquireString("RAW");
+  entry->description=ConstantString("Raw blue samples");
+  entry->format_type=ExplicitFormatType;
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("Y");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw yellow samples");
-  entry->module=AcquireString("RAW");
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Raw yellow samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("A");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw alpha samples");
-  entry->module=AcquireString("RAW");
+  entry->description=ConstantString("Raw alpha samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("O");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw opacity samples");
-  entry->module=AcquireString("RAW");
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Raw opacity samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("K");
-  entry->decoder=(DecoderHandler *) ReadRAWImage;
-  entry->encoder=(EncoderHandler *) WriteRAWImage;
+  entry->decoder=(DecodeImageHandler *) ReadRAWImage;
+  entry->encoder=(EncodeImageHandler *) WriteRAWImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw black samples");
-  entry->module=AcquireString("RAW");
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Raw black samples");
+  entry->module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -394,11 +441,10 @@ ModuleExport void UnregisterRAWImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
 {
@@ -408,6 +454,9 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   MagickOffsetType
     scene;
 
+  QuantumInfo
+    *quantum_info;
+
   QuantumType
     quantum_type;
 
@@ -417,14 +466,14 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
+  ssize_t
+    count;
+
   size_t
-    packet_size;
+    length;
 
   unsigned char
-    *scanline;
-
-  unsigned long
-    depth;
+    *pixels;
 
   /*
     Open output image file.
@@ -434,7 +483,7 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -518,41 +567,38 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   do
   {
     /*
-      Allocate memory for scanline.
+      Convert image to RAW raster pixels.
     */
-    depth=GetImageQuantumDepth(image,MagickTrue);
-    packet_size=(size_t) (depth/8);
-    scanline=(unsigned char *)
-      AcquireMagickMemory(packet_size*image->columns);
-    if (scanline == (unsigned char *) NULL)
+    quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    /*
-      Convert MIFF to RAW raster scanline.
-    */
+    pixels=GetQuantumPixels(quantum_info);
     for (y=0; y < (long) image->rows; y++)
     {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      (void) PopImagePixels(image,quantum_type,scanline);
-      (void) WriteBlob(image,packet_size*image->columns,scanline);
+      length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
+        quantum_type,pixels,&image->exception);
+      count=WriteBlob(image,length,pixels);
+      if (count != (ssize_t) length)
+        break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows) != MagickFalse)
-          {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-            if (status == MagickFalse)
-              break;
-          }
+        {
+          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
-    scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-    if (image->next == (Image *) NULL)
+    quantum_info=DestroyQuantumInfo(quantum_info);
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
