@@ -10,14 +10,14 @@
 %                            SSSSS   UUU   N   N                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write Sun Rasterfile Image Format.                  %
+%                    Read/Write Sun Rasterfile Image Format                   %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -53,8 +54,11 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Forward declarations.
@@ -82,12 +86,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsSUN(const unsigned char *magick,const size_t length)
 {
@@ -130,13 +132,12 @@ static MagickBooleanType IsSUN(const unsigned char *magick,const size_t length)
 %      must be at least equal to the number columns times the number of rows
 %      of the source pixels.
 %
-%
 */
 static MagickBooleanType DecodeImage(const unsigned char *compressed_pixels,
-  const size_t length,unsigned char *pixels)
+  const size_t length,unsigned char *pixels,size_t maxpixels)
 {
   register const unsigned char
-    *p;
+    *p, *l;
 
   register unsigned char
     *q;
@@ -152,7 +153,8 @@ static MagickBooleanType DecodeImage(const unsigned char *compressed_pixels,
   assert(pixels != (unsigned char *) NULL);
   p=compressed_pixels;
   q=pixels;
-  while ((size_t) (p-compressed_pixels) < length)
+  l=q+maxpixels;
+  while (((size_t) (p-compressed_pixels) < length) && (q < l))
   {
     byte=(*p++);
     if (byte != 128U)
@@ -165,7 +167,7 @@ static MagickBooleanType DecodeImage(const unsigned char *compressed_pixels,
         count=(ssize_t) (*p++);
         if (count > 0)
           byte=(*p++);
-        while (count >= 0)
+        while ((count >= 0) && (q < l))
         {
           *q++=byte;
           count--;
@@ -196,11 +198,10 @@ static MagickBooleanType DecodeImage(const unsigned char *compressed_pixels,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -213,7 +214,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   typedef struct _SUNInfo
   {
-    unsigned long
+    unsigned int
       magic,
       width,
       height,
@@ -233,6 +234,12 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickSizeType
+    number_pixels;
+
   register IndexPacket
     *indexes;
 
@@ -248,6 +255,9 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
+  size_t
+    length;
+
   ssize_t
     count;
 
@@ -259,9 +269,6 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *sun_pixels;
 
   unsigned int
-    status;
-
-  unsigned long
     bytes_per_line;
 
   /*
@@ -274,7 +281,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -284,7 +291,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read SUN raster header.
   */
-  ResetMagickMemory(&sun_info,0,sizeof(sun_info));
+  (void) ResetMagickMemory(&sun_info,0,sizeof(sun_info));
   sun_info.magic=ReadBlobMSBLong(image);
   do
   {
@@ -300,9 +307,11 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     sun_info.type=ReadBlobMSBLong(image);
     sun_info.maptype=ReadBlobMSBLong(image);
     sun_info.maplength=ReadBlobMSBLong(image);
-    image->columns= sun_info.width;
-    image->rows= sun_info.height;
-    image->depth=sun_info.depth <= 8 ? 8UL : QuantumDepth;
+    image->columns=sun_info.width;
+    image->rows=sun_info.height;
+    if ((sun_info.depth == 0) || (sun_info.depth > 32))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    image->depth=sun_info.depth <= 8 ? sun_info.depth : MAGICKCORE_QUANTUM_DEPTH;
     if (sun_info.depth < 24)
       {
         image->storage_class=PseudoClass;
@@ -321,7 +330,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Create linear color ramp.
             */
-            if (AllocateImageColormap(image,image->colors) == MagickFalse)
+            if (AcquireImageColormap(image,image->colors) == MagickFalse)
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         break;
@@ -334,9 +343,10 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read SUN raster colormap.
         */
-        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+        if (AcquireImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        sun_colormap=(unsigned char *) AcquireMagickMemory(image->colors);
+        sun_colormap=(unsigned char *) AcquireQuantumMemory(image->colors,
+          sizeof(*sun_colormap));
         if (sun_colormap == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         count=ReadBlob(image,image->colors,sun_colormap);
@@ -359,7 +369,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read SUN raster colormap.
         */
-        sun_colormap=(unsigned char *) AcquireMagickMemory(sun_info.maplength);
+        sun_colormap=(unsigned char *) AcquireQuantumMemory(sun_info.maplength,
+          sizeof(*sun_colormap));
         if (sun_colormap == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         count=ReadBlob(image,sun_info.maplength,sun_colormap);
@@ -374,17 +385,25 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->rows=sun_info.height;
     if (image_info->ping != MagickFalse)
       {
-        CloseBlob(image);
+        (void) CloseBlob(image);
         return(GetFirstImageInList(image));
       }
-    sun_data=(unsigned char *)
-      AcquireMagickMemory((size_t) sun_info.length*sizeof(*sun_data));
+    if ((sun_info.length*sizeof(*sun_data))/sizeof(*sun_data) !=
+        sun_info.length || !sun_info.length)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    number_pixels=(MagickSizeType) image->columns*image->rows;
+    if ((sun_info.depth >= 8) &&
+        ((number_pixels*((sun_info.depth+7)/8)) > sun_info.length))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    sun_data=(unsigned char *) AcquireQuantumMemory((size_t) sun_info.length,
+      sizeof(*sun_data));
     if (sun_data == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     count=(ssize_t) ReadBlob(image,sun_info.length,sun_data);
     if ((count == 0) && (sun_info.type != RT_ENCODED))
       ThrowReaderException(CorruptImageError,"UnableToReadImageData");
     sun_pixels=sun_data;
+    bytes_per_line=0;
     if (sun_info.type == RT_ENCODED)
       {
         unsigned long
@@ -394,11 +413,21 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Read run-length encoded raster pixels.
         */
         height=sun_info.height;
-        bytes_per_line=2*(sun_info.width*sun_info.depth+15)/16;
-        sun_pixels=(unsigned char *) AcquireMagickMemory(bytes_per_line*height);
+        bytes_per_line=sun_info.width*sun_info.depth;
+        if ((height == 0) || (sun_info.width == 0) || (sun_info.depth == 0) ||
+            ((bytes_per_line/sun_info.depth) != sun_info.width))
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        bytes_per_line+=15;
+        bytes_per_line<<=1;
+        if ((bytes_per_line >> 1) != (sun_info.width*sun_info.depth+15))
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        bytes_per_line>>=4;
+        sun_pixels=(unsigned char *) AcquireQuantumMemory(height,
+          bytes_per_line*sizeof(*sun_pixels));
         if (sun_pixels == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        (void) DecodeImage(sun_data,sun_info.length,sun_pixels);
+        (void) DecodeImage(sun_data,sun_info.length,sun_pixels,
+          bytes_per_line*height);
         sun_data=(unsigned char *) RelinquishMagickMemory(sun_data);
       }
     /*
@@ -408,10 +437,10 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (sun_info.depth == 1)
       for (y=0; y < (long) image->rows; y++)
       {
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < ((long) image->columns-7); x+=8)
         {
           for (bit=7; bit >= 0; bit--)
@@ -427,85 +456,91 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         if ((((image->columns/8)+(image->columns % 8 ? 1 : 0)) % 2) != 0)
           p++;
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
     else
       if (image->storage_class == PseudoClass)
-        for (y=0; y < (long) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
-          if (q == (PixelPacket *) NULL)
-            break;
-          indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
-            indexes[x]=(*p++);
-          if ((image->columns % 2) != 0)
-            p++;
-          if (SyncImagePixels(image) == MagickFalse)
-            break;
-          if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
+          length=image->rows*(image->columns+image->columns % 2);
+          if (((sun_info.type == RT_ENCODED) &&
+               (length > (bytes_per_line*image->rows))) ||
+              ((sun_info.type != RT_ENCODED) && (length > sun_info.length)))
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          for (y=0; y < (long) image->rows; y++)
+          {
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+            if (q == (PixelPacket *) NULL)
+              break;
+            indexes=GetAuthenticIndexQueue(image);
+            for (x=0; x < (long) image->columns; x++)
+              indexes[x]=(IndexPacket) (*p++);
+            if ((image->columns % 2) != 0)
+              p++;
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
+              break;
+            if (image->previous == (Image *) NULL)
               {
-                status=image->progress_monitor(LoadImageTag,y,image->rows,
-                  image->client_data);
+                status=SetImageProgress(image,LoadImageTag,y,image->rows);
                 if (status == MagickFalse)
                   break;
               }
+          }
         }
       else
-        for (y=0; y < (long) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
-          if (q == (PixelPacket *) NULL)
-            break;
-          for (x=0; x < (long) image->columns; x++)
+          length=image->rows*((image->columns*(image->matte != MagickFalse ? 4 :
+            3))+image->columns % 2);
+          if (((sun_info.type == RT_ENCODED) &&
+               (length > (bytes_per_line*image->rows))) ||
+              ((sun_info.type != RT_ENCODED) && (length > sun_info.length)))
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          for (y=0; y < (long) image->rows; y++)
           {
-            if (image->matte != MagickFalse)
-              q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
-            if (sun_info.type == RT_STANDARD)
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+            if (q == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (long) image->columns; x++)
+            {
+              if (image->matte != MagickFalse)
+                q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
+              if (sun_info.type == RT_STANDARD)
+                {
+                  q->blue=ScaleCharToQuantum(*p++);
+                  q->green=ScaleCharToQuantum(*p++);
+                  q->red=ScaleCharToQuantum(*p++);
+                }
+              else
+                {
+                  q->red=ScaleCharToQuantum(*p++);
+                  q->green=ScaleCharToQuantum(*p++);
+                  q->blue=ScaleCharToQuantum(*p++);
+                }
+              if (image->colors != 0)
+                {
+                  q->red=image->colormap[(long) q->red].red;
+                  q->green=image->colormap[(long) q->green].green;
+                  q->blue=image->colormap[(long) q->blue].blue;
+                }
+              q++;
+            }
+            if ((image->columns % 2) != 0)
+              p++;
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
+              break;
+            if (image->previous == (Image *) NULL)
               {
-                q->blue=ScaleCharToQuantum(*p++);
-                q->green=ScaleCharToQuantum(*p++);
-                q->red=ScaleCharToQuantum(*p++);
-              }
-            else
-              {
-                q->red=ScaleCharToQuantum(*p++);
-                q->green=ScaleCharToQuantum(*p++);
-                q->blue=ScaleCharToQuantum(*p++);
-              }
-            if (image->colors != 0)
-              {
-                q->red=image->colormap[q->red].red;
-                q->green=image->colormap[q->green].green;
-                q->blue=image->colormap[q->blue].blue;
-              }
-            q++;
-          }
-          if ((image->columns % 2) != 0)
-            p++;
-          if (SyncImagePixels(image) == MagickFalse)
-            break;
-          if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,y,image->rows,
-                  image->client_data);
+                status=SetImageProgress(image,LoadImageTag,y,image->rows);
                 if (status == MagickFalse)
                   break;
               }
+          }
         }
     if (image->storage_class == PseudoClass)
       (void) SyncImage(image);
@@ -528,23 +563,20 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImageTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (sun_info.magic == 0x59a66a95);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -568,27 +600,28 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterSUNImage method is:
 %
-%      RegisterSUNImage(void)
+%      unsigned long RegisterSUNImage(void)
 %
 */
-ModuleExport void RegisterSUNImage(void)
+ModuleExport unsigned long RegisterSUNImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("RAS");
-  entry->decoder=(DecoderHandler *) ReadSUNImage;
-  entry->encoder=(EncoderHandler *) WriteSUNImage;
-  entry->magick=(MagickHandler *) IsSUN;
-  entry->description=AcquireString("SUN Rasterfile");
-  entry->module=AcquireString("SUN");
+  entry->decoder=(DecodeImageHandler *) ReadSUNImage;
+  entry->encoder=(EncodeImageHandler *) WriteSUNImage;
+  entry->magick=(IsImageFormatHandler *) IsSUN;
+  entry->description=ConstantString("SUN Rasterfile");
+  entry->module=ConstantString("SUN");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SUN");
-  entry->decoder=(DecoderHandler *) ReadSUNImage;
-  entry->encoder=(EncoderHandler *) WriteSUNImage;
-  entry->description=AcquireString("SUN Rasterfile");
-  entry->module=AcquireString("SUN");
+  entry->decoder=(DecodeImageHandler *) ReadSUNImage;
+  entry->encoder=(EncodeImageHandler *) WriteSUNImage;
+  entry->description=ConstantString("SUN Rasterfile");
+  entry->module=ConstantString("SUN");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -635,11 +668,10 @@ ModuleExport void UnregisterSUNImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
 {
@@ -651,7 +683,7 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
 
   typedef struct _SUNInfo
   {
-    unsigned long
+    unsigned int
       magic,
       width,
       height,
@@ -674,12 +706,12 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
   MagickSizeType
     number_pixels;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -707,11 +739,15 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize SUN raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image->colorspace != RGBColorspace)
+      (void) TransformImageColorspace(image,RGBColorspace);
     sun_info.magic=0x59a66a95;
-    sun_info.width=image->columns;
-    sun_info.height=image->rows;
-    sun_info.type=(unsigned long) 
+    if ((image->columns != (unsigned int) image->columns) ||
+        (image->rows != (unsigned int) image->rows))
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+    sun_info.width=(unsigned int) image->columns;
+    sun_info.height=(unsigned int) image->rows;
+    sun_info.type=(unsigned int) 
       (image->storage_class == DirectClass ? RT_FORMAT_RGB : RT_STANDARD);
     sun_info.maptype=RMT_NONE;
     sun_info.maplength=0;
@@ -723,8 +759,8 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
         /*
           Full color SUN raster.
         */
-        sun_info.depth=(image->matte ? 32UL : 24UL);
-        sun_info.length=(unsigned long) ((image->matte ? 4 : 3)*number_pixels);
+        sun_info.depth=(image->matte ? 32U : 24U);
+        sun_info.length=(unsigned int) ((image->matte ? 4 : 3)*number_pixels);
         sun_info.length+=image->columns & 0x01 ? image->rows : 0;
       }
     else
@@ -734,7 +770,8 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             Monochrome SUN raster.
           */
           sun_info.depth=1;
-          sun_info.length=((image->columns+7) >> 3)*image->rows;
+          sun_info.length=(unsigned int) (((image->columns+7) >> 3)*
+            image->rows);
           sun_info.length+=((image->columns/8)+(image->columns % 8 ? 1 : 0)) %
             2 ? image->rows : 0;
         }
@@ -744,10 +781,10 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             Colormapped SUN raster.
           */
           sun_info.depth=8;
-          sun_info.length=(unsigned long) number_pixels;
+          sun_info.length=(unsigned int) number_pixels;
           sun_info.length+=image->columns & 0x01 ? image->rows : 0;
           sun_info.maptype=RMT_EQUAL_RGB;
-          sun_info.maplength=image->colors*3;
+          sun_info.maplength=(unsigned int) (3*image->colors);
         }
     /*
       Write SUN header.
@@ -779,8 +816,8 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
         /*
           Allocate memory for pixels.
         */
-        length=image->columns*sizeof(PixelPacket);
-        pixels=(unsigned char *) AcquireMagickMemory(length);
+        length=image->columns;
+        pixels=(unsigned char *) AcquireQuantumMemory(length,4*sizeof(*pixels));
         if (pixels == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         /*
@@ -788,14 +825,14 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
         */
         for (y=0; y < (long) image->rows; y++)
         {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
           q=pixels;
           for (x=0; x < (long) image->columns; x++)
           {
             if (image->matte != MagickFalse)
-              *q++=ScaleQuantumToChar(QuantumRange-p->opacity);
+              *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
             *q++=ScaleQuantumToChar(p->red);
             *q++=ScaleQuantumToChar(p->green);
             *q++=ScaleQuantumToChar(p->blue);
@@ -805,14 +842,11 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             *q++=0;  /* pad scanline */
           (void) WriteBlob(image,(size_t) (q-pixels),pixels);
           if (image->previous == (Image *) NULL)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(SaveImageTag,y,image->rows,
-                  image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
         pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       }
@@ -821,29 +855,24 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
         {
           register unsigned char
             bit,
-            byte,
-            polarity;
+            byte;
 
           /*
             Convert PseudoClass image to a SUN monochrome image.
           */
           (void) SetImageType(image,BilevelType);
-          polarity=PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2);
-          if (image->colors == 2)
-            polarity=PixelIntensityToQuantum(&image->colormap[0]) <
-              PixelIntensityToQuantum(&image->colormap[1]);
           for (y=0; y < (long) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=GetVirtualIndexQueue(image);
             bit=0;
             byte=0;
             for (x=0; x < (long) image->columns; x++)
             {
               byte<<=1;
-              if (indexes[x] != polarity)
+              if (PixelIntensity(p) < (MagickRealType) (QuantumRange/2.0))
                 byte|=0x01;
               bit++;
               if (bit == 8)
@@ -855,19 +884,16 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
               p++;
             }
             if (bit != 0)
-              (void) WriteBlobByte(image,byte << (8-bit));
+              (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
             if ((((image->columns/8)+
                 (image->columns % 8 ? 1 : 0)) % 2) != 0)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
-              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                  (QuantumTick(y,image->rows) != MagickFalse))
-                {
-                  status=image->progress_monitor(SaveImageTag,y,image->rows,
-                    image->client_data);
-                  if (status == MagickFalse)
-                    break;
-                }
+              {
+                status=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (status == MagickFalse)
+                  break;
+              }
           }
         }
       else
@@ -889,10 +915,10 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
           */
           for (y=0; y < (long) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=GetIndexes(image);
+            indexes=GetVirtualIndexQueue(image);
             for (x=0; x < (long) image->columns; x++)
             {
               (void) WriteBlobByte(image,(unsigned char) indexes[x]);
@@ -901,28 +927,21 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             if (image->columns & 0x01)
               (void) WriteBlobByte(image,0);  /* pad scanline */
             if (image->previous == (Image *) NULL)
-              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                  (QuantumTick(y,image->rows) != MagickFalse))
-                {
-                  status=image->progress_monitor(SaveImageTag,y,image->rows,
-                    image->client_data);
-                  if (status == MagickFalse)
-                    break;
-                }
+              {
+                status=SetImageProgress(image,SaveImageTag,y,image->rows);
+                if (status == MagickFalse)
+                  break;
+              }
           }
         }
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
