@@ -10,7 +10,7 @@
 %                             UUU   R  R   LLLLL                              %
 %                                                                             %
 %                                                                             %
-%                        Retrieve An Image Via a URL.                         %
+%                        Retrieve An Image Via URL.                           %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -18,7 +18,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,15 +51,18 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/resource_.h"
 #include "magick/string_.h"
-#if defined(HasXML)
+#include "magick/module.h"
+#if defined(MAGICKCORE_XML_DELEGATE)
 #  if defined(__WINDOWS__)
 #    if defined(__MINGW32__)
 #      define _MSC_VER
+#    else
+#      include <win32config.h>
 #    endif
-#    include <win32config.h>
 #  endif
 #  include <libxml/parser.h>
 #  include <libxml/xmlmemory.h>
@@ -78,7 +81,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadURLImage retrieves an image via a URL, decodes the image, and returns
+%  ReadURLImage retrieves an image via URL, decodes the image, and returns
 %  it.  It allocates the memory necessary for the new Image structure and
 %  returns a pointer to the new image.
 %
@@ -88,29 +91,33 @@
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-static void GetFTPData(void *userdata,const char *data,int length)
+#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
+static void GetFTPData(void *userdata,const char *data,int size)
 {
   FILE
     *file;
 
+  size_t
+    length;
+
   file=(FILE *) userdata;
   if (file == (FILE *) NULL)
     return;
-  if (length <= 0)
+  if (size <= 0)
     return;
-  (void) fwrite(data,length,1,file);
+  length=fwrite(data,size,1,file);
 }
+#endif
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
@@ -157,9 +164,13 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   LocaleLower(filename);
   (void) ConcatenateMagickString(filename,image_info->filename,MaxTextExtent);
   if (LocaleCompare(read_info->magick,"file") == 0)
-    (void) CopyMagickString(read_info->filename,image_info->filename+2,
-      MaxTextExtent);
-#if defined(HasXML) && defined(LIBXML_FTP_ENABLED)
+    {
+      (void) RelinquishUniqueFileResource(read_info->filename);
+      unique_file=(-1);
+      (void) CopyMagickString(read_info->filename,image_info->filename+2,
+        MaxTextExtent);
+    }
+#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
   if (LocaleCompare(read_info->magick,"ftp") == 0)
     {
       void
@@ -176,7 +187,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
 #endif
-#if defined(HasXML) && defined(LIBXML_HTTP_ENABLED)
+#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED)
   if (LocaleCompare(read_info->magick,"http") == 0)
     {
       char
@@ -194,8 +205,11 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (const char *) NULL,&type,(const char *) NULL,0);
       if (context != (void *) NULL)
         {
+          ssize_t
+            count;
+
           while ((bytes=xmlNanoHTTPRead(context,buffer,MaxBufferExtent)) > 0)
-            (void) fwrite(buffer,bytes,1,file);
+            count=(ssize_t) fwrite(buffer,bytes,1,file);
           xmlNanoHTTPClose(context);
           xmlFree(type);
           xmlNanoHTTPCleanup();
@@ -205,7 +219,8 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) fclose(file);
   *read_info->magick='\0';
   image=ReadImage(read_info,exception);
-  (void) RelinquishUniqueFileResource(read_info->filename);
+  if (unique_file != -1)
+    (void) RelinquishUniqueFileResource(read_info->filename);
   read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
@@ -233,36 +248,37 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterURLImage method is:
 %
-%      RegisterURLImage(void)
+%      unsigned long RegisterURLImage(void)
 %
 */
-ModuleExport void RegisterURLImage(void)
+ModuleExport unsigned long RegisterURLImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("HTTP");
-#if defined(HasXML) && defined(LIBXML_HTTP_ENABLED)
-  entry->decoder=(DecoderHandler *) ReadURLImage;
+#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED)
+  entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
-  entry->description=AcquireString("Uniform Resource Locator (http://)");
-  entry->module=AcquireString("URL");
+  entry->description=ConstantString("Uniform Resource Locator (http://)");
+  entry->module=ConstantString("URL");
   entry->stealth=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FTP");
-#if defined(HasXML) && defined(LIBXML_FTP_ENABLED)
-  entry->decoder=(DecoderHandler *) ReadURLImage;
+#if defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED)
+  entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
-  entry->description=AcquireString("Uniform Resource Locator (ftp://)");
-  entry->module=AcquireString("URL");
+  entry->description=ConstantString("Uniform Resource Locator (ftp://)");
+  entry->module=ConstantString("URL");
   entry->stealth=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FILE");
-  entry->decoder=(DecoderHandler *) ReadURLImage;
-  entry->description=AcquireString("Uniform Resource Locator (file://)");
-  entry->module=AcquireString("URL");
+  entry->decoder=(DecodeImageHandler *) ReadURLImage;
+  entry->description=ConstantString("Uniform Resource Locator (file://)");
+  entry->module=ConstantString("URL");
   entry->stealth=MagickTrue;
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
