@@ -10,14 +10,14 @@
 %                            X   X   W W   DDDD                               %
 %                                                                             %
 %                                                                             %
-%                Read/Write X Windows System Window Dump Format.              %
+%                Read/Write X Windows System Window Dump Format               %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,10 +40,11 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
+#include "magick/colormap-private.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -53,9 +54,13 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
-#if defined(HasX11)
+#include "magick/module.h"
+#if defined(MAGICKCORE_X11_DELEGATE)
 #include "magick/xwindow-private.h"
 #if !defined(vms)
 #include <X11/XWDFile.h>
@@ -67,7 +72,7 @@
 /*
   Forward declarations.
 */
-#if defined(HasX11)
+#if defined(MAGICKCORE_X11_DELEGATE)
 static MagickBooleanType
   WriteXWDImage(const ImageInfo *,Image *);
 #endif
@@ -92,12 +97,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 {
@@ -113,7 +116,7 @@ static MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
   return(MagickFalse);
 }
 
-#if defined(HasX11)
+#if defined(MAGICKCORE_X11_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -135,14 +138,17 @@ static MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
 static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+#define CheckOverflowException(length,width,height) \
+  (((height) != 0) && ((length)/((size_t) height) != ((size_t) width)))
+
   char
     *comment;
 
@@ -152,10 +158,16 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
+  int
+    x_status;
+
   long
     y;
 
   MagickBooleanType
+    authentic_colormap;
+
+  MagickStatusType
     status;
 
   register IndexPacket
@@ -201,7 +213,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -231,16 +243,15 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (header.header_size < sz_XWDheader)
     ThrowReaderException(CorruptImageError,"CorruptImage");
   length=(size_t) header.header_size-sz_XWDheader;
-  comment=(char *) AcquireMagickMemory(length+MaxTextExtent);
+  comment=(char *) AcquireQuantumMemory(length+1,sizeof(*comment));
   if (comment == (char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   count=ReadBlob(image,length,(unsigned char *) comment);
   comment[length]='\0';
-  (void) SetImageAttribute(image,"Comment",comment);
-  comment=(char *) RelinquishMagickMemory(comment);
-  if (count == 0)
-    ThrowReaderException(CorruptImageError,
-      "UnableToReadWindowNameFromDumpFile");
+  (void) SetImageProperty(image,"comment",comment);
+  comment=DestroyString(comment);
+  if (count != (ssize_t) length)
+    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
   /*
     Initialize the X image.
   */
@@ -262,33 +273,40 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ximage->red_mask=header.red_mask;
   ximage->green_mask=header.green_mask;
   ximage->blue_mask=header.blue_mask;
-  status=(MagickBooleanType) XInitImage(ximage);
-  if (status == MagickFalse)
-    ThrowReaderException(CorruptImageError,"UnableToReadImageHeader");
+  if ((ximage->depth < 0) || (ximage->width < 0) || (ximage->height < 0) ||
+      (ximage->bitmap_pad < 0) || (ximage->bytes_per_line < 0))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if ((ximage->bits_per_pixel > 32) || (ximage->bitmap_unit > 32))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  x_status=XInitImage(ximage);
+  if (x_status == 0)
+    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
   /*
     Read colormap.
   */
+  authentic_colormap=MagickFalse;
   colors=(XColor *) NULL;
   if (header.ncolors != 0)
     {
       XWDColor
         color;
 
-      colors=(XColor *)
-        AcquireMagickMemory((size_t) header.ncolors*sizeof(*colors));
+      length=(size_t) header.ncolors;
+      colors=(XColor *) AcquireQuantumMemory(length,sizeof(*colors));
       if (colors == (XColor *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (long) header.ncolors; i++)
       {
         count=ReadBlob(image,sz_XWDColor,(unsigned char *) &color);
         if (count == 0)
-          ThrowReaderException(CorruptImageError,
-            "UnableToReadColormapFromDumpFile");
+          ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
         colors[i].pixel=color.pixel;
         colors[i].red=color.red;
         colors[i].green=color.green;
         colors[i].blue=color.blue;
         colors[i].flags=(char) color.flags;
+        if (color.flags != 0)
+         authentic_colormap=MagickTrue;
       }
       /*
         Ensure the header byte-order is most-significant byte first.
@@ -299,23 +317,32 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         {
           MSBOrderLong((unsigned char *) &colors[i].pixel,
             sizeof(colors[i].pixel));
-          MSBOrderShort((unsigned char *) &colors[i].red,
-            3*sizeof(colors[i].red));
+          MSBOrderShort((unsigned char *) &colors[i].red,3*
+            sizeof(colors[i].red));
         }
     }
   /*
     Allocate the pixel buffer.
   */
-  if (ximage->format == ZPixmap)
-    length=(size_t) ximage->bytes_per_line*ximage->height;
-  else
-    length=(size_t) ximage->bytes_per_line*ximage->height*ximage->depth;
-  ximage->data=(char *) AcquireMagickMemory(length);
+  length=(size_t) ximage->bytes_per_line*ximage->height;
+  if (CheckOverflowException(length,ximage->bytes_per_line,ximage->height))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if (ximage->format != ZPixmap)
+    {
+      size_t
+        extent;
+
+      extent=length;
+      length*=ximage->depth;
+      if (CheckOverflowException(length,extent,ximage->depth))
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
+  ximage->data=(char *) AcquireQuantumMemory(length,sizeof(*ximage->data));
   if (ximage->data == (char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   count=ReadBlob(image,length,(unsigned char *) ximage->data);
   if (count == 0)
-    ThrowReaderException(CorruptImageError,"UnableToReadPixmapFromDumpFile");
+    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
   /*
     Convert image to MIFF format.
   */
@@ -371,63 +398,54 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Convert X image to DirectClass packets.
         */
-        if (image->colors != 0)
+        if ((image->colors != 0) && (authentic_colormap != MagickFalse))
           for (y=0; y < (long) image->rows; y++)
           {
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
             for (x=0; x < (long) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
               index=(IndexPacket) ((pixel >> red_shift) & red_mask);
-              q->red=ScaleShortToQuantum(colors[index].red);
+              q->red=ScaleShortToQuantum(colors[(long) index].red);
               index=(IndexPacket) ((pixel >> green_shift) & green_mask);
-              q->green=ScaleShortToQuantum(colors[index].green);
+              q->green=ScaleShortToQuantum(colors[(long) index].green);
               index=(IndexPacket) ((pixel >> blue_shift) & blue_mask);
-              q->blue=ScaleShortToQuantum(colors[index].blue);
+              q->blue=ScaleShortToQuantum(colors[(long) index].blue);
               q++;
             }
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
+              break;
+            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            if (status == MagickFalse)
               break;
-            if (QuantumTick(y,image->rows) != MagickFalse)
-              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                  (QuantumTick(y,image->rows) != MagickFalse))
-                {
-                  status=image->progress_monitor(LoadImageTag,y,image->rows,
-                    image->client_data);
-                  if (status == MagickFalse)
-                    break;
-                }
           }
         else
           for (y=0; y < (long) image->rows; y++)
           {
-            q=SetImagePixels(image,0,y,image->columns,1);
+            q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
             for (x=0; x < (long) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
               color=(pixel >> red_shift) & red_mask;
-              q->red=ScaleShortToQuantum((color*65535L)/red_mask);
+              color=(color*65535UL)/red_mask;
+              q->red=ScaleShortToQuantum((unsigned short) color);
               color=(pixel >> green_shift) & green_mask;
-              q->green=ScaleShortToQuantum((color*65535L)/green_mask);
+              color=(color*65535UL)/green_mask;
+              q->green=ScaleShortToQuantum((unsigned short) color);
               color=(pixel >> blue_shift) & blue_mask;
-              q->blue=ScaleShortToQuantum((color*65535L)/blue_mask);
+              color=(color*65535UL)/blue_mask;
+              q->blue=ScaleShortToQuantum((unsigned short) color);
               q++;
             }
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
+              break;
+            status=SetImageProgress(image,LoadImageTag,y,image->rows);
+            if (status == MagickFalse)
               break;
-            if (QuantumTick(y,image->rows) != MagickFalse)
-              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                  (QuantumTick(y,image->rows) != MagickFalse))
-                {
-                  status=image->progress_monitor(LoadImageTag,y,image->rows,
-                    image->client_data);
-                  if (status == MagickFalse)
-                    break;
-                }
           }
         break;
       }
@@ -436,7 +454,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Convert X image to PseudoClass packets.
         */
-        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+        if (AcquireImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i < (long) image->colors; i++)
         {
@@ -446,28 +464,22 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
+          indexes=GetAuthenticIndexQueue(image);
           for (x=0; x < (long) image->columns; x++)
           {
             index=ConstrainColormapIndex(image,XGetPixel(ximage,(int) x,
               (int) y));
             indexes[x]=index;
-            *q++=image->colormap[index];
+            *q++=image->colormap[(long) index];
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
+            break;
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          if (status == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows) != MagickFalse)
-            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(y,image->rows) != MagickFalse))
-              {
-                status=image->progress_monitor(LoadImageTag,y,image->rows,
-                  image->client_data);
-                if (status == MagickFalse)
-                  break;
-              }
         }
         break;
       }
@@ -477,12 +489,12 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   if (header.ncolors != 0)
     colors=(XColor *) RelinquishMagickMemory(colors);
-  ximage->data=(char *) RelinquishMagickMemory(ximage->data);
+  ximage->data=DestroyString(ximage->data);
   ximage=(XImage *) RelinquishMagickMemory(ximage);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 #endif
@@ -498,8 +510,8 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterXWDImage() adds attributes for the XWD image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterXWDImage() adds properties for the XWD image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -507,24 +519,25 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterXWDImage method is:
 %
-%      RegisterXWDImage(void)
+%      unsigned long RegisterXWDImage(void)
 %
 */
-ModuleExport void RegisterXWDImage(void)
+ModuleExport unsigned long RegisterXWDImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("XWD");
-#if defined(HasX11)
-  entry->decoder=(DecoderHandler *) ReadXWDImage;
-  entry->encoder=(EncoderHandler *) WriteXWDImage;
+#if defined(MAGICKCORE_X11_DELEGATE)
+  entry->decoder=(DecodeImageHandler *) ReadXWDImage;
+  entry->encoder=(EncodeImageHandler *) WriteXWDImage;
 #endif
-  entry->magick=(MagickHandler *) IsXWD;
+  entry->magick=(IsImageFormatHandler *) IsXWD;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("X Windows system window dump (color)");
-  entry->module=AcquireString("XWD");
+  entry->description=ConstantString("X Windows system window dump (color)");
+  entry->module=ConstantString("XWD");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -551,7 +564,7 @@ ModuleExport void UnregisterXWDImage(void)
   (void) UnregisterMagickInfo("XWD");
 }
 
-#if defined(HasX11)
+#if defined(MAGICKCORE_X11_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -572,16 +585,15 @@ ModuleExport void UnregisterXWDImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
 {
-  const ImageAttribute
-    *attribute;
+  const char
+    *value;
 
   long
     y;
@@ -589,12 +601,12 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -631,14 +643,16 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) TransformImageColorspace(image,RGBColorspace);
   /*
     Initialize XWD file header.
   */
+  (void) ResetMagickMemory(&xwd_info,0,sizeof(xwd_info));
   xwd_info.header_size=(CARD32) sz_XWDheader;
-  attribute=GetImageAttribute(image,"Comment");
-  if (attribute != (const ImageAttribute *) NULL)
-    xwd_info.header_size+=strlen(attribute->value);
+  value=GetImageProperty(image,"comment");
+  if (value != (const char *) NULL)
+    xwd_info.header_size+=(CARD32) strlen(value);
   xwd_info.header_size++;
   xwd_info.file_version=(CARD32) XWD_FILE_VERSION;
   xwd_info.pixmap_format=(CARD32) ZPixmap;
@@ -680,9 +694,8 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
   if ((int) (*(char *) &lsb_first) != 0)
     MSBOrderLong((unsigned char *) &xwd_info,sizeof(xwd_info));
   (void) WriteBlob(image,sz_XWDheader,(unsigned char *) &xwd_info);
-  if (attribute != (const ImageAttribute *) NULL)
-    (void) WriteBlob(image,strlen(attribute->value),(unsigned char *)
-      attribute->value);
+  if (value != (const char *) NULL)
+    (void) WriteBlob(image,strlen(value),(unsigned char *) value);
   (void) WriteBlob(image,1,(const unsigned char *) "\0");
   if (image->storage_class == PseudoClass)
     {
@@ -695,8 +708,9 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
       /*
         Dump colormap to file.
       */
-      colors=(XColor *)
-        AcquireMagickMemory((size_t) image->colors*sizeof(*colors));
+      (void) ResetMagickMemory(&color,0,sizeof(color));
+      colors=(XColor *) AcquireQuantumMemory((size_t) image->colors,
+        sizeof(*colors));
       if (colors == (XColor *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (long) image->colors; i++)
@@ -732,28 +746,28 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
   length=3*bytes_per_line;
   if (image->storage_class == PseudoClass)
     length=bytes_per_line;
-  pixels=(unsigned char *) AcquireMagickMemory(length);
+  pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  ResetMagickMemory(pixels,0,length);
+  (void) ResetMagickMemory(pixels,0,length);
   /*
     Convert MIFF to XWD raster pixels.
   */
   scanline_pad=(bytes_per_line-((image->columns*bits_per_pixel) >> 3));
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
     q=pixels;
     if (image->storage_class == PseudoClass)
       {
-        indexes=GetIndexes(image);
+        indexes=GetVirtualIndexQueue(image);
         for (x=0; x < (long) image->columns; x++)
           *q++=(unsigned char) indexes[x];
       }
     else
-      for (x=(long) image->columns-1; x >= 0; x--)
+      for (x=0; x < (long) image->columns; x++)
       {
         *q++=ScaleQuantumToChar(p->red);
         *q++=ScaleQuantumToChar(p->green);
@@ -763,18 +777,12 @@ static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
     for (x=0; x < (long) scanline_pad; x++)
       *q++='\0';
     (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-    if (image->previous == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(SaveImageTag,y,image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
 #endif
