@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,10 +13,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image annotate methods.
+  MagickCore image annotation methods.
 */
-#ifndef _MAGICK_ANNOTATE_H
-#define _MAGICK_ANNOTATE_H
+#ifndef _MAGICKCORE_ANNOTATE_H
+#define _MAGICKCORE_ANNOTATE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -24,6 +24,9 @@ extern "C" {
 
 #include "magick/draw.h"
 
+extern MagickExport long
+  FormatMagickCaption(Image *,DrawInfo *,TypeMetric *,char **);
+
 extern MagickExport MagickBooleanType
   AnnotateImage(Image *,const DrawInfo *),
   GetMultilineTypeMetrics(Image *,const DrawInfo *,TypeMetric *),
