@@ -571,18 +571,12 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   QuantumType
     quantum_type;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
+  SignatureInfo
+    *signature_info;
 
   register unsigned char
     *p;
 
-  SignatureInfo
-    *signature_info;
-
   size_t
     length;
 
@@ -595,7 +589,7 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
     output_block[AESBlocksize],
     *pixels;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   /*
@@ -653,6 +647,12 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
+    register long
+      x;
+
+    register PixelPacket
+      *__restrict q;
+
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
@@ -773,12 +773,6 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   QuantumType
     quantum_type;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
   register unsigned char
     *p;
 
@@ -797,7 +791,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
     output_block[AESBlocksize],
     *pixels;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   /*
@@ -862,6 +856,12 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
+    register long
+      x;
+
+    register PixelPacket
+      *__restrict q;
+
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
