@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/property.h"
 #include "magick/cache.h"
+#include "magick/cache-private.h"
 #include "magick/cache-view.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
@@ -134,7 +135,7 @@ static inline void ConvertXYZToLab(const double X,const double Y,const double Z,
   x=X/0.9504559271;
   if (x > (216/24389.0))
     x=pow(x,1.0/3.0);
-  else 
+  else
     x=(7.787*x)+(16.0/116.0);
   y=Y/1.00000;
   if (y > (216/24389.0))
@@ -178,7 +179,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     *y_map,
     *z_map;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   assert(image != (Image *) NULL);
@@ -198,11 +199,12 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       break;
     default:
     {
-      (void) SetImageColorspace(image,image->colorspace);
+      (void) TransformImageColorspace(image,image->colorspace);
       break;
     }
   }
-  image->colorspace=colorspace;
+  if (SetImageColorspace(image,colorspace) == MagickFalse)
+    return(MagickFalse);
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
@@ -230,7 +232,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -283,13 +285,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           pixel;
 
         register IndexPacket
-          *indexes;
+          *__restrict indexes;
 
         register long
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -345,7 +347,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -401,7 +403,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -457,7 +459,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -516,7 +518,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -608,7 +610,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -837,24 +839,24 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       /*
         Initialize CIE XYZ tables (ITU-R 709 RGB):
 
-          X = 0.4124240*R+0.3575790*G+0.1804640*B
-          Y = 0.2126560*R+0.7151580*G+0.0721856*B
-          Z = 0.0193324*R+0.1191930*G+0.9504440*B
+          X = 0.4124564*R+0.3575761*G+0.1804375*B
+          Y = 0.2126729*R+0.7151522*G+0.0721750*B
+          Z = 0.0193339*R+0.1191920*G+0.9503041*B
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4)
 #endif
       for (i=0; i <= (long) MaxMap; i++)
       {
-        x_map[i].x=0.4124240f*(MagickRealType) i;
-        y_map[i].x=0.3575790f*(MagickRealType) i;
-        z_map[i].x=0.1804640f*(MagickRealType) i;
-        x_map[i].y=0.2126560f*(MagickRealType) i;
-        y_map[i].y=0.7151580f*(MagickRealType) i;
-        z_map[i].y=0.0721856f*(MagickRealType) i;
-        x_map[i].z=0.0193324f*(MagickRealType) i;
-        y_map[i].z=0.1191930f*(MagickRealType) i;
-        z_map[i].z=0.9504440f*(MagickRealType) i;
+        x_map[i].x=0.4124564f*(MagickRealType) i;
+        y_map[i].x=0.3575761f*(MagickRealType) i;
+        z_map[i].x=0.1804375f*(MagickRealType) i;
+        x_map[i].y=0.2126729f*(MagickRealType) i;
+        y_map[i].y=0.7151522f*(MagickRealType) i;
+        z_map[i].y=0.0721750f*(MagickRealType) i;
+        x_map[i].z=0.0193339f*(MagickRealType) i;
+        y_map[i].z=0.1191920f*(MagickRealType) i;
+        z_map[i].z=0.9503041f*(MagickRealType) i;
       }
       break;
     }
@@ -1016,7 +1018,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         register unsigned long
           blue,
@@ -1099,13 +1101,14 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       break;
     }
   }
-  image->colorspace=colorspace;
   /*
     Relinquish resources.
   */
   z_map=(TransformPacket *) RelinquishMagickMemory(z_map);
   y_map=(TransformPacket *) RelinquishMagickMemory(y_map);
   x_map=(TransformPacket *) RelinquishMagickMemory(x_map);
+  if (SetImageColorspace(image,colorspace) == MagickFalse)
+    return(MagickFalse);
   return(status);
 }
 
@@ -1136,6 +1139,45 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
 */
 MagickExport MagickBooleanType SetImageColorspace(Image *image,
   const ColorspaceType colorspace)
+{
+  Cache
+    cache;
+
+  if (image->colorspace == colorspace)
+    return(MagickTrue);
+  image->colorspace=colorspace;
+  cache=GetImagePixelCache(image,MagickTrue,&image->exception);
+  image->colorspace=colorspace;  /* GRAY colorspace might get reset to RGB */
+  return(cache == (Cache) NULL ? MagickFalse: MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   T r a n s f o r m I m a g e C o l o r s p a c e                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransformImageColorspace() transforms an image colorspace.
+%
+%  The format of the TransformImageColorspace method is:
+%
+%      MagickBooleanType TransformImageColorspace(Image *image,
+%        const ColorspaceType colorspace)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o colorspace: the colorspace.
+%
+*/
+MagickExport MagickBooleanType TransformImageColorspace(Image *image,
+  const ColorspaceType colorspace)
 {
   MagickBooleanType
     status;
@@ -1146,7 +1188,8 @@ MagickExport MagickBooleanType SetImageColorspace(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (colorspace == UndefinedColorspace)
     {
-      image->colorspace=UndefinedColorspace;
+      if (SetImageColorspace(image,colorspace) == MagickFalse)
+        return(MagickFalse);
       return(MagickTrue);
     }
   if (image->colorspace == colorspace)
@@ -1196,6 +1239,7 @@ static inline void ConvertLabToXYZ(const double L,const double a,const double b,
   double *X,double *Y,double *Z)
 {
   double
+    cube,
     x,
     y,
     z;
@@ -1203,19 +1247,22 @@ static inline void ConvertLabToXYZ(const double L,const double a,const double b,
   assert(X != (double *) NULL);
   assert(Y != (double *) NULL);
   assert(Z != (double *) NULL);
-  y=((2.0*L-1.0)+0.160)/1.160;
-  x=(2.0*a-1.0)/5.000+y;
-  z=y-(2.0*b-1.0)/2.000;
-  if ((x*x*x) > (216.0/24389.0))
-    x=x*x*x;
-  else
-    x=(x-16.0/116.0)/7.787;
-  if ((y*y*y) > (216.0/24389.0))
-    y=y*y*y;
+  y=(100.0*L+16.0)/116.0;
+  x=255.0*(a > 0.5 ? a-1.0 : a)/500.0+y;
+  z=y-255.0*(b > 0.5 ? b-1.0 : b)/200.0;
+  cube=y*y*y;
+  if (cube > 0.008856)
+    y=cube;
   else
     y=(y-16.0/116.0)/7.787;
-  if ((z*z*z) > (216.0/24389.0))
-    z=z*z*z;
+  cube=x*x*x;
+  if (cube > 0.008856)
+    x=cube;
+  else
+    x=(x-16.0/116.0)/7.787;
+  cube=z*z*z;
+  if (cube > 0.008856)
+    z=cube;
   else
     z=(z-16.0/116.0)/7.787;
   *X=0.9504559271*x;
@@ -1246,9 +1293,21 @@ static inline void ConvertXYZToRGB(const double x,const double y,const double z,
   assert(red != (Quantum *) NULL);
   assert(green != (Quantum *) NULL);
   assert(blue != (Quantum *) NULL);
-  r=3.2407100*x-1.5372600*y-0.4985710*z;
-  g=(-0.9692580*x+1.8759900*y+0.0415557*z);
-  b=0.0556352*x-0.2039960*y+1.0570700*z;
+  r=3.2404542*x-1.5371385*y-0.4985314*z;
+  g=(-0.9692660*x+1.8760108*y+0.0415560*z);
+  b=0.0556434*x-0.2040259*y+1.0572252*z;
+  if (r > 0.0031308)
+    r=1.055*pow(r,1.0/2.4)-0.055;
+  else
+    r*=12.92;
+  if (g > 0.0031308)
+    g=1.055*pow(g,1.0/2.4)-0.055;
+  else
+    g*=12.92;
+  if (b > 0.0031308)
+    b=1.055*pow(b,1.0/2.4)-0.055;
+  else
+    b*=12.92;
   *red=RoundToQuantum((MagickRealType) QuantumRange*r);
   *green=RoundToQuantum((MagickRealType) QuantumRange*g);
   *blue=RoundToQuantum((MagickRealType) QuantumRange*b);
@@ -1323,7 +1382,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
     *x_map,
     *z_map;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   assert(image != (Image *) NULL);
@@ -1372,7 +1431,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1395,7 +1454,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case CMYKColorspace:
@@ -1427,13 +1487,13 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           pixel;
 
         register IndexPacket
-          *indexes;
+          *__restrict indexes;
 
         register long
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1458,7 +1518,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case HSBColorspace:
@@ -1491,7 +1552,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1515,7 +1576,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case HSLColorspace:
@@ -1548,7 +1610,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1572,7 +1634,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case HWBColorspace:
@@ -1605,7 +1668,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1629,7 +1692,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case LabColorspace:
@@ -1665,7 +1729,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1693,7 +1757,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case LogColorspace:
@@ -1757,7 +1822,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1781,7 +1846,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       }
       image_view=DestroyCacheView(image_view);
       logmap=(Quantum *) RelinquishMagickMemory(logmap);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     default:
@@ -1943,24 +2009,24 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       /*
         Initialize CIE XYZ tables (ITU R-709 RGB):
 
-          R =  3.2407100*X-1.5372600*Y-0.4985710*Z
-          G = -0.9692580*X+1.8759900*Y+0.0415557*Z
-          B =  0.0556352*X-0.2039960*Y+1.0570700*Z
+          R =  3.2404542*X-1.5371385*Y-0.4985314*Z
+          G = -0.9692660*X+1.8760108*Y+0.0415560*Z
+          B =  0.0556434*X-0.2040259*Y+1.057225*Z
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4)
 #endif
       for (i=0; i <= (long) MaxMap; i++)
       {
-        x_map[i].x=3.2407100f*(MagickRealType) i;
-        x_map[i].y=(-0.9692580f)*(MagickRealType) i;
-        x_map[i].z=0.0556352f*(MagickRealType) i;
-        y_map[i].x=(-1.5372600f)*(MagickRealType) i;
-        y_map[i].y=1.8759900f*(MagickRealType) i;
-        y_map[i].z=(-0.2039960f)*(MagickRealType) i;
-        z_map[i].x=(-0.4985710f)*(MagickRealType) i;
-        z_map[i].y=0.0415557f*(MagickRealType) i;
-        z_map[i].z=1.0570700f*(MagickRealType) i;
+        x_map[i].x=3.2404542f*(MagickRealType) i;
+        x_map[i].y=(-0.9692660f)*(MagickRealType) i;
+        x_map[i].z=0.0556434f*(MagickRealType) i;
+        y_map[i].x=(-1.5371385f)*(MagickRealType) i;
+        y_map[i].y=1.8760108f*(MagickRealType) i;
+        y_map[i].z=(-0.2040259f)*(MagickRealType) i;
+        z_map[i].x=(-0.4985314f)*(MagickRealType) i;
+        z_map[i].y=0.0415560f*(MagickRealType) i;
+        z_map[i].z=1.0572252f*(MagickRealType) i;
       }
       break;
     }
@@ -2126,7 +2192,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -2294,6 +2360,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
   z_map=(TransformPacket *) RelinquishMagickMemory(z_map);
   y_map=(TransformPacket *) RelinquishMagickMemory(y_map);
   x_map=(TransformPacket *) RelinquishMagickMemory(x_map);
-  image->colorspace=RGBColorspace;
+  if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+    return(MagickFalse);
   return(MagickTrue);
 }
