@@ -10,14 +10,14 @@
 %          CCCC   OOO   N   N  F      IIIII   GGG    UUU   R  R   EEEEE       %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Configure Methods                     %
+%                      MagickCore Image Configure Methods                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -52,21 +52,31 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
 
 /*
   Define declarations.
 */
 #define ConfigureFilename  "configure.xml"
 
+/*
+  Typedef declarations.
+*/
+typedef struct _ConfigureMapInfo
+{
+  const char
+    *name,
+    *value;
+} ConfigureMapInfo;
+
 /*
   Static declarations.
 */
-static const char
-  *ConfigureMap = (char *)
-    "<?xml version=\"1.0\"?>"
-    "<configuremap>"
-    "  <configure stealth=\"True\" />"
-    "</configuremap>";
+static const ConfigureMapInfo
+  ConfigureMap[] =
+  {
+    { "NAME", "ImageMagick" }
+  };
 
 static LinkedListInfo
   *configure_list = (LinkedListInfo *) NULL;
@@ -89,18 +99,41 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   D e s t r o y C o n f i g u r e L i s t                                   %
++   C o n f i g u r e C o m p o n e n t G e n e s i s                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DestroyConfigureList() deallocates memory associated with the configure list.
+%  ConfigureComponentGenesis() instantiates the configure component.
 %
-%  The format of the DestroyConfigureList method is:
+%  The format of the ConfigureComponentGenesis method is:
 %
-%      DestroyConfigureList(void)
+%      MagickBooleanType ConfigureComponentGenesis(void)
 %
+*/
+MagickExport MagickBooleanType ConfigureComponentGenesis(void)
+{
+  AcquireSemaphoreInfo(&configure_semaphore);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   C o n f i g u r e C o m p o n e n t T e r m i n u s                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConfigureComponentTerminus() destroys the configure component.
+%
+%  The format of the ConfigureComponentTerminus method is:
+%
+%      ConfigureComponentTerminus(void)
 %
 */
 
@@ -110,25 +143,30 @@ static void *DestroyConfigureElement(void *configure_info)
     *p;
 
   p=(ConfigureInfo *) configure_info;
-  if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
-  if (p->name != (char *) NULL)
-    p->name=(char *) RelinquishMagickMemory(p->name);
-  if (p->value != (char *) NULL)
-    p->value=(char *) RelinquishMagickMemory(p->value);
+  if (p->exempt == MagickFalse)
+    {
+      if (p->value != (char *) NULL)
+        p->value=DestroyString(p->value);
+      if (p->name != (char *) NULL)
+        p->name=DestroyString(p->name);
+      if (p->path != (char *) NULL)
+        p->path=DestroyString(p->path);
+    }
   p=(ConfigureInfo *) RelinquishMagickMemory(p);
   return((void *) NULL);
 }
 
-MagickExport void DestroyConfigureList(void)
+MagickExport void ConfigureComponentTerminus(void)
 {
-  AcquireSemaphoreInfo(&configure_semaphore);
+  if (configure_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&configure_semaphore);
+  (void) LockSemaphoreInfo(configure_semaphore);
   if (configure_list != (LinkedListInfo *) NULL)
     configure_list=DestroyLinkedList(configure_list,DestroyConfigureElement);
   configure_list=(LinkedListInfo *) NULL;
   instantiate_configure=MagickFalse;
-  RelinquishSemaphoreInfo(configure_semaphore);
-  configure_semaphore=DestroySemaphoreInfo(configure_semaphore);
+  (void) UnlockSemaphoreInfo(configure_semaphore);
+  DestroySemaphoreInfo(&configure_semaphore);
 }
 
 /*
@@ -151,8 +189,7 @@ MagickExport void DestroyConfigureList(void)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
-%
+%    o image: the image.
 %
 */
 
@@ -192,10 +229,9 @@ MagickExport LinkedListInfo *DestroyConfigureOptions(LinkedListInfo *options)
 %    o configure_info: GetConfigureInfo() searches the configure list for the
 %      specified name and if found returns attributes for that element.
 %
-%    o name: The configure name.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o name: the configure name.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
@@ -215,9 +251,9 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
     return((const ConfigureInfo *) GetValueFromLinkedList(configure_list,0));
   /*
-    Search for named configure.
+    Search for configure tag.
   */
-  AcquireSemaphoreInfo(&configure_semaphore);
+  (void) LockSemaphoreInfo(configure_semaphore);
   ResetLinkedListIterator(configure_list);
   p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   while (p != (const ConfigureInfo *) NULL)
@@ -226,10 +262,10 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
       break;
     p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   }
-  if (p == (ConfigureInfo *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
-      "NoSuchElement","`%s'",name);
-  RelinquishSemaphoreInfo(configure_semaphore);
+  if (p != (ConfigureInfo *) NULL)
+    (void) InsertValueInLinkedList(configure_list,0,
+      RemoveElementByValueFromLinkedList(configure_list,p));
+  (void) UnlockSemaphoreInfo(configure_semaphore);
   return(p);
 }
 
@@ -259,7 +295,7 @@ MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
 %    o number_options:  This integer returns the number of configure options in
 %    the list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -306,24 +342,24 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
   p=GetConfigureInfo("*",exception);
   if (p == (const ConfigureInfo *) NULL)
     return((const ConfigureInfo **) NULL);
-  options=(const ConfigureInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(configure_list)+1)*sizeof(*options));
+  options=(const ConfigureInfo **) AcquireQuantumMemory((size_t)
+    GetNumberOfElementsInLinkedList(configure_list)+1UL,sizeof(*options));
   if (options == (const ConfigureInfo **) NULL)
     return((const ConfigureInfo **) NULL);
   /*
     Generate configure list.
   */
-  AcquireSemaphoreInfo(&configure_semaphore);
+  (void) LockSemaphoreInfo(configure_semaphore);
   ResetLinkedListIterator(configure_list);
   p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   for (i=0; p != (const ConfigureInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
       options[i++]=p;
     p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   }
-  RelinquishSemaphoreInfo(configure_semaphore);
+  (void) UnlockSemaphoreInfo(configure_semaphore);
   qsort((void *) options,(size_t) i,sizeof(*options),ConfigureInfoCompare);
   options[i]=(ConfigureInfo *) NULL;
   *number_options=(unsigned long) i;
@@ -355,8 +391,7 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
 %
 %    o number_options:  This integer returns the number of options in the list.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -401,23 +436,21 @@ MagickExport char **GetConfigureList(const char *pattern,
   p=GetConfigureInfo("*",exception);
   if (p == (const ConfigureInfo *) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&configure_semaphore);
-  RelinquishSemaphoreInfo(configure_semaphore);
-  options=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(configure_list)+1)*sizeof(*options));
+  options=(char **) AcquireQuantumMemory((size_t)
+    GetNumberOfElementsInLinkedList(configure_list)+1UL,sizeof(*options));
   if (options == (char **) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&configure_semaphore);
+  (void) LockSemaphoreInfo(configure_semaphore);
   ResetLinkedListIterator(configure_list);
   p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   for (i=0; p != (const ConfigureInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
-      options[i++]=ConstantString(AcquireString(p->name));
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
+      options[i++]=ConstantString(p->name);
     p=(const ConfigureInfo *) GetNextValueInLinkedList(configure_list);
   }
-  RelinquishSemaphoreInfo(configure_semaphore);
+  (void) UnlockSemaphoreInfo(configure_semaphore);
   qsort((void *) options,(size_t) i,sizeof(*options),ConfigureCompare);
   options[i]=(char *) NULL;
   *number_options=(unsigned long) i;
@@ -429,6 +462,52 @@ MagickExport char **GetConfigureList(const char *pattern,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t C o n f i g u r e O p t i o n                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetConfigureOption() returns the value associated with the configure option.
+%
+%  The format of the GetConfigureOption method is:
+%
+%      char *GetConfigureOption(const char *option)
+%
+%  A description of each parameter follows:
+%
+%    o configure_info:  The configure info.
+%
+*/
+MagickExport char *GetConfigureOption(const char *option)
+{
+  const char
+    *value;
+
+  const ConfigureInfo
+    *configure_info;
+
+  ExceptionInfo
+    *exception;
+
+  assert(option != (const char *) NULL);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",option);
+  exception=AcquireExceptionInfo();
+  configure_info=GetConfigureInfo(option,exception);
+  exception=DestroyExceptionInfo(exception);
+  if (configure_info == (ConfigureInfo *) NULL)
+    return((char *) NULL);
+  value=GetConfigureValue(configure_info);
+  if ((value == (const char *) NULL) || (*value == '\0'))
+    return((char *) NULL);
+  return(ConstantString(value));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  G e t C o n f i g u r e O p t i o n s                                      %
 %                                                                             %
 %                                                                             %
@@ -445,10 +524,9 @@ MagickExport char **GetConfigureList(const char *pattern,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The configure file name.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o filename: the configure file name.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport LinkedListInfo *GetConfigureOptions(const char *filename,
@@ -501,9 +579,9 @@ MagickExport LinkedListInfo *GetConfigureOptions(const char *filename,
     if (blob != (char *) NULL)
       {
         xml=StringToStringInfo(blob);
-        (void) CopyMagickString(xml->path,filename,MaxTextExtent);
+        SetStringInfoPath(xml,filename);
         (void) AppendValueToLinkedList(options,xml);
-        blob=(char *) RelinquishMagickMemory(blob);
+        blob=DestroyString(blob);
       }
   }
 #endif
@@ -535,10 +613,9 @@ MagickExport LinkedListInfo *GetConfigureOptions(const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The configure file name.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o filename: the configure file name.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
@@ -556,51 +633,57 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
   (void) CopyMagickString(path,filename,MaxTextExtent);
   paths=NewLinkedList(0);
   {
-    const char
+    char
       *configure_path;
 
-    register char
-      *q;
-
-    register const char
-      *p;
-
     /*
       Search $MAGICK_CONFIGURE_PATH.
     */
-    configure_path=getenv("MAGICK_CONFIGURE_PATH");
-    if (configure_path != (const char *) NULL)
-      for (p=configure_path-1; p != (char *) NULL; )
+    configure_path=GetEnvironmentValue("MAGICK_CONFIGURE_PATH");
+    if (configure_path != (char *) NULL)
       {
-        (void) CopyMagickString(path,p+1,MaxTextExtent);
-        q=strchr(path,DirectoryListSeparator);
-        if (q != (char *) NULL)
-          *q='\0';
-        q=path+strlen(path)-1;
-        if ((q >= path) && (*q != *DirectorySeparator))
-          (void) ConcatenateMagickString(path,DirectorySeparator,
-            MaxTextExtent);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
-        p=strchr(p+1,DirectoryListSeparator);
+        register char
+          *p,
+          *q;
+
+        for (p=configure_path-1; p != (char *) NULL; )
+        {
+          (void) CopyMagickString(path,p+1,MaxTextExtent);
+          q=strchr(path,DirectoryListSeparator);
+          if (q != (char *) NULL)
+            *q='\0';
+          q=path+strlen(path)-1;
+          if ((q >= path) && (*q != *DirectorySeparator))
+            (void) ConcatenateMagickString(path,DirectorySeparator,
+              MaxTextExtent);
+          (void) AppendValueToLinkedList(paths,ConstantString(path));
+          p=strchr(p+1,DirectoryListSeparator);
+        }
+        configure_path=DestroyString(configure_path);
       }
   }
-#if defined(UseInstalledMagick)
-#if defined(MagickLibConfigPath)
-  (void) AppendValueToLinkedList(paths,AcquireString(MagickLibConfigPath));
+#if defined(MAGICKCORE_INSTALLED_SUPPORT)
+#if defined(MAGICKCORE_SHARE_CONFIGURE_PATH)
+  (void) AppendValueToLinkedList(paths,ConstantString(
+    MAGICKCORE_SHARE_CONFIGURE_PATH));
 #endif
-#if defined(MagickShareConfigPath)
-  (void) AppendValueToLinkedList(paths,AcquireString(MagickShareConfigPath));
+#if defined(MAGICKCORE_CONFIGURE_PATH)
+  (void) AppendValueToLinkedList(paths,ConstantString(
+    MAGICKCORE_CONFIGURE_PATH));
 #endif
-#if defined(MagickDocumentPath)
-  (void) AppendValueToLinkedList(paths,AcquireString(MagickDocumentPath));
+#if defined(MAGICKCORE_DOCUMENTATION_PATH)
+  (void) AppendValueToLinkedList(paths,ConstantString(
+    MAGICKCORE_DOCUMENTATION_PATH));
 #endif
-#if defined(MagickSharePath)
-  (void) AppendValueToLinkedList(paths,AcquireString(MagickSharePath));
+#if defined(MAGICKCORE_SHARE_PATH)
+  (void) AppendValueToLinkedList(paths,ConstantString(MAGICKCORE_SHARE_PATH));
 #endif
-#if defined(__WINDOWS__) && !(defined(MagickLibConfigPath) || defined(MagickShareConfigPath))
+#if defined(__WINDOWS__) && !(defined(MAGICKCORE_CONFIGURE_PATH) || defined(MAGICKCORE_SHARE_CONFIGURE_PATH))
   {
     char
-      *registry_key,
+      *registry_key;
+
+    unsigned char
       *key_value;
 
     /*
@@ -608,45 +691,47 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
     */
     registry_key="ConfigurePath";
     key_value=NTRegistryKeyLookup(registry_key);
-    if (key_value != (char *) NULL)
+    if (key_value != (unsigned char *) NULL)
       {
-        (void) FormatMagickString(path,MaxTextExtent,"%s%s",key_value,
+        (void) FormatMagickString(path,MaxTextExtent,"%s%s",(char *) key_value,
           DirectorySeparator);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
+        key_value=(unsigned char *) RelinquishMagickMemory(key_value);
       }
   }
 #endif
 #else
   {
-    const char
+    char
       *home;
 
     /*
       Search under MAGICK_HOME.
     */
-    home=getenv("MAGICK_HOME");
-    if (home != (const char *) NULL)
+    home=GetEnvironmentValue("MAGICK_HOME");
+    if (home != (char *) NULL)
       {
-#if !defined(POSIX)
+#if !defined(MAGICKCORE_POSIX_SUPPORT)
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",home,
           DirectorySeparator);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
 #else
         (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/",home,
-          MagickLibConfigSubDir);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+          MAGICKCORE_CONFIGURE_RELATIVE_PATH);
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
         (void) FormatMagickString(path,MaxTextExtent,"%s/share/%s/",home,
-          MagickShareConfigSubDir);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+          MAGICKCORE_SHARE_CONFIGURE_RELATIVE_PATH);
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
 #endif
+        home=DestroyString(home);
       }
     }
   if (*GetClientPath() != '\0')
     {
-#if !defined(POSIX)
+#if !defined(MAGICKCORE_POSIX_SUPPORT)
       (void) FormatMagickString(path,MaxTextExtent,"%s%s",GetClientPath(),
         DirectorySeparator);
-      (void) AppendValueToLinkedList(paths,AcquireString(path));
+      (void) AppendValueToLinkedList(paths,ConstantString(path));
 #else
       char
         prefix[MaxTextExtent];
@@ -656,28 +741,31 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
       */
       (void) CopyMagickString(prefix,GetClientPath(),MaxTextExtent);
       ChopPathComponents(prefix,1);
-      (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/",prefix,
-        MagickLibConfigSubDir);
-      (void) AppendValueToLinkedList(paths,AcquireString(path));
       (void) FormatMagickString(path,MaxTextExtent,"%s/share/%s/",prefix,
-        MagickShareConfigSubDir);
-      (void) AppendValueToLinkedList(paths,AcquireString(path));
+        MAGICKCORE_SHARE_CONFIGURE_RELATIVE_PATH);
+      (void) AppendValueToLinkedList(paths,ConstantString(path));
+      (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/",prefix,
+        MAGICKCORE_CONFIGURE_RELATIVE_PATH);
+      (void) AppendValueToLinkedList(paths,ConstantString(path));
 #endif
     }
 #endif
   {
-    const char
+    char
       *home;
 
-    home=getenv("HOME");
-    if (home != (const char *) NULL)
+    home=GetEnvironmentValue("HOME");
+    if (home == (char *) NULL)
+      home=GetEnvironmentValue("USERPROFILE");
+    if (home != (char *) NULL)
       {
         /*
           Search $HOME/.magick.
         */
-        (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",home,
-          *home == '/' ? "/.magick" : "",DirectorySeparator);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+        (void) FormatMagickString(path,MaxTextExtent,"%s%s.magick%s",home,
+          DirectorySeparator,DirectorySeparator);
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
+        home=DestroyString(home);
       }
   }
 #if defined(__WINDOWS__)
@@ -685,7 +773,8 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
     char
       module_path[MaxTextExtent];
 
-    if (NTGetModulePath("CORE_RL_magick_.dll",module_path) != MagickFalse)
+    if ((NTGetModulePath("CORE_RL_magick_.dll",module_path) != MagickFalse) ||
+        (NTGetModulePath("CORE_DB_magick_.dll",module_path) != MagickFalse))
       {
         char
           *element;
@@ -697,8 +786,8 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
           DirectorySeparator);
         element=(char *) RemoveElementByValueFromLinkedList(paths,path);
         if (element != (char *) NULL)
-          element=(char *) RelinquishMagickMemory(element);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+          element=DestroyString(element);
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
       }
     if (NTGetModulePath("Magick.dll",module_path) != MagickFalse)
       {
@@ -707,17 +796,17 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
         */
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",module_path,
           DirectorySeparator);
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",module_path,
           "\\inc\\lib\\auto\\Image\\Magick\\");
-        (void) AppendValueToLinkedList(paths,AcquireString(path));
+        (void) AppendValueToLinkedList(paths,ConstantString(path));
       }
   }
 #endif
   /*
     Search current directory.
   */
-  (void) AppendValueToLinkedList(paths,AcquireString(""));
+  (void) AppendValueToLinkedList(paths,ConstantString(""));
   return(paths);
 }
 
@@ -770,7 +859,7 @@ MagickExport const char *GetConfigureValue(const ConfigureInfo *configure_info)
 %
 %  A description of each parameter follows.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType InitializeConfigureList(ExceptionInfo *exception)
@@ -778,14 +867,16 @@ static MagickBooleanType InitializeConfigureList(ExceptionInfo *exception)
   if ((configure_list == (LinkedListInfo *) NULL) &&
       (instantiate_configure == MagickFalse))
     {
-      AcquireSemaphoreInfo(&configure_semaphore);
+      if (configure_semaphore == (SemaphoreInfo *) NULL)
+        AcquireSemaphoreInfo(&configure_semaphore);
+      (void) LockSemaphoreInfo(configure_semaphore);
       if ((configure_list == (LinkedListInfo *) NULL) &&
           (instantiate_configure == MagickFalse))
         {
           (void) LoadConfigureLists(ConfigureFilename,exception);
           instantiate_configure=MagickTrue;
         }
-      RelinquishSemaphoreInfo(configure_semaphore);
+      (void) UnlockSemaphoreInfo(configure_semaphore);
     }
   return(configure_list != (LinkedListInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -811,8 +902,7 @@ static MagickBooleanType InitializeConfigureList(ExceptionInfo *exception)
 %
 %    o file:  An pointer to a FILE.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
@@ -901,8 +991,7 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
 %
 %    o depth: depth of <include /> statements.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
@@ -910,13 +999,15 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
 {
   char
     keyword[MaxTextExtent],
-    *q,
     *token;
 
   ConfigureInfo
-    *configure_info = (ConfigureInfo *) NULL;
+    *configure_info;
 
-  MagickStatusType
+  const char
+    *q;
+
+  MagickBooleanType
     status;
 
   /*
@@ -935,8 +1026,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
-  if (xml == (char *) NULL)
-    xml=AcquireString(ConfigureMap);
+  configure_info=(ConfigureInfo *) NULL;
   token=AcquireString((char *) xml);
   for (q=(char *) xml; *q != '\0'; )
   {
@@ -992,10 +1082,16 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
                   if (*path != '\0')
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  if (*token == *DirectorySeparator)
+                    (void) CopyMagickString(path,token,MaxTextExtent);
+                  else
+                    (void) ConcatenateMagickString(path,token,MaxTextExtent);
                   xml=FileToString(path,~0,exception);
-                  status|=LoadConfigureList(xml,path,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadConfigureList(xml,path,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
                 }
             }
         }
@@ -1004,15 +1100,15 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
     if (LocaleCompare(keyword,"<configure") == 0)
       {
         /*
-          Allocate memory for the configure list.
+          Configure element.
         */
-        configure_info=(ConfigureInfo *)
-          AcquireMagickMemory(sizeof(*configure_info));
+        configure_info=(ConfigureInfo *) AcquireMagickMemory(
+          sizeof(*configure_info));
         if (configure_info == (ConfigureInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) ResetMagickMemory(configure_info,0,sizeof(*configure_info));
-        configure_info->path=ConstantString(AcquireString(filename));
+        configure_info->path=ConstantString(filename);
+        configure_info->exempt=MagickFalse;
         configure_info->signature=MagickSignature;
         continue;
       }
@@ -1027,7 +1123,10 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
             configure_info->name);
         configure_info=(ConfigureInfo *) NULL;
       }
-    GetMagickToken(q,(char **) NULL,token);
+    /*
+      Parse configure element.
+    */
+    GetMagickToken(q,(const char **) NULL,token);
     if (*token != '=')
       continue;
     GetMagickToken(q,&q,token);
@@ -1039,7 +1138,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            configure_info->name=ConstantString(AcquireString(token));
+            configure_info->name=ConstantString(token);
             break;
           }
         break;
@@ -1049,8 +1148,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"stealth") == 0)
           {
-            configure_info->stealth=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
+            configure_info->stealth=IsMagickTrue(token);
             break;
           }
         break;
@@ -1060,7 +1158,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"value") == 0)
           {
-            configure_info->value=ConstantString(AcquireString(token));
+            configure_info->value=ConstantString(token);
             break;
           }
         break;
@@ -1070,9 +1168,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
     }
   }
   token=(char *) RelinquishMagickMemory(token);
-  if (configure_list == (LinkedListInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  return(status);
 }
 
 /*
@@ -1086,7 +1182,7 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  LoadConfigureList() loads one or more configure configuration file which
+%  LoadConfigureList() loads one or more configure configuration files which
 %  provides a mapping between configure attributes and a configure name.
 %
 %  The format of the LoadConfigureLists method is:
@@ -1096,18 +1192,14 @@ static MagickBooleanType LoadConfigureList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o filename: the font file name.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType LoadConfigureLists(const char *filename,
   ExceptionInfo *exception)
 {
-#if defined(UseEmbeddableMagick)
-  return(LoadConfigureList(ConfigureMap,"built-in",0,exception));
-#else
   const StringInfo
     *option;
 
@@ -1117,21 +1209,64 @@ static MagickBooleanType LoadConfigureLists(const char *filename,
   MagickStatusType
     status;
 
+  register long
+    i;
+
+  /*
+    Load built-in configure map.
+  */
   status=MagickFalse;
+  if (configure_list == (LinkedListInfo *) NULL)
+    {
+      configure_list=NewLinkedList(0);
+      if (configure_list == (LinkedListInfo *) NULL)
+        {
+          ThrowFileException(exception,ResourceLimitError,
+            "MemoryAllocationFailed",filename);
+          return(MagickFalse);
+        }
+    }
+  for (i=0; i < (long) (sizeof(ConfigureMap)/sizeof(*ConfigureMap)); i++)
+  {
+    ConfigureInfo
+      *configure_info;
+
+    register const ConfigureMapInfo
+      *p;
+
+    p=ConfigureMap+i;
+    configure_info=(ConfigureInfo *) AcquireMagickMemory(
+      sizeof(*configure_info));
+    if (configure_info == (ConfigureInfo *) NULL)
+      {
+        (void) ThrowMagickException(exception,GetMagickModule(),
+          ResourceLimitError,"MemoryAllocationFailed","`%s'",
+          configure_info->name);
+        continue;
+      }
+    (void) ResetMagickMemory(configure_info,0,sizeof(*configure_info));
+    configure_info->path=(char *) "[built-in]";
+    configure_info->name=(char *) p->name;
+    configure_info->value=(char *) p->value;
+    configure_info->exempt=MagickTrue;
+    configure_info->signature=MagickSignature;
+    status=AppendValueToLinkedList(configure_list,configure_info);
+    if (status == MagickFalse)
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",
+        configure_info->name);
+  }
+  /*
+    Load external configure map.
+  */
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    status|=LoadConfigureList((const char *) option->datum,option->path,0,
-      exception);
+    status|=LoadConfigureList((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
-  if ((configure_list == (LinkedListInfo *) NULL) ||
-      (IsLinkedListEmpty(configure_list) != MagickFalse))
-    status|=LoadConfigureList(ConfigureMap,"built-in",0,exception);
-  else
-    (void) SetExceptionInfo(exception,UndefinedException);
   return(status != 0 ? MagickTrue : MagickFalse);
-#endif
 }
