@@ -54,6 +54,7 @@
 #include "magick/monitor-private.h"
 #include "magick/pixel-private.h"
 #include "magick/quantum.h"
+#include "magick/transform.h"
 
 /*
   Define declarations.
@@ -181,21 +182,16 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     shadow,
     trough;
 
-  register IndexPacket
-    *frame_indexes;
-
   register long
     x;
 
-  register PixelPacket
-    *q;
-
   unsigned long
     bevel_width,
     height,
     width;
 
-  ViewInfo
+  CacheView
+    *image_view,
     *frame_view;
 
   /*
@@ -286,11 +282,21 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     }
   status=MagickTrue;
   progress=0;
+  image_view=AcquireCacheView(image);
   frame_view=AcquireCacheView(frame_image);
   height=(unsigned long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
     frame_info->inner_bevel);
   if (height != 0)
     {
+      register IndexPacket
+        *__restrict frame_indexes;
+
+      register long
+        x;
+
+      register PixelPacket
+        *__restrict q;
+
       /*
         Draw top of ornamental border.
       */
@@ -399,13 +405,13 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *frame_indexes;
+      *__restrict frame_indexes;
 
     register long
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     /*
       Initialize scanline with matte color.
@@ -441,12 +447,39 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     /*
       Set frame interior to interior color.
     */
-    for (x=0; x < (long) image->columns; x++)
-    {
-      SetPixelPacket(frame_image,&interior,q,frame_indexes);
-      q++;
-      frame_indexes++;
-    }
+    if ((image->compose != CopyCompositeOp) &&
+        ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
+      for (x=0; x < (long) image->columns; x++)
+      {
+        SetPixelPacket(frame_image,&interior,q,frame_indexes);
+        q++;
+        frame_indexes++;
+      }
+    else
+      {
+        register const IndexPacket
+          *indexes;
+
+        register const PixelPacket
+          *p;
+
+        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+        if (p == (const PixelPacket *) NULL)
+          {
+            status=MagickFalse;
+            continue;
+          }
+        indexes=GetCacheViewVirtualIndexQueue(image_view);
+        (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
+        if ((image->colorspace == CMYKColorspace) &&
+            (frame_image->colorspace == CMYKColorspace))
+          {
+            (void) CopyMagickMemory(frame_indexes,indexes,image->columns*
+              sizeof(*indexes));
+            frame_indexes+=image->columns;
+          }
+        q+=image->columns;
+      }
     for (x=0; x < (long) frame_info->inner_bevel; x++)
     {
       SetPixelPacket(frame_image,&highlight,q,frame_indexes);
@@ -485,6 +518,15 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     frame_info->y-image->rows-bevel_width+frame_info->outer_bevel);
   if (height != 0)
     {
+      register IndexPacket
+        *__restrict frame_indexes;
+
+      register long
+        x;
+
+      register PixelPacket
+        *__restrict q;
+
       /*
         Draw bottom of ornamental border.
       */
@@ -584,11 +626,16 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
         }
     }
   frame_view=DestroyCacheView(frame_view);
-  x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
-    frame_info->inner_bevel);
-  y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
-    frame_info->inner_bevel);
-  (void) CompositeImage(frame_image,image->compose,image,x,y);
+  image_view=DestroyCacheView(image_view);
+  if ((image->compose != CopyCompositeOp) &&
+      ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
+    {
+      x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
+        frame_info->inner_bevel);
+      y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
+        frame_info->inner_bevel);
+      (void) CompositeImage(frame_image,image->compose,image,x,y);
+    }
   return(frame_image);
 }
 
@@ -646,7 +693,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     foreground,
     background;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   assert(image != (Image *) NULL);
@@ -683,7 +730,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -750,7 +797,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -806,7 +853,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
