@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,15 +13,18 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick drawing methods.
+  MagickCore drawing methods.
 */
-#ifndef _MAGICK_DRAW_H
-#define _MAGICK_DRAW_H
+#ifndef _MAGICKCORE_DRAW_H
+#define _MAGICKCORE_DRAW_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
+#include "magick/geometry.h"
+#include "magick/image.h"
+#include "magick/pixel.h"
 #include "magick/type.h"
 
 typedef enum
@@ -120,23 +123,42 @@ typedef enum
 {
   UndefinedSpread,
   PadSpread,
-  ReflectSpead,
+  ReflectSpread,
   RepeatSpread
 } SpreadMethod;
 
+typedef struct _PointInfo
+{ 
+  double
+    x,
+    y;
+} PointInfo;
+
+typedef struct _StopInfo
+{
+  MagickPixelPacket
+    color;
+
+  MagickRealType
+    offset;
+} StopInfo;
+
 typedef struct _GradientInfo
 {
   GradientType
     type;
 
-  PixelPacket
-    color;
+  RectangleInfo
+    bounding_box;
 
   SegmentInfo
-    stop;
+    gradient_vector;
+
+  StopInfo
+    *stops;
 
   unsigned long
-    length;
+    number_stops;
 
   SpreadMethod
     spread;
@@ -147,9 +169,11 @@ typedef struct _GradientInfo
   unsigned long
     signature;
 
-  struct _GradientInfo
-    *previous,
-    *next;
+  PointInfo
+    center;
+
+  MagickRealType
+    radius;
 } GradientInfo;
 
 typedef struct _ElementReference
@@ -269,7 +293,7 @@ typedef struct _DrawInfo
     *dash_pattern;
 
   char
-    *clip_path;
+    *clip_mask;
 
   SegmentInfo
     bounds;
@@ -291,14 +315,12 @@ typedef struct _DrawInfo
 
   unsigned long
     signature;
-} DrawInfo;
 
-typedef struct _PointInfo
-{
   double
-    x,
-    y;
-} PointInfo;
+    kerning,
+    interword_spacing,
+    interline_spacing;
+} DrawInfo;
 
 typedef struct _PrimitiveInfo
 {
@@ -334,15 +356,20 @@ typedef struct _TypeMetric
 
   SegmentInfo
     bounds;
+
+  PointInfo
+    origin;
 } TypeMetric;
 
 extern MagickExport DrawInfo
+  *AcquireDrawInfo(void),
   *CloneDrawInfo(const ImageInfo *,const DrawInfo *),
   *DestroyDrawInfo(DrawInfo *);
 
 extern MagickExport MagickBooleanType
   DrawAffineImage(Image *,const Image *,const AffineMatrix *),
   DrawClipPath(Image *,const DrawInfo *,const char *),
+  DrawGradientImage(Image *,const DrawInfo *),
   DrawImage(Image *,const DrawInfo *),
   DrawPatternPath(Image *,const DrawInfo *,const char *,Image **),
   DrawPrimitive(Image *,const DrawInfo *,const PrimitiveInfo *);
