@@ -10,14 +10,14 @@
 %            GGGG  EEEEE   OOO   M   M  EEEEE    T    R  R     Y              %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick Geometry Methods                          %
+%                       MagickCore Geometry Methods                           %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,6 +47,7 @@
 #include "magick/geometry.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
+#include "magick/token.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -79,7 +80,6 @@
 %    o width,height:  The width and height as determined by the geometry
 %      specification.
 %
-%
 */
 MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
   unsigned long *width,unsigned long *height)
@@ -89,6 +89,9 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
     pedantic_geometry[MaxTextExtent],
     *q;
 
+  double
+    value;
+
   MagickStatusType
     flags;
 
@@ -134,6 +137,12 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
         (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
+      case '^':
+      {
+        flags|=MinimumValue;
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
+        break;
+      }
       case '@':
       {
         flags|=AreaValue;
@@ -148,6 +157,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
       }
       case '-':
       case '.':
+      case ',':
       case '+':
       case '0':
       case '1':
@@ -159,6 +169,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
       case '7':
       case '8':
       case '9':
+      case -41:
       case 'x':
       case 'X':
       {
@@ -176,10 +187,10 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
   if (*p == '\0')
     return(flags);
   q=p;
-  (void) strtod(p,&q);
+  value=strtod(p,&q);
   if (LocaleNCompare(p,"0x",2) == 0)
-    (void) strtol(p,&q,10);
-  if ((*q == 'x') || (*q == 'X') || (*q == '\0'))
+    value=(double) strtol(p,&q,10);
+  if (((int) *q == -41) || (*q == 'x') || (*q == 'X') || (*q == '\0'))
     {
       /*
         Parse width.
@@ -192,7 +203,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
       if (p != q)
         flags|=WidthValue;
     }
-  if ((*p == 'x') || (*p == 'X'))
+  if (((int) *p == -41) || (*p == 'x') || (*p == 'X'))
     {
       p++;
       if ((*p != '+') && (*p != '-'))
@@ -257,7 +268,6 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
 %      The string is either a Postscript page name (e.g. A4) or a postscript
 %      page geometry (e.g. 612x792+36+36).
 %
-%
 */
 MagickExport char *GetPageGeometry(const char *page_geometry)
 {
@@ -273,65 +283,72 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
       { "10x13",  "720x936" },
       { "10x14",  "720x1008" },
       { "11x17",  "792x1224" },
-      { "A0",  "2384x3370" },
-      { "A1",  "1684x2384" },
-      { "A10", "73x105" },
-      { "A2",  "1191x1684" },
-      { "A3",  "842x1191" },
-      { "A4",  "595x842" },
-      { "A4SMALL", "595x842" },
-      { "A5",  "420x595" },
-      { "A6",  "297x420" },
-      { "A7",  "210x297" },
-      { "A8",  "148x210" },
-      { "A9",  "105x148" },
-      { "ARCHA", "648x864" },
-      { "ARCHB", "864x1296" },
-      { "ARCHC", "1296x1728" },
-      { "ARCHD", "1728x2592" },
-      { "ARCHE", "2592x3456" },
-      { "B0",  "2920x4127" },
-      { "B1",  "2064x2920" },
-      { "B10", "91x127" },
-      { "B2",  "1460x2064" },
-      { "B3",  "1032x1460" },
-      { "B4",  "729x1032" },
-      { "B5",  "516x729" },
-      { "B6",  "363x516" },
-      { "B7",  "258x363" },
-      { "B8",  "181x258" },
-      { "B9",  "127x181" },
-      { "C0",  "2599x3676" },
-      { "C1",  "1837x2599" },
-      { "C2",  "1298x1837" },
-      { "C3",  "918x1296" },
-      { "C4",  "649x918" },
-      { "C5",  "459x649" },
-      { "C6",  "323x459" },
-      { "C7",  "230x323" },
-      { "EXECUTIVE", "540x720" },
-      { "FLSA", "612x936" },
-      { "FLSE", "612x936" },
-      { "FOLIO",  "612x936" },
-      { "HALFLETTER", "396x612" },
-      { "ISOB0", "2835x4008" },
-      { "ISOB1", "2004x2835" },
-      { "ISOB10", "88x125" },
-      { "ISOB2", "1417x2004" },
-      { "ISOB3", "1001x1417" },
-      { "ISOB4", "709x1001" },
-      { "ISOB5", "499x709" },
-      { "ISOB6", "354x499" },
-      { "ISOB7", "249x354" },
-      { "ISOB8", "176x249" },
-      { "ISOB9", "125x176" },
-      { "LEDGER",  "1224x792" },
-      { "LEGAL",  "612x1008" },
-      { "LETTER", "612x792" },
-      { "LETTERSMALL",  "612x792" },
-      { "QUARTO",  "610x780" },
-      { "STATEMENT",  "396x612" },
-      { "TABLOID",  "792x1224" },
+      { "a0",  "2384x3370" },
+      { "a1",  "1684x2384" },
+      { "a10", "73x105" },
+      { "a2",  "1191x1684" },
+      { "a3",  "842x1191" },
+      { "a4",  "595x842" },
+      { "a4smaLL", "595x842" },
+      { "a5",  "420x595" },
+      { "a6",  "297x420" },
+      { "a7",  "210x297" },
+      { "a8",  "148x210" },
+      { "a9",  "105x148" },
+      { "archa", "648x864" },
+      { "archb", "864x1296" },
+      { "archC", "1296x1728" },
+      { "archd", "1728x2592" },
+      { "arche", "2592x3456" },
+      { "b0",  "2920x4127" },
+      { "b1",  "2064x2920" },
+      { "b10", "91x127" },
+      { "b2",  "1460x2064" },
+      { "b3",  "1032x1460" },
+      { "b4",  "729x1032" },
+      { "b5",  "516x729" },
+      { "b6",  "363x516" },
+      { "b7",  "258x363" },
+      { "b8",  "181x258" },
+      { "b9",  "127x181" },
+      { "c0",  "2599x3676" },
+      { "c1",  "1837x2599" },
+      { "c2",  "1298x1837" },
+      { "c3",  "918x1296" },
+      { "c4",  "649x918" },
+      { "c5",  "459x649" },
+      { "c6",  "323x459" },
+      { "c7",  "230x323" },
+      { "executive", "540x720" },
+      { "flsa", "612x936" },
+      { "flse", "612x936" },
+      { "folio",  "612x936" },
+      { "halfletter", "396x612" },
+      { "isob0", "2835x4008" },
+      { "isob1", "2004x2835" },
+      { "isob10", "88x125" },
+      { "isob2", "1417x2004" },
+      { "isob3", "1001x1417" },
+      { "isob4", "709x1001" },
+      { "isob5", "499x709" },
+      { "isob6", "354x499" },
+      { "isob7", "249x354" },
+      { "isob8", "176x249" },
+      { "isob9", "125x176" },
+      { "jisb0", "1030x1456" },
+      { "jisb1", "728x1030" },
+      { "jisb2", "515x728" },
+      { "jisb3", "364x515" },
+      { "jisb4", "257x364" },
+      { "jisb5", "182x257" },
+      { "jisb6", "128x182" },
+      { "ledger",  "1224x792" },
+      { "legal",  "612x1008" },
+      { "letter", "612x792" },
+      { "lettersmaLL",  "612x792" },
+      { "quarto",  "610x780" },
+      { "statement",  "396x612" },
+      { "tabloid",  "792x1224" },
       { (char *) NULL, (char *) NULL }
     };
 
@@ -372,6 +389,97 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
+%   G r a v i t y A d j u s t G e o m e t r y                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GravityAdjustGeometry() adjusts the offset of a region with regard to the
+%  given: width, height and gravity; against which it is positioned.
+%
+%  The region should also have an appropriate width and height to correctly
+%  set the right offset of the top left corner of the region.
+%
+%  The format of the GravityAdjustGeometry method is:
+%
+%      void GravityAdjustGeometry(const unsigned long width,
+%        const unsigned long height,const GravityType gravity,
+%        RectangleInfo *region);
+%
+%  A description of each parameter follows:
+%
+%    o width, height:  the larger area the region is relative to
+%
+%    o gravity: the edge/corner the current offset is relative to
+%
+%    o region:  The region requiring a offset adjustment relative to gravity
+%
+*/
+MagickExport void GravityAdjustGeometry(const unsigned long width,
+  const unsigned long height,const GravityType gravity,RectangleInfo *region)
+{
+  if (region->height == 0)
+    region->height=height;
+  if (region->width == 0)
+    region->width=width;
+  switch (gravity)
+  {
+    case NorthEastGravity:
+    case EastGravity:
+    case SouthEastGravity:
+    {
+      region->x=(long) (width-region->width-region->x);
+      break;
+    }
+    case NorthGravity:
+    case SouthGravity:
+    case CenterGravity:
+    case StaticGravity:
+    {
+      region->x+=(long) (width/2-region->width/2);
+      break;
+    }
+    case ForgetGravity:
+    case NorthWestGravity:
+    case WestGravity:
+    case SouthWestGravity:
+    default:
+      break;
+  }
+  switch (gravity)
+  {
+    case SouthWestGravity:
+    case SouthGravity:
+    case SouthEastGravity:
+    {
+      region->y=(long) (height-region->height-region->y);
+      break;
+    }
+    case EastGravity:
+    case WestGravity:
+    case CenterGravity:
+    case StaticGravity:
+    {
+      region->y+=(long) (height/2-region->height/2);
+      break;
+    }
+    case ForgetGravity:
+    case NorthWestGravity:
+    case NorthGravity:
+    case NorthEastGravity:
+    default:
+      break;
+  }
+  return;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +     I s G e o m e t r y                                                     %
 %                                                                             %
 %                                                                             %
@@ -389,7 +497,6 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 %
 %    o geometry: This string is the geometry specification.
 %
-%
 */
 MagickExport MagickBooleanType IsGeometry(const char *geometry)
 {
@@ -402,13 +509,16 @@ MagickExport MagickBooleanType IsGeometry(const char *geometry)
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(geometry,&geometry_info);
-  return((MagickBooleanType) (flags != NoValue));
+  if (flags == NoValue)
+    flags=ParseGeometry(geometry+1,&geometry_info);  /* i.e. +-4+-4 */
+  return(flags != NoValue ? MagickTrue : MagickFalse);
 }
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 +     I s S c e n e G e o m e t r y                                           %
 %                                                                             %
 %                                                                             %
@@ -430,7 +540,6 @@ MagickExport MagickBooleanType IsGeometry(const char *geometry)
 %    o pedantic: A value other than 0 invokes a more restrictive set of
 %      conditions for a valid specification (e.g. [1], [1-4], [4-1]).
 %
-%
 */
 MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
   const MagickBooleanType pedantic)
@@ -438,14 +547,16 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
   char
     *p;
 
+  double
+    value;
+
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   p=(char *) geometry;
-  (void) strtod(geometry,&p);
+  value=strtod(geometry,&p);
   if (p == geometry)
     return(MagickFalse);
-  if ((strchr(geometry,'x') != (char *) NULL) ||
-      (strchr(geometry,'X') != (char *) NULL))
+  if (strspn(geometry,"0123456789-, ") != strlen(geometry))
     return(MagickFalse);
   if ((pedantic != MagickFalse) && (strchr(geometry,',') != (char *) NULL))
     return(MagickFalse);
@@ -468,14 +579,13 @@ MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
 %  The format of the ParseAbsoluteGeometry method is:
 %
 %      MagickStatusType ParseAbsoluteGeometry(const char *geometry,
-%        RectangeInfo *region_info)
+%        RectangleInfo *region_info)
 %
 %  A description of each parameter follows:
 %
 %    o geometry:  The geometry (e.g. 100x100+10+10).
 %
-%    o region_info: The region as defined by the geometry string.
-%
+%    o region_info: the region as defined by the geometry string.
 %
 */
 MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
@@ -494,6 +604,78 @@ MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P a r s e A f f i n e G e o m e t r y                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ParseAffineGeometry() returns an affine matrix as defined by the geometry
+%  string.
+%
+%  The format of the ParseAffineGeometry method is:
+%
+%      MagickStatusType ParseAffineGeometry(const char *geometry,
+%        AffineMatrix *affine_matrix,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o geometry:  The geometry (e.g. 1.0,0.0,0.0,1.0,3.2,1.2).
+%
+%    o affine_matrix: the affine matrix as defined by the geometry string.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport MagickStatusType ParseAffineGeometry(const char *geometry,
+  AffineMatrix *affine_matrix,ExceptionInfo *exception)
+{
+  char
+    token[MaxTextExtent];
+
+  const char
+    *p;
+
+  double
+    determinant;
+
+  MagickStatusType
+    flags;
+
+  register long
+    i;
+
+  GetAffineMatrix(affine_matrix);
+  flags=NoValue;
+  p=(char *) geometry;
+  for (i=0; (*p != '\0') && (i < 6); i++)
+  {
+    GetMagickToken(p,&p,token);
+    if (*token == ',')
+      GetMagickToken(p,&p,token);
+    switch (i)
+    {
+      case 0: affine_matrix->sx=atof(token); break;
+      case 1: affine_matrix->rx=atof(token); break;
+      case 2: affine_matrix->ry=atof(token); break;
+      case 3: affine_matrix->sy=atof(token); break;
+      case 4: affine_matrix->tx=atof(token); flags|=XValue; break;
+      case 5: affine_matrix->ty=atof(token); flags|=YValue; break;
+    }
+  }
+  determinant=(affine_matrix->sx*affine_matrix->sy-affine_matrix->rx*
+    affine_matrix->ry);
+  if (fabs(determinant) < MagickEpsilon)
+    (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+      "InvalidGeometry","`%s'",geometry);
+  return(flags);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P a r s e G e o m e t r y                                                 %
 %                                                                             %
 %                                                                             %
@@ -517,7 +699,6 @@ MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %
 %    o geometry_info:  returns the parsed width/height/x/y in this structure.
 %
-%
 */
 MagickExport MagickStatusType ParseGeometry(const char *geometry,
   GeometryInfo *geometry_info)
@@ -576,6 +757,12 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
+      case '^':
+      {
+        flags|=MinimumValue;
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
+        break;
+      }
       case '@':
       {
         flags|=AreaValue;
@@ -589,8 +776,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         break;
       }
       case '-':
-      case '.':
       case '+':
+      case ',':
       case '0':
       case '1':
       case '2':
@@ -601,14 +788,21 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       case '7':
       case '8':
       case '9':
+      case -41:
       case 'x':
       case 'X':
-      case ',':
       case '/':
+      case ':':
       {
         p++;
         break;
       }
+      case '.':
+      {
+        p++;
+        flags|=DecimalValue;
+        break;
+      }
       default:
         return(flags);
     }
@@ -620,10 +814,11 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   if (*p == '\0')
     return(flags);
   q=p;
-  (void) strtod(p,&q);
+  value=strtod(p,&q);
   if (LocaleNCompare(p,"0x",2) == 0)
-    (void) strtol(p,&q,10);
-  if ((*q == 'x') || (*q == 'X') || (*q == '/') || (*q == ',') || (*q =='\0'))
+    value=(double) strtol(p,&q,10);
+  if (((int) *q == -41) || (*q == 'x') || (*q == 'X') || (*q == ',') ||
+      (*q == '/') || (*q == ':') || (*q =='\0'))
     {
       /*
         Parse rho.
@@ -640,7 +835,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         }
     }
   q=p;
-  if ((*p == 'x') || (*p == 'X') || (*p == ',') || (*p == '/'))
+  if (((int) *p == -41) || (*p == 'x') || (*p == 'X') || (*p == ',') ||
+      (*p == '/') || (*p == ':'))
     {
       /*
         Parse sigma.
@@ -648,7 +844,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       p++;
       while (isspace((int) ((unsigned char) *p)) != 0)
         p++;
-      if (((*q != 'x') && (*q != 'X')) || ((*p != '+') && (*p != '-')))
+      if ((((int) *q != -41) && (*q != 'x') && (*q != 'X')) ||
+          ((*p != '+') && (*p != '-')))
         {
           q=p;
           value=strtod(p,&p);
@@ -661,12 +858,12 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
     }
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
-  if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+  if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') || (*p == ':'))
     {
       /*
         Parse xi value.
       */
-      if ((*p == ',') || (*p == '/'))
+      if ((*p == ',') || (*p == '/') || (*p == ':'))
         p++;
       q=p;
       value=strtod(p,&p);
@@ -679,12 +876,13 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         }
       while (isspace((int) ((unsigned char) *p)) != 0)
         p++;
-      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') ||
+          (*p == ':'))
         {
           /*
             Parse psi value.
           */
-          if ((*p == ',') || (*p == '/'))
+          if ((*p == ',') || (*p == '/') || (*p == ':'))
             p++;
           q=p;
           value=strtod(p,&p);
@@ -698,12 +896,13 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         }
       while (isspace((int) ((unsigned char) *p)) != 0)
         p++;
-      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') ||
+          (*p == ':'))
         {
           /*
             Parse chi value.
           */
-          if ((*p == ',') || (*p == '/'))
+          if ((*p == ',') || (*p == '/') || (*p == ':'))
             p++;
           q=p;
           value=strtod(p,&p);
@@ -716,6 +915,27 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
             }
         }
     }
+  if (strchr(pedantic_geometry,':') != (char *) NULL)
+    {
+      /*
+        Normalize sampling factor (e.g. 4:2:2 => 2x1).
+      */
+      geometry_info->rho/=geometry_info->sigma;
+      geometry_info->sigma=1.0;
+      if (geometry_info->xi == 0.0)
+        geometry_info->sigma=2.0;
+    }
+  if (((flags & SigmaValue) == 0) && ((flags & XiValue) != 0) &&
+      ((flags & PsiValue) == 0))
+    {
+      /*
+        Support negative height values (e.g. 30x-20).
+      */
+      geometry_info->sigma=geometry_info->xi;
+      geometry_info->xi=0.0;
+      flags|=SigmaValue;
+      flags&=(~XiValue);
+    }
   return(flags);
 }
 
@@ -736,18 +956,20 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %  The format of the ParseGravityGeometry method is:
 %
 %      MagickStatusType ParseGravityGeometry(Image *image,const char *geometry,
-%        RectangeInfo *region_info)
+%        RectangeInfo *region_info,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o geometry:  The geometry (e.g. 100x100+10+10).
 %
-%    o region_info: The region as defined by the geometry string with
+%    o region_info: the region as defined by the geometry string with
 %      respect to the image dimensions and its gravity.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-MagickExport MagickStatusType ParseGravityGeometry(Image *image,
-  const char *geometry,RectangleInfo *region_info)
+MagickExport MagickStatusType ParseGravityGeometry(const Image *image,
+  const char *geometry,RectangleInfo *region_info,ExceptionInfo *exception)
 {
   MagickStatusType
     flags;
@@ -757,11 +979,15 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
     width;
 
   SetGeometry(image,region_info);
+  if (image->page.width != 0)
+    region_info->width=image->page.width;
+  if (image->page.height != 0)
+    region_info->height=image->page.height;
   flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        OptionError,"InvalidGeometry","`%s'",geometry);
+      (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+        "InvalidGeometry","`%s'",geometry);
       return(flags);
     }
   if ((flags & PercentValue) != 0)
@@ -770,7 +996,7 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
         geometry_info;
 
       MagickStatusType
-        flags;
+        status;
 
       PointInfo
         scale;
@@ -778,84 +1004,30 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
       /*
         Geometry is a percentage of the image size.
       */
-      flags=ParseGeometry(geometry,&geometry_info);
+      if (image->gravity != UndefinedGravity)
+        flags|=XValue | YValue;
+      status=ParseGeometry(geometry,&geometry_info);
       scale.x=geometry_info.rho;
-      if ((flags & RhoValue) == 0)
+      if ((status & RhoValue) == 0)
         scale.x=100.0;
       scale.y=geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
+      if ((status & SigmaValue) == 0)
         scale.y=scale.x;
       region_info->width=(unsigned long) ((scale.x*image->columns/100.0)+0.5);
       region_info->height=(unsigned long) ((scale.y*image->rows/100.0)+0.5);
     }
+  /*
+    Adjust offset according to gravity setting.
+  */
   width=region_info->width;
   height=region_info->height;
   if (width == 0)
-    {
-      if (image->page.width != 0)
-        width=image->page.width;
-      else
-        width=image->columns;
-    }
+    region_info->width=image->page.width | image->columns;
   if (height == 0)
-    {
-      if (image->page.height != 0)
-        height=image->page.height;
-      else
-        height=image->rows;
-    }
-  switch (image->gravity)
-  {
-    case ForgetGravity:
-    case NorthWestGravity:
-      break;
-    case NorthGravity:
-    {
-      region_info->x+=(long) (image->columns/2-width/2);
-      break;
-    }
-    case NorthEastGravity:
-    {
-      region_info->x=(long) (image->columns-width-region_info->x);
-      break;
-    }
-    case WestGravity:
-    {
-      region_info->y+=(long) (image->rows/2-height/2);
-      break;
-    }
-    case StaticGravity:
-    case CenterGravity:
-    default:
-    {
-      region_info->x+=(long) (image->columns/2-width/2);
-      region_info->y+=(long) (image->rows/2-height/2);
-      break;
-    }
-    case EastGravity:
-    {
-      region_info->x=(long) (image->columns-width-region_info->x);
-      region_info->y+=(long) (image->rows/2-height/2);
-      break;
-    }
-    case SouthWestGravity:
-    {
-      region_info->y=(long) (image->rows-height-region_info->y);
-      break;
-    }
-    case SouthGravity:
-    {
-      region_info->x+=(long) (image->columns/2-width/2);
-      region_info->y=(long) (image->rows-height-region_info->y);
-      break;
-    }
-    case SouthEastGravity:
-    {
-      region_info->x=(long) (image->columns-width-region_info->x);
-      region_info->y=(long) (image->rows-height-region_info->y);
-      break;
-    }
-  }
+    region_info->height=image->page.height | image->rows;
+  GravityAdjustGeometry(image->columns,image->rows,image->gravity,region_info);
+  region_info->width=width;
+  region_info->height=height;
   return(flags);
 }
 
@@ -889,6 +1061,15 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
 %      specification.
 %
 */
+
+static inline unsigned long MagickMax(const unsigned long x,
+  const unsigned long y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
 MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   long *y,unsigned long *width,unsigned long *height)
 {
@@ -937,13 +1118,17 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
       scale.y=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
         scale.y=scale.x;
-      *width=(unsigned long) ((scale.x*former_width/100.0)+0.5);
-      *height=(unsigned long) ((scale.y*former_height/100.0)+0.5);
+      *width=(unsigned long) (scale.x*former_width/100.0+0.5);
+      if (*width == 0)
+        *width=1;
+      *height=(unsigned long) (scale.y*former_height/100.0+0.5);
+      if (*height == 0)
+        *height=1;
       former_width=(*width);
       former_height=(*height);
     }
-  if (((flags & AspectValue) == 0) &&
-      ((*width != former_width) || (*height != former_height)))
+  if (((flags & AspectValue) == 0) && ((*width != former_width) ||
+      (*height != former_height)))
     {
       MagickRealType
         scale_factor;
@@ -957,20 +1142,43 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
         if (((flags & RhoValue) != 0) && (flags & SigmaValue) != 0)
           {
             scale_factor=(MagickRealType) *width/(MagickRealType) former_width;
-            if (scale_factor > ((MagickRealType) *height/
-                (MagickRealType) former_height))
-              scale_factor=(MagickRealType)
-                *height/(MagickRealType) former_height;
+            if ((flags & MinimumValue) == 0)
+              {
+                if (scale_factor > ((MagickRealType) *height/
+                    (MagickRealType) former_height))
+                  scale_factor=(MagickRealType) *height/(MagickRealType)
+                    former_height;
+              }
+            else
+              if (scale_factor < ((MagickRealType) *height/
+                  (MagickRealType) former_height))
+                scale_factor=(MagickRealType) *height/(MagickRealType)
+                  former_height;
           }
         else
           if ((flags & RhoValue) != 0)
-            scale_factor=(MagickRealType) *width/(MagickRealType) former_width;
+            {
+              scale_factor=(MagickRealType) *width/(MagickRealType)
+                former_width;
+              if (((flags & MinimumValue) != 0) &&
+                  (scale_factor < ((MagickRealType) *width/
+                    (MagickRealType) former_height)))
+                scale_factor=(MagickRealType) *width/(MagickRealType)
+                  former_height;
+            }
           else
-            scale_factor=(MagickRealType)
-              *height/(MagickRealType) former_height;
-    *width=(unsigned long) (scale_factor*former_width+0.5);
-    *height=(unsigned long) (scale_factor*former_height+0.5);
-  }
+            {
+              scale_factor=(MagickRealType) *height/(MagickRealType)
+                former_height;
+              if (((flags & MinimumValue) != 0) &&
+                  (scale_factor < ((MagickRealType) *height/
+                    (MagickRealType) former_width)))
+                scale_factor=(MagickRealType) *height/(MagickRealType)
+                  former_width;
+            }
+      *width=MagickMax((unsigned long) (scale_factor*former_width+0.5),1UL);
+      *height=MagickMax((unsigned long) (scale_factor*former_height+0.5),1UL);
+    }
   if ((flags & GreaterValue) != 0)
     {
       if (former_width < *width)
@@ -1000,10 +1208,9 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
       (void) ParseGeometry(geometry,&geometry_info);
       area=geometry_info.rho;
       distance=sqrt((double) former_width*former_height);
-      scale.x=(MagickRealType) former_width/(distance/sqrt((double) area));
-      scale.y=(MagickRealType) former_height/(distance/sqrt((double) area));
-      if ((scale.x < (MagickRealType) *width) ||
-          (scale.y < (MagickRealType) *height))
+      scale.x=(double) former_width/(double) (distance/sqrt((double) area));
+      scale.y=(double) former_height/(double) (distance/sqrt((double) area));
+      if ((scale.x < (double) *width) || (scale.y < (double) *height))
         {
           *width=(unsigned long) (former_width/(distance/sqrt((double) area)));
           *height=(unsigned long) (former_height/(distance/
@@ -1031,31 +1238,43 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
 %
 %  The format of the ParsePageGeometry method is:
 %
-%      MagickStatusType ParsePageGeometry(Image *image,const char *geometry,
-%        RectangeInfo *region_info)
+%      MagickStatusType ParsePageGeometry(const Image *image,
+%        const char *geometry,RectangeInfo *region_info,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o geometry:  The geometry (e.g. 100x100+10+10).
 %
-%    o region_info: The region as defined by the geometry string with
+%    o region_info: the region as defined by the geometry string with
 %      respect to the image and its gravity.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-MagickExport MagickStatusType ParsePageGeometry(Image *image,
-  const char *geometry,RectangleInfo *region_info)
+MagickExport MagickStatusType ParsePageGeometry(const Image *image,
+  const char *geometry,RectangleInfo *region_info,ExceptionInfo *exception)
 {
   MagickStatusType
     flags;
 
   SetGeometry(image,region_info);
+  if (image->page.width != 0)
+    region_info->width=image->page.width;
+  if (image->page.height != 0)
+    region_info->height=image->page.height;
   flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
-      (void) ThrowMagickException(&image->exception,GetMagickModule(),
-        OptionError,"InvalidGeometry","`%s'",geometry);
+      (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+        "InvalidGeometry","`%s'",geometry);
       return(flags);
     }
+  if ((flags & PercentValue) != 0)
+    {
+      region_info->width=image->columns;
+      region_info->height=image->rows;
+    }
   flags=ParseMetaGeometry(geometry,&region_info->x,&region_info->y,
     &region_info->width,&region_info->height);
   return(flags);
@@ -1066,30 +1285,32 @@ MagickExport MagickStatusType ParsePageGeometry(Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   P a r s e S i z e G e o m e t r y                                         %
+%   P a r s e R e g i o n G e o m e t r y                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ParseSizeGeometry() returns a region as defined by the geometry string with
-%  respect to the image dimensions and aspect ratio.
+%  ParseRegionGeometry() returns a region as defined by the geometry string
+%  with respect to the image dimensions and aspect ratio.
 %
-%  The format of the ParseSizeGeometry method is:
+%  The format of the ParseRegionGeometry method is:
 %
-%      MagickStatusType ParseSizeGeometry(Image *image,const char *geometry,
-%        RectangeInfo *region_info)
+%      MagickStatusType ParseRegionGeometry(const Image *image,
+%        const char *geometry,RectangeInfo *region_info,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o geometry:  The geometry (e.g. 100x100+10+10).
 %
-%    o region_info: The region as defined by the geometry string.
+%    o region_info: the region as defined by the geometry string.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport MagickStatusType ParseSizeGeometry(Image *image,
-  const char *geometry,RectangleInfo *region_info)
+MagickExport MagickStatusType ParseRegionGeometry(const Image *image,
+  const char *geometry,RectangleInfo *region_info,ExceptionInfo *exception)
 {
   MagickStatusType
     flags;
@@ -1098,7 +1319,7 @@ MagickExport MagickStatusType ParseSizeGeometry(Image *image,
   flags=ParseMetaGeometry(geometry,&region_info->x,&region_info->y,
     &region_info->width,&region_info->height);
   if (flags == NoValue)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),OptionError,
+    (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
       "InvalidGeometry","`%s'",geometry);
   return(flags);
 }
@@ -1114,7 +1335,7 @@ MagickExport MagickStatusType ParseSizeGeometry(Image *image,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SetGeometry sets the geometry to its default values.
+%  SetGeometry() sets the geometry to its default values.
 %
 %  The format of the SetGeometry method is:
 %
@@ -1122,10 +1343,9 @@ MagickExport MagickStatusType ParseSizeGeometry(Image *image,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
-%
-%    o geometry: The geometry.
+%    o image: the image.
 %
+%    o geometry: the geometry.
 %
 */
 MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
@@ -1137,11 +1357,7 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
   assert(geometry != (RectangleInfo *) NULL);
   (void) ResetMagickMemory(geometry,0,sizeof(*geometry));
   geometry->width=image->columns;
-  if (image->page.width != 0)
-    geometry->width=image->page.width;
   geometry->height=image->rows;
-  if (image->page.height != 0)
-    geometry->height=image->page.height;
 }
 
 /*
@@ -1163,8 +1379,7 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 %
 %  A description of each parameter follows:
 %
-%    o geometry_info: The geometry info structure.
-%
+%    o geometry_info: the geometry info structure.
 %
 */
 MagickExport void SetGeometryInfo(GeometryInfo *geometry_info)
