@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,18 +13,17 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick identify command method.
+  MagickCore image identify method.
 */
-#ifndef _MAGICK_IDENTIFY_H
-#define _MAGICK_IDENTIFY_H
+#ifndef _MAGICKCORE_IDENTIFY_H
+#define _MAGICKCORE_IDENTIFY_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
 extern MagickExport MagickBooleanType
-  IdentifyImage(Image *,FILE *,const MagickBooleanType),
-  IdentifyImageCommand(ImageInfo *,int,char **,char **,ExceptionInfo *);
+  IdentifyImage(Image *,FILE *,const MagickBooleanType);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
