@@ -100,18 +100,19 @@ static void ClearBounds(Image *image,RectangleInfo *bounds)
   long
     y;
 
-  register long
-    x;
-
-  register PixelPacket
-    *q;
-
-  if ( bounds->x < 0 ) return;
+  if (bounds->x < 0)
+    return;
   if (image->matte == MagickFalse)
     (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
   exception=(&image->exception);
   for (y=0; y < (long) bounds->height; y++)
   {
+    register long
+      x;
+
+    register PixelPacket
+      *__restrict q;
+
     q=GetAuthenticPixels(image,bounds->x,bounds->y+y,bounds->width,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
@@ -877,7 +878,7 @@ MagickExport Image *CompareImageLayers(const Image *image,
 %  DeconstructImages() compares each image with the next in a sequence and
 %  returns the minimum bounding region of all differences from the first image.
 %
-%  This function is depreciated in favor of the more universal
+%  This function is deprecated in favor of the more universal
 %  CompareImageLayers() function.
 %
 %  The format of the DeconstructImages method is:
