@@ -65,10 +65,13 @@
 #include "magick/random_.h"
 #include "magick/registry.h"
 #include "magick/resource_.h"
+#include "magick/policy.h"
 #include "magick/semaphore.h"
 #include "magick/signature-private.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
+#include "magick/thread_.h"
+#include "magick/thread-private.h"
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/xwindow-private.h"
@@ -121,40 +124,6 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   D e s t r o y M a g i c k L i s t                                         %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DestroyMagickList() deallocates memory associated with the MagickInfo list.
-%
-%  The format of the DestroyMagickList method is:
-%
-%      void DestroyMagickList(void)
-%
-*/
-MagickExport void DestroyMagickList(void)
-{
-  AcquireSemaphoreInfo(&magick_semaphore);
-  if (magick_list != (SplayTreeInfo *) NULL)
-    magick_list=DestroySplayTree(magick_list);
-  instantiate_magick=MagickFalse;
-  RelinquishSemaphoreInfo(magick_semaphore);
-  DestroySemaphoreInfo(&magick_semaphore);
-#if !defined(MAGICKCORE_BUILD_MODULES)
-  UnregisterStaticModules();
-#endif
-#if defined(MAGICKCORE_MODULES_SUPPORT)
-  DestroyModuleList();
-#endif
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 +   G e t I m a g e D e c o d e r                                             %
 %                                                                             %
 %                                                                             %
@@ -222,12 +191,13 @@ MagickExport EncodeImageHandler *GetImageEncoder(const MagickInfo *magick_info)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GetImageMagick() searches for an image format that matches the specified
-%  magick string.  If one is found the name is returned otherwise NULL.
+%  magick string.  If one is found, MagickTrue is returned otherwise
+%  MagickFalse.
 %
 %  The format of the GetImageMagick method is:
 %
-%      const char *GetImageMagick(const unsigned char *magick,
-%        const size_t length)
+%      MagickBooleanType GetImageMagick(const unsigned char *magick,
+%        const size_t length,char *format)
 %
 %  A description of each parameter follows:
 %
@@ -235,13 +205,18 @@ MagickExport EncodeImageHandler *GetImageEncoder(const MagickInfo *magick_info)
 %
 %    o length: the length of the binary string.
 %
+%    o format: the image format as determined by the magick bytes.
+%
 */
-MagickExport const char *GetImageMagick(const unsigned char *magick,
-  const size_t length)
+MagickExport MagickBooleanType GetImageMagick(const unsigned char *magick,
+  const size_t length,char *format)
 {
   ExceptionInfo
     *exception;
 
+  MagickBooleanType
+    status;
+
   register const MagickInfo
     *p;
 
@@ -251,21 +226,24 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
   p=GetMagickInfo("*",exception);
   exception=DestroyExceptionInfo(exception);
   if (p == (const MagickInfo *) NULL)
-    return((const char *) NULL);
-  AcquireSemaphoreInfo(&magick_semaphore);
+    return(MagickFalse);
+  status=MagickFalse;
+  (void) LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
   {
     if ((p->magick != (IsImageFormatHandler *) NULL) &&
         (p->magick(magick,length) != 0))
-      break;
+      {
+        status=MagickTrue;
+        (void) CopyMagickString(format,p->name,MaxTextExtent);
+        break;
+      }
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
-  RelinquishSemaphoreInfo(magick_semaphore);
-  if (p != (MagickInfo *) NULL)
-    return(p->name);
-  return((const char *) NULL);
+  (void) UnlockSemaphoreInfo(magick_semaphore);
+  return(status);
 }
 
 /*
@@ -371,7 +349,7 @@ MagickExport const char *GetMagickDescription(const MagickInfo *magick_info)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GetMagickEndianSupport() returns the MagickTrue if the coder respects
-%  endianess other than MSBEndian.
+%  endianness other than MSBEndian.
 %
 %  The format of the GetMagickEndianSupport method is:
 %
@@ -434,16 +412,16 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
       if (LocaleCompare(name,"*") == 0)
         (void) OpenModules(exception);
 #endif
-      AcquireSemaphoreInfo(&magick_semaphore);
+      (void) LockSemaphoreInfo(magick_semaphore);
       ResetSplayTreeIterator(magick_list);
       p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
-      RelinquishSemaphoreInfo(magick_semaphore);
+      (void) UnlockSemaphoreInfo(magick_semaphore);
       return(p);
     }
   /*
     Find name in list.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
+  (void) LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -467,7 +445,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
       }
     }
 #endif
-  RelinquishSemaphoreInfo(magick_semaphore);
+  (void) UnlockSemaphoreInfo(magick_semaphore);
   return(p);
 }
 
@@ -548,7 +526,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
   /*
     Generate magick list.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
+  (void) LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
@@ -558,7 +536,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
       formats[i++]=p;
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
-  RelinquishSemaphoreInfo(magick_semaphore);
+  (void) UnlockSemaphoreInfo(magick_semaphore);
   qsort((void *) formats,(size_t) i,sizeof(*formats),MagickInfoCompare);
   formats[i]=(MagickInfo *) NULL;
   *number_formats=(unsigned long) i;
@@ -638,7 +616,7 @@ MagickExport char **GetMagickList(const char *pattern,
     GetNumberOfNodesInSplayTree(magick_list)+1UL,sizeof(*formats));
   if (formats == (char **) NULL)
     return((char **) NULL);
-  AcquireSemaphoreInfo(&magick_semaphore);
+  (void) LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
@@ -648,7 +626,7 @@ MagickExport char **GetMagickList(const char *pattern,
       formats[i++]=ConstantString(p->name);
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
-  RelinquishSemaphoreInfo(magick_semaphore);
+  (void) UnlockSemaphoreInfo(magick_semaphore);
   qsort((void *) formats,(size_t) i,sizeof(*formats),MagickCompare);
   formats[i]=(char *) NULL;
   *number_formats=(unsigned long) i;
@@ -793,10 +771,13 @@ static void *DestroyMagickNode(void *magick_info)
 
 static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
 {
+  (void) exception;
   if ((magick_list == (SplayTreeInfo *) NULL) &&
       (instantiate_magick == MagickFalse))
     {
-      AcquireSemaphoreInfo(&magick_semaphore);
+      if (magick_semaphore == (SemaphoreInfo *) NULL)
+        AcquireSemaphoreInfo(&magick_semaphore);
+      (void) LockSemaphoreInfo(magick_semaphore);
       if ((magick_list == (SplayTreeInfo *) NULL) &&
           (instantiate_magick == MagickFalse))
         {
@@ -811,7 +792,7 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
           if (magick_list == (SplayTreeInfo *) NULL)
             ThrowFatalException(ResourceLimitFatalError,
               "MemoryAllocationFailed");
-          magick_info=SetMagickInfo("tmp");
+          magick_info=SetMagickInfo("ephemeral");
           magick_info->stealth=MagickTrue;
           status=AddValueToSplayTree(magick_list,magick_info->name,magick_info);
           if (status == MagickFalse)
@@ -838,7 +819,7 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
 #endif
           instantiate_magick=MagickTrue;
         }
-      RelinquishSemaphoreInfo(magick_semaphore);
+      (void) UnlockSemaphoreInfo(magick_semaphore);
     }
   return(magick_list != (SplayTreeInfo *) NULL ? MagickTrue : MagickFalse);
 }
@@ -1035,6 +1016,60 @@ MagickExport MagickBooleanType IsMagickInstantiated(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   M a g i c k C o m p o n e n t G e n e s i s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickComponentGenesis() instantiates the magick component.
+%
+%  The format of the MagickComponentGenesis method is:
+%
+%      MagickBooleanType MagickComponentGenesis(void)
+%
+*/
+MagickExport MagickBooleanType MagickComponentGenesis(void)
+{
+  AcquireSemaphoreInfo(&magick_semaphore);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   M a g i c k C o m p o n e n t T e r m i n u s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickComponentTerminus() destroys the magick component.
+%
+%  The format of the MagickComponentTerminus method is:
+%
+%      void MagickComponentTerminus(void)
+%
+*/
+MagickExport void MagickComponentTerminus(void)
+{
+  if (magick_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&magick_semaphore);
+  (void) LockSemaphoreInfo(magick_semaphore);
+  if (magick_list != (SplayTreeInfo *) NULL)
+    magick_list=DestroySplayTree(magick_list);
+  instantiate_magick=MagickFalse;
+  (void) UnlockSemaphoreInfo(magick_semaphore);
+  DestroySemaphoreInfo(&magick_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k C o r e G e n e s i s                                         %
 %                                                                             %
 %                                                                             %
@@ -1043,7 +1078,7 @@ MagickExport MagickBooleanType IsMagickInstantiated(void)
 %
 %  MagickCoreGenesis() initializes the MagickCore environment.
 %
-%  The format of the InitializeMagick function is:
+%  The format of the MagickCoreGenesis function is:
 %
 %      MagickCoreGenesis(const char *path,
 %        const MagickBooleanType establish_signal_handlers)
@@ -1095,7 +1130,7 @@ static void MagickSignalHandler(int signal_number)
 #if !defined(MAGICKCORE_HAVE_SIGACTION)
   (void) signal(signal_number,SIG_IGN);
 #endif
-  AsynchronousDestroyMagickResources();
+  AsynchronousResourceComponentTerminus();
   instantiate_magick=MagickFalse;
   (void) SetMagickSignalHandler(signal_number,signal_handlers[signal_number]);
 #if defined(MAGICKCORE_HAVE_RAISE)
@@ -1145,18 +1180,15 @@ MagickExport void MagickCoreGenesis(const char *path,
     execution_path[MaxTextExtent],
     filename[MaxTextExtent];
 
-  ExceptionInfo
-    *exception;
-
-  time_t
-    seconds;
-
   /*
     Initialize the Magick environment.
   */
-  seconds=time((time_t *) NULL);
-  srand((unsigned int) (seconds+getpid()));
-  InitializeSemaphore();
+  (void) setlocale(LC_ALL,"");
+  (void) setlocale(LC_NUMERIC,"C");
+  (void) SemaphoreComponentGenesis();
+  (void) LogComponentGenesis();
+  (void) LocaleComponentGenesis();
+  (void) RandomComponentGenesis();
   events=GetEnvironmentValue("MAGICK_DEBUG");
   if (events != (char *) NULL)
     {
@@ -1230,12 +1262,25 @@ MagickExport void MagickCoreGenesis(const char *path,
 #endif
     }
   /*
-    Initialize magick resources.
+    Instantiate magick resources.
   */
-  InitializeMagickResources();
-  exception=AcquireExceptionInfo();
-  (void) GetMagickInfo((char *) NULL,exception);
-  exception=DestroyExceptionInfo(exception);
+  (void) ConfigureComponentGenesis();
+  (void) PolicyComponentGenesis();
+  (void) CacheComponentGenesis();
+  (void) RegistryComponentGenesis();
+  (void) ResourceComponentGenesis();
+  (void) CoderComponentGenesis();
+  (void) MagickComponentGenesis();
+#if defined(MAGICKCORE_MODULES_SUPPORT)
+  (void) ModuleComponentGenesis();
+#endif
+  (void) DelegateComponentGenesis();
+  (void) MagicComponentGenesis();
+  (void) ColorComponentGenesis();
+  (void) TypeComponentGenesis();
+  (void) MimeComponentGenesis();
+  (void) ConstituteComponentGenesis();
+  (void) XComponentGenesis();
 }
 
 /*
@@ -1251,7 +1296,7 @@ MagickExport void MagickCoreGenesis(const char *path,
 %
 %  MagickCoreTerminus() destroys the MagickCore environment.
 %
-%  The format of the DestroyMagick function is:
+%  The format of the MagickCoreTerminus function is:
 %
 %      MagickCoreTerminus(void)
 %
@@ -1259,26 +1304,34 @@ MagickExport void MagickCoreGenesis(const char *path,
 MagickExport void MagickCoreTerminus(void)
 {
 #if defined(MAGICKCORE_X11_DELEGATE)
-  DestroyXResources();
+  XComponentTerminus();
 #endif
-  DestroyConstitute();
-  DestroyMimeList();
-  DestroyConfigureList();
-  DestroyTypeList();
-  DestroyColorList();
+  ConstituteComponentTerminus();
+  MimeComponentTerminus();
+  TypeComponentTerminus();
+  ColorComponentTerminus();
 #if defined(__WINDOWS__)
   NTGhostscriptUnLoadDLL();
 #endif
-  DestroyMagicList();
-  DestroyDelegateList();
-  DestroyMagickList();
-  DestroyCoderList();
-  DestroyMagickResources();
-  DestroyImageRegistry();
-  DestroyPixelCacheResources();
-  DestroyRandomReservoir();
-  DestroyLocaleList();
-  DestroyLogList();
+  MagicComponentTerminus();
+  DelegateComponentTerminus();
+  MagickComponentTerminus();
+#if !defined(MAGICKCORE_BUILD_MODULES)
+  UnregisterStaticModules();
+#endif
+#if defined(MAGICKCORE_MODULES_SUPPORT)
+  ModuleComponentTerminus();
+#endif
+  CoderComponentTerminus();
+  ResourceComponentTerminus();
+  RegistryComponentTerminus();
+  CacheComponentTerminus();
+  PolicyComponentTerminus();
+  ConfigureComponentTerminus();
+  RandomComponentTerminus();
+  LocaleComponentTerminus();
+  LogComponentTerminus();
+  SemaphoreComponentTerminus();
   instantiate_magick=MagickFalse;
 }
 
@@ -1411,7 +1464,7 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
     return(MagickFalse);
   if (GetNumberOfNodesInSplayTree(magick_list) == 0)
     return(MagickFalse);
-  AcquireSemaphoreInfo(&magick_semaphore);
+  (void) LockSemaphoreInfo(magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -1421,6 +1474,6 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
   status=DeleteNodeByValueFromSplayTree(magick_list,p);
-  RelinquishSemaphoreInfo(magick_semaphore);
+  (void) UnlockSemaphoreInfo(magick_semaphore);
   return(status);
 }
