@@ -70,6 +70,7 @@
 #define BlockHeader(block)  ((size_t *) (block)-1)
 #define BlockSize  4096
 #define BlockThreshold  1024
+#define AlignedSize  (16*sizeof(void *))
 #define MaxBlockExponent  16
 #define MaxBlocks ((BlockThreshold/(4*sizeof(size_t)))+MaxBlockExponent+1)
 #define MaxSegments  1024
@@ -130,12 +131,18 @@ typedef struct _MagickMemoryMethods
   DestroyMemoryHandler
     destroy_memory_handler;
 } MagickMemoryMethods;
+
 
 /*
   Global declarations.
 */
 static MagickMemoryMethods
-  memory_methods = { malloc, realloc, free };
+  memory_methods =
+  {
+    (AcquireMemoryHandler) malloc,
+    (ResizeMemoryHandler) realloc,
+    (DestroyMemoryHandler)free
+  };
 
 #if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 static MemoryInfo
@@ -154,6 +161,54 @@ static MagickBooleanType
   ExpandHeap(size_t);
 #endif
 
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   A c q u i r e A l i g n e d M e m o r y                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireAlignedMemory() returns a pointer to a block of memory at least size
+%  bytes whose address is a multiple of 16*sizeof(void *).
+%
+%  The format of the AcquireAlignedMemory method is:
+%
+%      void *AcquireAlignedMemory(const size_t count,const size_t quantum)
+%
+%  A description of each parameter follows:
+%
+%    o count: the number of quantum elements to allocate.
+%
+%    o quantum: the number of bytes in each quantum.
+%
+*/
+MagickExport void *AcquireAlignedMemory(const size_t count,const size_t quantum)
+{
+  size_t
+    size;
+
+  size=count*quantum;
+  if ((count == 0) || (quantum != (size/count)))
+    {
+      errno=ENOMEM;
+      return((void *) NULL);
+    }
+#if defined(MAGICKCORE_HAVE_POSIX_MEMALIGN)
+  {
+    void
+      *memory;
+
+    if (posix_memalign(&memory,AlignedSize,size) == 0)
+      return(memory);
+  }
+#endif
+  return(malloc(size));
+}
+
 #if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -334,9 +389,11 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 #if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   memory=memory_methods.acquire_memory_handler(size == 0 ? 1UL : size);
 #else
+  if (memory_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&memory_semaphore);
   if (free_segments == (DataSegmentInfo *) NULL)
     {
-      AcquireSemaphoreInfo(&memory_semaphore);
+      (void) LockSemaphoreInfo(memory_semaphore);
       if (free_segments == (DataSegmentInfo *) NULL)
         {
           register long
@@ -356,16 +413,16 @@ MagickExport void *AcquireMagickMemory(const size_t size)
           }
           free_segments=(&memory_info.segment_pool[0]);
         }
-      RelinquishSemaphoreInfo(memory_semaphore);
+      (void) UnlockSemaphoreInfo(memory_semaphore);
     }
-  AcquireSemaphoreInfo(&memory_semaphore);
+  (void) LockSemaphoreInfo(memory_semaphore);
   memory=AcquireBlock(size == 0 ? 1UL : size);
   if (memory == (void *) NULL)
     {
       if (ExpandHeap(size == 0 ? 1UL : size) != MagickFalse)
         memory=AcquireBlock(size == 0 ? 1UL : size);
     }
-  RelinquishSemaphoreInfo(memory_semaphore);
+  (void) UnlockSemaphoreInfo(memory_semaphore);
 #endif
   return(memory);
 }
@@ -491,8 +548,10 @@ MagickExport void DestroyMagickMemory(void)
   register long
     i;
 
-  AcquireSemaphoreInfo(&memory_semaphore);
-  RelinquishSemaphoreInfo(memory_semaphore);
+  if (memory_semaphore == (SemaphoreInfo *) NULL)
+    AcquireSemaphoreInfo(&memory_semaphore);
+  (void) LockSemaphoreInfo(memory_semaphore);
+  (void) UnlockSemaphoreInfo(memory_semaphore);
   for (i=0; i < (long) memory_info.number_segments; i++)
     if (memory_info.segments[i]->mapped == MagickFalse)
       memory_methods.destroy_memory_handler(
@@ -629,14 +688,45 @@ MagickExport void GetMagickMemoryMethods(
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   R e l i n q u i s h A l i g n e d M e m o r y                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  RelinquishAlignedMemory() frees memory acquired with AcquireAlignedMemory()
+%  or reuse.
+%
+%  The format of the RelinquishAlignedMemory method is:
+%
+%      void *RelinquishAlignedMemory(void *memory)
+%
+%  A description of each parameter follows:
+%
+%    o memory: A pointer to a block of memory to free for reuse.
+%
+*/
+MagickExport void *RelinquishAlignedMemory(void *memory)
+{
+  if (memory == (void *) NULL)
+    return((void *) NULL);
+  free(memory);
+  return((void *) NULL);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e l i n q u i s h M a g i c k M e m o r y                               %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RelinquishMagickMemory() zeros memory that has been allocated, frees it for
-%  reuse.
+%  RelinquishMagickMemory() frees memory acquired with AcquireMagickMemory()
+%  or AcquireQuantumMemory() for reuse.
 %
 %  The format of the RelinquishMagickMemory method is:
 %
@@ -656,7 +746,7 @@ MagickExport void *RelinquishMagickMemory(void *memory)
 #else
   assert((SizeOfBlock(memory) % (4*sizeof(size_t))) == 0);
   assert((*BlockHeader(NextBlock(memory)) & PreviousBlockBit) != 0);
-  AcquireSemaphoreInfo(&memory_semaphore);
+  (void) LockSemaphoreInfo(memory_semaphore);
   if ((*BlockHeader(memory) & PreviousBlockBit) == 0)
     {
       void
@@ -687,7 +777,7 @@ MagickExport void *RelinquishMagickMemory(void *memory)
   *BlockFooter(memory,SizeOfBlock(memory))=SizeOfBlock(memory);
   *BlockHeader(NextBlock(memory))&=(~PreviousBlockBit);
   InsertFreeBlock(memory,AllocationPolicy(SizeOfBlock(memory)));
-  RelinquishSemaphoreInfo(memory_semaphore);
+  (void) UnlockSemaphoreInfo(memory_semaphore);
 #endif
   return((void *) NULL);
 }
@@ -784,20 +874,20 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
   if (block == (void *) NULL)
     memory=RelinquishMagickMemory(memory);
 #else
-  AcquireSemaphoreInfo(&memory_semaphore);
+  (void) LockSemaphoreInfo(memory_semaphore);
   block=ResizeBlock(memory,size == 0 ? 1UL : size);
   if (block == (void *) NULL)
     {
       if (ExpandHeap(size == 0 ? 1UL : size) == MagickFalse)
         {
-          RelinquishSemaphoreInfo(memory_semaphore);
+          (void) UnlockSemaphoreInfo(memory_semaphore);
           memory=RelinquishMagickMemory(memory);
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         }
       block=ResizeBlock(memory,size == 0 ? 1UL : size);
       assert(block != (void *) NULL);
     }
-  RelinquishSemaphoreInfo(memory_semaphore);
+  (void) UnlockSemaphoreInfo(memory_semaphore);
   memory=RelinquishMagickMemory(memory);
 #endif
   return(block);
