@@ -49,6 +49,7 @@
 #include "magick/image.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/profile.h"
 #include "magick/property.h"
@@ -56,6 +57,7 @@
 #include "magick/quantum-private.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
+#include "magick/thread-private.h"
 #include "magick/token.h"
 #include "magick/utility.h"
 #if defined(MAGICKCORE_LCMS_DELEGATE)
@@ -309,6 +311,93 @@ MagickExport char *GetNextImageProfile(const Image *image)
 %
 */
 
+#if defined(MAGICKCORE_LCMS_DELEGATE)
+
+static unsigned short **DestroyPixelThreadSet(unsigned short **pixels)
+{
+  register long
+    i;
+
+  assert(pixels != (unsigned short **) NULL);
+  for (i=0; i < (long) GetOpenMPMaximumThreads(); i++)
+    if (pixels[i] != (unsigned short *) NULL)
+      pixels[i]=(unsigned short *) RelinquishMagickMemory(pixels[i]);
+  pixels=(unsigned short **) RelinquishAlignedMemory(pixels);
+  return(pixels);
+}
+
+static unsigned short **AcquirePixelThreadSet(const size_t columns,
+  const size_t channels)
+{
+  register long
+    i;
+
+  unsigned short
+    **pixels;
+
+  unsigned long
+    number_threads;
+
+  number_threads=GetOpenMPMaximumThreads();
+  pixels=(unsigned short **) AcquireAlignedMemory(number_threads,
+    sizeof(*pixels));
+  if (pixels == (unsigned short **) NULL)
+    return((unsigned short **) NULL);
+  (void) ResetMagickMemory(pixels,0,number_threads*sizeof(*pixels));
+  for (i=0; i < (long) number_threads; i++)
+  {
+    pixels[i]=(unsigned short *) AcquireQuantumMemory(columns,channels*
+      sizeof(**pixels));
+    if (pixels[i] == (unsigned short *) NULL)
+      return(DestroyPixelThreadSet(pixels));
+  }
+  return(pixels);
+}
+
+static cmsHTRANSFORM *DestroyTransformThreadSet(cmsHTRANSFORM *transform)
+{
+  register long
+    i;
+
+  assert(transform != (cmsHTRANSFORM *) NULL);
+  for (i=0; i < (long) GetOpenMPMaximumThreads(); i++)
+    if (transform[i] != (cmsHTRANSFORM) NULL)
+      cmsDeleteTransform(transform[i]);
+  transform=(cmsHTRANSFORM *) RelinquishAlignedMemory(transform);
+  return(transform);
+}
+
+static cmsHTRANSFORM *AcquireTransformThreadSet(
+  const cmsHPROFILE source_profile,const DWORD source_type,
+  const cmsHPROFILE target_profile,const DWORD target_type,const int intent,
+  const DWORD flags)
+{
+  cmsHTRANSFORM
+    *transform;
+
+  register long
+    i;
+
+  unsigned long
+    number_threads;
+
+  number_threads=GetOpenMPMaximumThreads();
+  transform=(cmsHTRANSFORM *) AcquireAlignedMemory(number_threads,
+    sizeof(*transform));
+  if (transform == (cmsHTRANSFORM *) NULL)
+    return((cmsHTRANSFORM *) NULL);
+  (void) ResetMagickMemory(transform,0,number_threads*sizeof(*transform));
+  for (i=0; i < (long) number_threads; i++)
+  {
+    transform[i]=cmsCreateTransform(source_profile,source_type,target_profile,
+      target_type,intent,flags);
+    if (transform[i] == (cmsHTRANSFORM) NULL)
+      return(DestroyTransformThreadSet(transform));
+  }
+  return(transform);
+}
+#endif
+
 static MagickBooleanType SetAdobeRGB1998ImageProfile(Image *image)
 {
   static unsigned char
@@ -695,11 +784,11 @@ static MagickBooleanType SetsRGBImageProfile(Image *image)
 }
 #if defined(MAGICKCORE_LCMS_DELEGATE)
 #if defined(LCMS_VERSION) && (LCMS_VERSION > 1010)
-static int LCMSErrorHandler(int severity,const char *context)
+static int LCMSErrorHandler(int severity,const char *message)
 {
   (void) LogMagickEvent(TransformEvent,GetMagickModule(),"lcms: #%d, %s",
-    severity,context != (char *) NULL ? context : "no context");
-  return(MagickTrue);
+    severity,message != (char *) NULL ? message : "no message");
+  return(1);
 }
 #endif
 #endif
@@ -708,6 +797,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
   const void *datum,const size_t length,
   const MagickBooleanType magick_unused(clone))
 {
+#define ProfileImageTag  "Profile/Image"
 #define ThrowProfileException(severity,tag,context) \
 { \
   (void) cmsCloseProfile(source_profile); \
@@ -810,6 +900,9 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
 #else
       if (icc_profile != (StringInfo *) NULL)
         {
+          CacheView
+            *image_view;
+
           ColorspaceType
             source_colorspace,
             target_colorspace;
@@ -819,7 +912,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             target_profile;
 
           cmsHTRANSFORM
-            transform;
+            *transform;
 
           DWORD
             flags,
@@ -829,23 +922,15 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           ExceptionInfo
             *exception;
 
-          IndexPacket
-            *indexes;
-
           int
             intent;
 
           long
+            progress,
             y;
 
-          register long
-            x;
-
-          register PixelPacket
-            *q;
-
-          register unsigned short
-            *p;
+          MagickBooleanType
+            status;
 
           size_t
             length,
@@ -853,8 +938,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             target_channels;
 
           unsigned short
-            *source_pixels,
-            *target_pixels;
+            **source_pixels,
+            **target_pixels;
 
           /*
             Transform pixel colors as defined by the color profiles.
@@ -881,11 +966,11 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               source_channels=4;
               break;
             }
-            case icSigYCbCrData:
+            case icSigGrayData:
             {
-              source_colorspace=YCbCrColorspace;
-              source_type=(DWORD) TYPE_YCbCr_16;
-              source_channels=3;
+              source_colorspace=GRAYColorspace;
+              source_type=(DWORD) TYPE_GRAY_16;
+              source_channels=1;
               break;
             }
             case icSigLabData:
@@ -902,13 +987,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               source_channels=3;
               break;
             }
-            case icSigGrayData:
-            {
-              source_colorspace=GRAYColorspace;
-              source_type=(DWORD) TYPE_GRAY_16;
-              source_channels=1;
-              break;
-            }
             case icSigRgbData:
             {
               source_colorspace=RGBColorspace;
@@ -916,6 +994,20 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               source_channels=3;
               break;
             }
+            case icSigXYZData:
+            {
+              source_colorspace=XYZColorspace;
+              source_type=(DWORD) TYPE_XYZ_16;
+              source_channels=3;
+              break;
+            }
+            case icSigYCbCrData:
+            {
+              source_colorspace=YCbCrColorspace;
+              source_type=(DWORD) TYPE_YCbCr_16;
+              source_channels=3;
+              break;
+            }
             default:
             {
               source_colorspace=UndefinedColorspace;
@@ -933,13 +1025,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=4;
               break;
             }
-            case icSigYCbCrData:
-            {
-              target_colorspace=YCbCrColorspace;
-              target_type=(DWORD) TYPE_YCbCr_16;
-              target_channels=3;
-              break;
-            }
             case icSigLabData:
             {
               target_colorspace=LabColorspace;
@@ -947,13 +1032,6 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=3;
               break;
             }
-            case icSigLuvData:
-            {
-              target_colorspace=YUVColorspace;
-              target_type=(DWORD) TYPE_YUV_16;
-              target_channels=3;
-              break;
-            }
             case icSigGrayData:
             {
               target_colorspace=GRAYColorspace;
@@ -961,6 +1039,13 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=1;
               break;
             }
+            case icSigLuvData:
+            {
+              target_colorspace=YUVColorspace;
+              target_type=(DWORD) TYPE_YUV_16;
+              target_channels=3;
+              break;
+            }
             case icSigRgbData:
             {
               target_colorspace=RGBColorspace;
@@ -968,6 +1053,20 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_channels=3;
               break;
             }
+            case icSigXYZData:
+            {
+              target_colorspace=XYZColorspace;
+              target_type=(DWORD) TYPE_XYZ_16;
+              target_channels=3;
+              break;
+            }
+            case icSigYCbCrData:
+            {
+              target_colorspace=YCbCrColorspace;
+              target_type=(DWORD) TYPE_YCbCr_16;
+              target_channels=3;
+              break;
+            }
             default:
             {
               target_colorspace=UndefinedColorspace;
@@ -977,16 +1076,32 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             }
           }
           exception=(&image->exception);
-          if (((source_colorspace == UndefinedColorspace) ||
-               (target_colorspace == UndefinedColorspace)) ||
-              ((source_colorspace == GRAYColorspace) &&
-               (!IsGrayImage(image,exception))) ||
-              ((source_colorspace == CMYKColorspace) &&
-               (image->colorspace != CMYKColorspace)) ||
-              ((source_colorspace != GRAYColorspace) &&
+          if ((source_colorspace == UndefinedColorspace) ||
+              (target_colorspace == UndefinedColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == GRAYColorspace) &&
+               (IsGrayImage(image,exception) == MagickFalse))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == CMYKColorspace) &&
+               (image->colorspace != CMYKColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == XYZColorspace) &&
+               (image->colorspace != XYZColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace == YCbCrColorspace) &&
+               (image->colorspace != YCbCrColorspace))
+            ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
+              name);
+           if ((source_colorspace != CMYKColorspace) &&
+               (source_colorspace != GRAYColorspace) &&
                (source_colorspace != LabColorspace) &&
-               (source_colorspace != CMYKColorspace) &&
-               (image->colorspace != RGBColorspace)))
+               (source_colorspace != XYZColorspace) &&
+               (source_colorspace != YCbCrColorspace) &&
+               (image->colorspace != RGBColorspace))
             ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
               name);
           switch (image->rendering_intent)
@@ -1002,39 +1117,72 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           if (image->black_point_compensation != MagickFalse)
             flags|=cmsFLAGS_BLACKPOINTCOMPENSATION;
 #endif
-          transform=cmsCreateTransform(source_profile,source_type,
+          transform=AcquireTransformThreadSet(source_profile,source_type,
             target_profile,target_type,intent,flags);
           (void) cmsCloseProfile(source_profile);
-          (void) cmsCloseProfile(target_profile);
-          if (transform == (cmsHTRANSFORM) NULL)
+          if (transform == (cmsHTRANSFORM *) NULL)
             ThrowBinaryException(ImageError,"UnableToCreateColorTransform",
               name);
           /*
             Transform image as dictated by the source and target image profiles.
           */
           length=(size_t) image->columns;
-          source_pixels=(unsigned short *) AcquireQuantumMemory(length,
-            source_channels*sizeof(*source_pixels));
-          target_pixels=(unsigned short *) AcquireQuantumMemory(length,
-            target_channels*sizeof(*target_pixels));
-          if ((source_pixels == (unsigned short *) NULL) ||
-              (target_pixels == (unsigned short *) NULL))
+          source_pixels=AcquirePixelThreadSet(image->columns,source_channels);
+          target_pixels=AcquirePixelThreadSet(image->columns,target_channels);
+          if ((source_pixels == (unsigned short **) NULL) ||
+              (target_pixels == (unsigned short **) NULL))
             {
-              cmsDeleteTransform(transform);
+              transform=DestroyTransformThreadSet(transform);
+              (void) cmsCloseProfile(target_profile);
               ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
                 image->filename);
             }
           if (SetImageStorageClass(image,DirectClass) == MagickFalse)
-            return(MagickFalse);
+            {
+              target_pixels=DestroyPixelThreadSet(target_pixels);
+              source_pixels=DestroyPixelThreadSet(source_pixels);
+              transform=DestroyTransformThreadSet(transform);
+              (void) cmsCloseProfile(target_profile);
+              return(MagickFalse);
+            }
           if (target_colorspace == CMYKColorspace)
-            image->colorspace=target_colorspace;
+            (void) SetImageColorspace(image,target_colorspace);
+          status=MagickTrue;
+          progress=0;
+          image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+          #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
           for (y=0; y < (long) image->rows; y++)
           {
-            q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+            MagickBooleanType
+              sync;
+
+            register IndexPacket
+              *__restrict indexes;
+
+            register long
+              id,
+              x;
+
+            register PixelPacket
+              *__restrict q;
+
+            register unsigned short
+              *p;
+
+            if (status == MagickFalse)
+              continue;
+            q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
+              exception);
             if (q == (PixelPacket *) NULL)
-              break;
-            p=source_pixels;
-            indexes=GetAuthenticIndexQueue(image);
+              {
+                status=MagickFalse;
+                continue;
+              }
+            indexes=GetCacheViewAuthenticIndexQueue(image_view);
+            id=GetOpenMPThreadId();
+            p=source_pixels[id];
             for (x=0; x < (long) image->columns; x++)
             {
               *p++=ScaleQuantumToShort(q->red);
@@ -1047,9 +1195,9 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 *p++=ScaleQuantumToShort(indexes[x]);
               q++;
             }
-            cmsDoTransform(transform,source_pixels,target_pixels,(unsigned int)
-              image->columns);
-            p=target_pixels;
+            cmsDoTransform(transform[id],source_pixels[id],target_pixels[id],
+              (unsigned int) image->columns);
+            p=target_pixels[id];
             q-=image->columns;
             for (x=0; x < (long) image->columns; x++)
             {
@@ -1071,10 +1219,25 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 }
               q++;
             }
-            if (SyncAuthenticPixels(image,exception) == MagickFalse)
-              break;
+            sync=SyncCacheViewAuthenticPixels(image_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
+            if (image->progress_monitor != (MagickProgressMonitor) NULL)
+              {
+                MagickBooleanType
+                  proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_ProfileImage)
+#endif
+                proceed=SetImageProgress(image,ProfileImageTag,progress++,
+                  image->rows);
+                if (proceed == MagickFalse)
+                  status=MagickFalse;
+              }
           }
-          image->colorspace=target_colorspace;
+          image_view=DestroyCacheView(image_view);
+          (void) SetImageColorspace(image,target_colorspace);
           switch (cmsGetColorSpace(target_profile))
           {
             case icSigRgbData:
@@ -1098,11 +1261,10 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             default:
               break;
           }
-          target_pixels=(unsigned short *) RelinquishMagickMemory(
-            target_pixels);
-          source_pixels=(unsigned short *) RelinquishMagickMemory(
-            source_pixels);
-          cmsDeleteTransform(transform);
+          target_pixels=DestroyPixelThreadSet(target_pixels);
+          source_pixels=DestroyPixelThreadSet(source_pixels);
+          transform=DestroyTransformThreadSet(transform);
+          (void) cmsCloseProfile(target_profile);
         }
 #endif
     }
@@ -1717,6 +1879,7 @@ MagickExport MagickBooleanType SyncImageProfiles(Image *image)
         {
           (void) WriteProfileLong(endian,(unsigned long)
             (image->x_resolution+0.5),p);
+          (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x011b:
@@ -1730,7 +1893,6 @@ MagickExport MagickBooleanType SyncImageProfiles(Image *image)
         {
           (void) WriteProfileShort(endian,(unsigned short)
             image->orientation,p);
-          (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x0128:
