@@ -10,14 +10,14 @@
 %               SSSSS  EEEEE   GGGG  M   M  EEEEE  N   N    T                 %
 %                                                                             %
 %                                                                             %
-%        Methods to Segment an Image with Thresholding Fuzzy c-Means          %
+%    MagickCore Methods to Segment an Image with Thresholding Fuzzy c-Means   %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -83,6 +83,7 @@
 */
 
 #include "magick/studio.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
@@ -91,7 +92,10 @@
 #include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
+#include "magick/quantum-private.h"
 #include "magick/segment.h"
 #include "magick/string_.h"
 
@@ -100,7 +104,6 @@
 */
 #define MaxDimension  3
 #define DeltaTau  0.5f
-#define FastClassify  1
 #if defined(FastClassify)
 #define WeightingExponent  2.0
 #define SegmentPower(ratio) (ratio)
@@ -203,8 +206,8 @@ static void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Classify() defines on ore more classes.  Each pixel is thresholded to
-%  determine which class it belongs to.  If not class is identified it is
+%  Classify() defines one or more classes.  Each pixel is thresholded to
+%  determine which class it belongs to.  If the class is not identified it is
 %  assigned to the closest class based on the fuzzy c-Means technique.
 %
 %  The format of the Classify method is:
@@ -216,7 +219,7 @@ static void
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o extrema:  Specifies a pointer to an array of integers.  They
 %      represent the peaks and valleys of the histogram for each color
@@ -231,12 +234,10 @@ static void
 %    o verbose:  A value greater than zero prints detailed information about
 %      the identified classes.
 %
-%
 */
 static MagickBooleanType Classify(Image *image,short **extrema,
   const MagickRealType cluster_threshold,
-  const MagickRealType magick_unused(weighting_exponent),
-  const MagickBooleanType verbose)
+  const MagickRealType weighting_exponent,const MagickBooleanType verbose)
 {
 #define SegmentImageTag  "Segment/Image"
 
@@ -246,50 +247,37 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     *last_cluster,
     *next_cluster;
 
+  ExceptionInfo
+    *exception;
+
   ExtentPacket
     blue,
     green,
     red;
 
-  IndexPacket
-    index;
-
   long
     count,
-    j,
-    k,
+    progress,
     y;
 
-  MagickBooleanType
+  MagickRealType
+    *free_squares;
+
+  MagickStatusType
     status;
 
-  MagickRealType
-    distance_squared,
-    *free_squares,
-    local_minima,
-    numerator,
-    ratio,
-    sum;
-
-  register const PixelPacket
-    *p;
-
-  register IndexPacket
-    *indexes;
-
   register long
-    i,
-    x;
+    i;
 
   register MagickRealType
     *squares;
 
-  register PixelPacket
-    *q;
-
   unsigned long
     number_clusters;
 
+  CacheView
+    *image_view;
+
   /*
     Form clusters.
   */
@@ -311,8 +299,8 @@ static MagickBooleanType Classify(Image *image,short **extrema,
         */
         if (head != (Cluster *) NULL)
           {
-            cluster->next=(Cluster *)
-              AcquireMagickMemory(sizeof(*cluster->next));
+            cluster->next=(Cluster *) AcquireMagickMemory(
+              sizeof(*cluster->next));
             cluster=cluster->next;
           }
         else
@@ -356,10 +344,20 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Count the pixels for each cluster.
   */
+  status=MagickTrue;
   count=0;
+  progress=0;
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    register const PixelPacket
+      *p;
+
+    register long
+      x;
+
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
@@ -382,26 +380,30 @@ static MagickBooleanType Classify(Image *image,short **extrema,
               Count this pixel.
             */
             count++;
-            cluster->red.center+=(MagickRealType)
-              ScaleQuantumToChar(p->red);
+            cluster->red.center+=(MagickRealType) ScaleQuantumToChar(p->red);
             cluster->green.center+=(MagickRealType)
               ScaleQuantumToChar(p->green);
-            cluster->blue.center+=(MagickRealType)
-              ScaleQuantumToChar(p->blue);
+            cluster->blue.center+=(MagickRealType) ScaleQuantumToChar(p->blue);
             cluster->count++;
             break;
           }
       p++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical (MagickCore_Classify)
+#endif
+        proceed=SetImageProgress(image,SegmentImageTag,progress++,
+          2*image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
+  image_view=DestroyCacheView(image_view);
   /*
     Remove clusters that do not meet minimum cluster threshold.
   */
@@ -442,6 +444,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       */
       (void) fprintf(stdout,"Fuzzy C-means Statistics\n");
       (void) fprintf(stdout,"===================\n\n");
+      (void) fprintf(stdout,"\tCluster Threshold = %g\n",(double)
+        cluster_threshold);
+      (void) fprintf(stdout,"\tWeighting Exponent = %g\n",(double)
+        weighting_exponent);
       (void) fprintf(stdout,"\tTotal Number of Clusters = %lu\n\n",
         number_clusters);
       /*
@@ -484,7 +490,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Speed up distance calculations.
   */
-  squares=(MagickRealType *) AcquireMagickMemory(513*sizeof(*squares));
+  squares=(MagickRealType *) AcquireQuantumMemory(513UL,sizeof(*squares));
   if (squares == (MagickRealType *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
@@ -494,31 +500,58 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Allocate image colormap.
   */
-  if (AllocateImageColormap(image,number_clusters) == MagickFalse)
+  if (AcquireImageColormap(image,number_clusters) == MagickFalse)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  cluster=head;
-  for (i=0; i < (long) number_clusters; i++)
+  i=0;
+  for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
   {
-    image->colormap[i].red=ScaleCharToQuantum(cluster->red.center);
-    image->colormap[i].green=ScaleCharToQuantum(cluster->green.center);
-    image->colormap[i].blue=ScaleCharToQuantum(cluster->blue.center);
-    cluster=cluster->next;
+    image->colormap[i].red=ScaleCharToQuantum((unsigned char)
+      (cluster->red.center+0.5));
+    image->colormap[i].green=ScaleCharToQuantum((unsigned char)
+      (cluster->green.center+0.5));
+    image->colormap[i].blue=ScaleCharToQuantum((unsigned char)
+      (cluster->blue.center+0.5));
+    i++;
   }
   /*
-    Do course grain storage_class.
+    Do course grain classes.
   */
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    Cluster
+      *cluster;
+
+    register const PixelPacket
+      *__restrict p;
+
+    register IndexPacket
+      *__restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *__restrict q;
+
+    if (status == MagickFalse)
+      continue;
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {
-      index=0;
-      cluster=head;
-      for (i=0; i < (long) number_clusters; i++)
+      indexes[x]=(IndexPacket) 0;
+      for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
       {
         if (((long) ScaleQuantumToChar(q->red) >=
              (cluster->red.left-SafeMargin)) &&
@@ -536,13 +569,23 @@ static MagickBooleanType Classify(Image *image,short **extrema,
             /*
               Classify this pixel.
             */
-            index=(IndexPacket) cluster->id;
+            indexes[x]=(IndexPacket) cluster->id;
             break;
           }
-        cluster=cluster->next;
       }
       if (cluster == (Cluster *) NULL)
         {
+          MagickRealType
+            distance_squared,
+            local_minima,
+            numerator,
+            ratio,
+            sum;
+
+          register long
+            j,
+            k;
+
           /*
             Compute fuzzy membership.
           */
@@ -576,27 +619,32 @@ static MagickBooleanType Classify(Image *image,short **extrema,
                   Classify this pixel.
                 */
                 local_minima=1.0/sum;
-                index=(IndexPacket) j;
+                indexes[x]=(IndexPacket) j;
               }
           }
         }
-      indexes[x]=index;
-      *q=image->colormap[index];
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical (MagickCore_Classify)
+#endif
+        proceed=SetImageProgress(image,SegmentImageTag,progress++,
+          2*image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
       }
   }
+  image_view=DestroyCacheView(image_view);
+  status&=SyncImage(image);
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster)
   {
@@ -635,8 +683,29 @@ static MagickBooleanType Classify(Image *image,short **extrema,
 %    o number_crossings: This unsigned long specifies the number of elements
 %      in the zero_crossing array.
 %
-%
 */
+
+static inline long MagickAbsoluteValue(const long x)
+{
+  if (x < 0)
+    return(-x);
+  return(x);
+}
+
+static inline long MagickMax(const long x,const long y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline long MagickMin(const long x,const long y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static void ConsolidateCrossings(ZeroCrossing *zero_crossing,
   const unsigned long number_crossings)
 {
@@ -669,12 +738,12 @@ static void ConsolidateCrossings(ZeroCrossing *zero_crossing,
       for (k=j-1; k > 0; k--)
         if (zero_crossing[i+1].crossings[k] != 0)
           break;
-      left=Max(k,0);
+      left=MagickMax(k,0);
       center=j;
       for (k=j+1; k < 255; k++)
         if (zero_crossing[i+1].crossings[k] != 0)
           break;
-      right=Min(k,255);
+      right=MagickMin(k,255);
       /*
         K is the zero crossing just left of j.
       */
@@ -753,7 +822,6 @@ static void ConsolidateCrossings(ZeroCrossing *zero_crossing,
 %    o extents:  This pointer to an ExtentPacket represent the extends
 %      of a particular peak or valley of a color component.
 %
-%
 */
 static long DefineRegion(const short *extrema,ExtentPacket *extents)
 {
@@ -810,7 +878,6 @@ static long DefineRegion(const short *extrema,ExtentPacket *extents)
 %      DerivativeHistogram to the derivative of the histogram using central
 %      differencing.
 %
-%
 */
 static void DerivativeHistogram(const MagickRealType *histogram,
   MagickRealType *derivative)
@@ -848,13 +915,13 @@ static void DerivativeHistogram(const MagickRealType *histogram,
 %
 %  The format of the GetImageDynamicThreshold method is:
 %
-%      MagickPixelPacket GetImageDynamicThreshold(const Image *image,
+%      MagickBooleanType GetImageDynamicThreshold(const Image *image,
 %        const double cluster_threshold,const double smooth_threshold,
-%        ExceptionInfo *)
+%        MagickPixelPacket *pixel,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o cluster_threshold:  This MagickRealType represents the minimum number of
 %      pixels contained in a hexahedra before it can be considered valid
@@ -864,13 +931,14 @@ static void DerivativeHistogram(const MagickRealType *histogram,
 %      derivative of the histogram.  As the value is increased, you can expect a
 %      smoother second derivative.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o pixel: return the dynamic threshold here.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
+MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
   const double cluster_threshold,const double smooth_threshold,
-  ExceptionInfo *exception)
+  MagickPixelPacket *pixel,ExceptionInfo *exception)
 {
   Cluster
     *background,
@@ -891,10 +959,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
     y;
 
   MagickBooleanType
-    status;
-
-  MagickPixelPacket
-    pixel;
+    proceed;
 
   MagickRealType
     threshold;
@@ -916,11 +981,11 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
+  GetMagickPixelPacket(image,pixel);
   for (i=0; i < MaxDimension; i++)
   {
-    histogram[i]=(long *) AcquireMagickMemory(256*sizeof(**histogram));
-    extrema[i]=(short *) AcquireMagickMemory(256*sizeof(**histogram));
+    histogram[i]=(long *) AcquireQuantumMemory(256UL,sizeof(**histogram));
+    extrema[i]=(short *) AcquireQuantumMemory(256UL,sizeof(**histogram));
     if ((histogram[i] == (long *) NULL) || (extrema[i] == (short *) NULL))
       {
         for (i-- ; i >= 0; i--)
@@ -930,7 +995,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         }
         (void) ThrowMagickException(exception,GetMagickModule(),
           ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-        return(pixel);
+        return(MagickFalse);
       }
   }
   /*
@@ -949,7 +1014,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   cluster=(Cluster *) NULL;
   head=(Cluster *) NULL;
   (void) ResetMagickMemory(&red,0,sizeof(red));
-	(void) ResetMagickMemory(&green,0,sizeof(green));
+  (void) ResetMagickMemory(&green,0,sizeof(green));
   (void) ResetMagickMemory(&blue,0,sizeof(blue));
   while (DefineRegion(extrema[Red],&red) != 0)
   {
@@ -964,8 +1029,8 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         */
         if (head != (Cluster *) NULL)
           {
-            cluster->next=(Cluster *)
-              AcquireMagickMemory(sizeof(*cluster->next));
+            cluster->next=(Cluster *) AcquireMagickMemory(
+              sizeof(*cluster->next));
             cluster=cluster->next;
           }
         else
@@ -978,7 +1043,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
             (void) ThrowMagickException(exception,GetMagickModule(),
               ResourceLimitError,"MemoryAllocationFailed","`%s'",
               image->filename);
-            return(pixel);
+            return(MagickFalse);
           }
         /*
           Initialize a new class.
@@ -1001,7 +1066,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-          return(pixel);
+          return(MagickFalse);
         }
       /*
         Initialize a new class.
@@ -1019,7 +1084,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   count=0;
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
@@ -1053,14 +1118,9 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
           }
       p++;
     }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,2*image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(SegmentImageTag,y,2*image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
+    if (proceed == MagickFalse)
+      break;
   }
   /*
     Remove clusters that do not meet minimum cluster threshold.
@@ -1114,13 +1174,16 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
       }
     }
   threshold=(background->red.center+object->red.center)/2.0;
-  pixel.red=(MagickRealType) ScaleCharToQuantum(threshold);
+  pixel->red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+    (threshold+0.5));
   threshold=(background->green.center+object->green.center)/2.0;
-  pixel.green=(MagickRealType) ScaleCharToQuantum(threshold);
+  pixel->green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+    (threshold+0.5));
   threshold=(background->blue.center+object->blue.center)/2.0;
-  pixel.blue=(MagickRealType) ScaleCharToQuantum(threshold);
+  pixel->blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+    (threshold+0.5));
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster)
   {
@@ -1132,7 +1195,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
     extrema[i]=(short *) RelinquishMagickMemory(extrema[i]);
     histogram[i]=(long *) RelinquishMagickMemory(histogram[i]);
   }
-  return(pixel);
+  return(MagickTrue);
 }
 
 /*
@@ -1160,7 +1223,6 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
 %    o histogram: Specifies an array of integers representing the number
 %      of pixels for each intensity of a particular color component.
 %
-%
 */
 static void InitializeHistogram(const Image *image,long **histogram,
   ExceptionInfo *exception)
@@ -1186,7 +1248,7 @@ static void InitializeHistogram(const Image *image,long **histogram,
   }
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
@@ -1225,7 +1287,6 @@ static void InitializeHistogram(const Image *image,long **histogram,
 %    o number_crossings: This unsigned long specifies the number of elements
 %      in the zero_crossing array.
 %
-%
 */
 
 static void InitializeList(IntervalTree **list,long *number_nodes,
@@ -1302,7 +1363,8 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
   /*
     Allocate interval tree.
   */
-  list=(IntervalTree **) AcquireMagickMemory(TreeLength*sizeof(*list));
+  list=(IntervalTree **) AcquireQuantumMemory((size_t) TreeLength,
+    sizeof(*list));
   if (list == (IntervalTree **) NULL)
     return((IntervalTree *) NULL);
   /*
@@ -1335,14 +1397,14 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
           {
             if (node == head)
               {
-                node->child=(IntervalTree *)
-                  AcquireMagickMemory(sizeof(*node->child));
+                node->child=(IntervalTree *) AcquireMagickMemory(
+                  sizeof(*node->child));
                 node=node->child;
               }
             else
               {
-                node->sibling=(IntervalTree *)
-                  AcquireMagickMemory(sizeof(*node->sibling));
+                node->sibling=(IntervalTree *) AcquireMagickMemory(
+                  sizeof(*node->sibling));
                 node=node->sibling;
               }
             node->tau=zero_crossing[i+1].tau;
@@ -1355,8 +1417,8 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
         }
       if (left != head->left)
         {
-          node->sibling=(IntervalTree *)
-            AcquireMagickMemory(sizeof(*node->sibling));
+          node->sibling=(IntervalTree *) AcquireMagickMemory(
+            sizeof(*node->sibling));
           node=node->sibling;
           node->tau=zero_crossing[i+1].tau;
           node->child=(IntervalTree *) NULL;
@@ -1403,7 +1465,6 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
 %      represent the peaks and valleys of the histogram for each color
 %      component.
 %
-%
 */
 
 static void ActiveNodes(IntervalTree **list,long *number_nodes,
@@ -1471,15 +1532,16 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
   /*
     Allocate interval tree.
   */
-  list=(IntervalTree **) AcquireMagickMemory(TreeLength*sizeof(*list));
+  list=(IntervalTree **) AcquireQuantumMemory((size_t) TreeLength,
+    sizeof(*list));
   if (list == (IntervalTree **) NULL)
     return(0.0);
   /*
     Allocate zero crossing list.
   */
   count=(unsigned long) ((max_tau-min_tau)/delta_tau)+2;
-  zero_crossing=(ZeroCrossing *)
-    AcquireMagickMemory((size_t) count*sizeof(*zero_crossing));
+  zero_crossing=(ZeroCrossing *) AcquireQuantumMemory((size_t) count,
+    sizeof(*zero_crossing));
   if (zero_crossing == (ZeroCrossing *) NULL)
     return(0.0);
   for (i=0; i < (long) count; i++)
@@ -1487,13 +1549,13 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
   /*
     Initialize zero crossing list.
   */
-  derivative=(MagickRealType *) AcquireMagickMemory(256*sizeof(*derivative));
-  second_derivative=(MagickRealType *)
-    AcquireMagickMemory(256*sizeof(*second_derivative));
+  derivative=(MagickRealType *) AcquireQuantumMemory(256,sizeof(*derivative));
+  second_derivative=(MagickRealType *) AcquireQuantumMemory(256,
+    sizeof(*second_derivative));
   if ((derivative == (MagickRealType *) NULL) ||
       (second_derivative == (MagickRealType *) NULL))
-    ThrowMagickFatalException(ResourceLimitFatalError,
-      "UnableToAllocateDerivatives","OptimalTau");
+    ThrowFatalException(ResourceLimitFatalError,
+      "UnableToAllocateDerivatives");
   i=0;
   for (tau=max_tau; tau >= min_tau; tau-=delta_tau)
   {
@@ -1567,7 +1629,8 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
     /*
       Find the value of the peak.
     */
-    peak=(MagickBooleanType) (zero_crossing[k].crossings[node->right] == -1);
+    peak=zero_crossing[k].crossings[node->right] == -1 ? MagickTrue :
+      MagickFalse;
     index=node->left;
     value=zero_crossing[k].histogram[index];
     for (x=node->left; x <= node->right; x++)
@@ -1605,7 +1668,7 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
     average_tau+=list[i]->tau;
   average_tau/=(MagickRealType) number_nodes;
   /*
-    Free resources.
+    Relinquish resources.
   */
   FreeNodes(root);
   zero_crossing=(ZeroCrossing *) RelinquishMagickMemory(zero_crossing);
@@ -1633,11 +1696,11 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
 %
 %  A description of each parameter follows.
 %
-%    o histogram: Specifies an array of MagickRealTypes representing the number of
-%      pixels for each intensity of a particular color component.
-%
+%    o histogram: Specifies an array of MagickRealTypes representing the number
+%      of pixels for each intensity of a particular color component.
 %
 */
+
 static void ScaleSpace(const long *histogram,const MagickRealType tau,
   MagickRealType *scale_histogram)
 {
@@ -1651,10 +1714,10 @@ static void ScaleSpace(const long *histogram,const MagickRealType tau,
     u,
     x;
 
-  gamma=(MagickRealType *) AcquireMagickMemory(256*sizeof(*gamma));
+  gamma=(MagickRealType *) AcquireQuantumMemory(256,sizeof(*gamma));
   if (gamma == (MagickRealType *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,
-      "UnableToAllocateGammaMap","ScaleSpace");
+    ThrowFatalException(ResourceLimitFatalError,
+      "UnableToAllocateGammaMap");
   alpha=1.0/(tau*sqrt(2.0*MagickPI));
   beta=(-1.0/(2.0*tau*tau));
   for (x=0; x <= 255; x++)
@@ -1669,7 +1732,7 @@ static void ScaleSpace(const long *histogram,const MagickRealType tau,
   {
     sum=0.0;
     for (u=0; u <= 255; u++)
-      sum+=(MagickRealType) histogram[u]*gamma[AbsoluteValue(x-u)];
+      sum+=(MagickRealType) histogram[u]*gamma[MagickAbsoluteValue(x-u)];
     scale_histogram[x]=alpha*sum;
   }
   gamma=(MagickRealType *) RelinquishMagickMemory(gamma);
@@ -1698,22 +1761,21 @@ static void ScaleSpace(const long *histogram,const MagickRealType tau,
 %
 %  A description of each parameter follows.
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o colorspace: Indicate the colorspace.
 %
-%    o verbose:  A value greater than zero prints detailed information about
-%      the identified classes.
+%    o verbose:  Set to MagickTrue to print detailed information about the
+%      identified classes.
 %
-%    o cluster_threshold:  This MagickRealType represents the minimum number of
-%      pixels contained in a hexahedra before it can be considered valid
-%      (expressed as a percentage).
+%    o cluster_threshold:  This represents the minimum number of pixels
+%      contained in a hexahedra before it can be considered valid (expressed
+%      as a percentage).
 %
 %    o smooth_threshold: the smoothing threshold eliminates noise in the second
 %      derivative of the histogram.  As the value is increased, you can expect a
 %      smoother second derivative.
 %
-%
 */
 MagickExport MagickBooleanType SegmentImage(Image *image,
   const ColorspaceType colorspace,const MagickBooleanType verbose,
@@ -1740,8 +1802,8 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   for (i=0; i < MaxDimension; i++)
   {
-    histogram[i]=(long *) AcquireMagickMemory(256*sizeof(**histogram));
-    extrema[i]=(short *) AcquireMagickMemory(256*sizeof(**extrema));
+    histogram[i]=(long *) AcquireQuantumMemory(256,sizeof(**histogram));
+    extrema[i]=(short *) AcquireQuantumMemory(256,sizeof(**extrema));
     if ((histogram[i] == (long *) NULL) || (extrema[i] == (short *) NULL))
       {
         for (i-- ; i >= 0; i--)
@@ -1754,7 +1816,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
       }
   }
   if (colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,colorspace);
+    (void) TransformImageColorspace(image,colorspace);
   /*
     Initialize histogram.
   */
@@ -1770,9 +1832,9 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
   */
   status=Classify(image,extrema,cluster_threshold,WeightingExponent,verbose);
   if (colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,colorspace);
+    (void) TransformImageColorspace(image,colorspace);
   /*
-    Free resources.
+    Relinquish resources.
   */
   for (i=0; i < MaxDimension; i++)
   {
@@ -1811,7 +1873,6 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
 %      -1, 0, or 1 representing the slope of the first derivative of the
 %      of a particular color component.
 %
-%
 */
 static void ZeroCrossHistogram(MagickRealType *second_derivative,
   const MagickRealType smooth_threshold,short *crossings)
