@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,20 +13,18 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image segment methods.
+  MagickCore image segment methods.
 */
-#ifndef _MAGICK_SEGMENT_H
-#define _MAGICK_SEGMENT_H
+#ifndef _MAGICKCORE_SEGMENT_H
+#define _MAGICKCORE_SEGMENT_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-extern MagickExport MagickPixelPacket
-  GetImageDynamicThreshold(const Image *,const double,const double,
-    ExceptionInfo *);
-
 extern MagickExport MagickBooleanType
+  GetImageDynamicThreshold(const Image *,const double,const double,
+    MagickPixelPacket *,ExceptionInfo *),
   SegmentImage(Image *,const ColorspaceType,const MagickBooleanType,
     const double,const double);
 
