@@ -10,7 +10,7 @@
 %        SSSSS  EEEEE  M   M  A   A  P      H   H   OOO   R  R   EEEEE        %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick Semaphore Methods                         %
+%                        MagickCore Semaphore Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                             William Radcliffe                               %
@@ -18,7 +18,7 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,35 +47,22 @@
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
-#if defined(HAVE_PTHREAD)
-#include <pthread.h>
-#endif
-#if defined(__WINDOWS__)
-#include <windows.h>
-#endif
+#include "magick/thread_.h"
+#include "magick/thread-private.h"
 
 /*
   Struct declaractions.
 */
 struct SemaphoreInfo
 {
-#if defined(HAVE_PTHREAD)
-  pthread_mutex_t
+  MagickMutexType
     mutex;
 
-  pthread_t
+  MagickThreadType
     id;
-#endif
-#if defined(__WINDOWS__)
-  CRITICAL_SECTION
-    mutex;
 
-  DWORD
-    id;
-#endif
-
-  MagickBooleanType
-    lock;
+  long
+    reference_count;
 
   unsigned long
     signature;
@@ -84,16 +71,24 @@ struct SemaphoreInfo
 /*
   Static declaractions.
 */
-#if defined(HAVE_PTHREAD)
+#if defined(MAGICKCORE_HAVE_PTHREAD)
 static pthread_mutex_t
   semaphore_mutex = PTHREAD_MUTEX_INITIALIZER;
-#endif
-
-#if defined(__WINDOWS__)
-static volatile LONG
+#elif defined(MAGICKCORE_HAVE_WINTHREADS)
+static LONG
+  semaphore_mutex = 0;
+#else
+static long
   semaphore_mutex = 0;
 #endif
 
+/*
+  Forward declaractions.
+*/
+static void
+  LockMagickMutex(void),
+  UnlockMagickMutex(void);
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -115,27 +110,17 @@ static volatile LONG
 %
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
-%
 */
 MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo **) NULL);
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_lock(&semaphore_mutex);
-#endif
-#if defined(__WINDOWS__)
-  while (InterlockedCompareExchange(&semaphore_mutex,1L,0L) != 0)
-    Sleep(10);
-#endif
   if (*semaphore_info == (SemaphoreInfo *) NULL)
-    *semaphore_info=AllocateSemaphoreInfo();
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_unlock(&semaphore_mutex);
-#endif
-#if defined(__WINDOWS__)
-  InterlockedExchange(&semaphore_mutex,0L);
-#endif
-  (void) LockSemaphoreInfo(*semaphore_info);
+    {
+      LockMagickMutex();
+      if (*semaphore_info == (SemaphoreInfo *) NULL)
+        *semaphore_info=AllocateSemaphoreInfo();
+      UnlockMagickMutex();
+    }
 }
 
 /*
@@ -155,12 +140,6 @@ MagickExport void AcquireSemaphoreInfo(SemaphoreInfo **semaphore_info)
 %
 %      SemaphoreInfo *AllocateSemaphoreInfo(void)
 %
-%  A description of each parameter follows:
-%
-%    o semaphore_info: Method AllocateSemaphoreInfo returns a pointer to an
-%      initialized SemaphoreInfo structure.
-%
-%
 */
 MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
 {
@@ -170,32 +149,45 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
   /*
     Allocate semaphore.
   */
-  semaphore_info=(SemaphoreInfo *) malloc(sizeof(SemaphoreInfo));
+  semaphore_info=(SemaphoreInfo *) AcquireAlignedMemory(1,
+    sizeof(SemaphoreInfo));
   if (semaphore_info == (SemaphoreInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,
-      "UnableToAllocateSemaphoreInfo",strerror(errno));
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) ResetMagickMemory(semaphore_info,0,sizeof(SemaphoreInfo));
   /*
     Initialize the semaphore.
   */
-#if defined(HAVE_PTHREAD)
+#if defined(MAGICKCORE_HAVE_PTHREAD)
   {
     int
       status;
 
-    status=pthread_mutex_init(&semaphore_info->mutex,
-      (const pthread_mutexattr_t *) NULL);
+    pthread_mutexattr_t
+      mutex_info;
+
+    status=pthread_mutexattr_init(&mutex_info);
     if (status != 0)
       {
-        free(semaphore_info);
-        return((SemaphoreInfo *) NULL);
+        semaphore_info=(SemaphoreInfo *) RelinquishAlignedMemory(
+          semaphore_info);
+        ThrowFatalException(ResourceLimitFatalError,
+          "UnableToInitializeSemaphore");
+      }
+    status=pthread_mutex_init(&semaphore_info->mutex,&mutex_info);
+    (void) pthread_mutexattr_destroy(&mutex_info);
+    if (status != 0)
+      {
+        semaphore_info=(SemaphoreInfo *) RelinquishAlignedMemory(
+          semaphore_info);
+        ThrowFatalException(ResourceLimitFatalError,
+          "UnableToInitializeSemaphore");
       }
   }
-#endif
-#if defined(__WINDOWS__)
+#elif defined(MAGICKCORE_HAVE_WINTHREADS)
   InitializeCriticalSection(&semaphore_info->mutex);
 #endif
-  semaphore_info->lock=MagickFalse;
+  semaphore_info->id=GetMagickThreadId();
+  semaphore_info->reference_count=0;
   semaphore_info->signature=MagickSignature;
   return(semaphore_info);
 }
@@ -205,32 +197,6 @@ MagickExport SemaphoreInfo *AllocateSemaphoreInfo(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   D e s t r o y S e m a p h o r e                                           %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DestroySemaphore() destroys the semaphore environment.
-%
-%  The format of the DestroySemaphore method is:
-%
-%      DestroySemaphore(void)
-%
-%
-*/
-MagickExport void DestroySemaphore(void)
-{
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_destroy(&semaphore_mutex);
-#endif
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   D e s t r o y S e m a p h o r e I n f o                                   %
 %                                                                             %
 %                                                                             %
@@ -241,41 +207,27 @@ MagickExport void DestroySemaphore(void)
 %
 %  The format of the DestroySemaphoreInfo method is:
 %
-%      SemaphoreInfo *DestroySemaphoreInfo(SemaphoreInfo *semaphore_info)
+%      void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 %
 %  A description of each parameter follows:
 %
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
-%
 */
-MagickExport SemaphoreInfo *DestroySemaphoreInfo(SemaphoreInfo *semaphore_info)
+MagickExport void DestroySemaphoreInfo(SemaphoreInfo **semaphore_info)
 {
-  assert(semaphore_info != (SemaphoreInfo *) NULL);
-  assert(semaphore_info->signature == MagickSignature);
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_lock(&semaphore_mutex);
+  assert(semaphore_info != (SemaphoreInfo **) NULL);
+  assert((*semaphore_info) != (SemaphoreInfo *) NULL);
+  assert((*semaphore_info)->signature == MagickSignature);
+  LockMagickMutex();
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  (void) pthread_mutex_destroy(&(*semaphore_info)->mutex);
+#elif defined(MAGICKCORE_HAVE_WINTHREADS)
+  DeleteCriticalSection(&(*semaphore_info)->mutex);
 #endif
-#if defined(__WINDOWS__)
-  while (InterlockedCompareExchange(&semaphore_mutex,1L,0L) != 0)
-    Sleep(10);
-#endif
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_destroy(&semaphore_info->mutex);
-#endif
-#if defined(__WINDOWS__)
-  DeleteCriticalSection(&semaphore_info->mutex);
-#endif
-  semaphore_info->signature=(~MagickSignature);
-  free(semaphore_info);
-  semaphore_info=(SemaphoreInfo *) NULL;
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_unlock(&semaphore_mutex);
-#endif
-#if defined(__WINDOWS__)
-  InterlockedExchange(&semaphore_mutex,0L);
-#endif
-  return(semaphore_info);
+  (*semaphore_info)->signature=(~MagickSignature);
+  *semaphore_info=(SemaphoreInfo *) RelinquishAlignedMemory(*semaphore_info);
+  UnlockMagickMutex();
 }
 
 /*
@@ -283,25 +235,29 @@ MagickExport SemaphoreInfo *DestroySemaphoreInfo(SemaphoreInfo *semaphore_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I n i t i a l i z e S e m a p h o r e                                     %
++   L o c k M a g i c k M u t e x                                             %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  InitializeSemaphore() initializes the semaphore environment.
+%  LockMagickMutex() locks a global mutex.  If it is already locked, the
+%  calling thread blocks until the mutex becomes available.
 %
-%  The format of the InitializeSemaphore method is:
-%
-%      InitializeSemaphore(void)
+%  The format of the LockMagickMutex method is:
 %
+%      void LockMagickMutex(void)
 %
 */
-MagickExport void InitializeSemaphore(void)
+static void LockMagickMutex(void)
 {
-#if defined(HAVE_PTHREAD)
-  (void) pthread_mutex_init(&semaphore_mutex,
-    (const pthread_mutexattr_t *) NULL);
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  if (pthread_mutex_lock(&semaphore_mutex) != 0)
+    (void) fprintf(stderr,"pthread_mutex_lock failed %s\n",
+      GetExceptionMessage(errno));
+#elif defined(MAGICKCORE_HAVE_WINTHREADS)
+  while (InterlockedCompareExchange(&semaphore_mutex,1L,0L) != 0)
+    Sleep(10);
 #endif
 }
 
@@ -326,22 +282,46 @@ MagickExport void InitializeSemaphore(void)
 %
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
-%
 */
 MagickExport MagickBooleanType LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo *) NULL);
-  assert(semaphore_info->signature == MagickSignature);
-#if defined(HAVE_PTHREAD)
-  if (pthread_mutex_lock(&semaphore_info->mutex) != 0)
-    return(MagickFalse);
-  semaphore_info->id=pthread_self();
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  {
+    int
+      status;
+
+    status=pthread_mutex_lock(&semaphore_info->mutex);
+    if (status != 0)
+      return(MagickFalse);
+#if defined(MAGICKCORE_DEBUG)
+    {
+      if ((semaphore_info->reference_count > 0) &&
+          (IsMagickThreadEqual(semaphore_info->id) != MagickFalse))
+        {
+          (void) fprintf(stderr,"Warning: unexpected recursive lock!\n");
+          (void) fflush(stderr);
+        }
+    }
 #endif
-#if defined(__WINDOWS__)
-  EnterCriticalSection(&semaphore_info->mutex);
-  semaphore_info->id=GetCurrentThreadId();
+  }
+#elif defined(MAGICKCORE_HAVE_WINTHREADS)
+  {
+    EnterCriticalSection(&semaphore_info->mutex);
+#if defined(MAGICKCORE_DEBUG)
+    {
+      if ((semaphore_info->reference_count > 0) &&
+          (IsMagickThreadEqual(semaphore_info->id) != MagickFalse))
+        {
+          (void) fprintf(stderr,"Warning: unexpected recursive lock!\n");
+          (void) fflush(stderr);
+        }
+    }
 #endif
-  semaphore_info->lock=MagickTrue;
+  }
+#endif
+  semaphore_info->id=GetMagickThreadId();
+  semaphore_info->reference_count++;
   return(MagickTrue);
 }
 
@@ -366,7 +346,6 @@ MagickExport MagickBooleanType LockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
-%
 */
 MagickExport void RelinquishSemaphoreInfo(SemaphoreInfo *semaphore_info)
 {
@@ -380,6 +359,82 @@ MagickExport void RelinquishSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S e m a p h o r e C o m p o n e n t G e n e s i s                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SemaphoreComponentGenesis() instantiates the semaphore environment.
+%
+%  The format of the SemaphoreComponentGenesis method is:
+%
+%      MagickBooleanType SemaphoreComponentGenesis(void)
+%
+*/
+MagickExport MagickBooleanType SemaphoreComponentGenesis(void)
+{
+  LockMagickMutex();
+  UnlockMagickMutex();
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e m a p h o r e C o m p o n e n t T e r m i n u s                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SemaphoreComponentTerminus() destroys the semaphore component.
+%
+%  The format of the SemaphoreComponentTerminus method is:
+%
+%      SemaphoreComponentTerminus(void)
+%
+*/
+MagickExport void SemaphoreComponentTerminus(void)
+{
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   U n l o c k M a g i c k M u t e x                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  UnlockMagickMutex() releases a global mutex.
+%
+%  The format of the LockMagickMutex method is:
+%
+%      void UnlockMagickMutex(void)
+%
+*/
+static void UnlockMagickMutex(void)
+{
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  if (pthread_mutex_unlock(&semaphore_mutex) != 0)
+    (void) fprintf(stderr,"pthread_mutex_unlock failed %s\n",
+      GetExceptionMessage(errno));
+#elif defined(MAGICKCORE_HAVE_WINTHREADS)
+  InterlockedExchange(&semaphore_mutex,0L);
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   U n l o c k S e m a p h o r e I n f o                                     %
 %                                                                             %
 %                                                                             %
@@ -388,7 +443,7 @@ MagickExport void RelinquishSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %  UnlockSemaphoreInfo() unlocks a semaphore.
 %
-%  The format of the LockSemaphoreInfo method is:
+%  The format of the UnlockSemaphoreInfo method is:
 %
 %      MagickBooleanType UnlockSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
@@ -396,23 +451,34 @@ MagickExport void RelinquishSemaphoreInfo(SemaphoreInfo *semaphore_info)
 %
 %    o semaphore_info: Specifies a pointer to an SemaphoreInfo structure.
 %
-%
 */
 MagickExport MagickBooleanType UnlockSemaphoreInfo(
   SemaphoreInfo *semaphore_info)
 {
   assert(semaphore_info != (SemaphoreInfo *) NULL);
-  assert(semaphore_info->signature == MagickSignature);
-  if (semaphore_info->lock == MagickFalse)
-    return(MagickFalse);
-  semaphore_info->lock=MagickFalse;
-#if defined(HAVE_PTHREAD)
-  assert(pthread_equal(semaphore_info->id,pthread_self()));
-  if (pthread_mutex_unlock(&semaphore_info->mutex) != 0)
-    return(MagickFalse);
+#if defined(MAGICKCORE_DEBUG)
+  assert(IsMagickThreadEqual(semaphore_info->id) != MagickFalse);
+  if (semaphore_info->reference_count == 0)
+    {
+      (void) fprintf(stderr,"Warning: semaphore lock already unlocked!\n");
+      (void) fflush(stderr);
+      return(MagickFalse);
+    }
+  semaphore_info->reference_count--;
 #endif
-#if defined(__WINDOWS__)
-  assert(GetCurrentThreadId() == semaphore_info->id);
+#if defined(MAGICKCORE_HAVE_PTHREAD)
+  {
+    int
+      status;
+
+    status=pthread_mutex_unlock(&semaphore_info->mutex);
+    if (status != 0)
+      {
+        semaphore_info->reference_count++;
+        return(MagickFalse);
+      }
+  }
+#elif defined(MAGICKCORE_HAVE_WINTHREADS)
   LeaveCriticalSection(&semaphore_info->mutex);
 #endif
   return(MagickTrue);
