@@ -72,6 +72,7 @@
 #include "magick/memory_.h"
 #include "magick/module.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/paint.h"
 #include "magick/pixel-private.h"
@@ -83,6 +84,7 @@
 #include "magick/signature-private.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/thread-private.h"
 #include "magick/timer.h"
 #include "magick/utility.h"
 #include "magick/version.h"
@@ -92,343 +94,250 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   G e t I m a g e B o u n d i n g B o x                                     %
+%     A v e r a g e I m a g e s                                               %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetImageBoundingBox() returns the bounding box of an image canvas.
+%  AverageImages() takes a set of images and averages them together.  Each
+%  image in the set must have the same width and height.  AverageImages()
+%  returns a single image with each corresponding pixel component of each
+%  image averaged.   On failure, a NULL image is returned and exception
+%  describes the reason for the failure.
 %
-%  The format of the GetImageBoundingBox method is:
+%  The format of the AverageImages method is:
 %
-%      RectangleInfo GetImageBoundingBox(const Image *image,
-%        ExceptionInfo *exception)
+%      Image *AverageImages(Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o bounds: Method GetImageBoundingBox returns the bounding box of an
-%      image canvas.
-%
-%    o image: the image.
+%    o image: the image sequence.
 %
 %    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
-  ExceptionInfo *exception)
+
+static MagickPixelPacket **DestroyPixelThreadSet(MagickPixelPacket **pixels)
 {
+  register long
+    i;
+
+  assert(pixels != (MagickPixelPacket **) NULL);
+  for (i=0; i < (long) GetOpenMPMaximumThreads(); i++)
+    if (pixels[i] != (MagickPixelPacket *) NULL)
+      pixels[i]=(MagickPixelPacket *) RelinquishMagickMemory(pixels[i]);
+  pixels=(MagickPixelPacket **) RelinquishAlignedMemory(pixels);
+  return(pixels);
+}
+
+static MagickPixelPacket **AcquirePixelThreadSet(const Image *image)
+{
+  register long
+    i,
+    j;
+
+  MagickPixelPacket
+    **pixels;
+
+  unsigned long
+    number_threads;
+
+  number_threads=GetOpenMPMaximumThreads();
+  pixels=(MagickPixelPacket **) AcquireAlignedMemory(number_threads,
+    sizeof(*pixels));
+  if (pixels == (MagickPixelPacket **) NULL)
+    return((MagickPixelPacket **) NULL);
+  (void) ResetMagickMemory(pixels,0,number_threads*sizeof(*pixels));
+  for (i=0; i < (long) number_threads; i++)
+  {
+    pixels[i]=(MagickPixelPacket *) AcquireQuantumMemory(image->columns,
+      sizeof(**pixels));
+    if (pixels[i] == (MagickPixelPacket *) NULL)
+      return(DestroyPixelThreadSet(pixels));
+    for (j=0; j < (long) image->columns; j++)
+      GetMagickPixelPacket(image,&pixels[i][j]);
+  }
+  return(pixels);
+}
+
+MagickExport Image *AverageImages(const Image *image,ExceptionInfo *exception)
+{
+#define AverageImageTag  "Average/Image"
+
+  CacheView
+    *average_view;
+
+  const Image
+    *next;
+
+  Image
+    *average_image;
+
   long
+    progress,
     y;
 
   MagickBooleanType
     status;
 
   MagickPixelPacket
-    target[3],
+    **average_pixels,
     zero;
 
-  RectangleInfo
-    bounds;
-
-  register const PixelPacket
-    *p;
-
-  ViewInfo
-    *image_view;
-
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  bounds.width=0;
-  bounds.height=0;
-  bounds.x=(long) image->columns;
-  bounds.y=(long) image->rows;
-  GetMagickPixelPacket(image,&target[0]);
-  image_view=AcquireCacheView(image);
-  p=GetCacheViewVirtualPixels(image_view,0,0,1,1,exception);
-  if (p == (const PixelPacket *) NULL)
-    {
-      image_view=DestroyCacheView(image_view);
-      return(bounds);
-    }
-  SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
-    &target[0]);
-  GetMagickPixelPacket(image,&target[1]);
-  p=GetCacheViewVirtualPixels(image_view,(long) image->columns-1,0,1,1,
-    exception);
-  SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
-    &target[1]);
-  GetMagickPixelPacket(image,&target[2]);
-  p=GetCacheViewVirtualPixels(image_view,0,(long) image->rows-1,1,1,exception);
-  SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
-    &target[2]);
-  status=MagickTrue;
-  GetMagickPixelPacket(image,&zero);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(status)
-#endif
-  for (y=0; y < (long) image->rows; y++)
-  {
-    MagickPixelPacket
-      pixel;
-
-    RectangleInfo
-      bounding_box;
-
-    register const IndexPacket
-      *indexes;
-
-    register const PixelPacket
-      *p;
-
-    register long
-      x;
-
-    if (status == MagickFalse)
-      continue;
-#if defined(HAVE_OPENMP)
-#  pragma omp critical (MagickCore_GetImageBoundingBox)
-#endif
-    bounding_box=bounds;
-    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-    if (p == (const PixelPacket *) NULL)
-      {
-        status=MagickFalse;
-        continue;
-      }
-    indexes=GetCacheViewVirtualIndexQueue(image_view);
-    pixel=zero;
-    for (x=0; x < (long) image->columns; x++)
-    {
-      SetMagickPixelPacket(image,p,indexes+x,&pixel);
-      if ((x < bounding_box.x) &&
-          (IsMagickColorSimilar(&pixel,&target[0]) == MagickFalse))
-        bounding_box.x=x;
-      if ((x > (long) bounding_box.width) &&
-          (IsMagickColorSimilar(&pixel,&target[1]) == MagickFalse))
-        bounding_box.width=(unsigned long) x;
-      if ((y < bounding_box.y) &&
-          (IsMagickColorSimilar(&pixel,&target[0]) == MagickFalse))
-        bounding_box.y=y;
-      if ((y > (long) bounding_box.height) &&
-          (IsMagickColorSimilar(&pixel,&target[2]) == MagickFalse))
-        bounding_box.height=(unsigned long) y;
-      p++;
-    }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical (MagickCore_GetImageBoundingBox)
-#endif
-    {
-      if (bounding_box.x < bounds.x)
-        bounds.x=bounding_box.x;
-      if (bounding_box.y < bounds.y)
-        bounds.y=bounding_box.y;
-      if (bounding_box.width > bounds.width)
-        bounds.width=bounding_box.width;
-      if (bounding_box.height > bounds.height)
-        bounds.height=bounding_box.height;
-    }
-  }
-  image_view=DestroyCacheView(image_view);
-  if ((bounds.width == 0) || (bounds.height == 0))
-    (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
-      "GeometryDoesNotContainImage","`%s'",image->filename);
-  else
-    {
-      bounds.width-=(bounds.x-1);
-      bounds.height-=(bounds.y-1);
-    }
-  return(bounds);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   G e t I m a g e C h a n n e l D e p t h                                   %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  GetImageChannelDepth() returns the depth of a particular image channel.
-%
-%  The format of the GetImageChannelDepth method is:
-%
-%      unsigned long GetImageDepth(const Image *image,ExceptionInfo *exception)
-%      unsigned long GetImageChannelDepth(const Image *image,
-%        const ChannelType channel,ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image.
-%
-%    o channel: the channel.
-%
-%    o exception: return any errors or warnings in this structure.
-%
-*/
-
-MagickExport unsigned long GetImageDepth(const Image *image,
-  ExceptionInfo *exception)
-{
-  return(GetImageChannelDepth(image,AllChannels,exception));
-}
-
-MagickExport unsigned long GetImageChannelDepth(const Image *image,
-  const ChannelType channel,ExceptionInfo *exception)
-{
-  long
-    y;
-
-  MagickBooleanType
-    status;
-
-  register long
-    id;
-
   unsigned long
-    *current_depth,
-    depth,
-    number_threads;
-
-  ViewInfo
-    *image_view;
+    number_images;
 
   /*
-    Compute image depth.
+    Ensure the image are the same size.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  number_threads=GetPixelCacheMaximumThreads();
-  current_depth=(unsigned long *) AcquireQuantumMemory(number_threads,
-    sizeof(*current_depth));
-  if (current_depth == (unsigned long *) NULL)
-    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  status=MagickTrue;
-  for (id=0; id < (long) number_threads; id++)
-    current_depth[id]=1;
-  if ((image->storage_class == PseudoClass) && (image->matte == MagickFalse))
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
+    if ((next->columns != image->columns) || (next->rows != image->rows))
+      ThrowImageException(OptionError,"ImageWidthsOrHeightsDiffer");
+  /*
+    Initialize average next attributes.
+  */
+  average_image=CloneImage(image,image->columns,image->rows,MagickTrue,
+    exception);
+  if (average_image == (Image *) NULL)
+    return((Image *) NULL);
+  if (SetImageStorageClass(average_image,DirectClass) == MagickFalse)
     {
-      register const PixelPacket
-        *p;
-
-      register long
-        i;
-
-      p=image->colormap;
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(status)
-#endif
-      for (i=0; i < (long) image->colors; i++)
-      {
-        if (status == MagickFalse)
-          continue;
-        id=GetPixelCacheThreadId();
-        while (current_depth[id] < MAGICKCORE_QUANTUM_DEPTH)
-        {
-          MagickStatusType
-            status;
-
-          QuantumAny
-            range;
-
-          status=0;
-          range=GetQuantumRange(current_depth[id]);
-          if ((channel & RedChannel) != 0)
-            status|=p->red != ScaleAnyToQuantum(ScaleQuantumToAny(p->red,
-              range),range);
-          if ((channel & GreenChannel) != 0)
-            status|=p->green != ScaleAnyToQuantum(ScaleQuantumToAny(p->green,
-              range),range);
-          if ((channel & BlueChannel) != 0)
-            status|=p->blue != ScaleAnyToQuantum(ScaleQuantumToAny(p->blue,
-              range),range);
-          if (status == 0)
-            break;
-          current_depth[id]++;
-        }
-        p++;
-      }
-      depth=current_depth[0];
-      for (id=1; id < (long) number_threads; id++)
-        if (depth < current_depth[id])
-          depth=current_depth[id];
-      current_depth=(unsigned long *) RelinquishMagickMemory(current_depth);
-      return(depth);
+      InheritException(exception,&average_image->exception);
+      average_image=DestroyImage(average_image);
+      return((Image *) NULL);
     }
-  image_view=AcquireCacheView(image);
+  average_pixels=AcquirePixelThreadSet(image);
+  if (average_pixels == (MagickPixelPacket **) NULL)
+    {
+      average_image=DestroyImage(average_image);
+      ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
+    }
+  /*
+    Average image pixels.
+  */
+  status=MagickTrue;
+  progress=0;
+  GetMagickPixelPacket(image,&zero);
+  number_images=GetImageListLength(image);
+  average_view=AcquireCacheView(average_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(status)
+  #pragma omp parallel for schedule(dynamic) shared(progress,status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (long) average_image->rows; y++)
   {
-    register const IndexPacket
-      *indexes;
+    CacheView
+      *image_view;
 
-    register const PixelPacket
-      *p;
+    const Image
+      *next;
+
+    MagickPixelPacket
+      pixel;
+
+    register IndexPacket
+      *__restrict average_indexes;
 
     register long
+      i,
       id,
       x;
 
+    register MagickPixelPacket
+      *average_pixel;
+
+    register PixelPacket
+      *__restrict q;
+
     if (status == MagickFalse)
       continue;
-    id=GetPixelCacheThreadId();
-    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-    if (p == (const PixelPacket *) NULL)
-      continue;
-    indexes=GetCacheViewVirtualIndexQueue(image_view);
-    for (x=0; x < (long) image->columns; x++)
-    {
-      while (current_depth[id] < MAGICKCORE_QUANTUM_DEPTH)
+    q=QueueCacheViewAuthenticPixels(average_view,0,y,average_image->columns,1,
+      exception);
+    if (q == (PixelPacket *) NULL)
       {
-        MagickStatusType
-          status;
-
-        QuantumAny
-          range;
-
-        status=0;
-        range=GetQuantumRange(current_depth[id]);
-        if ((channel & RedChannel) != 0)
-          status|=p->red != ScaleAnyToQuantum(ScaleQuantumToAny(p->red,range),
-            range);
-        if ((channel & GreenChannel) != 0)
-          status|=p->green != ScaleAnyToQuantum(ScaleQuantumToAny(p->green,
-            range),range);
-        if ((channel & BlueChannel) != 0)
-          status|=p->blue != ScaleAnyToQuantum(ScaleQuantumToAny(p->blue,range),
-            range);
-        if (((channel & OpacityChannel) != 0) && (image->matte != MagickFalse))
-          status|=p->opacity != ScaleAnyToQuantum(ScaleQuantumToAny(p->opacity,
-            range),range);
-        if (((channel & IndexChannel) != 0) &&
-            (image->colorspace == CMYKColorspace))
-          status|=indexes[x] != ScaleAnyToQuantum(ScaleQuantumToAny(indexes[x],
-            range),range);
-        if (status == 0)
-          break;
-        current_depth[id]++;
+        status=MagickFalse;
+        continue;
       }
-      p++;
+    average_indexes=GetCacheViewAuthenticIndexQueue(average_view);
+    pixel=zero;
+    id=GetOpenMPThreadId();
+    average_pixel=average_pixels[id];
+    for (x=0; x < (long) average_image->columns; x++)
+      average_pixel[x]=zero;
+    next=image;
+    for (i=0; i < (long) number_images; i++)
+    {
+      register const IndexPacket
+        *indexes;
+
+      register const PixelPacket
+        *p;
+
+      image_view=AcquireCacheView(next);
+      p=GetCacheViewVirtualPixels(image_view,0,y,next->columns,1,exception);
+      if (p == (const PixelPacket *) NULL)
+        {
+          image_view=DestroyCacheView(image_view);
+          break;
+        }
+      indexes=GetCacheViewVirtualIndexQueue(image_view);
+      for (x=0; x < (long) next->columns; x++)
+      {
+        SetMagickPixelPacket(next,p,indexes+x,&pixel);
+        average_pixel[x].red+=QuantumScale*pixel.red;
+        average_pixel[x].green+=QuantumScale*pixel.green;
+        average_pixel[x].blue+=QuantumScale*pixel.blue;
+        average_pixel[x].opacity+=QuantumScale*pixel.opacity;
+        if (average_image->colorspace == CMYKColorspace)
+          average_pixel[x].index+=QuantumScale*pixel.index;
+        p++;
+      }
+      image_view=DestroyCacheView(image_view);
+      next=GetNextImageInList(next);
     }
-    if (current_depth[id] == MAGICKCORE_QUANTUM_DEPTH)
+    for (x=0; x < (long) average_image->columns; x++)
+    {
+      average_pixel[x].red=(MagickRealType) (QuantumRange*
+        average_pixel[x].red/number_images);
+      average_pixel[x].green=(MagickRealType) (QuantumRange*
+        average_pixel[x].green/number_images);
+      average_pixel[x].blue=(MagickRealType) (QuantumRange*
+        average_pixel[x].blue/number_images);
+      average_pixel[x].opacity=(MagickRealType) (QuantumRange*
+        average_pixel[x].opacity/number_images);
+      if (average_image->colorspace == CMYKColorspace)
+        average_pixel[x].index=(MagickRealType) (QuantumRange*
+          average_pixel[x].index/number_images);
+      SetPixelPacket(average_image,&average_pixel[x],q,average_indexes+x);
+      q++;
+    }
+    if (SyncCacheViewAuthenticPixels(average_view,exception) == MagickFalse)
       status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical (MagickCore_AverageImages)
+#endif
+        proceed=SetImageProgress(image,AverageImageTag,progress++,
+          average_image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
   }
-  image_view=DestroyCacheView(image_view);
-  depth=current_depth[0];
-  for (id=1; id < (long) number_threads; id++)
-    if (depth < current_depth[id])
-      depth=current_depth[id];
-  current_depth=(unsigned long *) RelinquishMagickMemory(current_depth);
-  return(depth);
+  average_view=DestroyCacheView(average_view);
+  average_pixels=DestroyPixelThreadSet(average_pixels);
+  if (status == MagickFalse)
+    average_image=DestroyImage(average_image);
+  return(average_image);
 }
 
 /*
@@ -546,15 +455,6 @@ MagickExport MagickBooleanType GetImageChannelMean(const Image *image,
   long
     y;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
-  register long
-    x;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -564,6 +464,15 @@ MagickExport MagickBooleanType GetImageChannelMean(const Image *image,
   area=0.0;
   for (y=0; y < (long) image->rows; y++)
   {
+    register const IndexPacket
+      *__restrict indexes;
+
+    register const PixelPacket
+      *__restrict p;
+
+    register long
+      x;
+
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
@@ -675,15 +584,6 @@ MagickExport MagickBooleanType GetImageChannelKurtosis(const Image *image,
   long
     y;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
-  register long
-    x;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -698,6 +598,15 @@ MagickExport MagickBooleanType GetImageChannelKurtosis(const Image *image,
   sum_fourth_power=0.0;
   for (y=0; y < (long) image->rows; y++)
   {
+    register const IndexPacket
+      *__restrict indexes;
+
+    register const PixelPacket
+      *__restrict p;
+
+    register long
+      x;
+
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
@@ -822,15 +731,6 @@ MagickExport MagickBooleanType GetImageChannelRange(const Image *image,
   MagickPixelPacket
     pixel;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
-  register long
-    x;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -840,6 +740,15 @@ MagickExport MagickBooleanType GetImageChannelRange(const Image *image,
   GetMagickPixelPacket(image,&pixel);
   for (y=0; y < (long) image->rows; y++)
   {
+    register const IndexPacket
+      *__restrict indexes;
+
+    register const PixelPacket
+      *__restrict p;
+
+    register long
+      x;
+
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
@@ -957,15 +866,8 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
   QuantumAny
     range;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
   register long
-    i,
-    x;
+    i;
 
   size_t
     length;
@@ -995,9 +897,17 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
     channel_statistics[i].kurtosis=0.0;
     channel_statistics[i].skewness=0.0;
   }
-  y=(long) image->rows;
   for (y=0; y < (long) image->rows; y++)
   {
+    register const IndexPacket
+      *__restrict indexes;
+
+    register const PixelPacket
+      *__restrict p;
+
+    register long
+      x;
+
     p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
@@ -1040,18 +950,21 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
               continue;
             }
         }
-      if (channel_statistics[OpacityChannel].depth != MAGICKCORE_QUANTUM_DEPTH)
+      if (image->matte != MagickFalse)
         {
-          depth=channel_statistics[OpacityChannel].depth;
-          range=GetQuantumRange(depth);
-          status=p->opacity != ScaleAnyToQuantum(ScaleQuantumToAny(p->opacity,
-            range),range) ? MagickTrue : MagickFalse;
-          if (status != MagickFalse)
+          if (channel_statistics[OpacityChannel].depth != MAGICKCORE_QUANTUM_DEPTH)
             {
-              channel_statistics[OpacityChannel].depth++;
-              continue;
+              depth=channel_statistics[OpacityChannel].depth;
+              range=GetQuantumRange(depth);
+              status=p->opacity != ScaleAnyToQuantum(ScaleQuantumToAny(
+                p->opacity,range),range) ? MagickTrue : MagickFalse;
+              if (status != MagickFalse)
+                {
+                  channel_statistics[OpacityChannel].depth++;
+                  continue;
+                }
             }
-        }
+          }
       if (image->colorspace == CMYKColorspace)
         {
           if (channel_statistics[BlackChannel].depth != MAGICKCORE_QUANTUM_DEPTH)
@@ -1098,17 +1011,20 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
         p->blue*p->blue;
       channel_statistics[BlueChannel].skewness+=(double) p->blue*p->blue*
         p->blue;
-      if ((double) p->opacity < channel_statistics[OpacityChannel].minima)
-        channel_statistics[OpacityChannel].minima=(double) p->opacity;
-      if ((double) p->opacity > channel_statistics[OpacityChannel].maxima)
-        channel_statistics[OpacityChannel].maxima=(double) p->opacity;
-      channel_statistics[OpacityChannel].mean+=p->opacity;
-      channel_statistics[OpacityChannel].standard_deviation+=(double)
-        p->opacity*p->opacity;
-      channel_statistics[OpacityChannel].kurtosis+=(double) p->opacity*
-        p->opacity*p->opacity*p->opacity;
-      channel_statistics[OpacityChannel].skewness+=(double) p->opacity*
-        p->opacity*p->opacity;
+      if (image->matte != MagickFalse)
+        {
+          if ((double) p->opacity < channel_statistics[OpacityChannel].minima)
+            channel_statistics[OpacityChannel].minima=(double) p->opacity;
+          if ((double) p->opacity > channel_statistics[OpacityChannel].maxima)
+            channel_statistics[OpacityChannel].maxima=(double) p->opacity;
+          channel_statistics[OpacityChannel].mean+=p->opacity;
+          channel_statistics[OpacityChannel].standard_deviation+=(double)
+            p->opacity*p->opacity;
+          channel_statistics[OpacityChannel].kurtosis+=(double) p->opacity*
+            p->opacity*p->opacity*p->opacity;
+          channel_statistics[OpacityChannel].skewness+=(double) p->opacity*
+            p->opacity*p->opacity;
+        }
       if (image->colorspace == CMYKColorspace)
         {
           if ((double) indexes[x] < channel_statistics[BlackChannel].minima)
@@ -1193,204 +1109,3 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
   }
   return(channel_statistics);
 }
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   G e t I m a g e Q u a n t u m D e p t h                                   %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  GetImageQuantumDepth() returns the depth of the image rounded to a legal
-%  quantum depth: 8, 16, or 32.
-%
-%  The format of the GetImageQuantumDepth method is:
-%
-%      unsigned long GetImageQuantumDepth(const Image *image,
-%        const MagickBooleanType constrain)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image.
-%
-%    o constrain: A value other than MagickFalse, constrains the depth to
-%      a maximum of MAGICKCORE_QUANTUM_DEPTH.
-%
-*/
-MagickExport unsigned long GetImageQuantumDepth(const Image *image,
-  const MagickBooleanType constrain)
-{
-  unsigned long
-    depth;
-
-  depth=image->depth;
-  if (depth <= 8)
-    depth=8;
-  else
-    if (depth <= 16)
-      depth=16;
-    else
-      if (depth <= 32)
-        depth=32;
-      else
-        if (depth <= 64)
-          depth=64;
-  if (constrain != MagickFalse)
-    depth=(unsigned long) MagickMin((double) depth,(double)
-      MAGICKCORE_QUANTUM_DEPTH);
-  return(depth);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   S e t I m a g e C h a n n e l D e p t h                                   %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  SetImageChannelDepth() sets the depth of the image.
-%
-%  The format of the SetImageChannelDepth method is:
-%
-%      MagickBooleanType SetImageDepth(Image *image,const unsigned long depth)
-%      MagickBooleanType SetImageChannelDepth(Image *image,
-%        const ChannelType channel,const unsigned long depth)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image.
-%
-%    o channel: the channel.
-%
-%    o depth: the image depth.
-%
-*/
-
-MagickExport MagickBooleanType SetImageDepth(Image *image,
-  const unsigned long depth)
-{
-  return(SetImageChannelDepth(image,AllChannels,depth));
-}
-
-MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
-  const ChannelType channel,const unsigned long depth)
-{
-  ExceptionInfo
-    *exception;
-
-  long
-    y;
-
-  MagickBooleanType
-    status;
-
-  QuantumAny
-    range;
-
-  ViewInfo
-    *image_view;
-
-  assert(image != (Image *) NULL);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(image->signature == MagickSignature);
-  if (GetImageDepth(image,&image->exception) <= (unsigned long)
-      MagickMin((double) depth,(double) MAGICKCORE_QUANTUM_DEPTH))
-    {
-      image->depth=depth;
-      return(MagickTrue);
-    }
-  /*
-    Scale pixels to desired depth.
-  */
-  status=MagickTrue;
-  range=GetQuantumRange(depth);
-  exception=(&image->exception);
-  image_view=AcquireCacheView(image);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(status)
-#endif
-  for (y=0; y < (long) image->rows; y++)
-  {
-    register IndexPacket
-      *indexes;
-
-    register long
-      x;
-
-    register PixelPacket
-      *q;
-
-    if (status == MagickFalse)
-      continue;
-    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
-      exception);
-    if (q == (PixelPacket *) NULL)
-      {
-        status=MagickFalse;
-        continue;
-      }
-    indexes=GetCacheViewAuthenticIndexQueue(image_view);
-    for (x=0; x < (long) image->columns; x++)
-    {
-      if ((channel & RedChannel) != 0)
-        q->red=ScaleAnyToQuantum(ScaleQuantumToAny(q->red,range),range);
-      if ((channel & GreenChannel) != 0)
-        q->green=ScaleAnyToQuantum(ScaleQuantumToAny(q->green,range),range);
-      if ((channel & BlueChannel) != 0)
-        q->blue=ScaleAnyToQuantum(ScaleQuantumToAny(q->blue,range),range);
-      if (((channel & OpacityChannel) != 0) && (image->matte != MagickFalse))
-        q->opacity=ScaleAnyToQuantum(ScaleQuantumToAny(q->opacity,range),range);
-      if (((channel & IndexChannel) != 0) &&
-          (image->colorspace == CMYKColorspace))
-        indexes[x]=ScaleAnyToQuantum(ScaleQuantumToAny(indexes[x],range),range);
-      q++;
-    }
-    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
-      {
-        status=MagickFalse;
-        continue;
-      }
-  }
-  image_view=DestroyCacheView(image_view);
-  if (image->storage_class == PseudoClass)
-    {
-      QuantumAny
-        range;
-
-      register long
-        i;
-
-      register PixelPacket
-        *p;
-
-      p=image->colormap;
-      range=GetQuantumRange(depth);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(dynamic,4) shared(status)
-#endif
-      for (i=0; i < (long) image->colors; i++)
-      {
-        if ((channel & RedChannel) != 0)
-          p->red=ScaleAnyToQuantum(ScaleQuantumToAny(p->red,range),range);
-        if ((channel & GreenChannel) != 0)
-          p->green=ScaleAnyToQuantum(ScaleQuantumToAny(p->green,range),range);
-        if ((channel & BlueChannel) != 0)
-          p->blue=ScaleAnyToQuantum(ScaleQuantumToAny(p->blue,range),range);
-        if ((channel & OpacityChannel) != 0)
-          p->opacity=ScaleAnyToQuantum(ScaleQuantumToAny(p->opacity,range),
-            range);
-        p++;
-      }
-    }
-  image->depth=depth;
-  return(status);
-}
