@@ -1012,7 +1012,10 @@ MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
     j;
 
   static const char
-    *units[] = { "b", "kb", "mb", "gb", "tb", "pb", "eb", (char *) NULL };
+    *units[] =
+    {
+      "b", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB", (char *) NULL
+    };
 
 #if defined(_MSC_VER) && (_MSC_VER == 1200)
   length=(double) ((MagickOffsetType) size);
@@ -1021,7 +1024,7 @@ MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
 #endif
   for (i=0; (length >= 1024.0) && (units[i+1] != (const char *) NULL); i++)
     length/=1024.0;
-  for (j=2; j < 10; j++)
+  for (j=2; j < 12; j++)
   {
     count=FormatMagickString(format,MaxTextExtent,"%.*g%s",(int) (i+j),length,
       units[i]);
@@ -1128,41 +1131,47 @@ MagickExport long FormatMagickTime(const time_t time,const size_t length,
     count;
 
   struct tm
-    *local_time,
-    *utc_time;
-
-#if defined(MAGICKCORE_HAVE_LOCALTIME_R)
-  struct tm
-     local_time_storage,
-     utc_time_storage;
-#endif
+    gm_time,
+    local_time;
 
   time_t
     timezone;
 
   assert(timestamp != (char *) NULL);
+  (void) ResetMagickMemory(&local_time,0,sizeof(local_time));
+  (void) ResetMagickMemory(&gm_time,0,sizeof(gm_time));
 #if defined(MAGICKCORE_HAVE_LOCALTIME_R)
-  local_time=localtime_r(&time,&local_time_storage);
+  (void) localtime_r(&time,&local_time);
 #else
-  local_time=localtime(&time);
+  {
+    struct tm
+      *my_time;
+
+    my_time=localtime(&time);
+    if (my_time != (struct tm *) NULL)
+      (void) memcpy(&local_time,my_time,sizeof(local_time));
+  }
 #endif
-  if (local_time == (struct tm *) NULL)
-    return(0);
 #if defined(MAGICKCORE_HAVE_GMTIME_R)
-  utc_time=gmtime_r(&time,&utc_time_storage);
+  (void) gmtime_r(&time,&gm_time);
 #else
-  utc_time=gmtime(&time);
+  {
+    struct tm
+      *my_time;
+
+    my_time=gmtime(&time);
+    if (my_time != (struct tm *) NULL)
+      (void) memcpy(&gm_time,my_time,sizeof(gm_time));
+  }
 #endif
-  if (utc_time == (struct tm *) NULL)
-    return(0);
-  timezone=(time_t) ((local_time->tm_min-utc_time->tm_min)/60+
-    local_time->tm_hour-utc_time->tm_hour+24*((local_time->tm_year-
-    utc_time->tm_year) != 0 ? (local_time->tm_year-utc_time->tm_year) :
-    (local_time->tm_yday-utc_time->tm_yday)));
+  timezone=(time_t) ((local_time.tm_min-gm_time.tm_min)/60+
+    local_time.tm_hour-gm_time.tm_hour+24*((local_time.tm_year-
+    gm_time.tm_year) != 0 ? (local_time.tm_year-gm_time.tm_year) :
+    (local_time.tm_yday-gm_time.tm_yday)));
   count=FormatMagickString(timestamp,length,
-    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_time->tm_year+1900,
-    local_time->tm_mon+1,local_time->tm_mday,local_time->tm_hour,
-    local_time->tm_min,local_time->tm_sec,(long) timezone);
+    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_time.tm_year+1900,
+    local_time.tm_mon+1,local_time.tm_mday,local_time.tm_hour,
+    local_time.tm_min,local_time.tm_sec,(long) timezone);
   return(count);
 }
 
@@ -1328,18 +1337,17 @@ MagickExport long LocaleCompare(const char *p,const char *q)
   return((long) strcasecmp(p,q));
 #else
   {
-    register unsigned char
+    register int
       c,
-      d;
+      d,
+      i;
 
-    for ( ; ; )
+    for (i=0; ; i++)
     {
-      c=(unsigned char) *p;
-      d=(unsigned char) *q;
+      c=(int) p[i];
+      d=(int) q[i];
       if ((c == '\0') || (AsciiMap[c] != AsciiMap[d]))
         break;
-      p++;
-      q++;
     }
     return((long) AsciiMap[c]-AsciiMap[d]);
   }
@@ -1425,17 +1433,17 @@ MagickExport long LocaleNCompare(const char *p,const char *q,
   return((long) strncasecmp(p,q,length));
 #else
   {
-    register size_t
-      n;
-
-    register unsigned char
+    register int
       c,
       d;
 
+    register size_t
+      n;
+
     for (n=length; n != 0; n--)
     {
-      c=(unsigned char) *p;
-      d=(unsigned char) *q;
+      c=(int) *p;
+      d=(int) *q;
       if (AsciiMap[c] != AsciiMap[d])
         return(AsciiMap[c]-AsciiMap[d]);
       if (c == '\0')
@@ -1967,21 +1975,25 @@ MagickExport double StringToDouble(const char *string,const double interval)
     *q;
 
   double
+    scale,
     value;
 
   assert(string != (char *) NULL);
   value=strtod(string,&q);
+  scale=1000.0;
+  if ((*q != '\0') && (tolower((int) ((unsigned char) *(q+1))) == 'i'))
+    scale=1024.0;
   switch (tolower((int) ((unsigned char) *q)))
   {
-    case '%': value*=pow(1024.0,0)*interval/100.0; break;
-    case 'k': value*=pow(1024.0,1); break;
-    case 'm': value*=pow(1024.0,2); break;
-    case 'g': value*=pow(1024.0,3); break;
-    case 't': value*=pow(1024.0,4); break;
-    case 'p': value*=pow(1024.0,5); break;
-    case 'e': value*=pow(1024.0,6); break;
-    case 'z': value*=pow(1024.0,7); break;
-    case 'y': value*=pow(1024.0,8); break;
+    case '%': value*=pow(scale,0)*interval/100.0; break;
+    case 'k': value*=pow(scale,1); break;
+    case 'm': value*=pow(scale,2); break;
+    case 'g': value*=pow(scale,3); break;
+    case 't': value*=pow(scale,4); break;
+    case 'p': value*=pow(scale,5); break;
+    case 'e': value*=pow(scale,6); break;
+    case 'z': value*=pow(scale,7); break;
+    case 'y': value*=pow(scale,8); break;
     default:  break;
   }
   return(value);
@@ -2158,9 +2170,6 @@ MagickExport char **StringToList(const char *text)
   char
     **textlist;
 
-  register char
-    *q;
-
   register const char
     *p;
 
@@ -2178,6 +2187,9 @@ MagickExport char **StringToList(const char *text)
       break;
   if (*p == '\0')
     {
+      register const char
+        *q;
+
       /*
         Convert string to an ASCII list.
       */
@@ -2192,7 +2204,7 @@ MagickExport char **StringToList(const char *text)
       p=text;
       for (i=0; i < (long) lines; i++)
       {
-        for (q=(char *) p; *q != '\0'; q++)
+        for (q=p; *q != '\0'; q++)
           if ((*q == '\r') || (*q == '\n'))
             break;
         textlist[i]=(char *) AcquireQuantumMemory((size_t) (q-p)+MaxTextExtent,
@@ -2210,6 +2222,9 @@ MagickExport char **StringToList(const char *text)
       char
         hex_string[MaxTextExtent];
 
+      register char
+        *q;
+
       register long
         j;
 
@@ -2360,124 +2375,79 @@ MagickExport void StripString(char *message)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SubstituteString() performs string substitution on a buffer, replacing the
-%  buffer with the substituted version. Buffer must be allocate from the heap.
+%  SubstituteString() performs string substitution on a string, replacing the
+%  string with the substituted version. Buffer must be allocated from the heap.
+%  If the string is matched and status, MagickTrue is returned otherwise
+%  MagickFalse.
 %
 %  The format of the SubstituteString method is:
 %
-%      MagickBooleanType SubstituteString(char **buffer,const char *search,
+%      MagickBooleanType SubstituteString(char **string,const char *search,
 %        const char *replace)
 %
 %  A description of each parameter follows:
 %
-%    o buffer: the buffer to perform replacements on. Replaced with new
+%    o string: the string to perform replacements on;  replaced with new
 %      allocation if a replacement is made.
 %
-%    o search: String to search for.
+%    o search: search for this string.
 %
-%    o replace: Replacement string.
+%    o replace: replace any matches with this string.
 %
 */
-MagickExport MagickBooleanType SubstituteString(char **buffer,
+MagickExport MagickBooleanType SubstituteString(char **string,
   const char *search,const char *replace)
 {
-  char
-    *result;
+  MagickBooleanType
+    status;
 
-  const char
-    *match,
-    *source;
-
-  MagickOffsetType
-    destination_offset;
+  register char
+    *p;
 
   size_t
-    copy_length,
-    length,
-    replace_length,
-    result_length,
-    search_length;
+    extent,
+    replace_extent,
+    search_extent;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(buffer != (char **) NULL);
-  assert(*buffer != (char *) NULL);
-  assert(search != (const char*) NULL);
-  assert(replace != (const char*) NULL);
-  if (strcmp(search,replace) == 0)
-    return(MagickTrue);
-  source=(*buffer);
-  match=strstr(source,search);
-  if (match == (char *) NULL)
-    return(MagickFalse);
-  result=(char *) NULL;
-  length=strlen(source);
-  if (~length >= MaxTextExtent)
-    result=(char *) AcquireQuantumMemory(length+MaxTextExtent,sizeof(*result));
-  if (result == (char *) NULL)
-    ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  *result='\0';
-  result_length=0;
-  destination_offset=0;
-  replace_length=strlen(replace);
-  for (search_length=strlen(search); match != (char *) NULL; )
+  ssize_t
+    offset;
+
+  status=MagickFalse;
+  search_extent=0,
+  replace_extent=0;
+  for (p=strchr(*string,*search); p != (char *) NULL; p=strchr(p+1,*search))
   {
+    if (search_extent == 0)
+      search_extent=strlen(search);
+    if (strncmp(p,search,search_extent) != 0)
+      continue;
     /*
-      Copy portion before match.
+      We found a match.
     */
-    copy_length=(size_t) (match-source);
-    if (copy_length != 0)
+    status=MagickTrue;
+    if (replace_extent == 0)
+      replace_extent=strlen(replace);
+    if (replace_extent > search_extent)
       {
-        result_length+=copy_length;
-        if ((result_length+MaxTextExtent) >= length)
-          {
-            length+=copy_length;
-            result=(char *) ResizeQuantumMemory(result,length+MaxTextExtent,
-              sizeof(*result));
-            if (result == (char *) NULL)
-              ThrowFatalException(ResourceLimitFatalError,
-                "UnableToAcquireString");
-          }
-        (void) CopyMagickString(result+destination_offset,source,copy_length+1);
-        destination_offset+=copy_length;
-      }
-    /*
-      Copy replacement.
-    */
-    result_length+=replace_length;
-    if ((result_length+MaxTextExtent) >= length)
-      {
-        length+=replace_length;
-        result=(char *) ResizeQuantumMemory(result,length+MaxTextExtent,
-          sizeof(*result));
-        if (result == (char *) NULL)
+        /*
+          Make room for the replacement string.
+        */
+        offset=p-(*string);
+        extent=strlen(*string)+replace_extent-search_extent+1;
+        *string=(char *) ResizeQuantumMemory(*string,extent+MaxTextExtent,
+          sizeof(*p));
+        if (*string == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
+        p=(*string)+offset;
       }
-    (void) ConcatenateMagickString(result+destination_offset,replace,(size_t)
-      (length+MaxTextExtent)*sizeof(*result));
-    destination_offset+=replace_length;
     /*
-      Find next match.
+      Replace string.
     */
-    source=match;
-    source+=search_length;
-    match=strstr(source,search);
+    if (search_extent != replace_extent)
+      (void) CopyMagickMemory(p+replace_extent,p+search_extent,
+        strlen(p+search_extent)+1);
+    (void) CopyMagickMemory(p,replace,replace_extent);
+    p+=replace_extent-1;
   }
-  /*
-    Copy remaining string.
-  */
-  copy_length=strlen(source);
-  result_length+=copy_length;
-  if ((result_length+MaxTextExtent) >= length)
-    {
-      length+=copy_length;
-      result=(char *) ResizeQuantumMemory(result,length+MaxTextExtent,
-        sizeof(*result));
-      if (result == (char *) NULL)
-        ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-    }
-  (void) ConcatenateMagickString(result+destination_offset,source,(size_t)
-    (length+MaxTextExtent-destination_offset)*sizeof(*result));
-  (void) RelinquishMagickMemory(*buffer);
-  *buffer=result;
-  return(MagickTrue);
+  return(status);
 }
