@@ -77,6 +77,9 @@ extern "C" {
 # if defined(__cplusplus) || defined(c_plusplus)
 #  undef inline
 # endif
+#if defined(_magickcore_restrict) && !defined(__restrict)
+# define __restrict _magickcore_restrict
+#endif
 #endif
 
 #if defined(MAGICKCORE_NAMESPACE_PREFIX)
@@ -183,8 +186,11 @@ extern "C" {
 #include <signal.h>
 #include <assert.h>
 
-#if defined(MAGICKCORE_HAVE_PTHREAD )
+#if defined(MAGICKCORE_HAVE_PTHREAD)
 # include <pthread.h>
+#elif defined(__WINDOWS__)
+#  define MAGICKCORE_HAVE_WINTHREADS  1
+#include <windows.h>
 #endif
 #if defined(MAGICKCORE_HAVE_SYS_SYSLIMITS_H)
 # include <sys/syslimits.h>
@@ -193,9 +199,19 @@ extern "C" {
 # include <arm/limits.h>
 #endif
 
+#if defined(MAGICKCORE_HAVE_CL_CL_H)
+#  include <CL/cl.h>
+#  define MAGICKCORE_OPENCL_SUPPORT  1
+#endif
+
+#if defined(MAGICKCORE_HAVE_OPENCL_CL_H)
+#  include <OpenCL/cl.h>
+#  define MAGICKCORE_OPENCL_SUPPORT  1
+#endif
+
 #if defined(_OPENMP) && (_OPENMP >= 200203)
 #  include <omp.h>
-#  define MAGICKCORE_OPENMP_SUPPORT 1
+#  define MAGICKCORE_OPENMP_SUPPORT  1
 #endif
 
 #if defined(MAGICKCORE_HAVE_PREAD) && defined(MAGICKCORE_HAVE_DECL_PREAD) && !MAGICKCORE_HAVE_DECL_PREAD
@@ -305,8 +321,11 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 # include "magick/vms.h"
 #endif
 
+#undef HAVE_CONFIG_H
+#undef gamma
 #undef index
 #undef pipe
+#undef y1
 
 /*
   Review these platform specific definitions.
@@ -409,6 +428,10 @@ extern int vsnprintf(char *,size_t,const char *,va_list);
 #define O_BINARY  0x00
 #endif
 
+#if !defined(PATH_MAX)
+#define PATH_MAX  4096
+#endif
+
 #if defined(MAGICKCORE_LTDL_DELEGATE) || (defined(__WINDOWS__) && defined(_DLL) && !defined(_LIB))
 #  define MAGICKCORE_MODULES_SUPPORT
 #endif
