@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,10 +13,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick token methods.
+  MagickCore token methods.
 */
-#ifndef _MAGICK_TOKEN_H
-#define _MAGICK_TOKEN_H
+#ifndef _MAGICKCORE_TOKEN_H
+#define _MAGICKCORE_TOKEN_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -26,30 +26,22 @@ extern "C" {
   Typedef declarations.
 */
 typedef struct _TokenInfo
-{
-  int
-    state;
-
-  MagickStatusType
-    flag;
-
-  long
-    offset;
-
-  char
-    quote;
-} TokenInfo;
+  TokenInfo;
 
 extern MagickExport int
   Tokenizer(TokenInfo *,const unsigned int,char *,const size_t,const char *,
     const char *,const char *,const char *,const char,char *,int *,char *);
 
 extern MagickExport MagickBooleanType
-  GlobExpression(const char *,const char *),
+  GlobExpression(const char *,const char *,const MagickBooleanType),
   IsGlob(const char *);
 
+extern MagickExport TokenInfo
+  *AcquireTokenInfo(void),
+  *DestroyTokenInfo(TokenInfo *);
+
 extern MagickExport void
-  GetMagickToken(const char *,char **,char *);
+  GetMagickToken(const char *,const char **,char *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
