@@ -10,14 +10,14 @@
 %                 V    EEEEE  R  R   SSSSS  IIIII   OOO   N   N               %
 %                                                                             %
 %                                                                             %
-%                  ImageMagick Version and Copyright Methods                  %
+%                   MagickCore Version and Copyright Methods                  %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -68,7 +68,6 @@
 %
 %      const char *GetMagickCopyright(void)
 %
-%
 */
 MagickExport const char *GetMagickCopyright(void)
 {
@@ -80,6 +79,31 @@ MagickExport const char *GetMagickCopyright(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M a g i c k F e a t u r e s                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickFeatures() returns the ImageMagick features.
+%
+%  The format of the GetMagickFeatures method is:
+%
+%      const char *GetMagickFeatures(void)
+%
+%  No parameters are required.
+%
+*/
+MagickExport const char *GetMagickFeatures(void)
+{
+  return(MagickFeatures);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t M a g i c k H o m e U R L                                           %
 %                                                                             %
 %                                                                             %
@@ -92,7 +116,6 @@ MagickExport const char *GetMagickCopyright(void)
 %
 %      char *GetMagickHomeURL(void)
 %
-%
 */
 MagickExport char *GetMagickHomeURL(void)
 {
@@ -103,26 +126,26 @@ MagickExport char *GetMagickHomeURL(void)
     *element;
 
   ExceptionInfo
-    exception;
+    *exception;
 
   LinkedListInfo
     *paths;
 
-  GetExceptionInfo(&exception);
-  paths=GetConfigurePaths(MagickURLFilename,&exception);
-  DestroyExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  paths=GetConfigurePaths(MagickURLFilename,exception);
+  exception=DestroyExceptionInfo(exception);
   if (paths == (LinkedListInfo *) NULL)
-    return(AcquireString(MagickHomeURL));
+    return(ConstantString(MagickHomeURL));
   element=(const char *) GetNextValueInLinkedList(paths);
   while (element != (const char *) NULL)
   {
     (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",element,
       DirectorySeparator,MagickURLFilename);
-    if (IsAccessible(path) != MagickFalse)
-      return(AcquireString(path));
+    if (IsPathAccessible(path) != MagickFalse)
+      return(ConstantString(path));
     element=(const char *) GetNextValueInLinkedList(paths);
   }
-  return(AcquireString(MagickHomeURL));
+  return(ConstantString(MagickHomeURL));
 }
 
 /*
@@ -144,7 +167,6 @@ MagickExport char *GetMagickHomeURL(void)
 %
 %  No parameters are required.
 %
-%
 */
 MagickExport const char *GetMagickPackageName(void)
 {
@@ -170,14 +192,13 @@ MagickExport const char *GetMagickPackageName(void)
 %
 %  A description of each parameter follows:
 %
-%    o depth: The quantum depth is returned as a number.
-%
+%    o depth: the quantum depth is returned as a number.
 %
 */
 MagickExport const char *GetMagickQuantumDepth(unsigned long *depth)
 {
   if (depth != (unsigned long *) NULL)
-    *depth=(unsigned long) QuantumDepth;
+    *depth=(unsigned long) MAGICKCORE_QUANTUM_DEPTH;
   return(MagickQuantumDepth);
 }
 
@@ -200,8 +221,7 @@ MagickExport const char *GetMagickQuantumDepth(unsigned long *depth)
 %
 %  A description of each parameter follows:
 %
-%    o range: The quantum range is returned as a number.
-%
+%    o range: the quantum range is returned as a number.
 %
 */
 MagickExport const char *GetMagickQuantumRange(unsigned long *range)
@@ -230,7 +250,6 @@ MagickExport const char *GetMagickQuantumRange(unsigned long *range)
 %
 %  No parameters are required.
 %
-%
 */
 MagickExport const char *GetMagickReleaseDate(void)
 {
@@ -257,7 +276,7 @@ MagickExport const char *GetMagickReleaseDate(void)
 %
 %  A description of each parameter follows:
 %
-%    o version: The ImageMagick version is returned as a number.
+%    o version: the ImageMagick version is returned as a number.
 %
 */
 MagickExport const char *GetMagickVersion(unsigned long *version)
