@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,16 +13,16 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick X11 widget methods.
+  MagickCore X11 widget methods.
 */
-#ifndef _MAGICK_WIDGET_H
-#define _MAGICK_WIDGET_H
+#ifndef _MAGICKCORE_WIDGET_H
+#define _MAGICKCORE_WIDGET_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-#if defined(HasX11)
+#if defined(MAGICKCORE_X11_DELEGATE)
 
 #include "magick/xwindow-private.h"
 
