@@ -17,7 +17,7 @@
 %                            December 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,14 +43,8 @@
 /*
   Include declarations.
 */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
-#include "magick/ImageMagick.h"
-#if defined(__WINDOWS__)
-#include <windows.h>
-#endif
+#include "wand/studio.h"
+#include "wand/MagickWand.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -67,11 +61,8 @@
 */
 int main(int argc,char **argv)
 {
-  char
-    *option;
-
   ExceptionInfo
-    exception;
+    *exception;
 
   ImageInfo
     *image_info;
@@ -79,33 +70,13 @@ int main(int argc,char **argv)
   MagickBooleanType
     status;
 
-  register long
-    i;
-
-  InitializeMagick(*argv);
-  GetExceptionInfo(&exception);
-  for (i=1; i < (long) argc; i++)
-  {
-    option=argv[i];
-    if ((strlen(option) == 1) || ((*option != '-') && (*option != '+')))
-      continue;
-    if ((LocaleCompare("debug",option+1) == 0) && (i < (long) (argc-1)))
-      (void) SetLogEventMask(argv[++i]);
-    if (LocaleCompare("version",option+1) == 0)
-      {
-        (void) fprintf(stdout,"Version: %s\n",
-          GetMagickVersion((unsigned long *) NULL));
-        (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-        exit(0);
-      }
-  }
-  image_info=CloneImageInfo((ImageInfo *) NULL);
-  status=MogrifyImageCommand(image_info,argc,argv,(char **) NULL,&exception);
-  if (exception.severity != UndefinedException)
-    CatchException(&exception);
+  MagickCoreGenesis(*argv,MagickTrue);
+  exception=AcquireExceptionInfo();
+  image_info=AcquireImageInfo();
+  status=MagickCommandGenesis(image_info,MogrifyImageCommand,argc,argv,
+    (char **) NULL,exception);
   image_info=DestroyImageInfo(image_info);
-  DestroyExceptionInfo(&exception);
-  DestroyMagick();
-  exit(!status);
-  return(MagickFalse);
+  exception=DestroyExceptionInfo(exception);
+  MagickCoreTerminus();
+  return(status);
 }
