@@ -33,6 +33,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
+%  Use the animate program to animate an image sequence on any X server.
 %
 */
 
@@ -124,7 +125,8 @@ static MagickBooleanType AnimateUsage(void)
     },
     *settings[]=
     {
-      "-alpha option        activate, deactivate, reset, or set the alpha channel",
+      "-alpha option        on, activate, off, deactivate, set, opaque, copy",
+      "                     transparent, extract, background, or shape",
       "-authenticate password",
       "                     decipher image with this password",
       "-backdrop            display image centered on a backdrop",
@@ -183,7 +185,8 @@ static MagickBooleanType AnimateUsage(void)
     };
 
   (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
-  (void) printf("Copyright: %s\n\n",GetMagickCopyright());
+  (void) printf("Copyright: %s\n",GetMagickCopyright());
+  (void) printf("Features: %s\n\n",GetMagickFeatures());
   (void) printf("Usage: %s [options ...] file [ [options ...] file ...]\n",
     GetClientName());
   (void) printf("\nImage Settings:\n");
@@ -308,7 +311,8 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
         {
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
-          (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
           return(MagickFalse);
         }
     }
@@ -338,8 +342,6 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
       Check command line for server name.
     */
     option=argv[i];
-    if (IsMagickOption(option) == MagickFalse)
-      continue;
     if (LocaleCompare("display",option+1) == 0)
       {
         /*
@@ -435,7 +437,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
         /*
           Read input image.
         */
-        FireImageStack(MagickTrue,MagickTrue,pend);
+        FireImageStack(MagickFalse,MagickFalse,pend);
         filename=option;
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           {
@@ -632,6 +634,8 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("concurrent",option+1) == 0)
+          break;
         if (LocaleCompare("crop",option+1) == 0)
           {
             if (*option == '+')
@@ -767,6 +771,17 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             quantize_info->dither_method=(DitherMethod) method;
             break;
           }
+        if (LocaleCompare("duration",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowAnimateException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowAnimateInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowAnimateException(OptionError,"UnrecognizedOption",option);
       }
       case 'e':
@@ -1348,7 +1363,8 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
           {
             (void) fprintf(stdout,"Version: %s\n",
               GetMagickVersion((unsigned long *) NULL));
-            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
             break;
           }
         if (LocaleCompare("virtual-pixel",option+1) == 0)
@@ -1401,7 +1417,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
       default:
         ThrowAnimateException(OptionError,"UnrecognizedOption",option);
     }
-    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ? 
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ?
       MagickFalse : MagickTrue;
     if (fire != MagickFalse)
       FireImageStack(MagickFalse,MagickTrue,MagickTrue);
@@ -1411,9 +1427,14 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
     ThrowAnimateException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (image == (Image *) NULL)
     ThrowAnimateException(OptionError,"MissingAnImageFilename",argv[argc-1])
-  FinalizeImageSettings(image_info,image);
+  FinalizeImageSettings(image_info,image,MagickTrue);
+  if (image == (Image *) NULL)
+    ThrowAnimateException(OptionError,"MissingAnImageFilename",argv[argc-1])
   if (resource_info.window_id != (char *) NULL)
-    XAnimateBackgroundImage(display,&resource_info,image);
+    {
+      XAnimateBackgroundImage(display,&resource_info,image);
+      status&=MagickTrue;
+    }
   else
     {
       Image
@@ -1423,15 +1444,20 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
         Animate image to X server.
       */
       animate_image=XAnimateImages(display,&resource_info,argv,argc,image);
+      status&=animate_image != (Image *) NULL;
       while (animate_image != (Image *) NULL)
       {
         image=animate_image;
         animate_image=XAnimateImages(display,&resource_info,argv,argc,image);
+        if (animate_image != image)
+          image=DestroyImageList(image);
       }
     }
   DestroyAnimate();
   return(status != 0 ? MagickTrue : MagickFalse);
 #else
+  (void) argc;
+  (void) argv;
   (void) ThrowMagickException(exception,GetMagickModule(),MissingDelegateError,
     "DelegateLibrarySupportNotBuiltIn","`%s' (X11)",image_info->filename);
   return(AnimateUsage());
