@@ -33,7 +33,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%
+%  Use the compare program to mathematically and visually annotate the
+%  difference between an image and its reconstruction.
 %
 */
 
@@ -94,7 +95,8 @@ static MagickBooleanType CompareUsage(void)
     },
     *settings[]=
     {
-      "-alpha option        activate, deactivate, reset, or set the alpha channel",
+      "-alpha option        on, activate, off, deactivate, set, opaque, copy",
+      "                     transparent, extract, background, or shape",
       "-authenticate password",
       "                     decipher image with this password",
       "-channel type        apply option to select image channels",
@@ -106,6 +108,8 @@ static MagickBooleanType CompareUsage(void)
       "                     define one or more image format options",
       "-density geometry    horizontal and vertical density of the image",
       "-depth value         image depth",
+      "-dissimilarity-threshold value",
+      "                     maximum RMSE for (sub)image match",
       "-encipher filename   convert plain pixels to cipher pixels",
       "-extract geometry    extract area from image",
       "-format \"string\"     output formatted image characteristics",
@@ -130,6 +134,7 @@ static MagickBooleanType CompareUsage(void)
       "                     horizontal and vertical sampling factor",
       "-seed value          seed a new sequence of pseudo-random numbers",
       "-set attribute value set an image attribute",
+      "-quality value       JPEG/MIFF/PNG compression level",
       "-size geometry       width and height of image",
       "-transparent-color color",
       "                     transparent color",
@@ -142,7 +147,8 @@ static MagickBooleanType CompareUsage(void)
     };
 
   (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
-  (void) printf("Copyright: %s\n\n",GetMagickCopyright());
+  (void) printf("Copyright: %s\n",GetMagickCopyright());
+  (void) printf("Features: %s\n\n",GetMagickFeatures());
   (void) printf("Usage: %s [options ...] image reconstruct difference\n",
     GetClientName());
   (void) printf("\nImage Settings:\n");
@@ -166,8 +172,13 @@ static MagickBooleanType CompareUsage(void)
 WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   int argc,char **argv,char **metadata,ExceptionInfo *exception)
 {
+#define DefaultDissimilarityThreshold  0.2
 #define DestroyCompare() \
 { \
+  if (similarity_image != (Image *) NULL) \
+    similarity_image=DestroyImageList(similarity_image); \
+  if (difference_image != (Image *) NULL) \
+    difference_image=DestroyImageList(difference_image); \
   DestroyImageStack(); \
   for (i=0; i < (long) argc; i++) \
     argv[i]=DestroyString(argv[i]); \
@@ -200,13 +211,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     channels;
 
   double
-    distortion;
+    dissimilarity_threshold,
+    distortion,
+    similarity_metric;
 
   Image
     *difference_image,
     *image,
     *reconstruct_image,
-    *subimage;
+    *similarity_image;
 
   ImageStack
     image_stack[MaxImageStackDepth+1];
@@ -225,6 +238,9 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   MetricType
     metric;
 
+  RectangleInfo
+    offset;
+
   register long
     i;
 
@@ -244,7 +260,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         {
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
-          (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
           return(MagickFalse);
         }
     }
@@ -255,6 +272,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     }
   channels=AllChannels;
   difference_image=NewImageList();
+  similarity_image=NewImageList();
+  dissimilarity_threshold=DefaultDissimilarityThreshold;
   distortion=0.0;
   format=(char *) NULL;
   j=1;
@@ -272,7 +291,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
     ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
@@ -301,7 +320,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         /*
           Read input image.
         */
-        FireImageStack(MagickTrue,MagickTrue,pend);
+        FireImageStack(MagickFalse,MagickFalse,pend);
         filename=argv[i];
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
@@ -427,6 +446,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("concurrent",option+1) == 0)
+          break;
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'd':
@@ -495,6 +516,32 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("dissimilarity-threshold",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            if (*option == '+')
+              dissimilarity_threshold=DefaultDissimilarityThreshold;
+            else
+              dissimilarity_threshold=atof(argv[i]);
+            break;
+          }
+        if (LocaleCompare("duration",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'e':
@@ -831,7 +878,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           {
             (void) fprintf(stdout,"Version: %s\n",
               GetMagickVersion((unsigned long *) NULL));
-            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
             break;
           }
         if (LocaleCompare("virtual-pixel",option+1) == 0)
@@ -867,22 +915,55 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     ThrowCompareException(OptionError,"UnbalancedParenthesis",argv[i]);
   if (i-- != (long) (argc-1))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
-  if ((image == (Image *) NULL) ||
-      (GetImageListLength(image) < 2))
+  if ((image == (Image *) NULL) || (GetImageListLength(image) < 2))
+    ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
+  FinalizeImageSettings(image_info,image,MagickTrue);
+  if ((image == (Image *) NULL) || (GetImageListLength(image) < 2))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
-  FinalizeImageSettings(image_info,image);
   image=GetImageFromList(image,0);
   reconstruct_image=GetImageFromList(image,1);
-  subimage=ExtractSubimageFromImage(image,reconstruct_image,exception);
-  if (subimage == (Image *) NULL)
+  similarity_image=SimilarityImage(image,reconstruct_image,&offset,
+    &similarity_metric,exception);
+  if (similarity_metric > dissimilarity_threshold)
+    ThrowCompareException(ImageError,"ImagesTooDissimilar",image->filename);
+  if ((reconstruct_image->columns == image->columns) &&
+      (reconstruct_image->rows == image->rows))
     difference_image=CompareImageChannels(image,reconstruct_image,channels,
       metric,&distortion,exception);
   else
-    {
-      difference_image=CompareImageChannels(subimage,reconstruct_image,channels,
-        metric,&distortion,exception);
-      subimage=DestroyImage(subimage);
-    }
+    if (similarity_image != (Image *) NULL)
+      {
+        Image
+          *composite_image;
+
+        /*
+          Determine if reconstructed image is a subimage of the image.
+        */
+        composite_image=CloneImage(image,0,0,MagickTrue,exception);
+        if (composite_image == (Image *) NULL)
+          difference_image=CompareImageChannels(image,reconstruct_image,
+            channels,metric,&distortion,exception);
+        else
+          {
+            (void) CompositeImage(composite_image,CopyCompositeOp,
+              reconstruct_image,offset.x,offset.y);
+            difference_image=CompareImageChannels(image,composite_image,
+              channels,metric,&distortion,exception);
+            if (difference_image != (Image *) NULL)
+              {
+                difference_image->page.x=offset.x;
+                difference_image->page.y=offset.y;
+              }
+            composite_image=DestroyImage(composite_image);
+          }
+        if (difference_image == (Image *) NULL)
+          similarity_image=DestroyImage(similarity_image);
+        else
+          {
+            AppendImageToList(&difference_image,similarity_image);
+            similarity_image=(Image *) NULL;
+          }
+      }
   if (difference_image == (Image *) NULL)
     status=0;
   else
@@ -892,20 +973,6 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       if (*difference_image->magick == '\0')
         (void) CopyMagickString(difference_image->magick,image->magick,
           MaxTextExtent);
-      status&=WriteImages(image_info,difference_image,argv[argc-1],exception);
-      if ((metadata != (char **) NULL) && (format != (char *) NULL))
-        {
-          char
-            *text;
-
-          text=InterpretImageProperties(image_info,difference_image,format);
-          if (text == (char *) NULL)
-            ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-              strerror(errno));
-          (void) ConcatenateString(&(*metadata),text);
-          (void) ConcatenateString(&(*metadata),"\n");
-          text=DestroyString(text);
-        }
       if (image_info->verbose == MagickFalse)
         {
           switch (metric)
@@ -915,14 +982,24 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             case RootMeanSquaredErrorMetric:
             case PeakAbsoluteErrorMetric:
             {
-              (void) fprintf(stderr,"%g (%g)",distortion,(double)
-                (QuantumScale*distortion));
+              (void) fprintf(stderr,"%g (%g)",QuantumRange*distortion,(double)
+                distortion);
+              if ((reconstruct_image->columns != image->columns) ||
+                  (reconstruct_image->rows != image->rows))
+                (void) fprintf(stderr," @ %ld,%ld",difference_image->page.x,
+                  difference_image->page.y);
+              (void) fprintf(stderr,"\n");
               break;
             }
             case AbsoluteErrorMetric:
             case PeakSignalToNoiseRatioMetric:
             {
               (void) fprintf(stderr,"%g",distortion);
+              if ((reconstruct_image->columns != image->columns) ||
+                  (reconstruct_image->rows != image->rows))
+                (void) fprintf(stderr," @ %ld,%ld",difference_image->page.x,
+                  difference_image->page.y);
+              (void) fprintf(stderr,"\n");
               break;
             }
             case MeanErrorPerPixelMetric:
@@ -930,13 +1007,16 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               (void) fprintf(stderr,"%g (%g, %g)",distortion,
                 image->error.normalized_mean_error,
                 image->error.normalized_maximum_error);
+              if ((reconstruct_image->columns != image->columns) ||
+                  (reconstruct_image->rows != image->rows))
+                (void) fprintf(stderr," @ %ld,%ld",difference_image->page.x,
+                  difference_image->page.y);
+              (void) fprintf(stderr,"\n");
               break;
             }
             case UndefinedMetric:
               break;
           }
-          (void) fprintf(stderr," @ %ld,%ld\n",difference_image->page.x,
-            difference_image->page.y);
         }
       else
         {
@@ -946,8 +1026,10 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           channel_distortion=GetImageChannelDistortions(image,reconstruct_image,
             metric,&image->exception);
           (void) fprintf(stderr,"Image: %s\n",image->filename);
-          (void) fprintf(stderr,"Offset: %ld,%ld\n",difference_image->page.x,
-            difference_image->page.y);
+          if ((reconstruct_image->columns != image->columns) ||
+              (reconstruct_image->rows != image->rows))
+            (void) fprintf(stderr,"Offset: %ld,%ld\n",difference_image->page.x,
+              difference_image->page.y);
           (void) fprintf(stderr,"  Channel distortion: %s\n",
             MagickOptionToMnemonic(MagickMetricOptions,(long) metric));
           switch (metric)
@@ -963,54 +1045,54 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                 default:
                 {
                   (void) fprintf(stderr,"    red: %g (%g)\n",
-                    channel_distortion[RedChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[RedChannel],
                     channel_distortion[RedChannel]);
                   (void) fprintf(stderr,"    green: %g (%g)\n",
-                    channel_distortion[GreenChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[GreenChannel],
                     channel_distortion[GreenChannel]);
                   (void) fprintf(stderr,"    blue: %g (%g)\n",
-                    channel_distortion[BlueChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[BlueChannel],
                     channel_distortion[BlueChannel]);
                   if (image->matte != MagickFalse)
                     (void) fprintf(stderr,"    alpha: %g (%g)\n",
-                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      QuantumRange*channel_distortion[OpacityChannel],
                       channel_distortion[OpacityChannel]);
                   break;
                 }
                 case CMYKColorspace:
                 {
                   (void) fprintf(stderr,"    cyan: %g (%g)\n",
-                    channel_distortion[CyanChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[CyanChannel],
                     channel_distortion[CyanChannel]);
                   (void) fprintf(stderr,"    magenta: %g (%g)\n",
-                    channel_distortion[MagentaChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[MagentaChannel],
                     channel_distortion[MagentaChannel]);
                   (void) fprintf(stderr,"    yellow: %g (%g)\n",
-                    channel_distortion[YellowChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[YellowChannel],
                     channel_distortion[YellowChannel]);
                   (void) fprintf(stderr,"    black: %g (%g)\n",
-                    channel_distortion[BlackChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[BlackChannel],
                     channel_distortion[BlackChannel]);
                   if (image->matte != MagickFalse)
                     (void) fprintf(stderr,"    alpha: %g (%g)\n",
-                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      QuantumRange*channel_distortion[OpacityChannel],
                       channel_distortion[OpacityChannel]);
                   break;
                 }
                 case GRAYColorspace:
                 {
                   (void) fprintf(stderr,"    gray: %g (%g)\n",
-                    channel_distortion[GrayChannel],(double) QuantumScale*
+                    QuantumRange*channel_distortion[GrayChannel],
                     channel_distortion[GrayChannel]);
                   if (image->matte != MagickFalse)
                     (void) fprintf(stderr,"    alpha: %g (%g)\n",
-                      channel_distortion[OpacityChannel],(double) QuantumScale*
+                      QuantumRange*channel_distortion[OpacityChannel],
                       channel_distortion[OpacityChannel]);
                   break;
                 }
               }
               (void) fprintf(stderr,"    all: %g (%g)\n",
-                channel_distortion[AllChannels],(double) QuantumScale*
+                QuantumRange*channel_distortion[AllChannels],
                 channel_distortion[AllChannels]);
               break;
             }
@@ -1076,6 +1158,20 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           channel_distortion=(double *) RelinquishMagickMemory(
             channel_distortion);
         }
+      status&=WriteImages(image_info,difference_image,argv[argc-1],exception);
+      if ((metadata != (char **) NULL) && (format != (char *) NULL))
+        {
+          char
+            *text;
+
+          text=InterpretImageProperties(image_info,difference_image,format);
+          if (text == (char *) NULL)
+            ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
+              GetExceptionMessage(errno));
+          (void) ConcatenateString(&(*metadata),text);
+          (void) ConcatenateString(&(*metadata),"\n");
+          text=DestroyString(text);
+        }
       difference_image=DestroyImageList(difference_image);
     }
   DestroyCompare();
