@@ -33,7 +33,9 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%
+%  Use the montage program to create a composite image by combining several
+%  separate images. The images are tiled on the composite image optionally
+%  adorned with a border, frame, image name, and more.
 %
 */
 
@@ -99,9 +101,11 @@ static MagickBooleanType MontageUsage(void)
       "                     adaptively sharpen pixels; increase effect near edges",
       " annotate geometry text",
       "                     annotate the image with text",
+      "-auto-orient         automagically orient image",
       "-blur geometry      reduce image noise and reduce detail levels",
       "-border geometry     surround image with a border of color",
       "-crop geometry       preferred size and location of the cropped image",
+      "-extent geometry     set the image size",
       "-flatten             flatten a sequence of images",
       "-flip                flip image in the vertical direction",
       "-flop                flop image in the horizontal direction",
@@ -123,7 +127,8 @@ static MagickBooleanType MontageUsage(void)
     {
       "-adjoin              join images into a single multi-image file",
       "-affine matrix       affine transform matrix",
-      "-alpha option        activate, deactivate, reset, or set the alpha channel",
+      "-alpha option        on, activate, off, deactivate, set, opaque, copy",
+      "                     transparent, extract, background, or shape",
       "-authenticate password",
       "                     decipher image with this password",
       "-blue-primary point  chromaticity blue primary point",
@@ -198,6 +203,7 @@ static MagickBooleanType MontageUsage(void)
     *sequence_operators[]=
     {
       "-coalesce            merge a sequence of images",
+      "-composite           composite image",
       (char *) NULL
     },
     *stack_operators[]=
@@ -207,7 +213,8 @@ static MagickBooleanType MontageUsage(void)
     };
 
   (void) printf("Version: %s\n",GetMagickVersion((unsigned long *) NULL));
-  (void) printf("Copyright: %s\n\n",GetMagickCopyright());
+  (void) printf("Copyright: %s\n",GetMagickCopyright());
+  (void) printf("Features: %s\n\n",GetMagickFeatures());
   (void) printf("Usage: %s [options ...] file [ [options ...] file ...] file\n",
     GetClientName());
   (void) printf("\nImage Settings:\n");
@@ -320,7 +327,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
         {
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
-          (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+          (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
           return(MagickFalse);
         }
     }
@@ -349,7 +357,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
     ThrowMontageException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
@@ -375,7 +383,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
         Image
           *images;
 
-        FireImageStack(MagickTrue,MagickTrue,pend);
+        FireImageStack(MagickFalse,MagickFalse,pend);
         for (scene=first_scene; scene <= last_scene ; scene++)
         {
           char
@@ -471,6 +479,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
             i++;
             break;
           }
+        if (LocaleCompare("auto-orient",option+1) == 0)
+          break;
         if (LocaleCompare("authenticate",option+1) == 0)
           {
             if (*option == '+')
@@ -519,6 +529,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("border",option+1) == 0)
           {
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             montage_info->border_width=0;
             if (*option == '+')
               break;
@@ -676,6 +687,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("composite",option+1) == 0)
+          break;
         if (LocaleCompare("compress",option+1) == 0)
           {
             long
@@ -693,6 +706,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("concurrent",option+1) == 0)
+          break;
         if (LocaleCompare("crop",option+1) == 0)
           {
             if (*option == '+')
@@ -814,6 +829,17 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("duration",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowMontageInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
       }
       case 'e':
@@ -844,6 +870,17 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
                 argv[i]);
             break;
           }
+        if (LocaleCompare("extent",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowMontageInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
       }
       case 'f':
@@ -1501,8 +1538,6 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
         if (LocaleCompare("transparent",option+1) == 0)
           {
             transparent_color=(char *) NULL;
-            if (*option == '+')
-              break;
             i++;
             if (i == (long) argc)
               ThrowMontageException(OptionError,"MissingArgument",option);
@@ -1592,7 +1627,8 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           {
             (void) fprintf(stdout,"Version: %s\n",
               GetMagickVersion((unsigned long *) NULL));
-            (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Copyright: %s\n",GetMagickCopyright());
+            (void) fprintf(stdout,"Features: %s\n\n",GetMagickFeatures());
             break;
           }
         if (LocaleCompare("virtual-pixel",option+1) == 0)
@@ -1634,7 +1670,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
       default:
         ThrowMontageException(OptionError,"UnrecognizedOption",option)
     }
-    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ? 
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ?
       MagickFalse : MagickTrue;
     if (fire != MagickFalse)
       FireImageStack(MagickTrue,MagickTrue,MagickTrue);
@@ -1645,7 +1681,9 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
     ThrowMontageException(OptionError,"MissingAnImageFilename",argv[i]);
   if (image == (Image *) NULL)
     ThrowMontageException(OptionError,"MissingAnImageFilename",argv[argc-1]);
-  FinalizeImageSettings(image_info,image);
+  FinalizeImageSettings(image_info,image,MagickTrue);
+  if (image == (Image *) NULL)
+    ThrowMontageException(OptionError,"MissingAnImageFilename",argv[argc-1]);
   (void) CopyMagickString(montage_info->filename,argv[argc-1],MaxTextExtent);
   montage_image=MontageImageList(image_info,montage_info,image,exception);
   if (montage_image == (Image *) NULL)
@@ -1671,7 +1709,7 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
           text=InterpretImageProperties(image_info,montage_image,format);
           if (text == (char *) NULL)
             ThrowMontageException(ResourceLimitError,"MemoryAllocationFailed",
-              strerror(errno));
+              GetExceptionMessage(errno));
           (void) ConcatenateString(&(*metadata),text);
           (void) ConcatenateString(&(*metadata),"\n");
           text=DestroyString(text);
