@@ -23,7 +23,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,12 +47,11 @@
   Include declarations.
 */
 #include "wand/studio.h"
-#include "wand/magick-wand.h"
+#include "wand/MagickWand.h"
 #include "wand/magick-wand-private.h"
 #include "wand/pixel-iterator.h"
 #include "wand/pixel-wand.h"
 #include "wand/wand.h"
-#include "magick/ImageMagick.h"
 
 /*
   Define declarations.
@@ -71,19 +70,22 @@ struct _PixelIterator
     name[MaxTextExtent];
 
   ExceptionInfo
-    exception;
+    *exception;
 
-  ViewInfo
+  CacheView
     *view;
 
   RectangleInfo
     region;
 
+  MagickBooleanType
+    active;
+
   long
     y;
 
   PixelWand
-    **pixel_wand;
+    **pixel_wands;
 
   MagickBooleanType
     debug;
@@ -111,23 +113,21 @@ struct _PixelIterator
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
-%
+%    o iterator: the pixel iterator.
 %
 */
 WandExport void ClearPixelIterator(PixelIterator *iterator)
 {
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  CloseCacheView(iterator->view);
-  iterator->pixel_wand=DestroyPixelWands(iterator->pixel_wand,
+  iterator->pixel_wands=DestroyPixelWands(iterator->pixel_wands,
     iterator->region.width);
-  DestroyExceptionInfo(&iterator->exception);
-  GetExceptionInfo(&iterator->exception);
-  iterator->pixel_wand=NewPixelWands(iterator->region.width);
-  iterator->y=(-1);
+  ClearMagickException(iterator->exception);
+  iterator->pixel_wands=NewPixelWands(iterator->region.width);
+  iterator->active=MagickFalse;
+  iterator->y=0;
   iterator->debug=IsEventLogging();
 }
 
@@ -136,6 +136,61 @@ WandExport void ClearPixelIterator(PixelIterator *iterator)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C l o n e P i x e l I t e r a t o r                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ClonePixelIterator() makes an exact copy of the specified iterator.
+%
+%  The format of the ClonePixelIterator method is:
+%
+%      PixelIterator *ClonePixelIterator(const PixelIterator *iterator)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: the magick iterator.
+%
+*/
+WandExport PixelIterator *ClonePixelIterator(const PixelIterator *iterator)
+{
+  PixelIterator
+    *clone_iterator;
+
+  assert(iterator != (PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  clone_iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*clone_iterator));
+  if (clone_iterator == (PixelIterator *) NULL)
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      iterator->name);
+  (void) ResetMagickMemory(clone_iterator,0,sizeof(*clone_iterator));
+  clone_iterator->id=AcquireWandId();
+  (void) FormatMagickString(clone_iterator->name,MaxTextExtent,"%s-%lu",
+    PixelIteratorId,clone_iterator->id);
+  clone_iterator->exception=AcquireExceptionInfo();
+  InheritException(clone_iterator->exception,iterator->exception);
+  clone_iterator->view=CloneCacheView(iterator->view);
+  clone_iterator->region=iterator->region;
+  clone_iterator->active=iterator->active;
+  clone_iterator->y=iterator->y;
+  clone_iterator->pixel_wands=ClonePixelWands((const PixelWand **)
+    iterator->pixel_wands,iterator->region.width);
+  clone_iterator->debug=iterator->debug;
+  if (clone_iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",
+      clone_iterator->name);
+  clone_iterator->signature=WandSignature;
+  return(clone_iterator);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   D e s t r o y P i x e l I t e r a t o r                                   %
 %                                                                             %
 %                                                                             %
@@ -150,21 +205,20 @@ WandExport void ClearPixelIterator(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
-%
+%    o iterator: the pixel iterator.
 %
 */
 WandExport PixelIterator *DestroyPixelIterator(PixelIterator *iterator)
 {
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  CloseCacheView(iterator->view);
-  iterator->pixel_wand=DestroyPixelWands(iterator->pixel_wand,
+  iterator->view=DestroyCacheView(iterator->view);
+  iterator->pixel_wands=DestroyPixelWands(iterator->pixel_wands,
     iterator->region.width);
-  DestroyExceptionInfo(&iterator->exception);
-  iterator->signature=(~MagickSignature);
+  iterator->exception=DestroyExceptionInfo(iterator->exception);
+  iterator->signature=(~WandSignature);
   RelinquishWandId(iterator->id);
   iterator=(PixelIterator *) RelinquishMagickMemory(iterator);
   return(iterator);
@@ -190,17 +244,17 @@ WandExport PixelIterator *DestroyPixelIterator(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The magick iterator.
+%    o iterator: the magick iterator.
 %
 */
-MagickExport MagickBooleanType IsPixelIterator(const PixelIterator *iterator)
+WandExport MagickBooleanType IsPixelIterator(const PixelIterator *iterator)
 {
   size_t
     length;
 
   if (iterator == (const PixelIterator *) NULL)
     return(MagickFalse);
-  if (iterator->signature != MagickSignature)
+  if (iterator->signature != WandSignature)
     return(MagickFalse);
   length=strlen(PixelIteratorId);
   if (LocaleNCompare(iterator->name,PixelIteratorId,length) != 0)
@@ -227,7 +281,7 @@ MagickExport MagickBooleanType IsPixelIterator(const PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 */
 WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
@@ -244,37 +298,41 @@ WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
   unsigned long
     depth;
 
-  ViewInfo
+  CacheView
     *view;
 
-  depth=QuantumDepth;
+  depth=MAGICKCORE_QUANTUM_DEPTH;
   quantum=GetMagickQuantumDepth(&depth);
-  if (depth != QuantumDepth)
+  if (depth != MAGICKCORE_QUANTUM_DEPTH)
     ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
   assert(wand != (MagickWand *) NULL);
   image=GetImageFromMagickWand(wand);
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
-  view=OpenCacheView(image);
-  if (view == (ViewInfo *) NULL)
+  view=AcquireCacheView(image);
+  if (view == (CacheView *) NULL)
     return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   (void) ResetMagickMemory(iterator,0,sizeof(*iterator));
   iterator->id=AcquireWandId();
   (void) FormatMagickString(iterator->name,MaxTextExtent,"%s-%lu",
     PixelIteratorId,iterator->id);
-  GetExceptionInfo(&iterator->exception);
+  iterator->exception=AcquireExceptionInfo();
   iterator->view=view;
   SetGeometry(image,&iterator->region);
-  iterator->pixel_wand=NewPixelWands(iterator->region.width);
-  iterator->y=(-1);
+  iterator->region.width=image->columns;
+  iterator->region.height=image->rows;
+  iterator->region.x=0;
+  iterator->region.y=0;
+  iterator->pixel_wands=NewPixelWands(iterator->region.width);
+  iterator->y=0;
   iterator->debug=IsEventLogging();
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->signature=MagickSignature;
+  iterator->signature=WandSignature;
   return(iterator);
 }
 
@@ -298,17 +356,18 @@ WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The pixel wand.
+%    o wand: the pixel wand.
 %
 */
 WandExport MagickBooleanType PixelClearIteratorException(
   PixelIterator *iterator)
 {
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  return(SetExceptionInfo(&iterator->exception,UndefinedException));
+  ClearMagickException(iterator->exception);
+  return(MagickTrue);
 }
 
 /*
@@ -327,19 +386,18 @@ WandExport MagickBooleanType PixelClearIteratorException(
 %  The format of the NewPixelRegionIterator method is:
 %
 %      PixelIterator NewPixelRegionIterator(MagickWand *wand,const long x,
-%        const long y,const unsigned long columns,const unsigned long rows,
-%        const MagickBooleanType modify)
+%        const long y,const unsigned long width,const unsigned long height)
 %
 %  A description of each parameter follows:
 %
-%    o wand: The magick wand.
+%    o wand: the magick wand.
 %
 %    o x,y,columns,rows:  These values define the perimeter of a region of
 %      pixels.
 %
 */
 WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
-  const long y,const unsigned long columns,const unsigned long rows)
+  const long y,const unsigned long width,const unsigned long height)
 {
   const char
     *quantum;
@@ -353,43 +411,43 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
   unsigned long
     depth;
 
-  ViewInfo
+  CacheView
     *view;
 
   assert(wand != (MagickWand *) NULL);
-  depth=QuantumDepth;
+  depth=MAGICKCORE_QUANTUM_DEPTH;
   quantum=GetMagickQuantumDepth(&depth);
-  if (depth != QuantumDepth)
+  if (depth != MAGICKCORE_QUANTUM_DEPTH)
     ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
-  if ((columns == 0) || (rows == 0))
+  if ((width == 0) || (width == 0))
     ThrowWandFatalException(WandError,"ZeroRegionSize",quantum);
   image=GetImageFromMagickWand(wand);
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
-  view=OpenCacheView(image);
-  if (view == (ViewInfo *) NULL)
+  view=AcquireCacheView(image);
+  if (view == (CacheView *) NULL)
     return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      wand->name);
   (void) ResetMagickMemory(iterator,0,sizeof(*iterator));
   iterator->id=AcquireWandId();
   (void) FormatMagickString(iterator->name,MaxTextExtent,"%s-%lu",
     PixelIteratorId,iterator->id);
-  GetExceptionInfo(&iterator->exception);
+  iterator->exception=AcquireExceptionInfo();
   iterator->view=view;
   SetGeometry(image,&iterator->region);
+  iterator->region.width=width;
+  iterator->region.height=height;
   iterator->region.x=x;
   iterator->region.y=y;
-  iterator->region.width=columns;
-  iterator->region.height=rows;
-  iterator->pixel_wand=NewPixelWands(iterator->region.width);
-  iterator->y=(-1);
+  iterator->pixel_wands=NewPixelWands(iterator->region.width);
+  iterator->y=0;
   iterator->debug=IsEventLogging();
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->signature=MagickSignature;
+  iterator->signature=WandSignature;
   return(iterator);
 }
 
@@ -398,6 +456,72 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P i x e l G e t C u r r e n t I t e r a t o r R o w                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelGetCurrentIteratorRow() returns the current row as an array of pixel
+%  wands from the pixel iterator.
+%
+%  The format of the PixelGetCurrentIteratorRow method is:
+%
+%      PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
+%        unsigned long *number_wands)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: the pixel iterator.
+%
+%    o number_wands: the number of pixel wands.
+%
+*/
+WandExport PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
+  unsigned long *number_wands)
+{
+  register const IndexPacket
+    *indexes;
+
+  register const PixelPacket
+    *pixels;
+
+  register long
+    x;
+
+  assert(iterator != (PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  *number_wands=0;
+  iterator->active=MagickTrue;
+  pixels=GetCacheViewVirtualPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1,
+    iterator->exception);
+  if (pixels == (const PixelPacket *) NULL)
+    {
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
+      return((PixelWand **) NULL);
+    }
+  indexes=GetCacheViewVirtualIndexQueue(iterator->view);
+  for (x=0; x < (long) iterator->region.width; x++)
+    PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
+      PixelSetBlackQuantum(iterator->pixel_wands[x],indexes[x]);
+  if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+    for (x=0; x < (long) iterator->region.width; x++)
+      PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
+  *number_wands=iterator->region.width;
+  return(iterator->pixel_wands);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P i x e l G e t I t e r a t o r E x c e p t i o n                         %
 %                                                                             %
 %                                                                             %
@@ -414,18 +538,11 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o severity: The severity of the error is returned here.
+%    o severity: the severity of the error is returned here.
 %
 */
-
-WandExport char *PixelIteratorGetException(const PixelIterator *iterator,
-  ExceptionType *severity)
-{
-  return(PixelGetIteratorException(iterator,severity));
-}
-
 WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
   ExceptionType *severity)
 {
@@ -433,24 +550,25 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
     *description;
 
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   assert(severity != (ExceptionType *) NULL);
-  *severity=iterator->exception.severity;
-  description=(char *) AcquireMagickMemory(2*MaxTextExtent);
+  *severity=iterator->exception->severity;
+  description=(char *) AcquireQuantumMemory(2UL*MaxTextExtent,
+    sizeof(*description));
   if (description == (char *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+      iterator->name);
   *description='\0';
-  if (iterator->exception.reason != (char *) NULL)
+  if (iterator->exception->reason != (char *) NULL)
     (void) CopyMagickString(description,GetLocaleExceptionMessage(
-      iterator->exception.severity,iterator->exception.reason),MaxTextExtent);
-  if (iterator->exception.description != (char *) NULL)
+      iterator->exception->severity,iterator->exception->reason),MaxTextExtent);
+  if (iterator->exception->description != (char *) NULL)
     {
       (void) ConcatenateMagickString(description," (",MaxTextExtent);
       (void) ConcatenateMagickString(description,GetLocaleExceptionMessage(
-        iterator->exception.severity,iterator->exception.description),
+        iterator->exception->severity,iterator->exception->description),
         MaxTextExtent);
       (void) ConcatenateMagickString(description,")",MaxTextExtent);
     }
@@ -462,6 +580,70 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P i x e l G e t E x c e p t i o n T y p e                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelGetIteratorExceptionType() the exception type associated with the wand.
+%  If no exception has occurred, UndefinedExceptionType is returned.
+%
+%  The format of the PixelGetIteratorExceptionType method is:
+%
+%      ExceptionType PixelGetIteratorExceptionType(const PixelWand *wand)
+%
+%  A description of each parameter follows:
+%
+%    o wand: the magick wand.
+%
+*/
+WandExport ExceptionType PixelGetIteratorExceptionType(
+  const PixelIterator *wand)
+{
+  assert(wand != (const PixelIterator *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  return(wand->exception->severity);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   P i x e l G e t I t e r a t o r R o w                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelGetIteratorRow() returns the current pixel iterator row.
+%
+%  The format of the PixelGetIteratorRow method is:
+%
+%      MagickBooleanType PixelGetIteratorRow(PixelIterator *iterator)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: the pixel iterator.
+%
+*/
+WandExport long PixelGetIteratorRow(PixelIterator *iterator)
+{
+  assert(iterator != (const PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  return(iterator->y);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P i x e l G e t N e x t I t e r a t o r R o w                             %
 %                                                                             %
 %                                                                             %
@@ -471,64 +653,59 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
 %  PixelGetNextIteratorRow() returns the next row as an array of pixel wands
 %  from the pixel iterator.
 %
-%  The format of the PixelGetNextRow method is:
+%  The format of the PixelGetNextIteratorRow method is:
 %
-%      PixelWand **PixelGetNextRow(PixelIterator *iterator,
+%      PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
 %        unsigned long *number_wands)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o number_wands: The number of pixel wands.
+%    o number_wands: the number of pixel wands.
 %
 */
-
-WandExport PixelWand **PixelGetNextRow(PixelIterator *iterator)
-{
-  unsigned long
-    number_wands;
-
-  return(PixelGetNextIteratorRow(iterator,&number_wands));
-}
-
 WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
   unsigned long *number_wands)
 {
-  IndexPacket
+  register const IndexPacket
     *indexes;
 
   register const PixelPacket
-    *p;
+    *pixels;
 
   register long
     x;
 
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   *number_wands=0;
-  iterator->y++;
+  if (iterator->active != MagickFalse)
+    iterator->y++;
   if (PixelSetIteratorRow(iterator,iterator->y) == MagickFalse)
     return((PixelWand **) NULL);
-  p=AcquireCacheView(iterator->view,iterator->region.x,iterator->region.y+
-    iterator->y,iterator->region.width,1,&iterator->exception);
-  if (p == (const PixelPacket *) NULL)
-    return((PixelWand **) NULL);
-  indexes=GetCacheViewIndexes(iterator->view);
+  pixels=GetCacheViewVirtualPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1,
+    iterator->exception);
+  if (pixels == (const PixelPacket *) NULL)
+    {
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
+      return((PixelWand **) NULL);
+    }
+  indexes=GetCacheViewVirtualIndexQueue(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
-  {
-    PixelSetQuantumColor(iterator->pixel_wand[x],p);
-    if (iterator->view->image->colorspace == CMYKColorspace)
-      PixelSetBlackQuantum(iterator->pixel_wand[x],indexes[x]);
-    else
-      if (iterator->view->image->storage_class == PseudoClass)
-        PixelSetIndex(iterator->pixel_wand[x],indexes[x]);
-    p++;
-  }
+    PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
+      PixelSetBlackQuantum(iterator->pixel_wands[x],indexes[x]);
+  if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+    for (x=0; x < (long) iterator->region.width; x++)
+        PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
   *number_wands=iterator->region.width;
-  return(iterator->pixel_wand);
+  return(iterator->pixel_wands);
 }
 
 /*
@@ -545,16 +722,16 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
 %  PixelGetPreviousIteratorRow() returns the previous row as an array of pixel
 %  wands from the pixel iterator.
 %
-%  The format of the PixelGetPreviousRow method is:
+%  The format of the PixelGetPreviousIteratorRow method is:
 %
-%      PixelWand **PixelGetPreviousRow(PixelIterator *iterator,
+%      PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
 %        unsigned long *number_wands)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
-%    o number_wands: The number of pixel wands.
+%    o number_wands: the number of pixel wands.
 %
 */
 
@@ -569,40 +746,44 @@ WandExport PixelWand **PixelGetPreviousRow(PixelIterator *iterator)
 WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
   unsigned long *number_wands)
 {
-  IndexPacket
+  register const IndexPacket
     *indexes;
 
   register const PixelPacket
-    *p;
+    *pixels;
 
   register long
     x;
 
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   *number_wands=0;
-  iterator->y--;
+  if (iterator->active != MagickFalse)
+    iterator->y--;
   if (PixelSetIteratorRow(iterator,iterator->y) == MagickFalse)
     return((PixelWand **) NULL);
-  p=AcquireCacheView(iterator->view,iterator->region.x,iterator->region.y+
-    iterator->y,iterator->region.width,1,&iterator->exception);
-  if (p == (const PixelPacket *) NULL)
-    return((PixelWand **) NULL);
-  indexes=GetCacheViewIndexes(iterator->view);
+  pixels=GetCacheViewVirtualPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1,
+    iterator->exception);
+  if (pixels == (const PixelPacket *) NULL)
+    {
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
+      return((PixelWand **) NULL);
+    }
+  indexes=GetCacheViewVirtualIndexQueue(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
-  {
-    PixelSetQuantumColor(iterator->pixel_wand[x],p);
-    if (iterator->view->image->colorspace == CMYKColorspace)
-      PixelSetBlackQuantum(iterator->pixel_wand[x],indexes[x]);
-    else
-      if (iterator->view->image->storage_class == PseudoClass)
-        PixelSetIndex(iterator->pixel_wand[x],indexes[x]);
-    p++;
-  }
+    PixelSetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
+      PixelSetBlackQuantum(iterator->pixel_wands[x],indexes[x]);
+  if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+    for (x=0; x < (long) iterator->region.width; x++)
+      PixelSetIndex(iterator->pixel_wands[x],indexes[x]);
   *number_wands=iterator->region.width;
-  return(iterator->pixel_wand);
+  return(iterator->pixel_wands);
 }
 
 /*
@@ -617,7 +798,7 @@ WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  PixelResetIterator() resets the pixel iterator.  Use it in conjunction
-%  with PixelGetNextPixel() to iterate over all the pixels in a pixel
+%  with PixelGetNextIteratorRow() to iterate over all the pixels in a pixel
 %  container.
 %
 %  The format of the PixelResetIterator method is:
@@ -626,16 +807,17 @@ WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
+%    o iterator: the pixel iterator.
 %
 */
 WandExport void PixelResetIterator(PixelIterator *iterator)
 {
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->y=(-1);
+  iterator->active=MagickFalse;
+  iterator->y=0;
 }
 
 /*
@@ -657,16 +839,17 @@ WandExport void PixelResetIterator(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The magick iterator.
+%    o iterator: the magick iterator.
 %
 */
 WandExport void PixelSetFirstIteratorRow(PixelIterator *iterator)
 {
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->y=iterator->region.y-1;
+  iterator->active=MagickFalse;
+  iterator->y=iterator->region.y;
 }
 
 /*
@@ -689,19 +872,19 @@ WandExport void PixelSetFirstIteratorRow(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
-%
+%    o iterator: the pixel iterator.
 %
 */
 WandExport MagickBooleanType PixelSetIteratorRow(PixelIterator *iterator,
   const long row)
 {
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   if ((row < 0) || (row >= (long) iterator->region.height))
     return(MagickFalse);
+  iterator->active=MagickTrue;
   iterator->y=row;
   return(MagickTrue);
 }
@@ -725,16 +908,17 @@ WandExport MagickBooleanType PixelSetIteratorRow(PixelIterator *iterator,
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The magick iterator.
+%    o iterator: the magick iterator.
 %
 */
 WandExport void PixelSetLastIteratorRow(PixelIterator *iterator)
 {
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->y=(long) iterator->region.height;
+  iterator->active=MagickFalse;
+  iterator->y=(long) iterator->region.height-1;
 }
 
 /*
@@ -756,46 +940,48 @@ WandExport void PixelSetLastIteratorRow(PixelIterator *iterator)
 %
 %  A description of each parameter follows:
 %
-%    o iterator: The pixel iterator.
-%
+%    o iterator: the pixel iterator.
 %
 */
 WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
 {
-  IndexPacket
-    *indexes;
+  ExceptionInfo
+    *exception;
+
+  register IndexPacket
+    *__restrict indexes;
 
   register long
     x;
 
   register PixelPacket
-    *p;
+    *__restrict pixels;
 
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  p=GetCacheView(iterator->view,iterator->region.x,iterator->region.y+
-    iterator->y,iterator->region.width,1);
-  if (p == (PixelPacket *) NULL)
+  if (SetCacheViewStorageClass(iterator->view,DirectClass) == MagickFalse)
+    return(MagickFalse);
+  exception=iterator->exception;
+  pixels=GetCacheViewAuthenticPixels(iterator->view,iterator->region.x,
+    iterator->region.y+iterator->y,iterator->region.width,1,exception);
+  if (pixels == (PixelPacket *) NULL)
     {
-      InheritException(&iterator->exception,&iterator->view->image->exception);
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
       return(MagickFalse);
     }
-  indexes=GetCacheViewIndexes(iterator->view);
+  indexes=GetCacheViewAuthenticIndexQueue(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
-  {
-    PixelGetQuantumColor(iterator->pixel_wand[x],p);
-    if (iterator->view->image->colorspace == CMYKColorspace)
-      indexes[x]=PixelGetBlackQuantum(iterator->pixel_wand[x]);
-    else
-      if (iterator->view->image->storage_class == PseudoClass)
-        indexes[x]=PixelGetIndex(iterator->pixel_wand[x]);
-    p++;
-  }
-  if (SyncCacheView(iterator->view) == MagickFalse)
+    PixelGetQuantumColor(iterator->pixel_wands[x],pixels+x);
+  if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+    for (x=0; x < (long) iterator->region.width; x++)
+      indexes[x]=PixelGetBlackQuantum(iterator->pixel_wands[x]);
+  if (SyncCacheViewAuthenticPixels(iterator->view,exception) == MagickFalse)
     {
-      InheritException(&iterator->exception,&iterator->view->image->exception);
+      InheritException(iterator->exception,GetCacheViewException(
+        iterator->view));
       return(MagickFalse);
     }
   return(MagickTrue);
